! #####################################
! # Uniform-pdf model for Arrhenius
! # nonlinearity
! #####################################

FUNCTION PDF_UNIFORM_ARRH( mean, var, inlf, sgs_tact, &
     sgs_zst, sgs_smooth, d0, d1, c0, c1, c2 )

  IMPLICIT NONE

#include "types.h"

  TREAL mean,var,sgs_tact,sgs_zst,sgs_smooth,d0,d1,c0,c1,c2,&
       PDF_UNIFORM_ARRH
  TINTEGER inlf

  TREAL WORK,ABSERR,EPSABS,EPSREL
  TINTEGER IWORK,KEY,LAST,LENW,LIMIT,IER,NEVAL
  DIMENSION IWORK(100),WORK(400)

  TREAL PDF_UNIFORM_ARRH_INT, p(11), ARRHENIUS
  EXTERNAL PDF_UNIFORM_ARRH_INT
  TREAL DEL, C, A, B, A1, A2, meanhalf, lim_ab, lim_bc, &
       c13, c23, FUN, FUN2

  c13 = C_1_R/C_3_R
  c23 = C_2_R/C_3_R

  meanhalf = MIN(mean,(C_1_R-mean))
  lim_ab = var - meanhalf*(c23-meanhalf) 
  lim_bc = var - c13*meanhalf*meanhalf 

  IF ( lim_ab .GE. C_0_R ) THEN
     DEL = C_1_R
     C = C_6_R*(mean*(C_1_R-mean)-var)
     A = 1 - mean - C_05_R*C
     B = mean - C_05_R*C
     A1 = C_0_R
     A2 = C_1_R
  ELSE IF (lim_ab .LT. C_0_R .AND. lim_bc .GT. C_0_R ) THEN
     DEL = C_1_5_R*(meanhalf*meanhalf+var)/meanhalf
     C = C_2_R*meanhalf/(DEL*DEL)
     IF ( mean .GT. C_05_R ) THEN
        A = C_0_R
        B = C_1_R-C*DEL
        A1 = C_1_R - DEL
        A2 = C_1_R
     ELSE
        A = C_1_R-C*DEL
        B = C_0_R
        A1 = C_0_R
        A2 = DEL
     ENDIF
  ELSE
     DEL = SQRT(C_12_R*var)
     C = C_1_R/DEL
     A = C_0_R
     B = C_0_R
     A1 = mean - C_05_R*DEL
     A2 = mean + C_05_R*DEL
  ENDIF

! Convergence constants (check the value of the function)
  EPSABS = C_1EM5_R
  EPSREL = C_1EM5_R
  KEY = 6
  LIMIT = 100
  LENW = LIMIT*4

  p(3) = sgs_tact
  p(4) = sgs_zst
  p(5) = sgs_smooth
  p(6) = d0
  p(7) = d1
  p(8) = c0
  p(9) = c1
  p(10) = c2
  p(11) = M_REAL(inlf)

  IF ( ( inlf .EQ. 2 .OR. inlf .EQ. 4 ) .AND.&
       ( A1 .LT. sgs_zst .AND. A2 .GT. sgs_zst ) ) THEN
     CALL QUADAG(PDF_UNIFORM_ARRH_INT,p,A1,sgs_zst,EPSABS,&
          EPSREL,KEY,FUN,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)
     CALL QUADAG(PDF_UNIFORM_ARRH_INT,p,sgs_zst,A2,EPSABS,&
          EPSREL,KEY,FUN2,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)
     FUN = FUN + FUN2
  ELSE
     CALL QUADAG(PDF_UNIFORM_ARRH_INT,p,A1,A2,EPSABS,&
          EPSREL,KEY,FUN,ABSERR,NEVAL,IER,LIMIT,&
          LENW,LAST,IWORK,WORK)
  ENDIF

  PDF_UNIFORM_ARRH = FUN*C + &
       A*ARRHENIUS(inlf,C_0_R,p(3),p(4),p(5),&
       p(6),p(7),p(8),p(9),p(10)) + &
       B*ARRHENIUS(inlf,C_1_R,p(3),p(4),p(5),&
       p(6),p(7),p(8),p(9),p(10))

999 RETURN
END FUNCTION PDF_UNIFORM_ARRH

! #######################
! # Integrand
! #######################

FUNCTION PDF_UNIFORM_ARRH_INT(x,p)

  IMPLICIT NONE

#include "types.h"

  TREAL x, p(*), PDF_UNIFORM_ARRH_INT

  TINTEGER inlf
  TREAL ARRHENIUS, UNIFORMPDF, zero, one
  EXTERNAL ARRHENIUS, UNIFORMPDF

  zero = C_1EM5_R
  one = C_1_R - zero

!  IF ( x.LE.zero .OR. x.GE.one ) THEN
!     PDF_UNIFORM_ARRH_INT = C_0_R
!  ELSE
  inlf = INT(p(11))
  PDF_UNIFORM_ARRH_INT = ARRHENIUS(inlf,&
       x,p(3),p(4),p(5),p(6),p(7),p(8),p(9),p(10))
!     PDF_UNIFORM_ARRH_INT = UNIFORMPDF(p(1),p(2),x)*ARRHENIUS(inlf,
! $        x,p(3),p(4),p(5),p(6),p(7),p(8),p(9),p(10))
!  ENDIF

  RETURN
END FUNCTION PDF_UNIFORM_ARRH_INT

! #####################################
! # Uniform-pdf model for polynomial 
! # nonlinearity
! #####################################

FUNCTION PDF_UNIFORM_POLY( mean, var, nord )

  IMPLICIT NONE

#include "types.h"

  TREAL mean, var, PDF_UNIFORM_POLY
  TINTEGER nord

  TREAL B, C, DEL, c13, c23, meanhalf, lim_ab, lim_bc
  c13 = C_1_R/C_3_R
  c23 = C_2_R/C_3_R

  meanhalf = MIN(mean,(C_1_R-mean))
  lim_ab = var - meanhalf*(c23-meanhalf) 
  lim_bc = var - c13*meanhalf*meanhalf 

  IF ( lim_ab .GE. C_0_R ) THEN
     C = C_6_R*(mean*(C_1_R-mean)-var)
     B = mean - C_05_R*C
     PDF_UNIFORM_POLY = B + C/M_REAL(nord+1)
  ELSE IF (lim_ab .LT. C_0_R .AND. lim_bc .GT. C_0_R ) THEN
     DEL = C_1_5_R*(meanhalf*meanhalf+var)/meanhalf
     C = C_2_R*meanhalf/(DEL*DEL)
     B = C_1_R-C*DEL
     IF ( mean .GT. C_05_R ) THEN
        PDF_UNIFORM_POLY = B + C*(C_1_R-(C_1_R-DEL)**(nord+1))&
             /M_REAL(nord+1)
     ELSE
        PDF_UNIFORM_POLY = C*(DEL**(nord+1))/M_REAL(nord+1)
     ENDIF
  ELSE
     DEL = SQRT(C_12_R*var)
     PDF_UNIFORM_POLY = ((mean+C_05_R*DEL)**(nord+1)-&
          (mean-C_05_R*DEL)**(nord+1))/M_REAL(nord+1)/DEL
  ENDIF

999 RETURN
END FUNCTION PDF_UNIFORM_POLY

! #####################################
! # Uniform-pdf distribution 
! #####################################

SUBROUTINE UNIFPDF( mean, var, pdf, n )

  IMPLICIT NONE

#include "types.h"

  TINTEGER n
  TREAL mean, var, pdf(n+2)

  TREAL A1, A2, A, B, C, DEL, c13, c23, &
       meanhalf, lim_ab, lim_bc, x
  TINTEGER i
  c13 = C_1_R/C_3_R
  c23 = C_2_R/C_3_R

  meanhalf = MIN(mean,(C_1_R-mean))
  lim_ab = var - meanhalf*(c23-meanhalf) 
  lim_bc = var - c13*meanhalf*meanhalf 

  IF ( lim_ab .GE. C_0_R ) THEN
     DEL = C_1_R
     C = C_6_R*(mean*(C_1_R-mean)-var)
     A = 1 - mean - C_05_R*C
     B = mean - C_05_R*C
     A1 = C_0_R
     A2 = C_1_R
  ELSE IF (lim_ab .LT. C_0_R .AND. lim_bc .GT. C_0_R ) THEN
     DEL = C_1_5_R*(meanhalf*meanhalf+var)/meanhalf
     C = C_2_R*meanhalf/(DEL*DEL)
     IF ( mean .GT. C_05_R ) THEN
        A = C_0_R
        B = C_1_R-C*DEL
        A1 = C_1_R - DEL
        A2 = C_1_R
     ELSE
        A = C_1_R-C*DEL
        B = C_0_R
        A1 = C_0_R
        A2 = DEL
     ENDIF
  ELSE
     DEL = SQRT(C_12_R*var)
     C = C_1_R/DEL
     A = C_0_R
     B = C_0_R
     A1 = mean - C_05_R*DEL
     A2 = mean + C_05_R*DEL
  ENDIF

  DO i = 1,n
     x = (M_REAL(i)-C_05_R)/M_REAL(n)
     IF ( x .GE. A1 .AND. x .LE. A2 ) THEN
        pdf(i) = C
     ELSE
        pdf(i) = C_0_R
     ENDIF
  ENDDO

  pdf(n+1) = A
  pdf(n+2) = B

999 RETURN
END SUBROUTINE UNIFPDF

! #####################################

FUNCTION UNIFORMPDF(A,B,X)

  IMPLICIT NONE

#include "types.h"

  TREAL UNIFORMPDF, X,A,B

  TREAL dummy
  TINTEGER IER

  IER=0
  IF ( B .LE. C_0_R ) IER=1
  IF ( IER .NE. 0 ) GO TO 999

  dummy = ABS((X-A)/B*C_2_R)
  IF ( dummy .GT. C_1_R ) THEN
     UNIFORMPDF = C_0_R
  ELSE
     UNIFORMPDF = C_1_R/B
  ENDIF

999 IF ( IER .NE. 0 ) THEN
     WRITE(*,*) 'mathlib : uniform : ier = ',IER
     STOP
  ENDIF

  RETURN
END FUNCTION UNIFORMPDF
