SUBROUTINE SGS_ARM_RANS_A(imax, jmax, kmax, kmax_total, n0,&
     iflg_density, iunifx, iunify, iunifz, i1bc, j1bc, k1bc, area, &
     dx, dy, dz, rho, barvs,&
     mu, mu2sgs, a0, a1, a2, cf0x, cf0y, cf0z, tmp, wrk3d)

  ! # barvs is 1/bar0(bar0(rho))
  ! #
  ! # Calculate the model coeffcient c0 given the filtered field mu and the
  ! # expected value of the subfilter variance mu2sgs 
  ! # (Mellado et. al. 2003, Eq.(24))
  ! #
  ! # tmp size is 4

  IMPLICIT NONE

#include "types.h"
#include "nfile.h"
#include "integers.h"

  TINTEGER imax, jmax, kmax, kmax_total, n0, iflg_density
  TINTEGER iunifx, iunify, iunifz, i1bc, j1bc, k1bc
  TREAL area
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL rho(imax,jmax,kmax)
  TREAL mu(imax,jmax,kmax)
  TREAL barvs(imax,jmax,kmax)
  TREAL mu2sgs(*)
  TREAL a0(*)
  TREAL a1(*)
  TREAL a2(*)
  TREAL tmp(imax,jmax,kmax,*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL cf0x(*), cf0y(*), cf0z(*)

  TINTEGER ij, j, k
  TREAL r0, disc, AVG_IK

  r0 = C_0_R

#define BAR_MU(i,j,k)   tmp(i,j,k,1)
#define MU_SG(i,j,k)    tmp(i,j,k,2)
#define TMP1_LOC(i,j,k) tmp(i,j,k,3)
#define TMP2_LOC(i,j,k) tmp(i,j,k,4)

  a0(1:jmax) = C_0_R
  a1(1:jmax) = C_0_R
  a2(1:jmax) = C_0_R

  ! Defining BAR_MU and MU_SG
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, mu, &
       BAR_MU(1,1,1), wrk3d)

  DO k = 1,kmax
     DO ij = 1,imax*jmax
        MU_SG(ij,1,k) = mu(ij,1,k) - BAR_MU(ij,1,k)
     ENDDO
  ENDDO

  ! ##############
  ! # Polynomial #
  ! ##############
  ! --------------------------------------------------------------------
  !                                                   Coefficient a0
  ! --------------------------------------------------------------------
  IF ( iflg_density .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = mu(ij,1,k)*mu(ij,1,k)/rho(ij,1,k)
        ENDDO
     ENDDO
  ELSE
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = mu(ij,1,k)*mu(ij,1,k)
        ENDDO
     ENDDO
  ENDIF

  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, TMP1_LOC(1,1,1), &
       TMP2_LOC(1,1,1), wrk3d)
  DO j = 1,jmax
     a0(j) = AVG_IK(imax, jmax, kmax, j, &
          TMP2_LOC(1,1,1), dx, dz, area)
  ENDDO

  IF ( iflg_density .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = BAR_MU(ij,1,k)*BAR_MU(ij,1,k)*&
                barvs(ij,1,k)
        ENDDO
     ENDDO
  ELSE
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = BAR_MU(ij,1,k)*BAR_MU(ij,1,k)
        ENDDO
     ENDDO
  ENDIF

  DO j = 1,jmax
     a0(j) = a0(j) - AVG_IK(imax, jmax, kmax, j, &
          TMP1_LOC(1,1,1), dx, dz, area)
  ENDDO

  DO j = 1,jmax
     a0(j) = a0(j) - mu2sgs(j)
  ENDDO

  ! --------------------------------------------------------------------
  !                                                   Coefficient a1
  ! --------------------------------------------------------------------
  IF ( iflg_density .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = mu(ij,1,k)*MU_SG(ij,1,k)/rho(ij,1,k)
        ENDDO
     ENDDO
  ELSE
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = mu(ij,1,k)*MU_SG(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, TMP1_LOC(1,1,1), &
       TMP2_LOC(1,1,1), wrk3d)
  DO j = 1,jmax
     a1(j) = AVG_IK(imax, jmax, kmax, j, &
          TMP2_LOC(1,1,1), dx, dz, area)
  ENDDO

  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, MU_SG(1,1,1), &
       TMP1_LOC(1,1,1), wrk3d)
  IF ( iflg_density .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP2_LOC(ij,1,k) = BAR_MU(ij,1,k)*TMP1_LOC(ij,1,k)*&
                barvs(ij,1,k)
        ENDDO
     ENDDO
  ELSE
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP2_LOC(ij,1,k) = BAR_MU(ij,1,k)*TMP1_LOC(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
  DO j = 1,jmax
     a1(j) = C_2_R*(a1(j) - AVG_IK(imax, jmax, kmax, j, &
          TMP2_LOC(1,1,1), dx, dz, area))
  ENDDO

  ! --------------------------------------------------------------------
  !                                                   Coefficient a2
  ! --------------------------------------------------------------------
  IF ( iflg_density .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = TMP1_LOC(ij,1,k)*TMP1_LOC(ij,1,k)*&
                barvs(ij,1,k)
        ENDDO
     ENDDO
  ELSE
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = TMP1_LOC(ij,1,k)*TMP1_LOC(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
  DO j = 1,jmax
     a2(j) =-AVG_IK(imax, jmax, kmax, j, &
          TMP1_LOC(1,1,1), dx, dz, area)
  ENDDO

  IF ( iflg_density .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = MU_SG(ij,1,k)*MU_SG(ij,1,k)/rho(ij,1,k)
        ENDDO
     ENDDO
  ELSE
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           TMP1_LOC(ij,1,k) = MU_SG(ij,1,k)*MU_SG(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, TMP1_LOC(1,1,1), &
       TMP2_LOC(1,1,1), wrk3d)
  DO j = 1,jmax
     a2(j) = a2(j) + AVG_IK(imax, jmax, kmax, j, &
          TMP2_LOC(1,1,1), dx, dz, area)
  ENDDO

#ifdef _DEBUG
  CALL IO_WRITE_ASCII_FIELD('a0_rans.dat', i1, jmax, i1, a0)
  CALL IO_WRITE_ASCII_FIELD('a1_rans.dat', i1, jmax, i1, a1)
  CALL IO_WRITE_ASCII_FIELD('a2_rans.dat', i1, jmax, i1, a2)
#endif

#undef BAR_MU
#undef MU_SG
#undef TMP1_LOC
#undef TMP2_LOC

  RETURN
END SUBROUTINE SGS_ARM_RANS_A
