! ################################################
! # Compute the model coefficient c0 as a function
! # of gamma and Ret according to ARM model
! #
! # Definition of model spectrum
! # 
! # p(1) - cl
! # p(2) - beta 
! # p(3) - ret
! # 
! ################################################

      SUBROUTINE SGS_ARM_TABLE_A(ng, remin, dre, nre, vg, vret, c0)

      IMPLICIT NONE

#include "types.h"
#include "dns_error.h"
#include "nfile.h"
     
      TINTEGER ng, nre
      TREAL remin, dre
      TREAL c0(ng, nre)
      TREAL vg(ng, nre)
      TREAL vret(nre)

      TREAL gamma, rg
      TREAL params(9)
      TREAL ret
      TREAL A0, A1, A2, c34
      TINTEGER ire, ig
      TREAL A,B,EPSABS
      TINTEGER IER
      TREAL ESP_A_A0, ESP_A_A1, ESP_A_A2
      EXTERNAL ESP_A_A0, ESP_A_A1, ESP_A_A2
      CHARACTER*64 message

      c34 = C_3_R/C_4_R
      message = 'SGS_ARM_TABLE_A :Error in integration IER = '

! Integration and Iteration constants
      EPSABS = C_1EM8_R
      
! Parameters defining the spectrum
#ifdef SINGLE_PREC
! params(2) = 8.e0
! params(3) = 1.4e0
! params(6) = 1/1.67971e0
! params(7) = 1.25e0
! params(8) = 1.5e0
      params(2) = 9.0e0
      params(3) = 0.5e0
      params(6) = 1e0/2.8672e0
      params(7) = 1.2e0
      params(8) = 1.5e0
#else 
      params(2) = 9.0d0
      params(3) = 0.5d0
      params(6) = 1d0/2.8672d0
      params(7) = 1.2d0
      params(8) = 1.5d0
#endif

! Lower integration limit
      A = C_0_R

! Loop over the Reynolds number
      ret = remin
      DO ire = 1,nre

! Set reynolds table to log(ret)
         vret(ire) = LOG(ret)

! turbulent reynolds number
         params(4) = ret

! Minimum gamma is eta/2 L = (Re**-3/4)/2
         gamma = C_05_R/(ret**c34)

! Gamma factor is set so last gamma = 1/2 (Delta_f == L)
         rg = EXP(c34*LOG(ret)/M_REAL(ng-1))
         DO ig = 1, ng

! Set integral scale array to log
            vg(ig,ire) = LOG(gamma)

! Gamma parameter of model
            params(9) = gamma

! Calculate c0(Ret,gama)
            B = C_10_R/gamma
            CALL INTEGSPC(ESP_A_A0,params,A,B,EPSABS,A0,IER)
            IF (IER.NE.0) THEN
               CALL CONCATI(message,IER)
               CALL IO_WRITE_ASCII(efile,message)
               CALL DNS_STOP(LES_ERROR_ARM_INI)
            ENDIF
            CALL INTEGSPC(ESP_A_A1,params,A,B,EPSABS,A1,IER)
            IF (IER.NE.0) THEN
               CALL CONCATI(message,IER)
               CALL IO_WRITE_ASCII(efile,message)
               CALL DNS_STOP(LES_ERROR_ARM_INI)
            ENDIF
            CALL INTEGSPC(ESP_A_A2,params,A,B,EPSABS,A2,IER)
            IF (IER.NE.0) THEN
               CALL CONCATI(message,IER)
               CALL IO_WRITE_ASCII(efile,message)
               CALL DNS_STOP(LES_ERROR_ARM_INI)
            ENDIF
            
! Calculate c0 from root of quadratic equation
            c0(ig,ire) = (-A1+SQRT(A1**2-C_4_R*A0*A2))*C_05_R/A2

! Increase gamma
            gamma = gamma*rg
         ENDDO

! Increase Ret
         ret = ret*dre
      ENDDO

      RETURN
      END
      
