! #############################################
! # Kolmogorov model spectrum (from Pope)
! #
! # p(1) - p
! # p(3) - cl
! # p(6) - cK
! # p(7) - m (slope)
! #
! #############################################

      FUNCTION ESP_K(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c2, c34
      TREAL ESP_K

      c2 = (p(1)+p(7))*C_05_R
      c34 = C_3_R/C_4_R

      ESP_K = p(6)*(x**p(1))/((x*x+p(3))**c2)

      RETURN
      END

! ##################################################

      FUNCTION ESP_K_CL(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c2, c34
      TREAL ESP_K_CL

      c2 = (p(1)+p(7))*C_05_R
      c34 = C_3_R/C_4_R

      ESP_K_CL = -p(6)*c2*(x**p(1))/((x*x+p(3))**(c2+C_1_R))

      RETURN
      END
 
! ##################################################

      FUNCTION ESP_K_A0(x, params)

      IMPLICIT NONE

#include "types.h"

      TREAL x, params(*)
      TREAL G, S, norm
      TREAL FIL_1, ESP_K, ESP_K_A0

      S = params(9)*x
      G = FIL_1(S)
      norm = ESP_K(C_1_R/params(9),params)

      ESP_K_A0 = -ESP_K(x, params)&
           *(C_1_R-G*G)*(C_1_R-G*G)
      ESP_K_A0 = ESP_K_A0/norm

      RETURN
      END

! ##################################################

      FUNCTION ESP_K_A1(x, params)

      IMPLICIT NONE

#include "types.h"

      TREAL x, params(*)
      TREAL G, S, G2, norm
      TREAL FIL_1, ESP_K, ESP_K_A1

      S = params(9)*x
      G = FIL_1(S)
      norm = ESP_K(C_1_R/params(9),params)

      G2 = G*G
      ESP_K_A1 = C_2_R*ESP_K(x, params)*G2*&
           (C_1_R-G)*(C_1_R-G2)
      ESP_K_A1 = ESP_K_A1/norm

      RETURN
      END

! ##################################################

      FUNCTION ESP_K_A2(x, params)

      IMPLICIT NONE

#include "types.h"

      TREAL x, params(*)
      TREAL G, S, G2, norm
      TREAL FIL_1, ESP_K, ESP_K_A2

      S = params(9)*x
      G = FIL_1(S)
      norm = ESP_K(C_1_R/params(9),params)
      
      G2 = G*G
      ESP_K_A2 = ESP_K(x, params)&
           *G2*(C_1_R-G)*(C_1_R-G)*(C_1_R-G2)
      ESP_K_A2 = ESP_K_A2/norm

      RETURN
      END

! ##################################################

      FUNCTION ESP_K_CL_ROOT(cl, params)

      IMPLICIT NONE

#include "types.h"

      TREAL ESP_K_CL_ROOT
      TREAL cl
      TREAL params(*)

      TREAL A,B,ABSERR,EPSABS,EPSREL,FUN,DER
      TINTEGER IER,NEVAL
      TREAL WORK
      TINTEGER IWORK,KEY,LAST,LENW,LIMIT
      DIMENSION IWORK(100),WORK(400)
      TREAL ESP_K, ESP_K_CL
      EXTERNAL ESP_K, ESP_K_CL

! Integration Limits (if m is not 5/3 maybe B1 causes troubles, put it 1d7)
      A = C_0_R
      B = C_100_R*(params(6)/C_1EM3_R/(params(7)-C_1_R))**&
           (C_1_R/(params(7)-C_1_R))

! Convergence constants
      EPSABS = C_1EM8_R
      EPSREL = C_1EM8_R
      KEY = 6
      LIMIT = 100
      LENW = LIMIT*4

! Set the current value of cl
      params(3) = cl

      CALL QUADAG(ESP_K,params,A,B,EPSABS,EPSREL,KEY,FUN,&
           ABSERR,NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      CALL QUADAG(ESP_K_CL,params,A,B,EPSABS,EPSREL,KEY,DER,&
           ABSERR,NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      ESP_K_CL_ROOT = -(FUN-C_1_R)/DER

      RETURN
      END

! ##################################################
