! #############################################
! # Pope model spectrum
! #
! # p(1) - p
! # p(2) - beta
! # p(3) - cl
! # p(4) - ret
! # p(5) - cn
! # p(6) - cK
! # p(7) - m (slope)
! # p(8) - n (viscous exponent)
! # 
! #############################################

      FUNCTION ESP_P(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c34, c14, c2, eta
      TREAL ESP_P

      c34 = C_3_R/C_4_R
      c14 = C_1_R/C_4_R
      c2  = (p(1)+p(7))*C_05_R
      eta = x/EXP(c34*LOG(p(4)))

      eta = EXP(p(8)*LOG((eta**C_4_R+p(5)**C_4_R)**c14 - p(5))) 
      ESP_P = p(6)*(x**p(1))/((x*x+p(3))**c2)*&
           EXP(-p(2)*eta)

      RETURN
      END

! ##################################################

      FUNCTION DSP_P(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c2, c14, c32, eta, y
      TREAL DSP_P

      c2 = (p(1)+p(7))*C_05_R
      c32 = -C_3_R/C_2_R
      c14 = C_1_R/C_4_R

      y = (C_1_R-(C_3_R-p(7))*C_3_R/C_4_R)/C_3_R
      y = x*EXP(y*LOG(p(4)))      

      eta = EXP(p(8)*LOG((y**C_4_R+p(5)**C_4_R)**c14 - p(5))) 
      DSP_P = C_2_R*y*y*p(6)*(y**p(1))/&
           ( y*y + p(3)*EXP(c32*LOG(p(4))) )**c2*&
           EXP(-p(2)*eta)

      RETURN
      END

! ##################################################

      FUNCTION ESP_P_CL(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c2
      TREAL ESP_P_CL, ESP_P

      c2 = (p(1)+p(7))*C_05_R

      ESP_P_CL = -c2*ESP_P(x,p)/(x*x+p(3))

      RETURN
      END
 
! ##################################################

      FUNCTION DSP_P_CL(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c2, c32, y
      TREAL DSP_P_CL, DSP_P

      c2 = (p(1)+p(7))*C_05_R
      c32 = -C_3_R/C_2_R

      y = (C_1_R-(C_3_R-p(7))*C_3_R/C_4_R)/C_3_R
      y = x*EXP(y*LOG(p(4)))      

      DSP_P_CL = -c2*DSP_P(x,p)/( y*y + p(3)*EXP(c32*LOG(p(4))) )*&
           EXP(c32*LOG(p(4)))

      RETURN
      END

! ##################################################

      FUNCTION ESP_P_CN(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c14, c34, eta, dum
      TREAL ESP_P_CN, ESP_P

      c14 = C_1_R/C_4_R
      c34 = C_3_R/C_4_R
      eta = x/EXP(c34*LOG(p(4)))

      ESP_P_CN = -p(2)*ESP_P(x,p)*&
           ( (p(5)/(eta**C_4_R+p(5)**C_4_R)**c14)**C_3_R &
           - C_1_R )

      IF ( p(8) .NE. C_1_R ) THEN
         dum = p(8)*EXP((p(8)-C_1_R)*&
              LOG((eta**C_4_R+p(5)**C_4_R)**c14-p(5)))
         ESP_P_CN = ESP_P_CN*dum
      ENDIF

      RETURN
      END

! ##################################################

      FUNCTION DSP_P_CN(x, p)

      IMPLICIT NONE

#include "types.h"

      TREAL x, p(*)
      TREAL c14, dum, y
      TREAL DSP_P_CN, DSP_P

      c14 = C_1_R/C_4_R

      y = (C_1_R-(C_3_R-p(7))*C_3_R/C_4_R)/C_3_R
      y = x*EXP(y*LOG(p(4)))      

      DSP_P_CN = -p(2)*DSP_P(y,p)*&
           ( (p(5)/(y**C_4_R+p(5)**C_4_R)**c14)**C_3_R - C_1_R )

      IF ( p(8) .NE. C_1_R ) THEN
         dum = p(8)*EXP((p(8)-C_1_R)*&
              LOG((y**C_4_R+p(5)**C_4_R)**c14-p(5)))
         DSP_P_CN = DSP_P_CN*dum
      ENDIF
      
      RETURN
      END

! ##################################################
! # p(9) - gamma
! ##################################################

      FUNCTION ESP_P_A0(x, params)

      IMPLICIT NONE

#include "types.h"

      TREAL x, params(*)
      TREAL G, S, norm
      TREAL FIL_1, ESP_P, ESP_P_A0

      S = params(9)*x
      G = FIL_1(S)
      norm = ESP_P(C_1_R/params(9),params)

      ESP_P_A0 = -ESP_P(x, params)&
           *(C_1_R-G*G)*(C_1_R-G*G)
      ESP_P_A0 = ESP_P_A0/norm

      RETURN
      END

! ##################################################

      FUNCTION ESP_P_A1(x, params)

      IMPLICIT NONE

#include "types.h"

      TREAL x, params(*)
      TREAL G, S, G2, norm
      TREAL FIL_1, ESP_P, ESP_P_A1

      S = params(9)*x
      G = FIL_1(S)
      norm = ESP_P(C_1_R/params(9),params)

      G2 = G*G
      ESP_P_A1 = C_2_R*ESP_P(x, params)*G2*&
           (C_1_R-G)*(C_1_R-G2)
      ESP_P_A1 = ESP_P_A1/norm

      RETURN
      END

! ##################################################

      FUNCTION ESP_P_A2(x, params)

      IMPLICIT NONE

#include "types.h"

      TREAL x, params(*)
      TREAL G, S, G2, norm
      TREAL FIL_1, ESP_P, ESP_P_A2

      S = params(9)*x
      G = FIL_1(S)
      norm = ESP_P(C_1_R/params(9),params)
      
      G2 = G*G
      ESP_P_A2 = ESP_P(x, params)&
           *G2*(C_1_R-G)*(C_1_R-G)*(C_1_R-G2)
      ESP_P_A2 = ESP_P_A2/norm

      RETURN
      END

! ##################################################

      FUNCTION ESP_P_CL_ROOT(cl, params)

      IMPLICIT NONE

#include "types.h"

      TREAL ESP_P_CL_ROOT
      TREAL cl
      TREAL params(*)

      TREAL A,B,ABSERR,EPSABS,EPSREL,FUN,DER,dum
      TINTEGER IER,NEVAL
      TREAL WORK
      TINTEGER IWORK,KEY,LAST,LENW,LIMIT
      DIMENSION IWORK(100),WORK(400)

      TREAL ESP_P, ESP_P_CL
      EXTERNAL ESP_P, ESP_P_CL

! Integration Limits 
      A  = C_0_R
      IF ( params(7) .GT. C_1_R ) THEN
         B = C_10_R*(params(6)/C_1EM3_R/(params(7)-C_1_R))**&
              (C_1_R/(params(7)-C_1_R))
      ELSE
         B = C_BIG_R
      ENDIF
      dum = EXP(LOG(C_10_R/params(2))/params(8))*&
           params(4)**(C_3_R/C_4_R)
      B = MIN(dum,B)

! Convergence constants
      EPSABS = C_1EM8_R
      EPSREL = C_1EM8_R
      KEY = 6
      LIMIT = 100
      LENW = LIMIT*4

! Set the current value of cl
      params(3) = cl

      CALL QUADAG(ESP_P,params,A,B,EPSABS,EPSREL,KEY,FUN,&
           ABSERR,NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      CALL QUADAG(ESP_P_CL,params,A,B,EPSABS,EPSREL,KEY,DER,&
           ABSERR,NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)

      ESP_P_CL_ROOT = -(FUN-C_1_R)/DER

      RETURN
      END

! ##################################################

      SUBROUTINE ESP_P_CLCN_ROOT(cl, cn, params, d1, d2)

      IMPLICIT NONE

#include "types.h"

      TREAL cl, cn, d1, d2
      TREAL params(*)

      TREAL A,B,EPSABS
      TINTEGER IER

      TREAL f1, f2, f11, f12, f21, f22, det

      TREAL ESP_P, DSP_P, ESP_P_CL, DSP_P_CL, ESP_P_CN, DSP_P_CN
      EXTERNAL ESP_P, DSP_P, ESP_P_CL, DSP_P_CL, ESP_P_CN, DSP_P_CN 

! Integration parameters
      A = C_0_R
      B = C_5_R
      EPSABS = C_1EM8_R

! Set the current value of cl, cn
      params(3) = cl
      params(5) = cn

      CALL INTEGSPC(ESP_P,params,A,B,EPSABS,f1,IER)
      IF (IER.NE.0) THEN
         WRITE(*,*) 'Error in integration, ierr = ',IER
      ENDIF
      CALL INTEGSPC(DSP_P,params,A,B,EPSABS,f2,IER)
      IF (IER.NE.0) THEN
         WRITE(*,*) 'Error in integration, ierr = ',IER
      ENDIF

      CALL INTEGSPC(ESP_P_CL,params,A,B,EPSABS,f11,IER)
      IF (IER.NE.0) THEN
         WRITE(*,*) 'Error in integration, ierr = ',IER
      ENDIF
      CALL INTEGSPC(DSP_P_CL,params,A,B,EPSABS,f21,IER)
      IF (IER.NE.0) THEN
         WRITE(*,*) 'Error in integration, ierr = ',IER
      ENDIF
      CALL INTEGSPC(ESP_P_CN,params,A,B,EPSABS,f12,IER)
      IF (IER.NE.0) THEN
         WRITE(*,*) 'Error in integration, ierr = ',IER
      ENDIF
      CALL INTEGSPC(DSP_P_CN,params,A,B,EPSABS,f22,IER)
      IF (IER.NE.0) THEN
         WRITE(*,*) 'Error in integration, ierr = ',IER
      ENDIF

      f1 = f1 - C_1_R
      f2 = f2 - C_1_R
      det= f11*f22 - f12*f21
      d1 = (f12*f2 - f22*f1)/det
      d2 = (f21*f1 - f11*f2)/det

      RETURN
      END

! ##################################################

      FUNCTION ESP_P_CN_ROOT( cn, params )

      IMPLICIT NONE

#include "types.h"

      TREAL ESP_P_CN_ROOT
      TREAL cn, params(*)

      TREAL A,B,ABSERR,EPSABS,EPSREL
      TINTEGER IER,NEVAL
      TREAL WORK
      TINTEGER IWORK,KEY,LAST,LENW,LIMIT
      DIMENSION IWORK(100),WORK(400)
      TREAL FUN, DER, dum
      
      TREAL DSP_P, DSP_P_CN
      EXTERNAL DSP_P, DSP_P_CN

! Integration Limits 
      A  = C_0_R
      B = EXP( LOG(C_100_R/params(2)) / params(8) )

! Convergence constants
      EPSABS = C_1EM8_R
      EPSREL = C_1EM8_R
      KEY = 6
      LIMIT = 100
      LENW = LIMIT*4

! Set the current value of cn
      params(5) = cn

      CALL QUADAG(DSP_P,params,A,B,EPSABS,EPSREL,KEY,FUN,&
           ABSERR,NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)
      CALL QUADAG(DSP_P_CN,params,A,B,EPSABS,EPSREL,KEY,DER,&
           ABSERR,NEVAL,IER,LIMIT,LENW,LAST,IWORK,WORK)

      dum = C_1_R - (C_3_R-params(7))*C_3_R/C_4_R
      FUN = FUN - EXP(dum*LOG(params(4)))

      ESP_P_CN_ROOT = - FUN/DER

      RETURN
      END


