SUBROUTINE CGBC2U(imax, jmax, kmax, kmax_total, i1bc, j1bc, k1bc, dx, dy, dz, ci, cj, ck)

#include "types.h"
#include "dns_error.h"

#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER imax
  TINTEGER jmax
  TINTEGER kmax
  TINTEGER kmax_total
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TINTEGER i1bc
  TINTEGER j1bc
  TINTEGER k1bc
  TINTEGER i
  TINTEGER j
  TINTEGER k, kp
  TREAL ci(imax,2)
  TREAL cj(jmax,2)
  TREAL ck(kmax,2)
  TREAL diag
  TREAL chi
  TREAL mu
  TREAL eta

#ifdef PARALLEL
!  IF ( k1bc .NE. 0 ) THEN
!     CALL IO_WRITE_ASCII(efile, 'Poisson solver is periodic in k')
!     CALL DNS_STOP(DNS_ERROR_POISSON)
!  ENDIF
#endif

  chi = (dy(jmax/2)/dx(imax/2))**2

  IF ( kmax_total .GT. 1 ) THEN
     eta = (dy(jmax/2)/dz(kmax_total/2))**2
  ELSE
     eta = C_0_R
  ENDIF

  diag = C_2_R*(C_1_R+chi+eta)
  chi = chi / diag
  mu  = C_1_R / diag
  eta = eta / diag

  DO i=1, imax
     ci(i,1) = - chi
     ci(i,2) = - chi
     IF ( i1bc .NE. 0 ) THEN
        IF ( i .EQ. 1 ) THEN
           ci(i,1) = C_0_R
        ELSE IF ( i .EQ. imax ) THEN
           ci(i,2) = C_0_R
        ENDIF
     ENDIF
  ENDDO

  DO j=1, jmax
     cj(j,1) = - mu
     cj(j,2) = - mu
     IF ( j1bc .NE. 0 ) THEN
        IF ( j .EQ. 1 ) THEN
           cj(j,1) = C_0_R
        ELSE IF ( j .EQ. jmax ) THEN
           cj(j,2) = C_0_R
        ENDIF
     ENDIF
  ENDDO

  IF ( kmax_total .EQ. 1 ) THEN
     ck(1,1) = C_0_R
     ck(1,2) = C_0_R
  ELSE
     DO k=1, kmax
        ck(k,1) = - eta
        ck(k,2) = - eta
        IF ( k1bc .NE. 0 ) THEN
#ifdef PARALLEL
           kp = ims_offset_k + k
#else 
           kp = k
#endif
           IF ( kp .EQ. 1 ) THEN
              ck(k,1) = C_0_R
           ELSE IF ( kp .EQ. kmax_total ) THEN
              ck(k,2) = C_0_R
           ENDIF
        ENDIF
     ENDDO
  ENDIF

!#ifdef _DEBUG
!      DO i=1, imax
!         WRITE(*,*) i, ci(i,1), ci(i,2)
!      ENDDO
!      DO j=1, jmax
!         WRITE(*,*) j, cj(j,1), cj(j,2)
!      ENDDO
!      DO k=1, kmax
!         WRITE(*,*) k, ck(k,1), ck(k,2)
!      ENDDO
!#endif
  RETURN
END SUBROUTINE CGBC2U
