      SUBROUTINE CGMATRIX4(imax, jmax, kmax, kmax_total, &
           p, q, ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

#include "types.h"

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER kmax_total
      TINTEGER i
      TINTEGER j
      TINTEGER k
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)
      

! #################################################################
! #  Compute Matrix Product                                       #
! #
! #  Coefficients
! #  -------------------------------------------
! #  ci(i,1) = i-2  cj(j,1) = j-2  ck(k,1) = k-2
! #  ci(i,2) = i-1  cj(j,2) = j-1  ck(k,2) = k-1
! #  ci(i,3) = i+1  cj(j,3) = j+1  ck(k,3) = k+1
! #  ci(i,4) = i+2  cj(j,4) = j+2  ck(k,4) = k+2
! #
! #  Indexes
! #  -------------------------------------------------
! #  ipos(i,1) = i-2  jpos(j,1) = j-2  kpos(k,1) = k-2
! #  ipos(i,2) = i-1  jpos(j,2) = j-1  kpos(k,2) = k-1
! #  ipos(i,3) = i+1  jpos(j,3) = j+1  kpos(k,3) = k+1
! #  ipos(i,4) = i+2  jpos(j,4) = j+2  kpos(k,4) = k+2
! # 
! #  Boundaries
! #  ----------
! #  
! #  ---------- pboundary(i,j,4) ----------
! #  ---------- pboundary(i,j,3) ----------
! #  |XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX|
! #  |XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX|
! #  |XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX|
! #  ---------- pboundary(i,j,2) ----------
! #  ---------- pboundary(i,j,1) ----------
! # 
! #  Clashes ( (-) = boundary and (X) = inside domain )
! #  -------
! #
! #  kmax = 1   ->   BBTT
! #
! #  (-)(-)(X)(-)(-)     
! #
! #  kmax = 2   ->   BBT & BTT
! #
! #  (-)(-)(X)(X)(-)   &   (-)(X)(X)(-)(-)
! # 
! #  kmax = 3   ->   BB & BT & TT
! #
! #  (-)(-)(X)(X)(X)   &   (-)(X)(X)(X)(-)   &  (X)(X)(X)(-)(-)
! #
! #  kmax > 3   ->   BB & B & T & TT
! #  
! #  (-)(-)(X)(X)(X)   &   (-)(X)(X)(X)(X)   & (X)(X)(X)(X)(-)   &
! #  (X)(X)(X)(-)(-)
! #
! #################################################################

! #####################################
! # 2D Case                        
! #####################################

      IF ( kmax_total .EQ. 1 ) THEN

         DO j=1, jmax
            DO i=1, imax
               q(i,j,1) = p(i,j,1) &
                    + ci(i,1)*p(ipos(i,1),j,1)&
                    + ci(i,2)*p(ipos(i,2),j,1)&
                    + cj(j,1)*p(i,jpos(j,1),1)&
                    + cj(j,2)*p(i,jpos(j,2),1)&
                    + ci(i,3)*p(ipos(i,3),j,1)&
                    + ci(i,4)*p(ipos(i,4),j,1)&
                    + cj(j,3)*p(i,jpos(j,3),1)&
                    + cj(j,4)*p(i,jpos(j,4),1)
            ENDDO
         ENDDO

      ELSE
! ####################################
! # 3D Case
! ####################################

#ifdef PARALLEL
! ####################################
! # Parallel Case
! ####################################

         IF ( kmax .EQ. 1 ) THEN
            CALL CGMATRIX4_BBTT(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
         ELSE IF ( kmax .EQ. 2 ) THEN
            CALL CGMATRIX4_BBT(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
            CALL CGMATRIX4_BTT(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
         ELSE IF ( kmax .EQ. 3 ) THEN
            CALL CGMATRIX4_BB(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
            CALL CGMATRIX4_BT(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
            CALL CGMATRIX4_TT(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
         ELSE
            CALL CGMATRIX4_BB(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
            CALL CGMATRIX4_B(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)

            IF ( kmax .GT. 4 ) THEN
               DO k=3, kmax-2
                  DO j=1, jmax
                     DO i=1, imax
                        q(i,j,k) = p(i,j,k) &
                             + ci(i,1)*p(ipos(i,1),j,k)&
                             + ci(i,2)*p(ipos(i,2),j,k)&
                             + cj(j,1)*p(i,jpos(j,1),k)&
                             + cj(j,2)*p(i,jpos(j,2),k)&
                             + ck(k,1)*p(i,j,kpos(k,1))&
                             + ck(k,2)*p(i,j,kpos(k,2))&
                             + ci(i,3)*p(ipos(i,3),j,k)&
                             + ci(i,4)*p(ipos(i,4),j,k)&
                             + cj(j,3)*p(i,jpos(j,3),k)&
                             + cj(j,4)*p(i,jpos(j,4),k)&
                             + ck(k,3)*p(i,j,kpos(k,3))&
                             + ck(k,4)*p(i,j,kpos(k,4))
                     ENDDO
                  ENDDO
               ENDDO
            ENDIF
            
            CALL CGMATRIX4_T(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)
            
            CALL CGMATRIX4_TT(imax, jmax, kmax, p, q,&
                 ipos, jpos, kpos, ci, cj, ck, pboundary)

         ENDIF

#else
! ####################################
! # Serial Case
! ####################################

         DO k=1, kmax
            DO j=1, jmax
               DO i=1, imax
                  q(i,j,k) = p(i,j,k) &
                       + ci(i,1)*p(ipos(i,1),j,k)&
                       + ci(i,2)*p(ipos(i,2),j,k)&
                       + cj(j,1)*p(i,jpos(j,1),k)&
                       + cj(j,2)*p(i,jpos(j,2),k)&
                       + ck(k,1)*p(i,j,kpos(k,1))&
                       + ck(k,2)*p(i,j,kpos(k,2))&
                       + ci(i,3)*p(ipos(i,3),j,k)&
                       + ci(i,4)*p(ipos(i,4),j,k)&
                       + cj(j,3)*p(i,jpos(j,3),k)&
                       + cj(j,4)*p(i,jpos(j,4),k)&
                       + ck(k,3)*p(i,j,kpos(k,3))&
                       + ck(k,4)*p(i,j,kpos(k,4))
               ENDDO
            ENDDO
         ENDDO
#endif
      ENDIF

! #####################################
! # End Calculation of Matrix Product #
! #####################################

      RETURN
      END

      SUBROUTINE CGMATRIX4_BBTT(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,1) = p(i,j,1) &
                 + ci(i,1)*p(ipos(i,1),j,1)&
                 + ci(i,2)*p(ipos(i,2),j,1)&
                 + cj(j,1)*p(i,jpos(j,1),1)&
                 + cj(j,2)*p(i,jpos(j,2),1)&
                 + ck(1,1)*pboundary(i,j,1)&
                 + ck(1,2)*pboundary(i,j,2)&
                 + ci(i,3)*p(ipos(i,3),j,1)&
                 + ci(i,4)*p(ipos(i,4),j,1)&
                 + cj(j,3)*p(i,jpos(j,3),1)&
                 + cj(j,4)*p(i,jpos(j,4),1)&
                 + ck(1,3)*pboundary(i,j,3)&
                 + ck(1,4)*pboundary(i,j,4)
         ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE CGMATRIX4_BTT(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,kmax) = p(i,j,kmax) &
                 + ci(i,1)*p(ipos(i,1),j,kmax)&
                 + ci(i,2)*p(ipos(i,2),j,kmax)&
                 + cj(j,1)*p(i,jpos(j,1),kmax)&
                 + cj(j,2)*p(i,jpos(j,2),kmax)&
                 + ck(kmax,1)*pboundary(i,j,2)&
                 + ck(kmax,2)*p(i,j,kpos(kmax,2))&
                 + ci(i,3)*p(ipos(i,3),j,kmax)&
                 + ci(i,4)*p(ipos(i,4),j,kmax)&
                 + cj(j,3)*p(i,jpos(j,3),kmax)&
                 + cj(j,4)*p(i,jpos(j,4),kmax)&
                 + ck(kmax,3)*pboundary(i,j,3)&
                 + ck(kmax,4)*pboundary(i,j,4)
         ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE CGMATRIX4_BBT(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,1) = p(i,j,1) &
                 + ci(i,1)*p(ipos(i,1),j,1)&
                 + ci(i,2)*p(ipos(i,2),j,1)&
                 + cj(j,1)*p(i,jpos(j,1),1)&
                 + cj(j,2)*p(i,jpos(j,2),1)&
                 + ck(1,1)*pboundary(i,j,1)&
                 + ck(1,2)*pboundary(i,j,2)&
                 + ci(i,3)*p(ipos(i,3),j,1)&
                 + ci(i,4)*p(ipos(i,4),j,1)&
                 + cj(j,3)*p(i,jpos(j,3),1)&
                 + cj(j,4)*p(i,jpos(j,4),1)&
                 + ck(1,3)*p(i,j,kpos(1,3))&
                 + ck(1,4)*pboundary(i,j,3)
         ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE CGMATRIX4_BT(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,2) = p(i,j,2) &
                 + ci(i,1)*p(ipos(i,1),j,2)&
                 + ci(i,2)*p(ipos(i,2),j,2)&
                 + cj(j,1)*p(i,jpos(j,1),2)&
                 + cj(j,2)*p(i,jpos(j,2),2)&
                 + ck(2,1)*pboundary(i,j,2)&
                 + ck(2,2)*p(i,j,kpos(2,2))&
                 + ci(i,3)*p(ipos(i,3),j,2)&
                 + ci(i,4)*p(ipos(i,4),j,2)&
                 + cj(j,3)*p(i,jpos(j,3),2)&
                 + cj(j,4)*p(i,jpos(j,4),2)&
                 + ck(2,3)*p(i,j,kpos(2,3))&
                 + ck(2,4)*pboundary(i,j,3)
         ENDDO
      ENDDO

      RETURN
      END

      SUBROUTINE CGMATRIX4_BB(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,1) = p(i,j,1) &
                 + ci(i,1)*p(ipos(i,1),j,1)&
                 + ci(i,2)*p(ipos(i,2),j,1)&
                 + cj(j,1)*p(i,jpos(j,1),1)&
                 + cj(j,2)*p(i,jpos(j,2),1)&
                 + ck(1,1)*pboundary(i,j,1)&
                 + ck(1,2)*pboundary(i,j,2)&
                 + ci(i,3)*p(ipos(i,3),j,1)&
                 + ci(i,4)*p(ipos(i,4),j,1)&
                 + cj(j,3)*p(i,jpos(j,3),1)&
                 + cj(j,4)*p(i,jpos(j,4),1)&
                 + ck(1,3)*p(i,j,kpos(1,3))&
                 + ck(1,4)*p(i,j,kpos(1,4))
         ENDDO
      ENDDO

      RETURN
      END


      SUBROUTINE CGMATRIX4_TT(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,kmax) = p(i,j,kmax) &
                 + ci(i,1)*p(ipos(i,1),j,kmax)&
                 + ci(i,2)*p(ipos(i,2),j,kmax)&
                 + cj(j,1)*p(i,jpos(j,1),kmax)&
                 + cj(j,2)*p(i,jpos(j,2),kmax)&
                 + ck(kmax,1)*p(i,j,kpos(kmax,1))&
                 + ck(kmax,2)*p(i,j,kpos(kmax,2))&
                 + ci(i,3)*p(ipos(i,3),j,kmax)&
                 + ci(i,4)*p(ipos(i,4),j,kmax)&
                 + cj(j,3)*p(i,jpos(j,3),kmax)&
                 + cj(j,4)*p(i,jpos(j,4),kmax)&
                 + ck(kmax,3)*pboundary(i,j,3)&
                 + ck(kmax,4)*pboundary(i,j,4)
         ENDDO
      ENDDO

      RETURN
      END
      
      SUBROUTINE CGMATRIX4_B(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,2) = p(i,j,2) &
                 + ci(i,1)*p(ipos(i,1),j,2)&
                 + ci(i,2)*p(ipos(i,2),j,2)&
                 + cj(j,1)*p(i,jpos(j,1),2)&
                 + cj(j,2)*p(i,jpos(j,2),2)&
                 + ck(2,1)*pboundary(i,j,2)&
                 + ck(2,2)*p(i,j,kpos(2,2))&
                 + ci(i,3)*p(ipos(i,3),j,2)&
                 + ci(i,4)*p(ipos(i,4),j,2)&
                 + cj(j,3)*p(i,jpos(j,3),2)&
                 + cj(j,4)*p(i,jpos(j,4),2)&
                 + ck(2,3)*p(i,j,kpos(2,3))&
                 + ck(2,4)*p(i,j,kpos(2,4))
         ENDDO
      ENDDO
      

      RETURN
      END

      SUBROUTINE CGMATRIX4_T(imax, jmax, kmax, p, q,&
           ipos, jpos, kpos, ci, cj, ck, pboundary)

      IMPLICIT NONE

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i
      TINTEGER j
      TREAL p(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TINTEGER ipos(imax,4)
      TINTEGER jpos(jmax,4)
      TINTEGER kpos(kmax,4)
      TREAL ci(imax,4)
      TREAL cj(jmax,4)
      TREAL ck(kmax,4)
      TREAL pboundary(imax,jmax,4)

      DO j=1, jmax
         DO i=1, imax
            q(i,j,kmax-1) = p(i,j,kmax-1) &
                 + ci(i,1)*p(ipos(i,1),j,kmax-1)&
                 + ci(i,2)*p(ipos(i,2),j,kmax-1)&
                 + cj(j,1)*p(i,jpos(j,1),kmax-1)&
                 + cj(j,2)*p(i,jpos(j,2),kmax-1)&
                 + ck(kmax-1,1)*p(i,j,kpos(kmax-1,1))&
                 + ck(kmax-1,2)*p(i,j,kpos(kmax-1,2))&
                 + ci(i,3)*p(ipos(i,3),j,kmax-1)&
                 + ci(i,4)*p(ipos(i,4),j,kmax-1)&
                 + cj(j,3)*p(i,jpos(j,3),kmax-1)&
                 + cj(j,4)*p(i,jpos(j,4),kmax-1)&
                 + ck(kmax-1,3)*p(i,j,kpos(kmax-1,3))&
                 + ck(kmax-1,4)*pboundary(i,j,3)
         ENDDO
      ENDDO


      RETURN
      END
