#include "types.h"
#include "dns_error.h"

SUBROUTINE CGPOISSON(iverbose, imax, jmax, kmax, &
     kmax_total, i1bc, j1bc, k1bc, dx, dy, dz, u, r, p, q, &
     ipos, jpos, kpos, ci, cj, ck, wrk2d)
  
  USE DNS_GLOBAL, ONLY: efile, ofile
  USE CG_GLOBAL

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER iverbose
  TINTEGER imax, jmax, kmax, kmax_total
  TINTEGER i1bc, j1bc, k1bc
  TINTEGER ijk
  TINTEGER m

  TREAL, DIMENSION(*)              :: dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax) :: u, p, r, q

  TREAL rho
  TREAL rhom
  TREAL alpha
  TREAL beta
  TREAL limit
  TREAL cg_tol
  TREAL er
  TINTEGER nmax
  TINTEGER ipos(imax,4)
  TINTEGER jpos(jmax,4)
  TINTEGER kpos(kmax,4)
  TREAL ci(*)
  TREAL cj(*)
  TREAL ck(*)
  TREAL wrk2d(imax,jmax,4)
  CHARACTER*64 str

#ifdef PARALLEL
  TREAL sum
  TINTEGER k_bottom_disp, k_buff_disp
  INTEGER ims_err
  INTEGER ims_pro, ims_pron, ims_prop, ims_npro
  INTEGER ims_prop2, ims_pron2
  INTEGER ims_buff_len, ims_nm
  INTEGER cg_mpi_status(MPI_STATUS_SIZE,8)
  INTEGER cg_mpi_req(8)
  INTEGER cg_mpi_tag

#endif

  IF ( cg_ord .NE. 2 .AND. cg_ord .NE. 4 ) THEN
     CALL IO_WRITE_ASCII(efile, 'Calling CGPOISSON with'&
          //' cg_ord different of 2 or 4')
     CALL DNS_STOP(DNS_ERROR_CGORDER)
  ENDIF

  IF ( kmax_total .GT. 1 ) THEN
     IF ( (cg_ord .EQ. 2 .AND. kmax_total .LT. 3) .OR.  &
          (cg_ord .EQ. 4 .AND. kmax_total .LT. 5) ) THEN
        CALL IO_WRITE_ASCII(efile, 'Impossible to run Poisson solver with'&
             //' fewer than 3/5 planes')
        CALL DNS_STOP(DNS_ERROR_POISSON)
     ENDIF
  ENDIF

#ifdef PARALLEL
  CALL MPI_COMM_SIZE(MPI_COMM_WORLD,ims_npro,ims_err)
  CALL MPI_COMM_RANK(MPI_COMM_WORLD,ims_pro,ims_err)

  cg_mpi_tag = 5000

  ims_pron = ims_pro+1
  ims_pron2 = ims_pron+1
  ims_prop = ims_pro-1
  ims_prop2 = ims_prop-1

  IF ( ims_pro .EQ. 0 ) THEN
     ims_prop = ims_npro-1
     ims_prop2 = ims_prop-1
  ELSE IF ( ims_pro .EQ. ims_npro-1 ) THEN
     ims_pron = 0
     ims_pron2 = 1
  ELSE IF ( ims_pro .EQ. ims_npro-2 ) THEN
     ims_pron2 = 0
  ELSE IF ( ims_pro .EQ. 1 ) THEN
     ims_prop2 = ims_npro-1
  ENDIF

  IF ( cg_ord .EQ. 2 ) THEN
     ims_buff_len = imax*jmax
     k_bottom_disp = kmax
     k_buff_disp = 2
  ELSE
     ims_buff_len = imax*jmax*2
     k_bottom_disp = kmax-1
     k_buff_disp = 3
  ENDIF
  ims_nm = imax*jmax

#endif

! #####################################
! #       Set solution to zero        #
! #####################################

  limit = C_0_R
  rhom = C_1_R
  nmax = imax*jmax*kmax

  IF ( cg_unif .EQ. 0 ) THEN
     IF ( cg_ord .EQ. 2 ) THEN
        CALL CGBC2RSCU(imax, jmax, kmax, kmax_total, dx, dy, dz, r)
     ELSE IF ( cg_ord .EQ. 4 ) THEN
        CALL CGBC4RSC(imax, jmax, kmax, kmax_total, &
             i1bc, j1bc, k1bc, dx, dy, dz, r)
     ENDIF
  ELSE
     IF ( cg_ord .EQ. 2 ) THEN
        CALL CGBC2RSCNU(imax, jmax, kmax, kmax_total, dx, dy, dz, r)
     ELSE
        CALL IO_WRITE_ASCII(efile, 'Only 2nd order Poisson solver'&
             //' is nonuniform')
        CALL DNS_STOP(DNS_ERROR_POISSON)
     ENDIF
  ENDIF

  DO ijk=1, nmax
     u(ijk,1,1) = C_0_R
     limit = MAX(limit, ABS(r(ijk,1,1)))
  ENDDO

#ifdef PARALLEL
  IF ( ims_npro .GT. 1 ) THEN
     sum = limit
     CALL MPI_ALLREDUCE(sum, limit, 1, MPI_REAL8, MPI_SUM, &
          MPI_COMM_WORLD, ims_err)
  ENDIF
#endif

  IF ( limit .LT. cg_tol ) THEN
     GOTO 234
  ENDIF

!  limit = SQRT(limit)
  limit = limit * cg_tol

  DO m=1, cg_mmax

     rho = C_0_R

     DO ijk=1, nmax
        rho = rho + r(ijk,1,1)**2
     ENDDO

#ifdef PARALLEL
     IF ( ims_npro .GT. 1 ) THEN
        sum = rho
        CALL MPI_ALLREDUCE(sum, rho, 1, MPI_REAL8, MPI_SUM, &
             MPI_COMM_WORLD, ims_err)
     ENDIF
#endif

     IF ( m .EQ. 1 ) THEN
        DO ijk=1, nmax
           p(ijk,1,1) = r(ijk,1,1)
        ENDDO
     ELSE
        beta = rho/rhom
        DO ijk=1, nmax
           p(ijk,1,1) = r(ijk,1,1) + beta * p(ijk,1,1)
        ENDDO
     ENDIF

#ifdef PARALLEL
     IF ( ims_npro .GT. 1 ) THEN
!        Exchange Previous and Next planes

! #############################################################
! #  WARNING ! WARNING ! WARNING ! WARNING ! WARNING ! 
! #
! #  If all processors do not have 1 plane the following
! #  exchange of planes is incorrect.
! #
! #############################################################

        IF ( kmax .EQ. 1 .AND. cg_ord .EQ. 4 ) THEN
           CALL MPI_IRECV(wrk2d(1,1,1), ims_nm, MPI_REAL8, ims_prop2,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(1), ims_err)

           CALL MPI_IRECV(wrk2d(1,1,2), ims_nm, MPI_REAL8, ims_prop,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(2), ims_err)

           CALL MPI_IRECV(wrk2d(1,1,3), ims_nm, MPI_REAL8, ims_pron,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(3), ims_err)

           CALL MPI_IRECV(wrk2d(1,1,4), ims_nm, MPI_REAL8, ims_pron2,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(4), ims_err)

           CALL MPI_ISEND(p(1,1,1), ims_nm, MPI_REAL8, ims_prop,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(5), ims_err)

           CALL MPI_ISEND(p(1,1,1), ims_nm, MPI_REAL8, ims_prop2,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(6), ims_err)

           CALL MPI_ISEND(p(1,1,1), ims_nm, MPI_REAL8, ims_pron,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(7), ims_err)

           CALL MPI_ISEND(p(1,1,1), ims_nm, MPI_REAL8, ims_pron2,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(8), ims_err)

           CALL MPI_WAITALL(8, cg_mpi_req, cg_mpi_status, ims_err)

           cg_mpi_tag = cg_mpi_tag + 1
        ELSE
           CALL MPI_ISEND(p(1,1,1), ims_buff_len, MPI_REAL8, &
                ims_prop,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(1), ims_err)

           CALL MPI_IRECV(wrk2d(1,1,k_buff_disp), ims_buff_len, &
                MPI_REAL8, ims_pron,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(4), ims_err)

           cg_mpi_tag = cg_mpi_tag + 1

           CALL MPI_ISEND(p(1,1,k_bottom_disp), ims_buff_len, &
                MPI_REAL8, ims_pron,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(3), ims_err)

           CALL MPI_IRECV(wrk2d(1,1,1), ims_buff_len, MPI_REAL8, &
                ims_prop,&
                cg_mpi_tag, MPI_COMM_WORLD, cg_mpi_req(2), ims_err)

           cg_mpi_tag = cg_mpi_tag + 1

           CALL MPI_WAITALL(4, cg_mpi_req, cg_mpi_status, ims_err)
        ENDIF

     ENDIF
#endif

! ##################################
! #  Compute Matrix Product        #
! ##################################
     IF ( cg_unif .EQ. 0 ) THEN
        IF ( cg_ord .EQ. 2 ) THEN
           CALL CGMATRIX2(imax, jmax, kmax, kmax_total, p, q, &
                ipos, jpos, kpos, ci, cj, ck, wrk2d)
        ELSE IF ( cg_ord .EQ. 4 ) THEN
           CALL CGMATRIX4(imax, jmax, kmax, kmax_total, p, q, &
                ipos, jpos, kpos, ci, cj, ck, wrk2d)
        ENDIF
     ELSE
        IF ( cg_ord .EQ. 2 ) THEN
           CALL CGMATRIX2NU(imax, jmax, kmax, kmax_total,  &
                p, q, ipos, jpos, kpos, ci, cj, ck, wrk2d)
        ELSE
           CALL IO_WRITE_ASCII(efile, 'Only 2nd order Poisson solver'&
                //' is nonuniform')
           CALL DNS_STOP(DNS_ERROR_POISSON)
        ENDIF
     ENDIF

! #####################################
! # End Calculation of Matrix Product #
! #####################################

     alpha = C_0_R
     DO ijk=1, nmax
        alpha = alpha + p(ijk,1,1)*q(ijk,1,1)
     ENDDO

#ifdef PARALLEL
     IF ( ims_npro .GT. 1 ) THEN
        sum = alpha
        CALL MPI_ALLREDUCE(sum, alpha, 1, MPI_REAL8, MPI_SUM, &
             MPI_COMM_WORLD, ims_err)
     ENDIF
#endif

     alpha = rho/alpha
     er = C_0_R

     DO ijk=1, nmax
        u(ijk,1,1) = u(ijk,1,1) + alpha * p(ijk,1,1)
        r(ijk,1,1) = r(ijk,1,1) - alpha * q(ijk,1,1)
        er = MAX(er, ABS(r(ijk,1,1)))
     ENDDO

#ifdef PARALLEL
     IF ( ims_npro .GT. 1 ) THEN
        sum = er
        CALL MPI_ALLREDUCE(sum, er, 1, MPI_REAL8, MPI_SUM, &
             MPI_COMM_WORLD, ims_err)
     ENDIF
#endif

     rhom = rho
!     er = SQRT(er)

#ifdef _DEBUG
     IF ( iverbose .EQ. 1 ) THEN
        WRITE(*,*) 'Iteration #', m, '  Error = ', er
     ENDIF
#endif

     IF ( er .LT. limit ) THEN 
        GOTO 234
     ENDIF

  ENDDO

234 IF ( iverbose .EQ. 1 ) THEN
     WRITE(str,'(A11,1X,I6,1X,A8,E12.5E3)') 'Iteration #', m, 'Error = ', er
     CALL IO_WRITE_ASCII(ofile, str)
  ENDIF

  RETURN
END SUBROUTINE CGPOISSON
