      SUBROUTINE PCGPOISSON(imax, jmax, kmax, i1bc, j1bc, k1bc, &
           dx, dy, dz, u, f, p, r, q, z, error, mmax)

      IMPLICIT NONE

#include "types.h"

      TINTEGER imax
      TINTEGER jmax
      TINTEGER kmax
      TINTEGER i1bc
      TINTEGER j1bc
      TINTEGER k1bc
      TINTEGER i
      TINTEGER j
      TINTEGER k
      TINTEGER m
      TINTEGER mmax
      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax)
      TREAL u(imax, jmax, kmax)
      TREAL f(imax, jmax, kmax)
      TREAL p(imax, jmax, kmax)
      TREAL r(imax, jmax, kmax)
      TREAL q(imax, jmax, kmax)
      TREAL z(imax, jmax, kmax)
      TREAL chi
      TREAL mu
      TREAL eta
      TREAL rho
      TREAL rhom
      TREAL alpha
      TREAL beta
      TREAL limit
      TREAL error
      TREAL er
      TREAL diag
      TINTEGER imax2
      TINTEGER jmax2
      TINTEGER kmax2

      imax2 = imax/2
      jmax2 = jmax/2
      IF ( kmax .GT. 1 ) THEN
         kmax2 = kmax/2
      ELSE
         kmax2 = 1
      ENDIF

      chi = (dy(jmax2)/dx(imax2))**2
      eta = (dy(jmax2)/dz(kmax2))**2
      diag = 2.0*(1.0+chi+eta)
      chi = chi / diag
      mu  = 1.0 / diag
      eta = eta / diag

! #####################################
! #       Set solution to zero        #
! #####################################

      limit = 0.0
      DO k=1, kmax
         DO j=1, jmax
            DO i=1, imax
               u(i,j,k) = 0.0
               r(i,j,k) = dy(jmax2)**2 * f(i,j,k) / diag
               limit = limit + r(i,j,k)**2
            ENDDO
         ENDDO
      ENDDO

      limit = SQRT(limit)*error

      CALL ILUDELTA(imax, jmax, kmax, chi, mu, eta, f)

      DO m=1, mmax

! ###########################
! #    Solve Mz = r         #
! ###########################

         CALL PCGSOLVER(imax, jmax, kmax, i1bc, j1bc, k1bc,&
              chi, mu, eta, f, z, r)

         rho = 0.0
         
         DO k=1, kmax
            DO j=1, jmax
               DO i=1, imax
                  rho = rho + r(i,j,k)*z(i,j,k)
               ENDDO
            ENDDO
         ENDDO
      
         IF ( m .EQ. 1 ) THEN
            DO k=1, kmax
               DO j=1, jmax
                  DO i=1, imax
                     p(i,j,k) = z(i,j,k)
                  ENDDO
               ENDDO
            ENDDO
         ELSE
            beta = rho/rhom
            DO k=1, kmax
               DO j=1, jmax
                  DO i=1, imax
                     p(i,j,k) = z(i,j,k) + beta * p(i,j,k)
                  ENDDO
               ENDDO
            ENDDO
         ENDIF

! ##################################
! #  Compute Matrix Product        #
! ##################################

         CALL CGMATRIX(imax, jmax, kmax, i1bc, j1bc, k1bc, &
              chi, mu, eta, p, q)

! #####################################
! # End Calculation of Matrix Product #
! #####################################

         alpha = 0.0
         DO k=1, kmax
            DO j=1, jmax
               DO i=1, imax
                  alpha = alpha + p(i,j,k)*q(i,j,k)
               ENDDO
            ENDDO
         ENDDO
         
         alpha = rho/alpha
         er = 0.0
         
         DO k=1, kmax
            DO j=1, jmax
               DO i=1, imax
                  u(i,j,k) = u(i,j,k) + alpha * p(i,j,k)
                  r(i,j,k) = r(i,j,k) - alpha * q(i,j,k)
                  er = er + r(i,j,k)**2
               ENDDO
            ENDDO
         ENDDO

         rhom = rho
         er = SQRT(er)

         PRINT *, 'Iteration #', m, '  Error = ', er

         IF ( er .LT. limit ) THEN 
            GOTO 234
         ENDIF

      ENDDO

 234  RETURN
      END
