!########################################################################
!# Library DNS
!#
!########################################################################
!# HISTORY
!#
!# 1999/01/01 - C. Pantano
!#              Created (reactini.f)
!# 2007/05/09 - J.P. Mellado
!#              Cleaned
!#
!########################################################################
!# DESCRIPTION
!#
!# Global stoichiometry. Only one global reaction, detail mechanism
!# is specific in each routine CHEM_*
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE CHEM_INIT

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"

  USE DNS_GLOBAL
  USE CHEM_GLOBAL

  IMPLICIT NONE

! -------------------------------------------------------------------
  TINTEGER icp,is
  TREAL DHREF
  CHARACTER*46 str
  TINTEGER im_o2, im_n2_f, im_n2_o, im_ch4
  TINTEGER is_fuel, is_ox, is_inrt, is_prd(5)
  TINTEGER ins_prd

! For Methane
#define is_h2o  is_prd(1)
#define is_co2  is_prd(2)
! For Single Step
#define is_prod is_prd(1)

! ###################################################################
! Reaction Coefficients nupp(isp,ir), nup(isp,ir)
! ###################################################################
  IF ( imech_type .EQ. CHEM_TYPE_UNIDECOMP ) THEN
!    Unimolecular reaction
     NUP(1) = C_1_R
     NUP(2) = C_0_R

     NUPP(1) = C_0_R
     NUPP(2) = C_1_R

     ins_prd = 1
     is_fuel = 1
     is_prod = 2
     is_inrt = -1

  ELSE IF ( imech_type .EQ. CHEM_TYPE_ONESTEP ) THEN
!    Single Step reaction
     NUP(1) = C_1_R
     NUP(2) = C_1_R     ! Change if other reaction wanted
     NUP(3) = C_0_R
     NUP(4) = C_0_R

     NUPP(1) = C_0_R
     NUPP(2) = C_0_R
     NUPP(3) = C_2_R    ! Change if other reaction wanted
     NUPP(4) = C_0_R

     ins_prd = 1
     is_fuel = 1
     is_ox   = 2
     is_prod = 3
     is_inrt = NSP

  ELSE
!    Methane Combustion
     NUP(1) = C_1_R
     NUP(2) = C_2_R
     NUP(3) = C_0_R
     NUP(4) = C_0_R
     NUP(5) = C_0_R

     NUPP(1) = C_0_R
     NUPP(2) = C_0_R
     NUPP(3) = C_2_R
     NUPP(4) = C_1_R
     NUPP(5) = C_0_R

     ins_prd = 2
     is_fuel = 1
     is_ox  = 2
     is_h2o = 3
     is_co2 = 4
     is_inrt  = NSP

  ENDIF

! ###################################################################
! Nondimensional heat of reaction
! ###################################################################
  DHREF = C_0_R
  DO is = 1,NSP
! HREF(is), already nondimensionalized by (CPREF*TREF)
     CHW_HI = C_0_R
     DO icp = NCP_CHEMKIN, 1, -1
        CHW_HI = CHW_HI + THERMO_AI(icp,2,is)/M_REAL(icp)
     ENDDO
     CHW_HI = CHW_HI + THERMO_AI(6,2,is)

     DHREF = DHREF + WGHT(is)*(NUP(is)-NUPP(is))*CHW_HI
  ENDDO

! DHREF is already nondimensionalized by (CPREF*TREF)
  DHREF = DHREF / WREF

! -------------------------------------------------------------------
! Output
! -------------------------------------------------------------------
  CALL IO_WRITE_ASCII(lfile, '########################################')
  CALL IO_WRITE_ASCII(lfile, '# Heat Released')
  CALL IO_WRITE_ASCII(lfile, '########################################')
  WRITE(str,'(A8,1X,E12.5E3)') 'DHREF = ', DHREF
  CALL IO_WRITE_ASCII(lfile, str)

  IF ( ichem_config .EQ. CHEM_PREMIXED ) THEN
     CALL CHEM_INIT_PREMIXED(im_o2, im_n2_f, im_n2_o, im_ch4,&
          is_fuel, is_ox, is_inrt, is_prd, DHREF)
  ELSE
     CALL CHEM_INIT_NONPREMIXED(im_o2, im_n2_f, im_n2_o, im_ch4,&
          is_fuel, is_ox, is_inrt, is_prd, ins_prd, DHREF)
  ENDIF

  RETURN
END SUBROUTINE CHEM_INIT
