SUBROUTINE CHEM_INIT_NONPREMIXED(im_o2, im_n2_f, im_n2_o, im_ch4, &
     is_fuel, is_ox, is_inrt, is_prd, ins_prd, DHREF)
  
#include "types.h"
#include "dns_const.h"

  USE DNS_GLOBAL
  USE CHEM_GLOBAL

  IMPLICIT NONE

  TINTEGER icp,is,im
  TREAL DHREF
  TREAL TOX, TFUEL, WOX, WFUEL
  TREAL HO2, HN2_o, HCH4, HN2_f, HE, CPN2
  CHARACTER*46 str
  TINTEGER im_o2, im_n2_f, im_n2_o, im_ch4
  TINTEGER is_fuel, is_ox, is_inrt, is_prd(*)
  TINTEGER ins_prd

  TREAL TV, CHW_RCOI, CHW_ERROR, CHW_INC

! ###############################################################
! # Burke-Schumann Constants 
! ###############################################################
! \nu Y_F,1 / Y_o2,2
  NU = WGHT(is_ox)*(NUPP(is_ox)-NUP(is_ox))*YFUEL&
       /(WGHT(is_fuel)*(NUPP(is_fuel)-NUP(is_fuel))*YOX)
! Stoichimoetric mixture fraction Z_st
  XIST = C_1_R/(C_1_R+NU) 

  CPN2 = C_0_R
  DO icp = NCP_CHEMKIN, 1, -1
     CPN2 = CPN2 + THERMO_AI(icp,2,is_inrt)
  ENDDO

  CHW_CE = DHREF*WREF*YFUEL*XIST/(NUP(is_fuel)-NUPP(is_fuel))/WGHT(is_fuel)/CPN2

  CST(is_fuel) = YFUEL*(C_1_R+NU)/NU ! Y_F,1 /(1-Z_st)
  CST(is_ox)   =-YOX  *(C_1_R+NU)    ! Y_o2,2/Z_st
  DO is=1, ins_prd
     CST(is_prd(is)) = WGHT(is_prd(is))*(NUPP(is_prd(is))-NUP(is_prd(is)))*YFUEL/WGHT(is_fuel)
  ENDDO

  CHW_ETA0  = -XIST/dsmooth
  CHW_ETA1  = (C_1_R-XIST)/dsmooth
  CHW_LN0   = EXP(C_2_R*CHW_ETA0)+C_1_R
  CHW_LN1   = EXP(C_2_R*CHW_ETA1)+C_1_R
  CHW_DHALF = dsmooth*C_05_R

  WOX   = C_1_R/(WREF*(YOX/WGHT(is_ox)+(C_1_R-YOX)/WGHT(is_inrt)))
  WFUEL = C_1_R/(WREF*(YFUEL/WGHT(is_fuel)+(C_1_R-YFUEL)/WGHT(is_inrt)))
  TOX   = MRATIO*p_init*WOX  /(mean_rho - C_05_R*delta_rho)
  TFUEL = MRATIO*p_init*WFUEL/(mean_rho + C_05_R*delta_rho)

! Compute enthalpy of free streams.
  IF ( TOX .LT. THERMO_TLIM(3,is_ox) ) THEN
     im_o2 = 2
  ELSE
     im_o2 = 1
  ENDIF
  IF ( TOX .LT. THERMO_TLIM(3,is_inrt) ) THEN
     im_n2_o = 2
  ELSE
     im_n2_o = 1
  ENDIF
  IF ( TFUEL .LT. THERMO_TLIM(3,is_fuel) ) THEN
     im_ch4 = 2
  ELSE
     im_ch4 = 1
  ENDIF
  IF ( TFUEL .LT. THERMO_TLIM(3,is_inrt) ) THEN
     im_n2_f = 2
  ELSE
     im_n2_f = 1
  ENDIF
  HO2 =   C_0_R
  HN2_o = C_0_R
  HCH4 =  C_0_R
  HN2_f = C_0_R
  DO icp=NCP_CHEMKIN, 1, -1
     CHW_RCOI = C_1_R/M_REAL(icp)
     HO2   = HO2  *TOX   + THERMO_AI(icp,im_o2,is_ox)    *CHW_RCOI
     HN2_o = HN2_o*TOX   + THERMO_AI(icp,im_n2_o,is_inrt)*CHW_RCOI
     HCH4  = HCH4 *TFUEL + THERMO_AI(icp,im_ch4,is_fuel) *CHW_RCOI
     HN2_f = HN2_f*TFUEL + THERMO_AI(icp,im_n2_f,is_inrt)*CHW_RCOI
  ENDDO
  HO2   = HO2*TOX     + THERMO_AI(6,im_o2,is_ox)
  HN2_o = HN2_o*TOX   + THERMO_AI(6,im_n2_o,is_inrt)
  HCH4  = HCH4*TFUEL  + THERMO_AI(6,im_ch4,is_fuel)
  HN2_f = HN2_f*TFUEL + THERMO_AI(6,im_n2_f,is_inrt)

  HOX =   HO2*YOX    + HN2_o*(C_1_R-YOX)
  HFUEL = HCH4*YFUEL + HN2_f*(C_1_R-YFUEL)

! Compute adiabatic flame temperature
  DO is = 1,NSP
     YMASS(is) = C_0_R
  ENDDO

#define MACRO_ZINPUT XIST
#include "dns_chem_mass.h"

  TFLAME = CHW_CE + C_1_R
  HE = (HOX + (HFUEL-HOX)*XIST)

#define MACRO_TINPUT TFLAME
#define MACRO_HINPUT HE
#include "dns_chem_inv.h"

  WRITE(str,'(A8,1X,E12.5E3)') 'Q     = ', CHW_CE
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'XIST  = ', XIST
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'NU    = ', NU
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'HOX   = ', HOX
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'HFUEL = ', HFUEL
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'TFLAME= ', TFLAME
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'TOX=    ', TOX
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'TFUEL=  ', TFUEL
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'WOX=    ', WOX
  CALL IO_WRITE_ASCII(lfile, str)
  WRITE(str,'(A8,1X,E12.5E3)') 'WFUEL=  ', WFUEL
  CALL IO_WRITE_ASCII(lfile, str)
  TV = C_1_R
  DO is=1, NSP
     IF ( TV .LT. THERMO_TLIM(3,is) ) THEN
        im = 2
     ELSE
        im = 1
     ENDIF
     CHW_CP = C_0_R
     DO icp=NCP_CHEMKIN, 1, -1
        CHW_CP = CHW_CP*TV + THERMO_AI(icp,im,is)
     ENDDO
     WRITE(str,'(A3,A3,A3,1X,E12.5E3)') 'CP(', THERMO_SPNAME(is), ')= ', CHW_CP
     CALL IO_WRITE_ASCII(lfile, str)
     CHW_HI = C_0_R
     DO icp=NCP_CHEMKIN, 1, -1
        CHW_HI = CHW_HI*TV + THERMO_AI(icp,im,is)/M_REAL(icp)
     ENDDO
     CHW_HI = CHW_HI*TV + THERMO_AI(6,im,is)
     WRITE(str,'(A3,A3,A3,1X,E12.5E3)') 'HI(', THERMO_SPNAME(is), ')= ', CHW_HI
     CALL IO_WRITE_ASCII(lfile, str)
  ENDDO

  RETURN
END SUBROUTINE CHEM_INIT_NONPREMIXED
