SUBROUTINE CHEM_PETERS1991(iop, rho, T, GM, z1, zh1, h4, wrk)

#include "types.h"
#include "dns_error.h"
  
  USE DNS_GLOBAL
  USE CHEM_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER iop
  TREAL rho(imax,jmax,kmax)
  TREAL T(imax,jmax,kmax)
  TREAL GM(imax,jmax,kmax)
  TREAL z1(imax,jmax,kmax,*)
  TREAL zh1(imax,jmax,kmax,*)
  TREAL h4(imax,jmax,kmax)
  TREAL wrk(imax,jmax,kmax,*)

  TINTEGER ij, k
  TINTEGER is, icp, im, ik
  TREAL cda1, cda2, cs(7)
  TREAL dtmp, tmpA, tmpB, tmpC, tmpD, lnT, tmpE
  TREAL a, b, c, omegaT
  TREAL c_ch4, c_o2, c_h2o, c_co2, c_co, c_h2, c_h, c_n2
  TREAL c_m, c_oh, c_o, c_ch3, c_ch3o, c_ch2o, c_hco
  TREAL c_ho2, c_h2o2
  TREAL omega(25), yomega(7), csw(20)
  TREAL omegaI, omegaII, omegaIII, omegaIV
  TREAL kr(REAC_MAX)
  TREAL CDLT, TMAX
  TREAL kcorr
  TREAL WC, WH, WO

#define k1f   kr(1)
#define k1b   kr(2)
#define k2f   kr(3)
#define k2b   kr(4)
#define k3f   kr(5)
#define k3b   kr(6)
#define k4f   kr(7)
#define k4b   kr(8)
#define k5    kr(9)
#define k6    kr(10)
#define k7    kr(11)
#define k8    kr(12)
#define k9f   kr(13)
#define k9b   kr(14)
#define k10f  kr(15)
#define k10b  kr(16)
#define k11f  kr(17)
#define k11b  kr(18)
#define k12f  kr(19)
#define k12b  kr(20)
#define k13   kr(21)
#define k14   kr(22)
#define k15   kr(23)
#define k16   kr(24)
#define k17   kr(25)
#define k18   kr(26)
#define k19   kr(27)
#define k20   kr(28)
#define k21   kr(29)
#define k22f  kr(30)
#define k22b  kr(31)
#define k23f  kr(32)
#define k23b  kr(33)
#define k24   kr(34)
#define k25   kr(35)
#define kfall kr(36)

  IF ( NMECH .NE. 36 ) THEN
     CALL IO_WRITE_ASCII(efile, 'Error: Wrong number of reactions for'&
          //' Peters1991 mechanism')
     CALL DNS_STOP(DNS_ERROR_NBREAC)
  ENDIF

  cda1 = C_1_R/(gama0*mach*mach)
  cda2 = C_1_R/((gama0-C_1_R)*mach*mach)

! Normalized molecular weight
  DO is=1, NSP
     csw(is) = WGHT(is)/WREF
  ENDDO

  WC = 12.0e0
  WH = 1.0e0
  WO = 16.0e0

  csw(9)  = (WO+WH)/WREF
  csw(10) = (WO)/WREF
  csw(11) = (WC+3*WH)/WREF
  csw(12) = (WC+3*WH+WO)/WREF
  csw(13) = (WC+2*WH+WO)/WREF
  csw(14) = (WC+WH+WO)/WREF
  csw(15) = (WH+2*WO)/WREF
  csw(16) = (2*WH+2*WO)/WREF

  cs(1) = - csw(1)*damkohler(inb_scal)
  cs(2) = - csw(2)*damkohler(inb_scal)
  cs(3) = csw(3)*damkohler(inb_scal)
  cs(4) = csw(4)*damkohler(inb_scal)
  cs(5) = csw(5)*damkohler(inb_scal)
  cs(6) = csw(6)*damkohler(inb_scal)
  cs(7) = csw(7)*C_2_R*damkohler(inb_scal)

  CDLT = 2.0e-5
  TMAX = C_2_R
  TGFM = C_0_R

  IF ( damkohler(inb_scal) .EQ. C_0_R ) RETURN

  DO k=1, kmax
     DO ij=1, imax*jmax

        IF ( T(ij,1,k) .GT. TMAX ) THEN

           lnT = LOG(T(ij,1,k))

           DO ik=1,NMECH
              kr(ik) = mech_pre(ik)*&
                   EXP(-mech_tac(ik)/T(ij,1,k)&
                   +mech_beta(ik)*lnT)
           ENDDO

!       Check all mass fractions are positive
           DO is=1, NSP-1
              z1(ij,1,k,is) = MAX(z1(ij,1,k,is),C_0_R)
           ENDDO

!       Concentrations of intervening species
           c_ch4 = rho(ij,1,k)*z1(ij,1,k,1)/csw(1)
           c_o2  = rho(ij,1,k)*z1(ij,1,k,2)/csw(2)
           c_h2o = rho(ij,1,k)*z1(ij,1,k,3)/csw(3)
           c_co2 = rho(ij,1,k)*z1(ij,1,k,4)/csw(4)
           c_co  = rho(ij,1,k)*z1(ij,1,k,5)/csw(5)
           c_h2  = rho(ij,1,k)*z1(ij,1,k,6)/csw(6) 
           c_h   = rho(ij,1,k)*z1(ij,1,k,7)/csw(7) 

           dtmp = C_1_R
           DO is=1,NSP-1
              dtmp = dtmp - z1(ij,1,k,is)
           ENDDO
           c_n2  = rho(ij,1,k)*dtmp/csw(NSP)

!       Compute Reaction Rates Here

           c_m = 6.5*c_ch4+6.5*c_h2o+1.5*c_co2+0.75*c_co+0.4*c_o2&
                +0.4*c_n2+c_h+c_h2

           kcorr = C_1_R/(C_1_R+kfall/c_m)
           k10f = k10f*kcorr
           k10b = k10b*kcorr

           c_oh = k3b*c_h2o*c_h/(k3f*c_h2+CDLT)
           c_o = k4f*c_oh*c_oh/(k4b*c_h2o+CDLT)
           c_oh = c_h*(k3b*c_h2o+k1f*c_o2)/(k3f*c_h2&
                +k1b*c_o+CDLT)

           tmpA = (k1f*c_o2+k2b*c_oh)*c_h+k4f*c_oh*c_oh
           tmpB = k1b*c_oh+k2f*c_h2+k4b*c_h2o
           tmpC = (k11f*c_h+k12f*c_oh)*c_ch4
           tmpD = k10b*c_h+k11b*c_h2+k12b*c_h2o

           a = C_2_R*k13*tmpB
           b = tmpB*tmpD+k13*(tmpC-tmpA)
           c = -tmpA*tmpD

           c_o = (-b+sqrt(b*b-C_2_R*a*c))/(a+CDLT)
           c_ch3 = tmpC/(tmpD+k13*c_o+CDLT)
           tmpE = k19*c_h+k20*c_m
           c_ch3o = k18*c_ch3*c_o2/(tmpE+CDLT)
           c_ch2o = (k13*c_ch3*c_o+tmpE*c_ch3o)&
                /(k14*c_h+k15*c_oh+CDLT)
           c_hco = (k14*c_h+k15*c_oh)*c_ch2o&
                /(k16*c_h+k17*c_m+CDLT)
           c_ho2 = k5*c_h*c_o2*c_m/((k6+k7)*c_h&
                +k8*c_oh+CDLT)
           c_h2o2 = (k21*c_ho2*c_ho2+k22b*c_oh*c_oh*c_m&
                +k23b*c_h2o*c_ho2)/(k22f*c_m+k23f*c_oh+CDLT)

           IF ( iop .EQ. 1 ) THEN
              wrk(ij,1,k, 1) = c_oh  *csw(9)/rho(ij,1,k)
              wrk(ij,1,k, 2) = c_o   *csw(10)/rho(ij,1,k)
              wrk(ij,1,k, 3) = c_ch3 *csw(11)/rho(ij,1,k)
              wrk(ij,1,k, 4) = c_ch3o*csw(12)/rho(ij,1,k)
              wrk(ij,1,k, 5) = c_ch2o*csw(13)/rho(ij,1,k)
              wrk(ij,1,k, 6) = c_hco *csw(14)/rho(ij,1,k)
              wrk(ij,1,k, 7) = c_ho2 *csw(15)/rho(ij,1,k)
              wrk(ij,1,k, 8) = c_h2o2*csw(16)/rho(ij,1,k)
           ENDIF

           omega(1) = k1f*c_h*c_o2-k1b*c_oh*c_o
           omega(5) = k5*c_h*c_o2*c_m
           omega(6) = k6*c_h*c_ho2
           omega(9) = k9f*c_co*c_oh-k9b*c_co2*c_h
           omega(10) = k10f*c_ch4-k10b*c_ch3*c_h
           omega(11) = k11f*c_ch4*c_h-k11b*c_ch3*c_h2
           omega(12) = k12f*c_ch4*c_oh-k12b*c_ch3*c_h2o
           omega(16) = k16*c_hco*c_h
           omega(18) = k18*c_ch3*c_o2
           omega(19) = k19*c_ch3o*c_h
           omega(21) = k21*c_ho2*c_ho2
           omega(23) = k23f*c_h2o2*c_oh-k23b*c_h2o*c_ho2
           omega(24) = k24*c_oh*c_h*c_m
           omega(25) = k25*c_h*c_h*c_m

           omegaI = omega(10)+omega(11)+omega(12)
           omegaII= omega(9)
           omegaIII = omega(5)-omega(10)+omega(16)-omega(18)&
                +omega(19)-omega(21)+omega(23)+omega(24)+omega(25)
           omegaIV = omega(1) + omega(6) + omega(18) + omega(21) &
                - omega(23)

           yomega(1) = cs(1)*omegaI
           yomega(2) = cs(2)*omegaIV
           yomega(3) = cs(3)*(2*omegaIV-omegaI-omegaII)
           yomega(4) = cs(4)*omegaII
           yomega(5) = cs(5)*(omegaI-omegaII)
           yomega(6) = cs(6)*(4*omegaI+omegaII+omegaIII-3*omegaIV)
           yomega(7) = cs(7)*(omegaIV-omegaI-omegaIII)

!        ymax = C_0_R
!        DO is=1, NSP-1
!           ymax = MAX(ymax, ABS(yomega(is)))
!        ENDDO

           IF ( iop .EQ. 0 ) THEN
!       Reaction rate is zero if zero mass fraction of this species
!       and negative reaction rate. Not possible to consume where 
!       there is no reactant.
              DO is=1, NSP-1
                 IF ( z1(ij,1,k,is) .LE. C_0_R .AND. &
                      yomega(is) .LT. C_0_R ) THEN
                    yomega(is) = C_0_R
                 ENDIF
              ENDDO

              omegaT = C_0_R
              DO is=1, NSP-1
                 IF ( T(ij,1,k) .LT. THERMO_TLIM(3,is) ) THEN
                    im = 2
                 ELSE
                    im = 1
                 ENDIF

! Enthalpy
                 CHW_HI = C_0_R
                 DO icp=NCP_CHEMKIN, 1, -1
                    CHW_HI = CHW_HI*T(ij,1,k)&
                         + THERMO_AI(icp,im,is)/M_REAL(icp)
                 ENDDO
                 CHW_HI = CHW_HI*T(ij,1,k) + THERMO_AI(6,im,is)

                 omegaT = omegaT + (cda1*GM(ij,1,k)*T(ij,1,k)/csw(is)&
                      -cda2*(GM(ij,1,k)-C_1_R)*CHW_HI)*yomega(is)

                 zh1(ij,1,k,is) = zh1(ij,1,k,is) + yomega(is)
              ENDDO
              h4(ij,1,k) = h4(ij,1,k) + omegaT
              TGFM = MAX(TGFM, ABS(omegaT))
           ENDIF
        ENDIF
     ENDDO
  ENDDO

! Distribute TGFM across processors
#ifdef PARALLEL

  CALL MPI_ALLREDUCE(TGFM, dtmp, 1, MPI_REAL8, MPI_MAX, &
       MPI_COMM_WORLD, ims_err)
  TGFM = dtmp

#endif

!  PRINT *, 'TGFM = ', TGFM

!  PRINT *, ymax
!  CALL MINMAX(imax, jmax, kmax, rho, a, b)
!  PRINT *, 'Rho ', a, b
!  CALL MINMAX(imax, jmax, kmax, T, a, b)
!  PRINT *, 'T ', a, b
!  CALL MINMAX(imax, jmax, kmax, h4, a, b)
!  PRINT *, 'h4 ', a, b
!  DO is=1, NSP-1
!     CALL MINMAX(imax, jmax, kmax, z1(1,1,1,is), a, b)
!     PRINT *, is, a, b
!     CALL MINMAX(imax, jmax, kmax, zh1(1,1,1,is), a, b)
!     PRINT *, is, a, b
!  ENDDO

  RETURN
END SUBROUTINE CHEM_PETERS1991
