SUBROUTINE CHEM_READ_MECH(name)
  
#include "types.h"
#include "dns_error.h"
#include "dns_const.h"
  
  USE CHEM_GLOBAL
  USE DNS_GLOBAL

  IMPLICIT NONE

  CHARACTER*(*) name
  TINTEGER i23, im, l1, l2, ik, in
  TINTEGER ir
  TREAL pre, beta, tac
  TREAL EREF, PREF, CREF
  TREAL k0mx, k1, dn
  CHARACTER*6 tag
  CHARACTER*80 line
  CHARACTER*80 wrk
  CHARACTER*24 stmp
  CHARACTER*12 units(4)
  CHARACTER*2 str
  TINTEGER i, lwrk, iu

  TREAL rho1, rho2, beta1, alpha, sigma, sigma_as, vo

  i23 = 23
  in = 0

! Reset all reaction rates to zero
  DO ik=1, REAC_MAX
     mech_pre(ik) = C_0_R
     mech_beta(ik) = C_0_R
     mech_tac(ik) = C_0_R
     mech_order(ik) = 2.0
     rate_type(ik) = 1
  ENDDO

! Read file
  OPEN(i23, file=name, status='old')

  DO WHILE ( .TRUE. ) 
     READ(i23,'(A80)',END=50) line
!     CALL IO_WRITE_ASCII(lfile, line)
!    Check for comments
     IF ( LEN_TRIM(line) .NE. 0 .AND. line(1:1) .NE. '#' ) THEN
        READ(line(1:6),'(A6)') tag
        l2 = LEN_TRIM(tag)
        IF ( tag(l2:l2) .NE. ':' ) THEN
           CALL IO_WRITE_ASCII(efile, 'Error: Mechanism format error')
           CALL DNS_STOP(DNS_ERROR_MECHFORMAT)
        ENDIF
        l2 = l2 - 1

        IF ( tag(1:l2) .EQ. 'units' ) THEN
!          Read Units 
           iu = 1
           wrk=line(7:80)
34         wrk=TRIM(ADJUSTL(wrk))
           lwrk = LEN_TRIM(wrk)
           DO i=1,lwrk
              IF ( wrk(i:i) .EQ. ' ' .OR. i .EQ. lwrk ) THEN 
                 READ(wrk(1:i), *) stmp
                 units(iu) = TRIM(ADJUSTL(stmp))
                 iu = iu + 1
                 wrk = wrk(i:lwrk)
                 IF ( i .LT. lwrk ) GOTO 34
              ENDIF
           ENDDO

           IF ( iu .LT. 5 ) THEN
              CALL IO_WRITE_ASCII(efile, 'Error: Please specify all units')
              CALL DNS_STOP(DNS_ERROR_UNITS)
           ENDIF

           CALL IO_WRITE_ASCII(lfile, 'Units: '//units(1)//' '//units(2)&
                //' '//units(3)//' '//units(4))
        ELSE

           READ(line(7:80),*) pre, beta, tac, dn

           IF ( tag(l2:l2) .EQ. 'f' ) THEN 
!             Forward rate
              im = 1
              l2 = l2 - 1
           ELSE IF ( tag(l2:l2) .EQ. 'b' ) THEN
!             Backward rate
              im = 2
              l2 = l2 - 1
           ELSE IF ( tag(l2:l2) .EQ. 'w' ) THEN
!             Falloff constant
              im = 3
              l2 = l2 - 1
           ELSE IF ( tag(l2:l2) .EQ. 'e' ) THEN
!             Equilibrium constant
              im = 4
              l2 = l2 - 1
           ELSE
              im = 1
           ENDIF

           READ(tag(1:l2), *) ik

           in = in + 1 
           mech_number(in) = ik
           mech_pre(in) = pre
           mech_beta(in) = beta
           mech_tac(in) = tac
           mech_order(in) = dn
           rate_type(in) = im

        ENDIF
     ENDIF
  ENDDO

50 CLOSE(i23)

  NMECH = in
  WRITE(line, *) 'Reaction rates found : ', NMECH
  CALL IO_WRITE_ASCII(lfile, line)

! Normalize Constants
  EREF = RGAS*TREF
  IF ( units(4) .EQ. 'cal' ) THEN
!    E is in calories/kelvin mol.
     EREF = EREF/4.184e+03
  ELSE IF ( units(4) .EQ. 'kcal' ) THEN
!    E is in kcalories/kelvin mol.
     EREF = EREF/4.184e+06
  ELSE IF ( units(4) .EQ. 'kj' .OR.&
       units(4) .EQ. 'kjoules') THEN
!    E is in kj/kelvin mol
     EREF = EREF*1.0e-6
  ELSE IF ( units(4) .EQ. 'j' .OR.&
       units(4) .EQ. 'joules' ) THEN
!    E is in joules/kelvin mol
     EREF = EREF*1.0e-3
  ENDIF

  DO ik=1, NMECH
     mech_tac(ik) = mech_tac(ik)/EREF
  ENDDO

! Do not forget that TFLAME contains the non-dimensional 
! adiabatic flame temperature.

  PREF = 1.0e+5                         ! N/m^2
  CREF = PREF*1.0e-3/(RGAS*TREF)        ! mol/cm^3
!  CREF = 1.0e-3/WREF

  WRITE(line, *) 'Pressure reference constant : ', CREF
  CALL IO_WRITE_ASCII(lfile, line)

  ir = 0
  k0mx = C_0_R 
  DO ik=1, NMECH
     IF ( rate_type(ik) .LT. 3 ) THEN
        k1 = mech_pre(ik)*EXP(-mech_tac(ik)&
             +mech_beta(ik)*LOG(TREF))&
             *(CREF**mech_order(ik))
        IF ( k1 .GT. k0mx ) THEN
           k0mx = k1
           ir = ik
        ENDIF
     ENDIF
  ENDDO

  WRITE(line, *) 'Maximum reaction rate : ', k0mx
  CALL IO_WRITE_ASCII(lfile, line)
  IF ( ir .GT. 0 ) THEN
     WRITE(line, '(A11,I4,A1,I1,A1)') 'Reaction : ', &
          mech_number(ir), &
          '(',rate_type(ir),')'
     CALL IO_WRITE_ASCII(lfile, line)
  ENDIF

  IF ( ichem_config .EQ. CHEM_PREMIXED .AND.&
       imech_type .EQ. CHEM_TYPE_UNIDECOMP ) THEN
! We assume m=rho u0 = 1.0
     rho1 = mean_rho + delta_rho/C_2_R
     rho2 = mean_rho - delta_rho/C_2_R
     beta1 = mech_tac(1)/TFLAME
     WRITE(line,*) 'Beta1 =', beta1
     CALL IO_WRITE_ASCII(lfile, line)
     WRITE(line,*) 'Alpha1=', CHW_CE
     CALL IO_WRITE_ASCII(lfile, line)
     alpha = CHW_CE/(C_1_R+CHW_CE)
     beta = beta1*alpha
     WRITE(line,*) 'Beta  =', beta
     CALL IO_WRITE_ASCII(lfile, line)
     sigma = (rho2*TFLAME*damkohler(inb_scal)*CREF*mech_pre(1)/(reynolds*prandtl))&
          *EXP(-beta1)
     WRITE(line,*) 'Sigma    =', sigma
     CALL IO_WRITE_ASCII(lfile, line)
     sigma_as = beta*(beta/C_2_R+C_3_R*alpha-1.344e0)
     WRITE(line,*) 'Sigma(as)=', sigma_as
     CALL IO_WRITE_ASCII(lfile, line)
     vo = (sqrt(C_2_R*rho2*TFLAME*damkohler(inb_scal)*CREF*mech_pre(1)&
          /(reynolds*prandtl))&
          /(beta*rho1))*EXP(-beta1/C_2_R)*(C_1_R&
          +(1.344-C_3_R*alpha)/beta)
     WRITE(line,*) 'Vo = ', vo
     CALL IO_WRITE_ASCII(lfile, line)
  ENDIF

  IF ( k0mx .EQ. C_0_R ) THEN
     k0mx = C_1_R
  ENDIF

! Normalize constants
  IF ( ichem_normalized .EQ. 1 ) THEN
     DO ik=1, NMECH
        mech_pre(ik) = mech_pre(ik)*&
             (TREF**mech_beta(ik))*&
             (CREF**mech_order(ik))
        IF ( rate_type(ik) .LT. 3 ) THEN
           mech_pre(ik) = mech_pre(ik)/k0mx
        ENDIF

        IF ( rate_type(ik) .EQ. 1 ) THEN
           str = 'f:'
        ELSE IF ( rate_type(ik) .EQ. 2 ) THEN
           str = 'b:'
        ELSE IF ( rate_type(ik) .EQ. 3 ) THEN
           str = 'w:'
        ELSE IF ( rate_type(ik) .EQ. 4 ) THEN
           str = 'e:'
        ENDIF
        WRITE(line, '(I4,A2,4(1X,E12.5))') &
             mech_number(ik), str, mech_pre(ik), &
             mech_beta(ik), mech_tac(ik), &
             mech_order(ik)
        CALL IO_WRITE_ASCII(lfile, line)
     ENDDO
  ENDIF

  RETURN
END SUBROUTINE CHEM_READ_MECH
