! Expected format
! x rho u  y_ch4 y_o2 y_h2o y_co2 y_co y_h2 y_h zeta
! 1  2  3   4   5     6    7     8     9    10   11

SUBROUTINE CHEM_READ_TEXT(file_name, inb, ix, ipos, imax, x_shift, x, z1, iwrk_size, wrk)

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"

  USE DNS_CONSTANTS, ONLY : efile

  IMPLICIT NONE

  CHARACTER*(*) file_name
  TINTEGER inb, ipos, ix
  TINTEGER imax
  TINTEGER iwrk_size
  TREAL x_shift
  TREAL x(imax)
  TREAL z1(imax)
  TREAL wrk(*)

  TINTEGER MAX_SIZE, MAX_BUFFER
  PARAMETER(MAX_SIZE=2048)
  PARAMETER(MAX_BUFFER=128)
  TREAL x_f(MAX_SIZE)
  TREAL y_f(MAX_SIZE)
  TINTEGER iopt, ifmax
  TINTEGER nest, itu, ic
  TINTEGER lwrk, iwrk, iiwrk, inu
  TREAL s, fp, xb, xe
  TREAL input_buffer(MAX_BUFFER)
  TINTEGER ier
  TINTEGER i, k, i1, kord
  CHARACTER*80 msg

  IF ( inb .GT. MAX_BUFFER ) THEN
     CALL IO_WRITE_ASCII(efile, 'Error: Buffer overflow')
     CALL DNS_STOP(DNS_ERROR_WRKOVERFLW)
  ENDIF

! Load Flame Data
  OPEN(23,file=file_name, status='old')

  READ(23,*) ifmax

  IF ( ifmax .GT. MAX_SIZE ) THEN
     CALL IO_WRITE_ASCII(efile, 'Error: Working array overflow')
     CALL DNS_STOP(DNS_ERROR_WRKOVERFLW)
  ENDIF

  DO k=1, ifmax
     READ(23,*) (input_buffer(i),i=1,inb)
     x_f(k) = input_buffer(ix) + x_shift
     y_f(k) = input_buffer(ipos)
  ENDDO

  CLOSE(23)

! Interpolation perameters
  i1 = 1
  iopt = 0
  s = C_0_R
  kord = 5
  nest = ifmax+kord+1
  inu = ifmax+kord+1
  lwrk = ifmax*(kord+1)+nest*(7+3*kord)

  itu = 1 + ifmax 
  ic = itu + nest 
  iwrk = ic + nest 
  iiwrk = iwrk + lwrk 

  IF ( iwrk_size .LT. ifmax + 3*nest + lwrk ) THEN
     CALL IO_WRITE_ASCII(efile, 'Error: DNS_READ_TEXT - '&
          //'Working array too small ')
     WRITE(msg,*) ifmax + 3*nest + lwrk
     CALL IO_WRITE_ASCII(efile,'Required = '//msg)
     WRITE(msg,*) iwrk_size
     CALL IO_WRITE_ASCII(efile,'Provided = '//msg)
     CALL DNS_STOP(DNS_ERROR_WRKSIZE)
  ENDIF
  xb = x_f(1)
  xe = x_f(ifmax)

! ########################################################
! Scalar field
  DO i=1, ifmax
     wrk(i) = C_1_R
  ENDDO

  call curfit(iopt, ifmax, x_f, y_f, wrk(1), xb, xe, kord, s, &
       nest, inu, wrk(itu), wrk(ic), fp, wrk(iwrk), lwrk, &
       wrk(iiwrk), ier)

  IF ( ier .NE. 0 .AND. ier .NE. -1 ) THEN
     WRITE(msg,*) 'Error: DNS_READ_TEXT - CURFIT returned = ',ier
     CALL IO_WRITE_ASCII(efile, msg)
     CALL DNS_STOP(DNS_ERROR_REGRID)
  ENDIF

  DO i=1, imax
     call splev(wrk(itu), inu, wrk(ic), kord, x(i), z1(i), &
          i1, ier)

     IF ( ier .NE. 0 .AND. ier .NE. -1 ) THEN
        WRITE(msg,*) 'Error: DNS_READ_TEXT - SPLEV returned = ',&
             ier
        CALL IO_WRITE_ASCII(efile, msg)
        CALL DNS_STOP(DNS_ERROR_REGRID)
     ENDIF
  ENDDO

  RETURN
END SUBROUTINE CHEM_READ_TEXT
