SUBROUTINE CORR1V3PTX(imax,jmax,kmax, dx, dy, dz, u, rux, lux, vol, wc)

#include "types.h"
#include "dns_error.h"

  USE DNS_GLOBAL, ONLY: efile

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER LOCAL_P_WRK
  PARAMETER(LOCAL_P_WRK=2048)

  TINTEGER imax, jmax, kmax
  TREAL dx(imax), dy(jmax), dz(kmax)
  TREAL u(imax, jmax, kmax)
  TREAL wc(imax, jmax, kmax)
  TREAL rux(imax/2+1)
  TREAL lux, vol

  TINTEGER p_wrk(LOCAL_P_WRK)
  TREAL AVG1V3D, SIMPSON, ref
  TINTEGER ilen
  TINTEGER i, j, k, id, index
#ifdef PARALLEL
  INTEGER ims_pro, ims_err

  CALL MPI_COMM_RANK(MPI_COMM_WORLD,ims_pro,ims_err)
#endif

! check size
  IF ( imax .GT. LOCAL_P_WRK ) THEN
     CALL IO_WRITE_ASCII(efile,'CORR1V3PTX. Working array size too small')
     CALL DNS_STOP(DNS_ERROR_WRKSIZE)
  ENDIF

! maximum off-set
  ilen = imax/2+1

! translational off-set
  DO id = 1,ilen

! defining pointer to save time
     DO i = 1,imax
        index = i+id-2
        p_wrk(i) = MOD(index,imax)+1
     ENDDO

     DO k = 1, kmax
        DO j = 1,jmax
           DO i = 1,imax
!              index = i+id-2
!              wc(i,j,k) = u(i,j,k)*u(MOD(index,imax)+1,j,k)
              wc(i,j,k) = u(i,j,k)*u(p_wrk(i),j,k)
           ENDDO
        ENDDO
     ENDDO
     rux(id) = AVG1V3D(imax, jmax, kmax, i1, wc)
  ENDDO

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif

     IF ( rux(1) .NE. C_0_R ) THEN
        lux = SIMPSON(ilen, rux, dx)/rux(1)
        ref = rux(1)
        DO id=1,ilen
           rux(id) = rux(id)/ref
        ENDDO
     ELSE
        lux = C_0_R
     ENDIF

#ifdef PARALLEL
  ENDIF
#endif

  RETURN
END SUBROUTINE CORR1V3PTX
