SUBROUTINE CORRT_SPA(nvar, nvarcross, ntimes, kmax, kmax_total, &
     nstatlin, statlin_i, statlin_j, t_dat, l_dat, autocrl, crosscrl, wrk1d )
  
! l_dat enters with the mean already removed
  
! correlation without normalization (i.e. first element in autocorrelations
! has the variance)

#include "types.h"

#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER nvar, nvarcross, ntimes, kmax, kmax_total, nstatlin

  TREAL t_dat(2,ntimes)
  TREAL l_dat(ntimes,nstatlin,nvar,kmax)

  TINTEGER statlin_i(nstatlin), statlin_j(nstatlin)

  TREAL autocrl(ntimes,nstatlin,nvar)
  TREAL crosscrl(ntimes,nstatlin,nvarcross)
  TREAL wrk1d(ntimes,*)

  TINTEGER i, j, k, n, it, ivar, itmax
  TREAL delta_t
  CHARACTER*32 filename
#ifdef PARALLEL
  INTEGER len
#endif

! ################
! # Initializing #
! ################
  DO it = 1,ntimes*nvar*nstatlin
     autocrl(it,1,1) = C_0_R
  ENDDO
  DO it = 1,ntimes*nvarcross*nstatlin
     crosscrl(it,1,1) = C_0_R
  ENDDO
  DO it = 1,ntimes
     wrk1d(it,1) = t_dat(2,it)
  ENDDO

! #############################
! # Calculate autocorrelation #
! #############################
  DO k = 1,kmax
     DO ivar = 1,nvar
        DO n = 1,nstatlin
           CALL MK_CST_DT( ntimes, l_dat(1,n,ivar,k), wrk1d(1,1), delta_t, wrk1d(1,3) )
           CALL CORR1DP(ntimes, wrk1d(1,3), wrk1d(1,3), wrk1d(1,4))
           DO it = 1,ntimes
              autocrl(it,n,ivar) = autocrl(it,n,ivar) + wrk1d(it,4)
           ENDDO
        ENDDO
     ENDDO
  ENDDO

! ##############################
! # Calculate crosscorrelation #
! ##############################
  DO k = 1,kmax
     DO n = 1,nstatlin
        CALL MK_CST_DT( ntimes, l_dat(1,n,1,k), wrk1d(1,1), delta_t, wrk1d(1,2) ) ! u
        CALL MK_CST_DT( ntimes, l_dat(1,n,2,k), wrk1d(1,1), delta_t, wrk1d(1,3) ) ! v
        CALL MK_CST_DT( ntimes, l_dat(1,n,3,k), wrk1d(1,1), delta_t, wrk1d(1,4) ) ! w

        CALL CORR1DPN(ntimes, wrk1d(1,2), wrk1d(1,3), wrk1d(1,5))
        DO it = 1,ntimes
           crosscrl(it,n,1) = crosscrl(it,n,1) + wrk1d(it,5)
        ENDDO
        CALL CORR1DPN(ntimes, wrk1d(1,2), wrk1d(1,4), wrk1d(1,5))
        DO it = 1,ntimes
           crosscrl(it,n,2) = crosscrl(it,n,2) + wrk1d(it,5)
        ENDDO
        CALL CORR1DPN(ntimes, wrk1d(1,3), wrk1d(1,4), wrk1d(1,5))
        DO it = 1,ntimes
           crosscrl(it,n,3) = crosscrl(it,n,3) + wrk1d(it,5)
        ENDDO

     ENDDO
  ENDDO

! Average in k for autocorrelations
  itmax = ntimes*nstatlin*nvar
#ifdef PARALLEL
  len = itmax
  CALL MPI_ALLREDUCE(autocrl, l_dat, len, MPI_REAL8, MPI_SUM, MPI_COMM_WORLD, ims_err)
  DO it = 1,itmax
     autocrl(it,1,1) = l_dat(it,1,1,1)
  ENDDO
#endif
  DO it = 1,itmax
     autocrl(it,1,1) = autocrl(it,1,1)/M_REAL(kmax_total*ntimes)
  ENDDO

! Average in k for crosscorrelations
  itmax = ntimes*nstatlin*nvarcross
#ifdef PARALLEL
  len = itmax
  CALL MPI_ALLREDUCE(crosscrl, l_dat, len, MPI_REAL8, MPI_SUM, MPI_COMM_WORLD, ims_err)
  DO it = 1,itmax
     crosscrl(it,1,1) = l_dat(it,1,1,1)
  ENDDO
#endif
  DO it = 1,itmax
     crosscrl(it,1,1) = crosscrl(it,1,1)/M_REAL(kmax_total*ntimes)
  ENDDO

! ###############
! # Tkstat file #
! ###############

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif

     it = INT(t_dat(1,ntimes))
     WRITE(filename,*) it; filename = 'tcr'//TRIM(ADJUSTL(filename))

     OPEN(unit=23,file=filename)
     WRITE(23, *) 'RTIME = ', t_dat(2,ntimes)
     WRITE(23, *) 'NUMBEROFPOINTS = ', ntimes
     WRITE(23, *) 'I J time Cuu(t) Cvv(t) Cww(t) Cpp(t) Crr(t) Css(t) Cuv(t) Cuw(t) Cvw(t)'

     DO n = 1,nstatlin
        i = statlin_i(n)
        j = statlin_j(n)
        DO it = 1, ntimes
           WRITE(23,1020) i, j, delta_t*M_REAL(it-1),&
                (autocrl(it,n,ivar), ivar=1,nvar), (crosscrl(it,n,ivar), ivar=1,nvarcross)
        ENDDO
     ENDDO

     CLOSE(23)

#ifdef PARALLEL
  ENDIF
#endif

  RETURN

1020 FORMAT(I3,1X,I3,10(1X,E12.5E3))

END SUBROUTINE CORRT_SPA

