SUBROUTINE CORRY_SPA (itime, itxc, imax, jmax, kmax, kmax_total,&
     x, y, z, dy, nvar, nstatpln, statpln, ntimes, t_dat, p_dat, crl, wrk1d, tmp)

#include "types.h"

  USE DNS_GLOBAL, ONLY : efile
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER itime, itxc
  TINTEGER imax, jmax, kmax, kmax_total
  TINTEGER nvar, nstatpln, ntimes

  TREAL x(imax) 
  TREAL y(jmax) 
  TREAL z(kmax_total) 
  TREAL dy(jmax) 

  TREAL t_dat(2,ntimes)
  TREAL p_dat(ntimes,nstatpln,jmax,nvar,kmax)

  TINTEGER statpln(nstatpln)

  TREAL crl(jmax/2+1,nvar,nstatpln)
  TREAL wrk1d(jmax,*)
  TREAL tmp((jmax/2+1)*nvar*nstatpln)

  TINTEGER i, j, k, n, it
  TREAL delta_y 
  CHARACTER*32 filename
#ifdef PARALLEL
  INTEGER len
#endif

! ################
! # Initializing #
! ################

  IF ( (jmax/2+1)*nvar*nstatpln .GT. itxc ) THEN
     CALL IO_WRITE_ASCII(efile,'CORRY_SPA: not enough space in tmp')
     CALL DNS_STOP(345)
  ENDIF

  DO j = 1,(jmax/2+1)*nvar*nstatpln
     crl(j,1,1) = C_0_R
  ENDDO

! p_dat has to enter with the mean removed
! ###################
! # Remove the mean #
! ###################
!
!
!  DO n = 1,nvar
!     DO j = 1,jmax
!        DO i = 1,nstatpln
!
!           sum = C_0_R
!           DO k = 1,kmax
!              DO it = 1,ntimes 
!                 sum = sum + p_dat(it,i,j,n,k)
!              ENDDO
!           ENDDO
!           sum = sum/M_REAL(ntimes)
!#ifdef PARALLEL
!           sum2 = sum/M_REAL(kmax_total)
!           CALL MPI_ALLREDUCE(sum2, sum, 1, MPI_REAL8, MPI_SUM, 
! $              MPI_COMM_WORLD, ims_err)
!#else 
!           sum = sum/M_REAL(kmax_total)
!#endif
!           DO k = 1,kmax
!              DO it = 1,ntimes 
!                 p_dat(it,i,j,n,k) = p_dat(it,i,j,n,k) - sum
!              ENDDO
!           ENDDO
!              
!        ENDDO
!     ENDDO
!  ENDDO

! #############################
! # Calculate autocorrelation #
! #############################

  DO k = 1,kmax
     DO n = 1,2
        DO i = 1,nstatpln
           DO it = 1,ntimes

              DO j = 1,jmax
                 wrk1d(j,1) = p_dat(it,i,j,n,k)
              ENDDO
              CALL MK_CST_DT( jmax, wrk1d(1,1), y, delta_y, wrk1d(1,2) ) 
              CALL CORR1DNP ( i1, jmax, wrk1d(1,2), wrk1d(1,2), wrk1d(1,3) )
              DO j = 1,jmax/2+1
                 crl(j,n,i) = crl(j,n,i) + wrk1d(j,3)
              ENDDO

           ENDDO
        ENDDO
     ENDDO
  ENDDO

  IF ( kmax_total .GT. 1 ) THEN
     DO k = 1,kmax
        DO i = 1,nstatpln
           DO it = 1,ntimes
              DO j = 1,jmax
                 wrk1d(j,1) = p_dat(it,i,j,3,k)
              ENDDO
              CALL MK_CST_DT( jmax, wrk1d(1,1), y, delta_y, wrk1d(1,2) ) 
              CALL CORR1DNP ( i1, jmax, wrk1d(1,2), wrk1d(1,2),wrk1d(1,3) )
              DO j = 1,jmax/2+1
                 crl(j,3,i) = crl(j,3,i) + wrk1d(j,3)
              ENDDO
           ENDDO
        ENDDO
     ENDDO
  ENDIF

  DO k = 1,kmax
     DO n = 4,nvar
        DO i = 1,nstatpln
           DO it = 1,ntimes

              DO j = 1,jmax
                 wrk1d(j,1) = p_dat(it,i,j,n,k)
              ENDDO
              CALL MK_CST_DT( jmax, wrk1d(1,1), y, delta_y, wrk1d(1,2) ) 
              CALL CORR1DNP ( i1, jmax, wrk1d(1,2), wrk1d(1,2), wrk1d(1,3) )
              DO j = 1,jmax/2+1
                 crl(j,n,i) = crl(j,n,i) + wrk1d(j,3)
              ENDDO

           ENDDO
        ENDDO
     ENDDO
  ENDDO

! Average in k for autocorrelations

#ifdef PARALLEL
  len = (jmax/2+1)*nstatpln*nvar
  CALL MPI_ALLREDUCE(crl, tmp, len, MPI_REAL8, MPI_SUM, MPI_COMM_WORLD, ims_err)
  DO j = 1,(jmax/2+1)*nvar*nstatpln
     crl(j,1,1) = tmp(j)
  ENDDO
#endif

  DO j = 1,(jmax/2+1)*nvar*nstatpln
     crl(j,1,1) = crl(j,1,1)/M_REAL(ntimes*kmax_total)
  ENDDO

! ###############
! # Tkstat file #
! ###############

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif

     WRITE(filename,*) itime; filename = 'ycr'//TRIM(ADJUSTL(filename))

     OPEN(unit=i23,file=filename)
     WRITE(i23, *) 'RTIME = ', t_dat(2,ntimes)
     WRITE(i23, *) 'I J ry Cuu(y) Cvv(y) Cww(y) Cpp(y) Crr(y) Css(y)'

     DO n = 1,nstatpln
        i = statpln(n)
        DO j = 1, jmax/2+1
           WRITE(i23,1020) i, jmax/2, delta_y*M_REAL(j-1), &
                crl(j,1,n), crl(j,2,n), crl(j,3,n), crl(j,4,n), crl(j,5,n), crl(j,6,n)
        ENDDO
     ENDDO

     CLOSE(i23)

#ifdef PARALLEL
  ENDIF
#endif

  RETURN

1020 FORMAT(I3,1X,I3,8(1X,E12.5E3))

END SUBROUTINE CORRY_SPA

