#include "types.h"
#ifdef PARALLEL
#include "dns_const_mpi.h"
#endif

SUBROUTINE DNS_CHECK(nx,ny,nz, a, txc, wrk2d,wrk3d)

  USE DNS_GLOBAL, ONLY : isize_field,isize_txc_field, isize_wrk2d
  USE DNS_GLOBAL, ONLY : imax_total,kmax_total !,jmax_total 
  USE DNS_GLOBAL, ONLY : ifourier, fft_reordering
  USE DNS_CONSTANTS, ONLY : lfile
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif 

  TINTEGER nx,ny,nz
  TREAL, DIMENSION(isize_field,2)     :: a
  TREAL, DIMENSION(isize_txc_field,3) :: txc
  TREAL, DIMENSION(isize_txc_field)   :: wrk3d
  TREAL, DIMENSION(isize_wrk2d,2)     :: wrk2d
 
! -------------------------------------------------------------------
  TREAL residual
  TINTEGER t_srt,t_end,t_dif, PROC_CYCLES, MAX_CYCLES
  TREAL norm
  CHARACTER*64 str
  CHARACTER*256 line

#ifdef PARALLEL
  TREAL dummy
  TINTEGER idummy, id
#endif

! ###################################################################
! Create random array
  CALL RANDOM_NUMBER(a(1:isize_field,1))

! -------------------------------------------------------------------
! Transposition along OX
! -------------------------------------------------------------------
#ifdef PARALLEL         
  IF ( ims_npro_i .GT. 1 ) THEN
     id = DNS_MPI_I_PARTIAL

     CALL SYSTEM_CLOCK(t_srt,PROC_CYCLES,MAX_CYCLES)
     CALL DNS_MPI_TRPF_I(a(1,1), wrk3d, ims_ds_i(1,id), ims_dr_i(1,id), ims_ts_i(1,id), ims_tr_i(1,id))
     CALL DNS_MPI_TRPB_I(wrk3d, a(1,2), ims_ds_i(1,id), ims_dr_i(1,id), ims_ts_i(1,id), ims_tr_i(1,id))
     CALL SYSTEM_CLOCK(t_end,PROC_CYCLES,MAX_CYCLES)

     idummy = t_end-t_srt
     CALL MPI_REDUCE(idummy, t_dif, 1, MPI_INT, MPI_MAX, 0, MPI_COMM_WORLD, ims_err) 
     WRITE(str,100) M_REAL(t_dif)/PROC_CYCLES

     dummy = MAXVAL(ABS(a(1:isize_field,1)-a(1:isize_field,2)))
     CALL MPI_REDUCE(dummy, residual, 1, MPI_REAL8, MPI_MAX, 0, MPI_COMM_WORLD, ims_err)

     WRITE(line,100) residual
     line = 'Checking MPI transposition for Ox derivatives: Residual '&
          //TRIM(ADJUSTL(line))//'. Max. elapsed time '//TRIM(ADJUSTL(str))//' sec.'
     CALL IO_WRITE_ASCII(lfile,line)
     
  ENDIF
#endif

! -------------------------------------------------------------------
! Transposition along OZ
! -------------------------------------------------------------------
#ifdef PARALLEL         
  IF ( ims_npro_k .GT. 1 ) THEN
     id = DNS_MPI_K_PARTIAL

     CALL SYSTEM_CLOCK(t_srt,PROC_CYCLES,MAX_CYCLES)
     CALL DNS_MPI_TRPF_K(a(1,1), wrk3d, ims_ds_k(1,id), ims_dr_k(1,id), ims_ts_k(1,id), ims_tr_k(1,id))
     CALL DNS_MPI_TRPB_K(wrk3d, a(1,2), ims_ds_k(1,id), ims_dr_k(1,id), ims_ts_k(1,id), ims_tr_k(1,id))
     CALL SYSTEM_CLOCK(t_end,PROC_CYCLES,MAX_CYCLES)

     idummy = t_end-t_srt
     CALL MPI_REDUCE(idummy, t_dif, 1, MPI_INT, MPI_MAX, 0, MPI_COMM_WORLD, ims_err) 
     WRITE(str,100) M_REAL(t_dif)/PROC_CYCLES 

     dummy = MAXVAL(ABS(a(1:isize_field,1)-a(1:isize_field,2)))
     CALL MPI_REDUCE(dummy, residual, 1, MPI_REAL8, MPI_MAX, 0, MPI_COMM_WORLD, ims_err)

     WRITE(line,100) residual
     line = 'Checking MPI transposition for Oz derivatives: Residual '&
          //TRIM(ADJUSTL(line))//'. Max. elapsed time '//TRIM(ADJUSTL(str))//' sec.'
     CALL IO_WRITE_ASCII(lfile,line)
     
  ENDIF
#endif
  
! -------------------------------------------------------------------
! Poisson FFT
! -------------------------------------------------------------------
  IF ( ifourier .EQ. 1) THEN

     wrk2d(:,1:2) = C_0_R
     txc(1:isize_field,3) = a(1:isize_field,1)

!     fft_reordering = 1
     CALL SYSTEM_CLOCK(t_srt,PROC_CYCLES,MAX_CYCLES)
     CALL OPR_FOURIER_F(i2, nx,ny,nz, txc(1,3),txc(1,1), txc(1,2), wrk2d,wrk3d)
     CALL OPR_FOURIER_B(i2, nx,ny,nz, txc(1,1),txc(1,2), wrk3d)
     CALL SYSTEM_CLOCK(t_end,PROC_CYCLES,MAX_CYCLES)
!     fft_reordering = 0

     a(1:isize_field,2) = txc(1:isize_field,2)

#ifdef PARALLEL         
     idummy = t_end-t_srt
     CALL MPI_REDUCE(idummy, t_dif, 1, MPI_INT, MPI_MAX, 0, MPI_COMM_WORLD, ims_err) 
#else
     t_dif  = t_end-t_srt
#endif
     WRITE(str,100) M_REAL(t_dif)/PROC_CYCLES

     norm = C_1_R/M_REAL(imax_total*kmax_total)
!     norm = norm /M_REAL(jmax_total) ! for large domains we need to do it in two steps !

#ifdef PARALLEL         
     dummy = MAXVAL(ABS(norm*a(1:isize_field,2)-a(1:isize_field,1)))
     CALL MPI_REDUCE(dummy, residual, 1, MPI_REAL8, MPI_MAX, 0, MPI_COMM_WORLD, ims_err)
#else
     residual = MAXVAL(ABS(norm*a(1:isize_field,2)-a(1:isize_field,1)))
#endif

     WRITE(line,100) residual
     line = 'Checking FFT routines: Residual '&
          //TRIM(ADJUSTL(line))//'. Max. elapsed time '//TRIM(ADJUSTL(str))//' sec.'
     CALL IO_WRITE_ASCII(lfile,line)

  ENDIF

  RETURN

100 FORMAT(G_FORMAT_R)

END SUBROUTINE DNS_CHECK
