!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/12 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the scalar gradient production term as given by -(G_i G_j s_ij)
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE FI_GRADIENT_PRODUCTION&
     (imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, z1, u, v, w, result, grad_x, grad_y, grad_z, tmp1, tmp2, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "types.h"
#include "integers.h"

  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax)

  TREAL z1(*), u(*), v(*), w(*), result(*)
  TREAL grad_x(*), grad_y(*), grad_z(*)
  TREAL tmp1(*), tmp2(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

! -------------------------------------------------------------------
  TINTEGER ij

! ###################################################################
! Vorticity vector
! ###################################################################
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, z1, grad_x, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, z1, grad_y, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, z1, grad_z, i0, i0, wrk1d, wrk2d, wrk3d)

! ###################################################################
! Production term
! ###################################################################
! Ux, Vy, Wz
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = tmp1(ij)*grad_x(ij)*grad_x(ij) + tmp2(ij)*grad_y(ij)*grad_y(ij)
  ENDDO
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp2(ij)*grad_z(ij)*grad_z(ij)
  ENDDO

! Uy, Vx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + (tmp1(ij)+tmp2(ij))*grad_x(ij)*grad_y(ij)
  ENDDO

! Uz, Wx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + (tmp1(ij)+tmp2(ij))*grad_x(ij)*grad_z(ij)
  ENDDO

! Vz, Wy
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = -(result(ij) + (tmp1(ij)+tmp2(ij))*grad_y(ij)*grad_z(ij))
  ENDDO

  RETURN
END SUBROUTINE FI_GRADIENT_PRODUCTION
