!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/12 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate second invariant of the velocity gradient tensor 
!# Second invariant of the velocity gradient tensor, Q, as defined
!# by Chong et al. (1990). Regions with high positive values of Q are 
!# vorticity dominated, regions with high negative values of Q are 
!# strain dominated.
!# If incompressible, P=0 and then Q=(w^2-2s^2)/4, where P is the first
!# invariant, P=-div(v). 
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"

SUBROUTINE FI_INVARIANT_Q(imode_fdm, nx,ny,nz, i1bc,j1bc,k1bc, &
     dx,dy,dz, u,v,w, result, tmp1,tmp2,tmp3, wrk1d,wrk2d,wrk3d)

  IMPLICIT NONE

#include "integers.h"

  TINTEGER imode_fdm, nx,ny,nz, i1bc,j1bc,k1bc

  TREAL, DIMENSION(*)        :: dx,dy,dz
  TREAL, DIMENSION(nx,ny,nz) :: u,v,w, result
  TREAL, DIMENSION(nx,ny,nz) :: tmp1,tmp2,tmp3
  TREAL, DIMENSION(*)        :: wrk1d,wrk2d,wrk3d

! ###################################################################
! -------------------------------------------------------------------
! diagonal terms
! -------------------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, nx,ny,nz, i1bc, dx, u, tmp1, i0,i0, wrk1d,wrk2d,wrk3d)
  CALL PARTIAL_Y(imode_fdm, nx,ny,nz, j1bc, dy, v, tmp2, i0,i0, wrk1d,wrk2d,wrk3d)
  CALL PARTIAL_Z(imode_fdm, nx,ny,nz, k1bc, dz, w, tmp3, i0,i0, wrk1d,wrk2d,wrk3d)
  result = tmp1*tmp2 + tmp2*tmp3 + tmp3*tmp1

! -------------------------------------------------------------------
! off-diagonal terms
! -------------------------------------------------------------------
! UyVx
  CALL PARTIAL_X(imode_fdm, nx,ny,nz, i1bc, dx, v, tmp1, i0,i0, wrk1d,wrk2d,wrk3d)
  CALL PARTIAL_Y(imode_fdm, nx,ny,nz, j1bc, dy, u, tmp2, i0,i0, wrk1d,wrk2d,wrk3d)
  result = result - tmp1*tmp2

! UzWx
  CALL PARTIAL_X(imode_fdm, nx,ny,nz, i1bc, dx, w, tmp1, i0,i0, wrk1d,wrk2d,wrk3d)
  CALL PARTIAL_Z(imode_fdm, nx,ny,nz, k1bc, dz, u, tmp2, i0,i0, wrk1d,wrk2d,wrk3d)
  result = result - tmp1*tmp2

! WyVz
  CALL PARTIAL_Y(imode_fdm, nx,ny,nz, j1bc, dy, w, tmp1, i0,i0, wrk1d,wrk2d,wrk3d)
  CALL PARTIAL_Z(imode_fdm, nx,ny,nz, k1bc, dz, v, tmp2, i0,i0, wrk1d,wrk2d,wrk3d)
  result = result - tmp1*tmp2

  RETURN
END SUBROUTINE FI_INVARIANT_Q
