!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/17 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the strain diffusion term as given by s_ij lap s_ij
!# The kinematic viscosity \nu is not multiplied here.
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"

SUBROUTINE FI_STRAIN_DIFFUSION&
     (iunifx, iunify, iunifz, imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx,dy,dz, u, v, w, result, strain, tmp1, tmp2, tmp3, tmp4, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "integers.h"

  TINTEGER iunifx, iunify, iunifz
  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL, DIMENSION(*) :: dx, dy, dz

  TREAL u(*), v(*), w(*), result(*)
  TREAL strain(*)
  TREAL tmp1(*), tmp2(*), tmp3(*), tmp4(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

! -------------------------------------------------------------------
  TINTEGER ij

! ###################################################################
! -------------------------------------------------------------------
! S_11 = U,x
! -------------------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, u, strain, i0, i0, wrk1d, wrk2d, wrk3d)

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, strain, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, strain, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, strain, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = strain(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! S_22 = V,y
! -------------------------------------------------------------------
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, v, strain, i0, i0, wrk1d, wrk2d, wrk3d)

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, strain, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, strain, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, strain, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + strain(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! S_33 = W,z
! -------------------------------------------------------------------
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, w, strain, i0, i0, wrk1d, wrk2d, wrk3d)

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, strain, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, strain, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, strain, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + strain(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! S_12 = (V,x + U,y)/2
! -------------------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     strain(ij) = tmp1(ij)+tmp2(ij)
  ENDDO

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, strain, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, strain, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, strain, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + C_05_R*strain(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! S_13 = (U,z + W,x)/2
! -------------------------------------------------------------------
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     strain(ij) = tmp1(ij)+tmp2(ij)
  ENDDO

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, strain, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, strain, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, strain, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + C_05_R*strain(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! S_23 = (W,y + V,z)/2
! -------------------------------------------------------------------
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     strain(ij) = tmp1(ij)+tmp2(ij)
  ENDDO

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, strain, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, strain, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, strain, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + C_05_R*strain(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO


  RETURN
END SUBROUTINE FI_STRAIN_DIFFUSION
