!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/11 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the strain-pressure term as given by -s_ij p,ij:
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"

SUBROUTINE FI_STRAIN_PRESSURE&
     (iunifx, iunify, iunifz, imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, u, v, w, p, result, tmp1, tmp2, tmp3, tmp4, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "integers.h"

  TINTEGER iunifx, iunify, iunifz
  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL, DIMENSION(*) :: dx, dy, dz

  TREAL u(*), v(*), w(*), p(*), result(*)
  TREAL tmp1(*), tmp2(*), tmp3(*), tmp4(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

  ! -------------------------------------------------------------------
  TINTEGER ij

  ! ###################################################################
  ! -------------------------------------------------------------------
  ! diagonal terms
  ! -------------------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, p, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = tmp1(ij)*tmp2(ij)
  ENDDO

  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, p, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp1(ij)*tmp2(ij)
  ENDDO

  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, p, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp1(ij)*tmp2(ij)
  ENDDO

  ! -------------------------------------------------------------------
  ! off-diagonal terms
  ! -------------------------------------------------------------------
  ! 2 s_12 p,xy
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, p, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, tmp2, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp3, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp1(ij)*(tmp2(ij)+tmp3(ij))
  ENDDO

  ! 2 s_13 p,xz
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, p, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, tmp2, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp3, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp1(ij)*(tmp2(ij)+tmp3(ij))
  ENDDO

  ! 2 s_23 p,yz
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, p, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, tmp2, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp3, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + tmp1(ij)*(tmp2(ij)+tmp3(ij))
  ENDDO

  ! right sign
  DO ij = 1,imax*jmax*kmax
     result(ij) =-result(ij)
  ENDDO

  RETURN
END SUBROUTINE FI_STRAIN_PRESSURE
