!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/11 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate components of the strain tensor
!#
!########################################################################
!# ARGUMENTS 
!#
!# result    Out     Array containing the tensor elements:
!#                   1  Sxx 
!#                   2  Syy
!#                   3  Szz
!#                   4  Sxy
!#                   5  Sxz
!#                   6  Syz
!#
!########################################################################
SUBROUTINE FI_STRAIN_TENSOR(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, u, v, w, result, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "types.h"
#include "integers.h"

  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax)

  TREAL u(*), v(*), w(*), result(imax*jmax*kmax,6)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

  ! -------------------------------------------------------------------
  TINTEGER ij

! ###################################################################
! Off diagonal terms
! result1 used as auxiliar array
! ###################################################################
! Uy, Vx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, result(1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, result(1,4), i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij,4) = C_05_R*(result(ij,4)+result(ij,1))
  ENDDO

! Uz, Wx
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, result(1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, result(1,5), i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij,5) = C_05_R*(result(ij,5)+result(ij,1))
  ENDDO

! Vz, Wy
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, result(1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, result(1,6), i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     result(ij,6) = C_05_R*(result(ij,6)+result(ij,1))
  ENDDO

! ###################################################################
! diagonal terms
! ###################################################################
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, u, result(1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, v, result(1,2), i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, w, result(1,3), i0, i0, wrk1d, wrk2d, wrk3d)

  RETURN
END SUBROUTINE FI_STRAIN_TENSOR
