!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/11 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the vorticity as given by w_i w_i:
!# (v,x-u,y)^2 + (u,z-w,x)^2 + (w,y-v,z)^2
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"

SUBROUTINE FI_VORTICITY(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, u, v, w, result, tmp1, tmp2, wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "integers.h"

  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL, DIMENSION(*) :: dx, dy, dz

  TREAL u(*), v(*), w(*), result(*)
  TREAL tmp1(*), tmp2(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

! -------------------------------------------------------------------
  TINTEGER ij
  TREAL dummy

! ###################################################################
! v,x-u,y
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     dummy = tmp1(ij)-tmp2(ij)
     result(ij) = dummy*dummy
  ENDDO

! u,z-w,x
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     dummy = tmp1(ij)-tmp2(ij)
     result(ij) = result(ij) + dummy*dummy
  ENDDO

! w,y-v,z
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     dummy = tmp1(ij)-tmp2(ij)
     result(ij) = result(ij) + dummy*dummy
  ENDDO

  RETURN
END SUBROUTINE FI_VORTICITY
