!########################################################################
!# Tool/Library FIELDS
!#
!########################################################################
!# HISTORY
!#
!# 2007/09/17 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the vorticity diffusion term as given by w_i lap w_i
!# The kinematic viscosity \nu is not multiplied here.
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
#include "types.h"

SUBROUTINE FI_VORTICITY_DIFFUSION&
     (iunifx, iunify, iunifz, imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
     dx, dy, dz, u, v, w, result, vort, tmp1, tmp2, tmp3, tmp4, &
     wrk1d, wrk2d, wrk3d)

  IMPLICIT NONE

#include "integers.h"

  TINTEGER iunifx, iunify, iunifz
  TINTEGER imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc

  TREAL, DIMENSION(*) :: dx, dy, dz

  TREAL u(*), v(*), w(*), result(*)
  TREAL vort(*)
  TREAL tmp1(*), tmp2(*), tmp3(*), tmp4(*)

  TREAL wrk1d(imax,*)
  TREAL wrk2d(imax,*)
  TREAL wrk3d(imax,jmax,kmax)

! -------------------------------------------------------------------
  TINTEGER ij

! ###################################################################
! -------------------------------------------------------------------
! W_z = V,x - U,y
! -------------------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, v, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, u, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     vort(ij) = tmp1(ij)-tmp2(ij)
  ENDDO

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, vort, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, vort, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, vort, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = vort(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! W_y = U,z - W,x
! -------------------------------------------------------------------
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, u, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, w, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     vort(ij) = tmp1(ij)-tmp2(ij)
  ENDDO

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, vort, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, vort, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, vort, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + vort(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

! -------------------------------------------------------------------
! W_z = W,y - V,z
! -------------------------------------------------------------------
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, w, tmp1, i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, v, tmp2, i0, i0, wrk1d, wrk2d, wrk3d)
  DO ij = 1,imax*jmax*kmax
     vort(ij) = tmp1(ij)-tmp2(ij)
  ENDDO

  CALL PARTIAL_ZZ(i0, iunifz, imode_fdm, imax, jmax, kmax, k1bc,&
       dz, vort, tmp3, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_YY(i0, iunify, imode_fdm, imax, jmax, kmax, j1bc,&
       dy, vort, tmp2, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_XX(i0, iunifx, imode_fdm, imax, jmax, kmax, i1bc,&
       dx, vort, tmp1, i0, i0, i0, i0, tmp4, wrk1d, wrk2d, wrk3d)

  DO ij = 1,imax*jmax*kmax
     result(ij) = result(ij) + vort(ij)*( tmp1(ij)+tmp2(ij)+tmp3(ij) )
  ENDDO

  RETURN
END SUBROUTINE FI_VORTICITY_DIFFUSION
