#include "types.h"
#include "dns_const.h"

!########################################################################
!# FLow/Library
!#
!########################################################################
!# HISTORY
!#
!# 2015/06/25 - A de Lozar
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate the transport terms due to selling
!# There are two options for the settling model.The first corresponds to a 
!# constant settling velocity while in the second the settling velocity 
!# assumes that the droplet spectrum is represented by a log-normal distrubution.
!# For each case we consider a simplified calculaition and a more exact one (differences of 10^-4)
!########################################################################
SUBROUTINE  FLOW_TRANSPORT(itransport, nx,ny,nz,s,is, is_trans, param, settling,tmp, &
                           imode_fdm, j1bc, dy, wrk1d,wrk2d,wrk3d ,trans)
  USE DNS_GLOBAL, ONLY : jmax_total

  IMPLICIT NONE

#include "integers.h"

  TINTEGER,                          INTENT(IN)    :: itransport, nx,ny,nz
  TREAL, DIMENSION(nx*ny*nz,*),      INTENT(IN)       :: s          ! Array with all  scalars
  TINTEGER,                          INTENT(IN)    :: is         ! Scalar for which the transport term is calculated
  TINTEGER,                          INTENT(IN)    :: is_trans   ! Scalar used to calculate the transport
  TREAL, DIMENSION(*),               INTENT(IN)    :: param      ! Transport Parameters
  TREAL,                             INTENT(IN)    :: settling   ! Settling Parameter
  TREAL, DIMENSION(nx*ny*nz,*),      INTENT(INOUT) :: tmp        ! It saves some calculations to include some calculations outside the function in some cases
  TREAL, DIMENSION(nx*ny*nz,*),      INTENT(OUT)    :: trans     ! Transport compomemt. It can have three directions
 
! Parameters for the Y derivative
  TINTEGER imode_fdm, j1bc
  TREAL, DIMENSION(jmax_total,*)         :: dy
  TREAL, DIMENSION(nx*ny*nz)             :: wrk3d
  TREAL, DIMENSION(*)                    :: wrk1d 
  TREAL, DIMENSION(nx*nz)                :: wrk2d



! -----------------------------------------------------------------------

  TREAL dummy, dummy2
  TINTEGER isize_field,ij

  isize_field = nx*ny*nz
     
    IF (itransport .EQ. EQNS_TRANS_C_SET ) THEN ! Cosntant Settling forumlation in simplified way
       dummy = settling*param(is)
       DO ij = 1,isize_field    
          trans(ij,1) =  dummy*tmp(ij,1) ! Linear formulation (all sizes with the same velocity). Assume ds/dy in tmp
       ENDDO
    ELSEIF (itransport .EQ. EQNS_TRANS_LN_SET ) THEN
       dummy = settling*param(is)*C_5_R/C_3_R
       DO ij = 1,isize_field
          trans(ij,1) = dummy*tmp(ij,1) ! Linear formulation (all sizes with the same velocity). Assume ds/dy*s^(2/3) in tmp 
       ENDDO
        
    ELSEIF (itransport .EQ. EQNS_TRANS_C_SET_FULL ) THEN
       DO ij = 1,isize_field ! Loop to calculate the vector flux
          tmp(ij,1) = settling*(param(is) - param(3)*s(ij,is))*s(ij,is_trans) ! constant settling velocity
       ENDDO
       CALL PARTIAL_Y(imode_fdm, nx,ny,nz, j1bc, dy, tmp(1,1), trans(1,1), i0,i0, wrk1d,wrk2d,wrk3d) !Gradient of the vector flux

    ELSEIF (itransport .EQ. EQNS_TRANS_LN_SET_FULL ) THEN
       dummy2 = C_5_R/C_3_R
       DO ij = 1,isize_field
          trans(ij,1) = settling*(param(is) - param(3)*s(ij,is))*(s(ij,is_trans)**dummy2) ! constant settling velocity
       ENDDO
       CALL PARTIAL_Y(imode_fdm, nx,ny,nz, j1bc, dy, tmp(1,1), trans(1,1), i0,i0, wrk1d,wrk2d,wrk3d) !Gradient of the vector flux

    ENDIF
  





END SUBROUTINE FLOW_TRANSPORT

