!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# inlf  In  1 smoothed T profile and reaction rate
!#           2 tent-like T profile and reaction rate
!#           3 smoothed T profile and radiative term
!#           4 tent-like T profile and radiative term
!#           5 smoothed T profile and rho(T)=1/T
!#           6 tent-like T profile and rho(T)=1/T
!#           7 smoothed T profile
!#           8 tent-like T profile 
!########################################################################
#include "types.h" 

FUNCTION ARRHENIUS(inlf, z_m, tact, zst, zsmooth, a, b, c0,c1,c2)

  IMPLICIT NONE

  TINTEGER inlf
  TREAL ARRHENIUS
  TREAL z_m, tact, zst, zsmooth, a, b, c0, c1, c2

! -----------------------------------------------------------------------
  TREAL tdec, cdec, tfl
  TINTEGER i2

! #######################################################################
  i2 = 2

  IF ( MOD(inlf,i2) .EQ. 0 ) THEN
     IF ( z_m .LT. zst ) THEN; tdec = C_1_R + a* z_m
     ELSE;                         tdec = C_1_R - b*(z_m - C_1_R); ENDIF
  ELSE
     cdec = EXP((z_m - C_2_R*zst)/zsmooth) + EXP(-z_m/zsmooth)
     tdec = C_1_R + c1*z_m - c2*LOG(cdec/c0)
  ENDIF

! normalizing by the values of the function at Tflame or wherever
  tfl = a*zst+C_1_R
  IF      ( inlf .EQ. 1 .OR. inlf .EQ. 2 ) THEN; ARRHENIUS = EXP(-tact*(C_1_R/tdec-C_1_R/tfl))
  ELSE IF ( inlf .EQ. 3 .OR. inlf .EQ. 4 ) THEN; ARRHENIUS = (tdec/tfl)**C_4_R
  ELSE IF ( inlf .EQ. 5 .OR. inlf .EQ. 6 ) THEN; ARRHENIUS = C_1_R/tdec
  ELSE;                                          ARRHENIUS = tdec/tfl
  ENDIF

  RETURN
END FUNCTION ARRHENIUS

! #######################################################################
! Derivatives
! #######################################################################
FUNCTION ARRHENIUS2(inlf, z_m, tact, zst, zsmooth, a, b, c0,c1,c2)

  IMPLICIT NONE

  TINTEGER inlf
  TREAL ARRHENIUS2
  TREAL z_m, tact, zst, zsmooth, a, b, c0, c1, c2

! -----------------------------------------------------------------------
  TREAL tdec0, tdec1, tdec2, tdum
  TREAL ddum, cdec0, cdec1
  TREAL fdec1, fdec2

! #######################################################################
  IF ( inlf .EQ. 1 ) THEN
     ddum = C_1_R/zsmooth
     cdec0 = EXP((z_m - 2*zst)*ddum) + EXP(-z_m*ddum)
     cdec1 = EXP((z_m - 2*zst)*ddum) - EXP(-z_m*ddum)
     tdum = cdec1/cdec0
     tdec0 = C_1_R + c1*z_m - c2*LOG(cdec0/c0)
     tdec1 = c1-c2*tdum*ddum
     tdec2 = c2*(tdum*tdum-C_1_R)*ddum*ddum
  ELSE
     IF ( z_m .LT. zst ) THEN
        tdec0 = C_1_R + a*z_m
        tdec1 = a
     ELSE
        tdec0 = C_1_R - b*(z_m - C_1_R)
        tdec1 = -b
     ENDIF
     tdec2 = C_0_R
  ENDIF

! normalize by w(Zst) from Burke-Schumman soln.
  tdum = C_1_R/tdec0
  fdec1 = tact*tdum*tdum*EXP(-tact*(tdum-C_1_R/(a*zst+C_1_R)))
  fdec2 = fdec1*(tact*tdum-C_2_R)*tdum

  ARRHENIUS2 = fdec2*tdec1*tdec1 + fdec1*tdec2

  RETURN
END FUNCTION ARRHENIUS2

! #######################################################################
! #######################################################################
FUNCTION ARRHENIUS3(inlf, z_m, tact, zst, zsmooth, a, b, c0,c1,c2)

  IMPLICIT NONE

  TINTEGER inlf
  TREAL ARRHENIUS3
  TREAL z_m, tact, zst, zsmooth, a, b, c0, c1, c2

! -----------------------------------------------------------------------
  TREAL tdec0, tdec1, tdec2, tdec3, tdum
  TREAL ddum, cdec0, cdec1
  TREAL fdec1, fdec2, fdec3

! #######################################################################
  IF ( inlf .EQ. 1 ) THEN
     ddum = C_1_R/zsmooth
     cdec0 = EXP((z_m - 2*zst)*ddum) + EXP(-z_m*ddum)
     cdec1 = EXP((z_m - 2*zst)*ddum) - EXP(-z_m*ddum)
     tdum = cdec1/cdec0
     tdec0 = C_1_R + c1*z_m - c2*LOG(cdec0/c0)
     tdec1 = c1-c2*tdum*ddum
     tdec2 = c2*(tdum*tdum-C_1_R)*ddum*ddum
     tdec3 =-tdec2*tdum*C_2_R*ddum
  ELSE
     IF ( z_m .LT. zst ) THEN
        tdec0 = C_1_R + a*z_m
        tdec1 = a
     ELSE
        tdec0 = C_1_R - b*(z_m - C_1_R)
        tdec1 = -b
     ENDIF
     tdec2 = C_0_R
     tdec3 = C_0_R
  ENDIF

! normalize by w(Zst) from Burke-Schumman soln.
  tdum = C_1_R/tdec0
  fdec1 = tact*tdum*tdum*EXP(-tact*(tdum-C_1_R/(a*zst+C_1_R)))
  fdec2 = fdec1*(tact*tdum-C_2_R)*tdum
  fdec3 = fdec2*(tact*tdum-C_3_R)*tdum-tact*tdum**C_3_R*fdec1

  ARRHENIUS3 = fdec3*tdec1**C_3_R+C_3_R*fdec2*tdec1*tdec2+fdec1*tdec3

  RETURN
END FUNCTION ARRHENIUS3
