SUBROUTINE LES_AVG_CHEM_BS(x, y, z, dx, dy, dz, rho, &
     u, z1, gama, tmp, mean2d, chi, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, fdf)

#include "types.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL u(imax,jmax,kmax)
  TREAL rho(imax,jmax,kmax)
  TREAL z1(imax,jmax,kmax)
  TREAL gama(imax,jmax,kmax)

  TREAL chi(imax,jmax,kmax)
  TREAL mean2d(jmax,*)
  TREAL tmp(imax,jmax,kmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL cf0x(*), cf0y(*), cf0z(*)
  TREAL fdf(*)

#define rR(j)          mean2d(j,1)
#define C0(j)          mean2d(j,2)
#define AVG_Z2SG(j)    mean2d(j,3)

#define AVG_CHI_SG(j)  mean2d(j,4)
#define VAR_CHI_SG(j)  mean2d(j,5)
#define SKE_CHI_SG(j)  mean2d(j,6)
#define FLA_CHI_SG(j)  mean2d(j,7)

#define AVG_CHI(j)     mean2d(j,8)
#define VAR_CHI(j)     mean2d(j,9)
#define SKE_CHI(j)     mean2d(j,10)
#define FLA_CHI(j)     mean2d(j,11)

#define AVG_CHI_Z(j)   mean2d(j,12)
#define AVG_FDF(j)     mean2d(j,13)

#define AVG_OMEGAP(j)  mean2d(j,14)
#define VAR_OMEGAP(j)  mean2d(j,15)
#define SKE_OMEGAP(j)  mean2d(j,16)
#define FLA_OMEGAP(j)  mean2d(j,17)

#define BACK_PERC(j)   mean2d(j,18)
#define BACK_CHI(j)    mean2d(j,19)

#define L_AVGMAX 19

  TINTEGER i, j, ij, k, icp, p2, p3, iback_perc
  TREAL AVG_IK, SIMPSON_NU
  TREAL delta_w, y_center, CPN2
  TREAL omega_prime, chi_prime
  TREAL zmin, zmax, dummy, back_chi, chi_coef
  CHARACTER*32 name 

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_STS_CHEM_BS')
#endif

  mean2d(:,1:L_AVGMAX) = C_0_R

  CPN2 = C_0_R
  DO icp = NCP_CHEMKIN, 1, -1
     CPN2 = CPN2 + THERMO_AI(icp,2,NSP)
  ENDDO

  LES_CHW_Qp = C_05_R*CHW_CE*CPN2/XIST/(1-XIST)

! #####################
! # Global quantities #
! #####################
! -----------------------------------------------------------
!                                             Shear layer
! -----------------------------------------------------------
  IF (imode_flow .EQ. DNS_FLOW_SHEAR) THEN
#define fU(j)        wrk1d(j,2)
#define fU_y(j)      wrk1d(j,3)
#define WRK1D_LOC(j) wrk1d(j,4)
     DO j = 1,jmax
        rR(j) = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
     ENDDO

     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = rho(i,j,k)*u(i,j,k)
           ENDDO
        ENDDO
        fU(j) = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, &
             area)/rR(j)
     ENDDO

     CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, fU(1), &
          fU_y(1), i0, i0, WRK1D_LOC(1), wrk2d, wrk3d)
     delta_w = delta_u/MAXVAL(ABS(fU_y(1:jmax)))
#undef fU
#undef fU_y
#undef WRK1D_LOC
! -----------------------------------------------------------
!                                                     Jet
! -----------------------------------------------------------
  ELSE IF (imode_flow .EQ. DNS_FLOW_JET) THEN
  ENDIF

! ##############
! # Statistics #
! ##############
! SGS dissipation
  DO j = 1,jmax
     AVG_CHI_SG(j) = C_2_R*AVG_IK(imax, jmax, kmax, j, &
          chi, dx, dz, area)/rR(j)
     DO k = 1,kmax
        DO i = 1,imax
           chi_prime = C_2_R*chi(i,j,k)/rho(i,j,k) - &
                AVG_CHI_SG(j)
           wrk3d(i,1,k) = rho(i,j,k)*chi_prime*chi_prime
           wrk3d(i,2,k) = wrk3d(i,1,k)*chi_prime
           wrk3d(i,3,k) = wrk3d(i,2,k)*chi_prime
        ENDDO
     ENDDO
     VAR_CHI_SG(j) = AVG_IK(imax, jmax, kmax, i1, &
          wrk3d, dx, dz, area)/rR(j)
     SKE_CHI_SG(j) = AVG_IK(imax, jmax, kmax, i2, &
          wrk3d, dx, dz, area)/rR(j)
     FLA_CHI_SG(j) = AVG_IK(imax, jmax, kmax, i3, &
          wrk3d, dx, dz, area)/rR(j)
  ENDDO

#define OMEGAP(i,j,k)  tmp(i,j,k,1)
#define Z2SG(i,j,k)    tmp(i,j,k,2)
#define FDF(i,j,k)     tmp(i,j,k,4)
#define CHI_Z(i,j,k)   tmp(i,j,k,5)

! model coefficient returns in wrk1d(1,1)
! tmp(1,1,1,3) returns FDF or broader term
  p2 = i1 + les_fdf_bs_maxmean
  p3 = p2 + les_fdf_bs_maxmean*les_fdf_bs_maxvar
  CALL SGS_HEAT_BS(x, y, z, dx, dy, dz, rho, &
       u, z1, gama, OMEGAP(1,1,1), &
       Z2SG(1,1,1), chi, wrk1d, wrk2d, wrk3d,&
       cf0x, cf0y, cf0z, fdf(1), fdf(p2), fdf(p3))

! computing the percentage of backscatter
!  DO j = 1,jmax
!     iback_perc = 0
!     back_chi = C_0_R
!     DO k = 1,kmax
!        DO i = 1,imax
!           IF ( chi(i,j,k) .LT. C_0_R ) THEN
!              iback_perc = iback_perc + 1
!              back_chi = back_chi + chi(i,j,k)
!           ENDIF
!        ENDDO
!     ENDDO
!     BACK_PERC(j) = M_REAL(iback_perc)/M_REAL(imax*kmax_total)
!     BACK_CHI(j) =-C_2_R*back_chi/rR(j)/M_REAL(imax*kmax_total)
!  ENDDO

! final operations in the scalar dissipation
!c     do not forget the factor 2 in chi, and force it positive !!!
!  DO k = 1,kmax
!     DO ij = 1,imax*jmax
!        CHI_Z(ij,1,k) = MAX(r0,chi(ij,1,k))
!     ENDDO
!  ENDDO
!  DO j = 1,jmax
!     wrk1d(j,2) = AVG_IK(imax, jmax, kmax, j, chi, 
! $        dx, dz, area)
!     wrk1d(j,3) = AVG_IK(imax, jmax, kmax, j, CHI_Z(1,1,1), 
! $        dx, dz, area)
!  ENDDO
!  chi_coef = C_2_R*SIMPSON_NU(jmax, wrk1d(1,2), y)/
! $     SIMPSON_NU(jmax, wrk1d(1,3), y)
!  DO k = 1,kmax
!     DO ij = 1,imax*jmax
!        chi(ij,1,k) = chi_coef*CHI_Z(ij,1,k)
!     ENDDO
!  ENDDO

! Computing the beta FDF
  IF ( iles_type_disZchem .EQ. LES_DISZCHEM_MEAN ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           FDF(ij,1,k) = tmp(ij,1,k,3)
        ENDDO
     ENDDO
  ELSE
     CALL FDF_BETA_STO(imax, jmax, kmax, XIST, z1, Z2SG(1,1,1), &
          FDF(1,1,1))
  ENDIF

! Computing \chi_s
  IF ( iles_type_disZchem .EQ. LES_DISZCHEM_MEAN ) THEN
  ELSE
     zmin = fdf(2)
     zmax = fdf(les_fdf_bs_maxmean-1)
     DO k = 1,kmax
        DO ij = 1,imax*jmax
! preventing singularities
           IF ( z1(ij,1,k) .LT. zmin .OR.&
                z1(ij,1,k) .GT. zmax ) THEN
              CHI_Z(ij,1,k) = C_0_R
           ELSE
              dummy = z1(ij,1,k)*(C_1_R-z1(ij,1,k))
              IF ( Z2SG(ij,1,k) .LT. zmin*dummy .OR.&
                   Z2SG(ij,1,k) .GT. zmax*dummy ) THEN
                 CHI_Z(ij,1,k) = C_0_R
              ELSE
                 CHI_Z(ij,1,k) = chi(ij,1,k)*tmp(ij,1,k,3)&
                      /FDF(ij,1,k)
              ENDIF
           ENDIF
        ENDDO
     ENDDO
  ENDIF

  DO j = 1,jmax

     C0(j) = wrk1d(j,1)

     AVG_CHI(j) = AVG_IK(imax, jmax, kmax, j, &
          chi, dx, dz, area)/rR(j)

! statistics of S=w/\rho
     AVG_OMEGAP(j) = AVG_IK(imax, jmax, kmax, j, &
          OMEGAP(1,1,1), dx, dz, area)/rR(j)

     DO k = 1,kmax
        DO i = 1,imax
           omega_prime = OMEGAP(i,j,k)/rho(i,j,k) - AVG_OMEGAP(j)
           wrk3d(i,1,k) = rho(i,j,k)*omega_prime*omega_prime
           wrk3d(i,2,k) = wrk3d(i,1,k)*omega_prime
           wrk3d(i,3,k) = wrk3d(i,2,k)*omega_prime

           wrk3d(i,4,k) = Z2SG(i,j,k)*rho(i,j,k)
           wrk3d(i,5,k) = FDF(i,j,k)*rho(i,j,k)

           chi_prime = chi(i,j,k)/rho(i,j,k) - AVG_CHI(j)
           wrk3d(i,6,k) = rho(i,j,k)*chi_prime*chi_prime
           wrk3d(i,7,k) = wrk3d(i,6,k)*chi_prime
           wrk3d(i,8,k) = wrk3d(i,7,k)*chi_prime
        ENDDO
     ENDDO
     VAR_OMEGAP(j) = AVG_IK(imax, jmax, kmax, i1, &
          wrk3d, dx, dz, area)/rR(j)
     SKE_OMEGAP(j) = AVG_IK(imax, jmax, kmax, i2, &
          wrk3d, dx, dz, area)/rR(j)
     FLA_OMEGAP(j) = AVG_IK(imax, jmax, kmax, i3, &
          wrk3d, dx, dz, area)/rR(j)

     AVG_Z2SG(j) = AVG_IK(imax, jmax, kmax, i4, &
          wrk3d, dx, dz, area)/rR(j)
     AVG_FDF(j) = AVG_IK(imax, jmax, kmax, i5, &
          wrk3d, dx, dz, area)/rR(j)
     AVG_CHI_Z(j) = AVG_IK(imax, jmax, kmax, j, &
          CHI_Z(1,1,1), dx, dz, area)/rR(j)

     VAR_CHI(j) = AVG_IK(imax, jmax, kmax, i6, &
          wrk3d, dx, dz, area)/rR(j)
     SKE_CHI(j) = AVG_IK(imax, jmax, kmax, i7, &
          wrk3d, dx, dz, area)/rR(j)
     FLA_CHI(j) = AVG_IK(imax, jmax, kmax, i8, &
          wrk3d, dx, dz, area)/rR(j)
  ENDDO

#undef OMEGAP
#undef Z2SG
#undef FDF
#undef CHI_Z

! ####################################
! # Saving the data in TkStat format #
! ####################################

  name = 'leschm '
  CALL CONCATI(name,itime)

#ifdef PARALLEL
  CALL MPI_COMM_RANK(MPI_COMM_WORLD,ims_pro,ims_err)
  IF ( ims_pro .EQ. 0 ) THEN
#endif

#ifdef USE_RECLEN
     OPEN(UNIT=i23,RECL=339,FILE=name,STATUS='unknown')
#else
     OPEN(UNIT=i23,FILE=name,STATUS='unknown')
#endif     

     WRITE(i23,1010) '#################'
     WRITE(i23,1010) '#  Version 1.0  #'
     WRITE(i23,1010) '#################'
     WRITE(i23,'(A8,E14.7E3)') 'RTIME = ', rtime

     WRITE(i23,1010) 'I J Y SW rR cO fZ2_SG '&
          //'X_sg (X_sg)f2 (X_sg)f3 (X_sg)f4 '&
          //'fchi fchif2 fchif3 fchif4 fchi_Z fFDF '&
          //'fSP fSPf2 fSPf3 fSPf4 Back_percentage Back_chi'
1010 FORMAT(A)

     y_center = y(1) + C_05_R*scaley

     DO j = 1,jmax
        IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
           WRITE(i23,1300) i, j, y(j), (y(j)-y_center)/delta_w, (mean2d(j,k),k=1,L_AVGMAX)
        ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
        ENDIF
     ENDDO

     CLOSE(i23)

#ifdef PARALLEL
  ENDIF
#endif

1300 FORMAT(I3,1X,I3,2(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

  RETURN
END SUBROUTINE LES_AVG_CHEM_BS
