!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# txc size is 10 if dynamic ssmflg is 0
!# txc size is 13 if dynamic ssmflg is 1
!# txc size is 12 if ssmflg is 1
!#
!########################################################################
SUBROUTINE LES_AVG_FLOW_TEMPORAL_LAYER&
     (x,y,z,dx,dy,dz, rho,u,v,w,e, u_sg,v_sg,w_sg,aux_sg, txc, mean2d, wrk1d,wrk2d,wrk3d, &
     cf0x,cf0y,cf0z,cf1x,cf1y,cf1z,cf2x,cf2y,cf2z, carm_uf,carm_pf,carm_uh,carm_ph)

#include "types.h"
#include "dns_error.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: u, v, w, rho, e, u_sg, v_sg, w_sg, aux_sg, wrk3d
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL mean2d(jmax,*)

  TREAL wrk1d(jmax,*)
  TREAL wrk2d(*)

  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z
  TREAL, DIMENSION(*) :: carm_uf, carm_uh, carm_pf, carm_ph

! -----------------------------------------------------------------------
#define smg_cs(j)      mean2d(j,1)
#define smg_mu(j)      mean2d(j,2)

#define smg_tauxx(j)   mean2d(j,3)
#define smg_tauyy(j)   mean2d(j,4)
#define smg_tauzz(j)   mean2d(j,5)
#define smg_tauxy(j)   mean2d(j,6)
#define smg_tauxz(j)   mean2d(j,7) 
#define smg_tauyz(j)   mean2d(j,8)
#define smg_eps(j)     mean2d(j,9)
#define smg_trans(j)   mean2d(j,10)
#define smg_force_u(j) mean2d(j,11)
#define smg_force_v(j) mean2d(j,12)

#define ssm_cs(j)      mean2d(j,13)

#define ssm_tauxx(j)   mean2d(j,14)
#define ssm_tauyy(j)   mean2d(j,15)
#define ssm_tauzz(j)   mean2d(j,16)
#define ssm_tauxy(j)   mean2d(j,17)
#define ssm_tauxz(j)   mean2d(j,18) 
#define ssm_tauyz(j)   mean2d(j,19)
#define ssm_eps(j)     mean2d(j,20)
#define ssm_trans(j)   mean2d(j,21)
#define ssm_force_u(j) mean2d(j,22)
#define ssm_force_v(j) mean2d(j,23)

#define smg_csp(j)     mean2d(j,24)
#define smg_rhoa(j)    mean2d(j,25)

#define smg_qpx(j)     mean2d(j,26)
#define smg_qpy(j)     mean2d(j,27)
#define smg_qpz(j)     mean2d(j,28)

#define ssm_csp(j)     mean2d(j,29)

#define ssm_qpx(j)     mean2d(j,30)
#define ssm_qpy(j)     mean2d(j,31)
#define ssm_qpz(j)     mean2d(j,32)

#define tot_eps(j)     mean2d(j,33)
#define tot_trans(j)   mean2d(j,34)
#define tot_mean(j)    mean2d(j,35)
#define tot_force_u(j) mean2d(j,36)
#define tot_force_v(j) mean2d(j,37)

#define tot_Rxx(j)     mean2d(j,38)
#define tot_Ryy(j)     mean2d(j,39)
#define tot_Rzz(j)     mean2d(j,40)
#define tot_Rxy(j)     mean2d(j,41)
#define tot_Rxz(j)     mean2d(j,42)
#define tot_Ryz(j)     mean2d(j,43)

#define L_AVGMAX 43

  TINTEGER i, j, k, ij
  TREAL AVG_IK, UPPER_THRESHOLD, LOWER_THRESHOLD
  TREAL delta_w, SimVar1, SimVar2, dummy
  TREAL delta_ht01, delta_hb01, delta_h01
  TREAL r23, r13, coefdelta
  TREAL favU, favV, reyU, reyV, dreyUdy, dreyVdy, den
  TREAL int_rxx, int_ryy, int_rzz, int_diss
  TINTEGER n0, n1, l1, l2, ll1, ll2, ssmflg
  TINTEGER ip_den, ip_tmp
  CHARACTER*32 num
  CHARACTER*250 line1
  CHARACTER*500 line2

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_AVG_FLOW_TEMPORAL_LAYER' )
#endif

  r13 = C_1_R/C_3_R
  r23 = C_2_R/C_3_R

  n0 = isgs_f0size
  n1 = isgs_f1size
  coefdelta = (M_REAL(n1)/M_REAL(n0))**C_2_R + C_1_R

  IF ( iles_type_tran .EQ. LES_TRAN_NONE ) THEN 
     ssmflg = 0
  ELSE
     ssmflg = 1
  ENDIF

  IF ( MAX_AVG_TEMPORAL .LT. L_AVGMAX ) THEN
     CALL IO_WRITE_ASCII(efile,'LES_AVG_FLOW_TEMPORAL_LAYER. Not enough space.')
     CALL DNS_STOP(LES_ERROR_AVGTMP)
  ENDIF
  mean2d(:,1:L_AVGMAX) = C_0_R

! ###################################################################
! Header
! ###################################################################
  num='lesavg '
  CALL CONCATI(num, itime)

#ifdef PARALLEL
  CALL MPI_COMM_RANK(MPI_COMM_WORLD,ims_pro,ims_err)

  IF ( ims_pro .EQ. 0 ) THEN
#endif

#ifdef USE_RECLEN
     OPEN(UNIT=23, RECL=694, FILE=num, STATUS='unknown')
#else
     OPEN(UNIT=23, FILE=num, STATUS='unknown')
#endif

     WRITE(i23,1010) '#################'
     WRITE(i23,1010) '#  Version 1.1  #'
     WRITE(i23,1010) '#################'
     WRITE(i23,'(A8,E14.7E3)') 'RTIME = ', rtime


     line2='I J Y S1'
     CALL CLIP(line2,ll1,ll2)

     line1 ='Cs rho*nu_sg smg_tauxx smg_tauyy smg_tauzz '&
          //'smg_tauxy smg_tauxz smg_tauyz smg_eps smg_trans smg_force_u smg_force_v'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = SMG_U '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='Cssm ssm_tauxx ssm_tauyy ssm_tauzz '&
          //'ssm_tauxy ssm_tauxz ssm_tauyz ssm_eps ssm_trans ssm_force_u ssm_force_v'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = SSM_U '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='CsP rho*alpha_sg smg_qpx smg_qpy smg_qpz '
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = SMG_P '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='CssmP ssm_qpx ssm_qpy ssm_qpz'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = SSM_P '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='tot_eps tot_trans tot_mean tot_force_u tot_force_v'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = Balances '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     line1 ='Rxx Ryy Rzz Rxy Rxz Ryz'
     CALL CLIP(line1,l1,l2)
     WRITE(i23,1010) 'GROUP = Reynolds_stresses '//line1(l1:l2)
     line2=line2(ll1:ll2)//' '//line1(l1:l2)
     CALL CLIP(line2,ll1,ll2)

     WRITE(i23,1010) line2(ll1:ll2)

1010 FORMAT(A)

#ifdef PARALLEL
  ENDIF
#endif

! ########################################################################
! Scale similarity  part
! ########################################################################
#define BARVS(i,j,k)    txc(i,j,k,1)
#define MU_M(i,j,k)     txc(i,j,k,2)
#define MV_M(i,j,k)     txc(i,j,k,3)
#define MW_M(i,j,k)     txc(i,j,k,4)
#define BARMU_M(i,j,k)  txc(i,j,k,5)
#define BARMV_M(i,j,k)  txc(i,j,k,6)
#define BARMW_M(i,j,k)  txc(i,j,k,7)
#define TAU_M(i,j,k)    txc(i,j,k,8)
#define TMP1_LOC(i,j,k) txc(i,j,k,9)
#define TMP2_LOC(i,j,k) txc(i,j,k,10)
  IF ( ssmflg .EQ. 1 ) THEN
! ------------------------------------------------------------
!                                Defining the model fields
! ------------------------------------------------------------
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, rho, BARVS(1,1,1), wrk3d)
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           BARVS(ij,1,k) = C_1_R/BARVS(ij,1,k)
        ENDDO
     ENDDO
! ------------------------------------------------------------
! Remove mean if needed
! ------------------------------------------------------------
! txc_size=1
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, u, u_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, v, v_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)
!     CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
!          cf0x, cf0y, cf0z, rho, w, w_sg, TMP1_LOC(1,1,1), wrk2d, wrk3d)

     IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              MU_M(ij,1,k) = rho(ij,1,k)*u(ij,1,k)
              MV_M(ij,1,k) = rho(ij,1,k)*v(ij,1,k)
              MW_M(ij,1,k) = rho(ij,1,k)*w(ij,1,k)
!              MU_M(ij,1,k) = rho(ij,1,k)*u_sg(ij,1,k)
!              MV_M(ij,1,k) = rho(ij,1,k)*v_sg(ij,1,k)
!              MW_M(ij,1,k) = rho(ij,1,k)*w_sg(ij,1,k)
           ENDDO
        ENDDO
     ELSE 
! txc size is 4 and goes into BAR_MU
!            CALL SGS_ARM_U(n0, x, y, z, dx, dy, dz, &
!                 rho, u_sg, v_sg, w_sg, BARVS(1,1,1), carm_uf, &
!                 MU_M(1,1,1), MV_M(1,1,1), MW_M(1,1,1),&
!                 BARMU_M(1,1,1), &
!                 wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z) 
     ENDIF

     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, MU_M(1,1,1), BARMU_M(1,1,1), wrk3d)
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, MV_M(1,1,1), BARMV_M(1,1,1), wrk3d)
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, MW_M(1,1,1), BARMW_M(1,1,1), wrk3d)

! ------------------------------------------------------------
!                       Velocity (isotropic part retained)
! ------------------------------------------------------------
! tauxx = rho*(U*U)_sg
     CALL SGS_RECTN_TAUIJ(n0, rho, MU_M(1,1,1), MU_M(1,1,1), &
          BARVS(1,1,1), BARMU_M(1,1,1), BARMU_M(1,1,1), &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, u, &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TAU_M(i,j,k)*TMP1_LOC(i,j,k)
           ENDDO
        ENDDO
        ssm_tauxx(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        ssm_eps(j) = ssm_eps(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, TAU_M(1,1,1), &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        ssm_force_u(j) = ssm_force_u(j) -&
             AVG_IK(imax, jmax, kmax, j, TMP1_LOC(1,1,1), &
             dx, dz, area)
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TMP1_LOC(i,j,k)*u(i,j,k)
           ENDDO
        ENDDO
        ssm_trans(j) = ssm_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

! tauyy = rho*(V*V)_sg
     CALL SGS_RECTN_TAUIJ(n0, rho, MV_M(1,1,1), MV_M(1,1,1), &
          BARVS(1,1,1), BARMV_M(1,1,1), BARMV_M(1,1,1), &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, v, &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TAU_M(i,j,k)*TMP1_LOC(i,j,k)
           ENDDO
        ENDDO
        ssm_tauyy(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        ssm_eps(j) = ssm_eps(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, TAU_M(1,1,1), &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        ssm_force_v(j) = ssm_force_v(j) -&
             AVG_IK(imax, jmax, kmax, j, TMP1_LOC(1,1,1), &
             dx, dz, area)
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TMP1_LOC(i,j,k)*v(i,j,k)
           ENDDO
        ENDDO
        ssm_trans(j) = ssm_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

     IF ( kmax_total .GT. 1 ) THEN
! tauzz = rho*(W*W)_sg
        CALL SGS_RECTN_TAUIJ(n0, rho, MW_M(1,1,1), MW_M(1,1,1), &
             BARVS(1,1,1), BARMW_M(1,1,1), BARMW_M(1,1,1), &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, w, &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = TAU_M(i,j,k)*TMP1_LOC(i,j,k)
              ENDDO
           ENDDO
           ssm_tauzz(j) = AVG_IK(imax, jmax, kmax, j, &
                TAU_M(1,1,1), dx, dz, area)
           ssm_eps(j) = ssm_eps(j) + &
                AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
        ENDDO
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
             TAU_M(1,1,1), &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = TMP1_LOC(i,j,k)*w(i,j,k)
              ENDDO
           ENDDO
           ssm_trans(j) = ssm_trans(j) + &
                AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
        ENDDO
     ENDIF

! tauxy = rho(U*V)_sg
     CALL SGS_RECTN_TAUIJ(n0, rho, MU_M(1,1,1), MV_M(1,1,1), &
          BARVS(1,1,1), BARMU_M(1,1,1), BARMV_M(1,1,1), &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, v, &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, u, &
          TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TAU_M(i,j,k)*&
                   (TMP1_LOC(i,j,k)+TMP2_LOC(i,j,k))
           ENDDO
        ENDDO
        ssm_tauxy(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        ssm_eps(j) = ssm_eps(j) + &
             AVG_IK(imax, jmax, kmax, i1,&
             wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, TAU_M(1,1,1), &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, TAU_M(1,1,1), &
          TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        ssm_force_u(j) = ssm_force_u(j) -&
             AVG_IK(imax, jmax, kmax, j, TMP2_LOC(1,1,1), &
             dx, dz, area)
        ssm_force_v(j) = ssm_force_v(j) -&
             AVG_IK(imax, jmax, kmax, j, TMP1_LOC(1,1,1), &
             dx, dz, area)
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TMP1_LOC(i,j,k)*v(i,j,k)+&
                   TMP2_LOC(i,j,k)*u(i,j,k)
           ENDDO
        ENDDO
        ssm_trans(j) = ssm_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

     IF ( kmax_total .GT. 1 ) THEN
! tauxz = rho(U*W)_sg
        CALL SGS_RECTN_TAUIJ(n0, rho, MU_M(1,1,1), MW_M(1,1,1), &
             BARVS(1,1,1), BARMU_M(1,1,1), BARMW_M(1,1,1), &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, w, &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, u, &
             TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = TAU_M(i,j,k)*&
                      (TMP1_LOC(i,j,k)+TMP2_LOC(i,j,k))
              ENDDO
           ENDDO
           ssm_tauxz(j) = AVG_IK(imax, jmax, kmax, j, &
                TAU_M(1,1,1), dx, dz, area)
           ssm_eps(j) = ssm_eps(j) + &
                AVG_IK(imax, jmax, kmax, i1,&
                wrk3d, dx, dz, area)
        ENDDO
        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
             TAU_M(1,1,1), TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = TMP1_LOC(i,j,k)*w(i,j,k)+&
                      TMP2_LOC(i,j,k)*u(i,j,k)
              ENDDO
           ENDDO
           ssm_trans(j) = ssm_trans(j) + &
                AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
        ENDDO

! tauyz = rho(V*W)_sg
        CALL SGS_RECTN_TAUIJ(n0, rho, MV_M(1,1,1), MW_M(1,1,1), &
             BARVS(1,1,1), BARMV_M(1,1,1), BARMW_M(1,1,1), &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, v, &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, w, &
             TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = TAU_M(i,j,k)*&
                      (TMP1_LOC(i,j,k)+TMP2_LOC(i,j,k))
              ENDDO
           ENDDO
           ssm_tauyz(j) = AVG_IK(imax, jmax, kmax, j, &
                TAU_M(1,1,1), dx, dz, area)
           ssm_eps(j) = ssm_eps(j) + &
                AVG_IK(imax, jmax, kmax, i1,&
                wrk3d, dx, dz, area)
        ENDDO
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
             TAU_M(1,1,1), TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = TMP1_LOC(i,j,k)*v(i,j,k)+&
                      TMP2_LOC(i,j,k)*w(i,j,k)
              ENDDO
           ENDDO
           ssm_trans(j) = ssm_trans(j) + &
                AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
        ENDDO
     ENDIF
#undef TAU_M
#undef TMP1_LOC
#undef TMP2_LOC

#define P_M(i,j,k)      txc(i,j,k,8)
#define BARP_M(i,j,k)   txc(i,j,k,9)
#define Q_M(i,j,k)      txc(i,j,k,10)
#define TMP1_LOC(i,j,k) txc(i,j,k,11)

! ------------------------------------------------------------
! Energy
! ------------------------------------------------------------
! model fields
     IF ( iles_type_tran .EQ. LES_TRAN_SSM ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              P_M(ij,1,k) = rho(ij,1,k)*e(ij,1,k)
           ENDDO
        ENDDO
     ELSE 
! txc size is 4 and goes into BARP_M => txc is 12
!         CALL SGS_ARM_P(n0, x, y, z, dx, dy, dz, &
!              rho, p, BARVS(1,1,1), carm_pf, P_M(1,1,1), &
!              BARP_M(1,1,1), &
!              wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z)
     ENDIF
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n0, cf0x, cf0y, cf0z, P_M(1,1,1), BARP_M(1,1,1), wrk3d)

! X gradient
     CALL SGS_RECTN_TAUIJ(n0, rho, P_M(1,1,1), MU_M(1,1,1), &
          BARVS(1,1,1), BARP_M(1,1,1), BARMU_M(1,1,1), Q_M(1,1,1),&
          TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     DO j = 1,jmax
        ssm_qpx(j) = AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
     ENDDO

! Y gradient
     CALL SGS_RECTN_TAUIJ(n0, rho, P_M(1,1,1), MV_M(1,1,1), &
          BARVS(1,1,1), BARP_M(1,1,1), BARMV_M(1,1,1), Q_M(1,1,1),&
          TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     DO j = 1,jmax
        ssm_qpy(j) = AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
     ENDDO

     IF ( kmax_total .GT. 1 ) THEN
! Z gradient
        CALL SGS_RECTN_TAUIJ(n0, rho, P_M(1,1,1), MW_M(1,1,1), &
             BARVS(1,1,1), BARP_M(1,1,1), BARMW_M(1,1,1), Q_M(1,1,1),&
             TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
        DO j = 1,jmax
           ssm_qpz(j) = AVG_IK(imax, jmax, kmax, j, Q_M(1,1,1), dx, dz, area)
        ENDDO
     ENDIF

  ENDIF
#undef BARVS
#undef BARMU_M
#undef BARMV_M
#undef BARMW_M
#undef MU_M
#undef MV_M
#undef MW_M
#undef Q_M
#undef TMP1_LOC
#undef P_M
#undef BARP_M

! output data
!  DO j = 1,jmax
!     ssm_cs(j)  = carm_uf(j)
!     ssm_csp(j) = carm_pf(j)
!  ENDDO

! ########################################################################
! Smagorinsky part
! ########################################################################
! defining the array distribution
  ip_den = 3
  IF ( ssmflg .EQ. 1 ) THEN
     ip_tmp = ip_den + 3
  ELSE
     ip_tmp = ip_den + 1
  ENDIF

#define STRAIN0(i,j,k)        txc(i,j,k,1)
#define STRAIN1(i,j,k)        txc(i,j,k,2)
#define RHOINV1(i,j,k)        txc(i,j,k,ip_den)
#define DIFFSG(i,j,k)         txc(i,j,k,ip_tmp)

  IF ( iles_type_regu .NE. LES_REGU_NONE ) THEN
!-------------------------------------------------------------
!                                                 Velocity
!-------------------------------------------------------------
     IF ( iles_type_regu .EQ. LES_REGU_SMGDYN .OR. &
          iles_type_regu .EQ. LES_REGU_SMGDYNRMS ) THEN
! Specific volumes (txc_size is 1)
        CALL SGS_DYN_BARVS(n0, n1, rho, RHOINV1(1,1,1), txc(1,1,1,ip_tmp), wrk3d, &
             cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z)
! remove mean if needed (txc_size is 1)
        CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
             cf0x, cf0y, cf0z, rho, u, u_sg, txc(1,1,1,ip_tmp), wrk2d, wrk3d)
        CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
             cf0x, cf0y, cf0z, rho, v, v_sg, txc(1,1,1,ip_tmp), wrk2d, wrk3d)
        CALL SGS_FIELDS(iles_type_regu, n0, dx, dy, dz, &
             cf0x, cf0y, cf0z, rho, w, w_sg, txc(1,1,1,ip_tmp), wrk2d, wrk3d)
! ARM coefficients (txc_size is 6)    
!        IF ( iles_type_tran .EQ. LES_TRAN_ARMRANS .OR.&
!             iles_type_tran .EQ. LES_TRAN_ARMSPTL .OR.&
!             iles_type_tran .EQ. LES_TRAN_ARMINVS ) THEN
!            CALL SGS_DYN_ARM_U(n0, n1, x, y, z, dx, dy, dz, &
!                 rho, u_sg, v_sg, w_sg, RHOINV1(1,1,1), &
!                 txc(1,1,1,ip_tmp), wrk1d, wrk2d, wrk3d, &
!                 cf0x, cf0y, cf0z,&
!                 cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, &
!                 carm_uf, carm_uh)
!        ENDIF
! Norms of strain rate tensor (txc_size is 6)
        CALL SGS_DYN_SIJ(n0, n1, x, y, z, dx, dy, dz,&
             rho, u_sg, v_sg, w_sg, STRAIN0(1,1,1), STRAIN1(1,1,1), &
             RHOINV1(1,1,1), txc(1,1,1,ip_tmp), wrk1d, wrk2d, wrk3d, cf1x, cf1y, cf1z)
! Dynamic coefficient
        CALL SGS_DYN_U(ssmflg, coefdelta, n0, n1, x,y,z,dx,dy,dz, rho,u,v,w, u_sg,v_sg,w_sg, &
             STRAIN0(1,1,1), STRAIN1(1,1,1), RHOINV1(1,1,1), txc(1,1,1,ip_tmp), &
             smg_cs(1),wrk1d, wrk2d, wrk3d, &
             cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh)
! 2*rho*nu_sg in txc(ip_tmp), 2*Cs in wrk1d(1)
        DO k = 1,kmax
           DO j = 1,jmax
              DO i = 1,imax
                 txc(i,j,k,ip_tmp) = smg_cs(j)*STRAIN0(i,j,k)
              ENDDO
           ENDDO
        ENDDO
        DO j = 1,jmax
           smg_cs(j) = C_05_R*smg_cs(j)
           smg_mu(j) = C_05_R*AVG_IK(imax, jmax, kmax, j, txc(1,1,1,ip_tmp), dx, dz, area)
        ENDDO
     ELSE IF ( iles_type_regu .EQ. LES_REGU_SMGSTA ) THEN
! static part to be developed
     ENDIF

#define SIJSG(i,j,k)     txc(i,j,k,ip_tmp+1)
#define SIJR(i,j,k)      txc(i,j,k,ip_tmp+2)
#define TAU_M(i,j,k)     txc(i,j,k,ip_tmp+3)
#define TMP_LOC(i,j,k)   txc(i,j,k,ip_tmp+4)

!-------------------------------------------------------------
!                                     tauxy = - mu_s * Sxy      
!-------------------------------------------------------------
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, u_sg, SIJSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, v_sg, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1,kmax
        DO ij=1,imax*jmax
           SIJSG(ij,1,k) = &
                C_05_R*(SIJSG(ij,1,k)+TMP_LOC(ij,1,k))
        ENDDO
     ENDDO
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, u, SIJR(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, v, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1,kmax
        DO ij=1,imax*jmax
           SIJR(ij,1,k) = &
                C_05_R*(SIJR(ij,1,k)+TMP_LOC(ij,1,k))
        ENDDO
     ENDDO

     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              TAU_M(i,j,k) =&
                   -SIJSG(i,j,k)*DIFFSG(i,j,k)
              wrk3d(i,1,k) = TAU_M(i,j,k)*SIJR(i,j,k)
           ENDDO
        ENDDO
        smg_tauxy(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        smg_eps(j) = smg_eps(j) + &
             C_2_R*AVG_IK(imax, jmax, kmax, i1,&
             wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, TAU_M(1,1,1), SIJSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)

     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, TAU_M(1,1,1), SIJR(1,1,1), i0, i0,  wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        smg_force_u(j) = smg_force_u(j) -&
             AVG_IK(imax, jmax, kmax, j, SIJSG(1,1,1), dx, dz, area)
        smg_force_v(j) = smg_force_v(j) -&
             AVG_IK(imax, jmax, kmax, j, SIJR(1,1,1), dx, dz, area)
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = SIJSG(i,j,k)*u(i,j,k)+ SIJR(i,j,k)*v(i,j,k)
           ENDDO
        ENDDO
        smg_trans(j) = smg_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

!-------------------------------------------------------------
!                                     tauxz = - mu_s * Sxz      
!-------------------------------------------------------------
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, u_sg, SIJSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, w_sg, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1,kmax
        DO ij=1,imax*jmax
           SIJSG(ij,1,k) = &
                C_05_R*(SIJSG(ij,1,k)+TMP_LOC(ij,1,k))
        ENDDO
     ENDDO
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, u, SIJR(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, w, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1,kmax
        DO ij=1,imax*jmax
           SIJR(ij,1,k) = &
                C_05_R*(SIJR(ij,1,k)+TMP_LOC(ij,1,k))
        ENDDO
     ENDDO
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              TAU_M(i,j,k) =&
                   -SIJSG(i,j,k)*DIFFSG(i,j,k)
              wrk3d(i,1,k) = TAU_M(i,j,k)*SIJR(i,j,k)
           ENDDO
        ENDDO
        smg_tauxz(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        smg_eps(j) = smg_eps(j) + &
             C_2_R*AVG_IK(imax, jmax, kmax, i1,&
             wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, TAU_M(1,1,1), SIJSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)      

     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, TAU_M(1,1,1), SIJR(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)  
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = SIJSG(i,j,k)*u(i,j,k)+&
                   SIJR(i,j,k)*w(i,j,k)
           ENDDO
        ENDDO
        smg_trans(j) = smg_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

!-------------------------------------------------------------
!                                     tauyz = - mu_s * Syz      
!-------------------------------------------------------------
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, v_sg, SIJSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, w_sg, TMP_LOC(1,1,1), i0, i0,  wrk1d, wrk2d, wrk3d)
     DO k=1,kmax
        DO ij=1,imax*jmax
           SIJSG(ij,1,k) = &
                C_05_R*(SIJSG(ij,1,k)+TMP_LOC(ij,1,k))
        ENDDO
     ENDDO
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, v, SIJR(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, w, TMP_LOC(1,1,1), i0, i0,  wrk1d, wrk2d, wrk3d)
     DO k=1,kmax
        DO ij=1,imax*jmax
           SIJR(ij,1,k) = &
                C_05_R*(SIJR(ij,1,k)+TMP_LOC(ij,1,k))
        ENDDO
     ENDDO
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              TAU_M(i,j,k) =&
                   -SIJSG(i,j,k)*DIFFSG(i,j,k)
              wrk3d(i,1,k) = TAU_M(i,j,k)*SIJR(i,j,k)
           ENDDO
        ENDDO
        smg_tauyz(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        smg_eps(j) = smg_eps(j) + &
             C_2_R*AVG_IK(imax, jmax, kmax, i1,&
             wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, TAU_M(1,1,1), SIJSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)  
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, TAU_M(1,1,1), SIJR(1,1,1), i0, i0,  wrk1d, wrk2d, wrk3d)      
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = SIJSG(i,j,k)*v(i,j,k)+&
                   SIJR(i,j,k)*w(i,j,k)
           ENDDO
        ENDDO
        smg_trans(j) = smg_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

#undef SIJSG
#undef SIJR
#undef TAU_M
#undef TMP_LOC

#define UXSG(i,j,k)     txc(i,j,k,ip_tmp+1)
#define VYSG(i,j,k)     txc(i,j,k,ip_tmp+2)
#define WZSG(i,j,k)     txc(i,j,k,ip_tmp+3)
#define TAU_M(i,j,k)    txc(i,j,k,ip_tmp+4)
#define TMP_LOC(i,j,k)  txc(i,j,k,ip_tmp+5)

     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, u_sg, UXSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, v_sg, VYSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, w_sg, WZSG(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
!-------------------------------------------------------------
!                                     tauxx = - mu_s * Sxx
!-------------------------------------------------------------
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, u, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              TAU_M(i,j,k) = -DIFFSG(i,j,k)*&
                   (r23*UXSG(i,j,k)-&
                   r13*(VYSG(i,j,k)+WZSG(i,j,k)))
              wrk3d(i,1,k) = TAU_M(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_tauxx(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        smg_eps(j) = smg_eps(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, TAU_M(1,1,1), TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)     
     DO j = 1,jmax
        smg_force_u(j) = smg_force_u(j) -&
             AVG_IK(imax, jmax, kmax, j, &
             TMP_LOC(1,1,1), dx, dz, area)
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TMP_LOC(i,j,k)*u(i,j,k)
           ENDDO
        ENDDO
        smg_trans(j) = smg_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

!-------------------------------------------------------------
!                                     tauyy = - mu_s * Syy
!-------------------------------------------------------------
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, v, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              TAU_M(i,j,k) = -DIFFSG(i,j,k)*&
                   (r23*VYSG(i,j,k)-&
                   r13*(UXSG(i,j,k)+WZSG(i,j,k)))
              wrk3d(i,1,k) = TAU_M(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_tauyy(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        smg_eps(j) = smg_eps(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, TAU_M(1,1,1), TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)     
     DO j = 1,jmax
        smg_force_v(j) = smg_force_v(j) -&
             AVG_IK(imax, jmax, kmax, j, &
             TMP_LOC(1,1,1), dx, dz, area)
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TMP_LOC(i,j,k)*v(i,j,k)
           ENDDO
        ENDDO
        smg_trans(j) = smg_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

!-------------------------------------------------------------
!                                     tauzz = - mu_s * Szz
!-------------------------------------------------------------
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, w, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              TAU_M(i,j,k) = -DIFFSG(i,j,k)*&
                   (r23*WZSG(i,j,k)-&
                   r13*(UXSG(i,j,k)+VYSG(i,j,k)))
              wrk3d(i,1,k) = TAU_M(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_tauzz(j) = AVG_IK(imax, jmax, kmax, j, &
             TAU_M(1,1,1), dx, dz, area)
        smg_eps(j) = smg_eps(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, TAU_M(1,1,1), TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)     
     DO j = 1,jmax
        DO k = 1,kmax
           DO i = 1,imax
              wrk3d(i,1,k) = TMP_LOC(i,j,k)*w(i,j,k)
           ENDDO
        ENDDO
        smg_trans(j) = smg_trans(j) + &
             AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

#undef VYSG
#undef WZSG
#undef TAU_M
#undef TMP_LOC

!-------------------------------------------------------------
!                                                 
!-------------------------------------------------------------
     IF ( iles_type_regu .EQ. LES_REGU_SMGDYN      .OR.&
          iles_type_regu .EQ. LES_REGU_SMGDYNRMS ) THEN
! Dynamic coefficients
     CALL SGS_DYN_Z(ssmflg, coefdelta, n0, n1, x, y, z, dx, dy, dz, &
          rho, u, v, w, e, u_sg, v_sg, w_sg, aux_sg,&
          STRAIN0(1,1,1), STRAIN1(1,1,1), RHOINV1(1,1,1), txc(1,1,1,ip_tmp), &
          smg_csp(1),wrk1d,wrk2d,wrk3d, &
          cf0x,cf0y,cf0z,cf1x,cf1y,cf1z,cf2x,cf2y,cf2z, carm_uf,carm_uh,carm_pf,carm_ph)

! rho*nu_sg = rho*Csp*delta_f^2*S in txc(ip_tmp)
        DO k = 1,kmax
           DO j = 1,jmax
              DO i = 1,imax
                 txc(i,j,k,ip_tmp) = smg_cs(j)*STRAIN0(i,j,k)
              ENDDO
           ENDDO
        ENDDO
        DO j = 1,jmax
           smg_rhoa(j) = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,ip_tmp), dx, dz, area)
        ENDDO
     ELSE
! static part to be developed
     ENDIF
#ifdef _DEBUG
     CALL IO_WRITE_ASCII_FIELD('mu_p.dat', imax, jmax, kmax, txc(1,1,1,ip_tmp))
#endif

#define TMP_LOC(i,j,k)    txc(i,j,k,ip_tmp+2)

! X gradient
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, e, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1, kmax
           DO i = 1, imax
              wrk3d(i,1,k) = DIFFSG(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_qpx(j) =-AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

! Y gradient
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, e, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1, kmax
           DO i = 1, imax
              wrk3d(i,1,k) = DIFFSG(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_qpy(j) =-AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

! Z gradient
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, e, TMP_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO j = 1,jmax
        DO k = 1, kmax
           DO i = 1, imax
              wrk3d(i,1,k) = DIFFSG(i,j,k)*TMP_LOC(i,j,k)
           ENDDO
        ENDDO
        smg_qpz(j) =-AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)
     ENDDO

#undef TMP_LOC

  ENDIF

#undef STRAIN0
#undef STRAIN1
#undef RHOINV1
#undef DIFFSG

! ########################################################################
! Total quantities
! ########################################################################
! Signs: (smg,ssm)_qij ....: +<Q_U>_ij
!    (smg,ssm)_force ..: -<Q_U>_ij,i
!    (smg,ssm)_dis ....: +<(Q_U)_ij U_i,j>
!    (smg,ssm)_trans ..: +<(Q_U)_ij,i U_j>

  DO j = 1,jmax
     wrk1d(j,1) = AVG_IK(imax, jmax, kmax, j, u, dx, dz, area)
     wrk1d(j,2) = AVG_IK(imax, jmax, kmax, j, v, dx, dz, area)
  ENDDO
  CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, wrk1d(1,1), &
       wrk1d(1,3), i0, i0, wrk1d(1,5), wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, wrk1d(1,2), &
       wrk1d(1,4), i0, i0, wrk1d(1,5), wrk2d, wrk3d)
  DO j = 1,jmax
     den = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
     DO k = 1,kmax
        DO i = 1,imax
           wrk3d(i,1,k) = rho(i,j,k)*u(i,j,k)
           wrk3d(i,2,k) = rho(i,j,k)*v(i,j,k)
        ENDDO
     ENDDO
     reyU = wrk1d(j,1)
     reyV = wrk1d(j,2)
     dreyUdy = wrk1d(j,3)
     dreyVdy = wrk1d(j,4)
     favU = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)/den
     favV = AVG_IK(imax, jmax, kmax, i2, wrk3d, dx, dz, area)/den

! Reynolds streeses, <Q_U>_ij/<\rho>
     tot_Rxx(j) = (smg_tauxx(j)+ssm_tauxx(j))/den
     tot_Ryy(j) = (smg_tauyy(j)+ssm_tauyy(j))/den
     tot_Rzz(j) = (smg_tauzz(j)+ssm_tauzz(j))/den
     tot_Rxy(j) = (smg_tauxy(j)+ssm_tauxy(j))/den
     tot_Rxz(j) = (smg_tauxz(j)+ssm_tauxz(j))/den
     tot_Ryz(j) = (smg_tauyz(j)+ssm_tauyz(j))/den

! Force, -<Q_U>_ij,i
     tot_force_u(j) = smg_force_u(j)+&
          ssm_force_u(j)
     tot_force_v(j) = smg_force_v(j)+&
          ssm_force_v(j)

! Dissipation (-), <(Q_U)'_ij U''_i,j>/<\rho> 
     tot_eps(j) = (smg_eps(j)+&
          ssm_eps(j)-&
          (smg_tauxy(j)+ssm_tauxy(j))*dreyUdy-&
          (smg_tauyy(j)+ssm_tauyy(j))*dreyVdy)/den

! Transport (-), -<(Q_U)'_ij U''_i>,j/<\rho> <(Careful w sign tot_force)
     tot_trans(j) =-tot_eps(j)-&
          (smg_trans(j)+&
          ssm_trans(j)+&
          tot_force_u(j)*reyU+&
          tot_force_v(j)*reyV)/den

! Mean, -<U''_i><Q_U>_ij,j/<\rho>
     tot_mean(j) = (&
          (reyU-favU)*tot_force_u(j)+&
          (reyV-favV)*tot_force_v(j))/&
          den
  ENDDO

! ########################################################################
! Global quantities 
! ########################################################################
#define rR(j)    wrk1d(j,1)
  IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

! -------------------------------------------------------------------
! Based on delta_rho
! -------------------------------------------------------------------
! 1% thickness
     IF ( imode_eqns .NE. DNS_EQNS_INCOMPRESSIBLE .AND. &
          imode_eqns .NE. DNS_EQNS_ANELASTIC      .AND. &
          ABS(delta_rho) .GT. C_SMALL_R          )THEN
        DO j = 1,jmax
           rR(j) = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
        ENDDO
        dummy = mean_rho + (C_05_R-C_1EM2_R)*delta_rho
        delta_hb01 = LOWER_THRESHOLD(jmax, dummy, rR(1), y)
        dummy = mean_rho - (C_05_R-C_1EM2_R)*delta_rho
        delta_ht01 = UPPER_THRESHOLD(jmax, dummy, rR(1), y)

        delta_hb01 = (y(1) + scaley*ycoor_rho) - delta_hb01  
        delta_ht01 = delta_ht01 - (y(1) + scaley*ycoor_rho)
        delta_h01  = delta_ht01 + delta_hb01

     ELSE
        delta_h01 = C_1_R

     ENDIF

! -------------------------------------------------------------------
! Based on delta_u 
! -------------------------------------------------------------------
! Vorticity thickness 
     IF ( ABS(delta_u) .GT. C_SMALL_R ) THEN
#define fU(j)    wrk1d(j,2)
#define fU_y(j)  wrk1d(j,3)
        DO j = 1,jmax
           rR(j) = AVG_IK(imax, jmax, kmax, j, rho, dx, dz, area)
        ENDDO
        
        DO j = 1,jmax
           DO k = 1,kmax
              DO i = 1,imax
                 wrk3d(i,1,k) = rho(i,j,k)*u(i,j,k)
              ENDDO
           ENDDO
           fU(j) = AVG_IK(imax, jmax, kmax, i1, wrk3d, dx, dz, area)/rR(j)
        ENDDO
        
        CALL PARTIAL_Y(imode_fdm, i1, jmax, i1, j1bc, dy, fU(1), &
             fU_y(1), i0, i0, wrk1d(1,4), wrk2d, wrk3d)
        delta_w = delta_u/MAXVAL(ABS(fU_y(1:jmax)))

     ELSE
        delta_w = C_1_R

     ENDIF

#undef fU
#undef fU_y

! -----------------------------------------------------------
!                                                     Jet
! -----------------------------------------------------------
  ELSE IF (imode_flow .EQ. DNS_FLOW_JET) THEN
! not developed yet

  ENDIF

#undef rR

! ################################
! # Final writing to TkStat file #
! ################################
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     DO j = 1,jmax
        IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
           SimVar1 = (y(j)-scaley*ycoor_u-y(1))/delta_w
           SimVar2 = (y(j)-scaley*ycoor_rho-y(1))/delta_h01
           WRITE(i23,1300) i1, j, y(j), SimVar1, SimVar2, (mean2d(j,l1),l1=1,L_AVGMAX)

        ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN ! for compatibility of format
           WRITE(i23,1300) i1, j, y(j), y(j), y(j), (mean2d(j,l1),l1=1,L_AVGMAX)

        ENDIF
     ENDDO
#ifdef PARALLEL
  ENDIF
#endif

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CLOSE(23)
#ifdef PARALLEL
  ENDIF
#endif

1300 FORMAT(I5,1X,I5,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

  RETURN
END SUBROUTINE LES_AVG_FLOW_TEMPORAL_LAYER
