! Probably to be splitted into two
! LES_INITIALIZE_01 Filters
! LES_INITIALIZE_02 SGS viscosity
!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE LES_INI(x,y,z,dx,dy,dz, q,z1,h,zh1, txc, vaux, wrk1d,wrk2d,wrk3d)

#include "types.h"
#include "dns_const.h"
#include "les_const.h"
#include "avgij_map.h"
#include "les_const.h"
#include "les_error.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL
  USE NFILE,     ONLY : lfile, efile

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax*jmax*kmax,*) :: q, z1, h, zh1, txc
  TREAL, DIMENSION(*)                :: wrk1d, wrk2d, wrk3d, vaux

! -----------------------------------------------------------------------
  CHARACTER*46 str
  TINTEGER l1, l2, nx0, nx1, ip_mean_les
  TINTEGER icp, ssmflg, ichi
  TREAL CPN2
  TINTEGER p1, p2, p3, i, j, nxy

#ifdef PARALLEL
  INTEGER icount
  TINTEGER fsize_max
#endif

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_INI' )
#endif

  lesfile = 'dns.les'

  nx0 = isgs_f0size
  nx1 = isgs_f1size

  nxy = imax*jmax

  ichi = inb_txc - 1 ! only used in chem active

! Initialize header of log files
  CALL LES_LOGS(i1)
  
! #######################################################################
! Initialize coefficients for top-hat filter
! #######################################################################
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif

     CALL FILTHINI(iunifx, i1bc, imax, nx0, scalex, x, vaux(vindex(VA_LES_FLT0X)), wrk1d)
     CALL FILTHINI(iunify, j1bc, jmax, nx0, scaley, y, vaux(vindex(VA_LES_FLT0Y)), wrk1d)
     IF ( kmax_total .GT. 1 ) THEN
        CALL FILTHINI(iunifz, k1bc, kmax_total, nx0, scalez, z, vaux(vindex(VA_LES_FLT0Z)), wrk1d)
     ENDIF

     CALL FILTHINI(iunifx, i1bc, imax, nx1, scalex, x, vaux(vindex(VA_LES_FLT1X)), wrk1d)
     CALL FILTHINI(iunify, j1bc, jmax, nx1, scaley, y, vaux(vindex(VA_LES_FLT1Y)), wrk1d)
     IF ( kmax_total .GT. 1 ) THEN
        CALL FILTHINI(iunifz, k1bc, kmax_total, nx1, scalez, z, vaux(vindex(VA_LES_FLT1Z)), wrk1d)
     ENDIF

     IF ( iles_type_regu .EQ. LES_REGU_SMGDYN      .OR. &
          iles_type_regu .EQ. LES_REGU_SMGDYNRMS ) THEN
        CALL FILTH2INI(iunifx,i1bc,imax,nx0,nx1,scalex, x, vaux(vindex(VA_LES_FLT2X)), txc, wrk1d)
        CALL FILTH2INI(iunify,j1bc,jmax,nx0,nx1,scaley, y, vaux(vindex(VA_LES_FLT2Y)), txc, wrk1d)
        IF ( kmax_total .GT. 1 ) THEN
           CALL FILTH2INI(iunifz,k1bc,kmax_total,nx0,nx1,scalez, z, vaux(vindex(VA_LES_FLT2Z)), txc, wrk1d)
        ENDIF
     ENDIF

#ifdef PARALLEL
  ENDIF
  icount = vsize(VA_LES_FLT0X) + vsize(VA_LES_FLT0Y) + vsize(VA_LES_FLT0Z) + &
           vsize(VA_LES_FLT1X) + vsize(VA_LES_FLT1Y) + vsize(VA_LES_FLT1Z) + &
           vsize(VA_LES_FLT2X) + vsize(VA_LES_FLT2Y) + vsize(VA_LES_FLT2Z)
  CALL MPI_BCAST(vaux(vindex(VA_LES_FLT0X)), icount, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
#endif

! ####################################
! #  Initialize table ARM LES model  #
! ####################################

!  IF ( iles_inviscid .NE. 1 ) THEN
!     IF ( iles_type_tran .EQ. LES_TRAN_ARMSPTL .OR. &
!          iles_type_chem .EQ. LES_CHEM_STAT_BS .OR.&
!          iles_type_chem .EQ. LES_CHEM_STAT_PS ) THEN
!        IF ( imode_sim .EQ. DNS_MODE_TEMPORAL ) THEN
!           CALL SGS_ARM_SPTL_INIT(iarm_spc, jmax, iarm_nl, &
!                iarm_nre,&
!                arm_remin, arm_dre, vaux(vindex(VA_ARM_WRK)), &
!                vaux(vindex(VA_ARM_C0)))
!        ELSE IF ( imode_sim .EQ. DNS_MODE_SPATIAL ) THEN
!           CALL SGS_ARM_SPTL_INIT(iarm_spc, nxy, iarm_nl, &
!                iarm_nre, &
!                arm_remin, arm_dre, vaux(vindex(VA_ARM_WRK)), &
!                vaux(vindex(VA_ARM_C0)))
!        ENDIF
!     ENDIF
!  ENDIF

! #######################################################################
! Calculate initial SGS-viscosity
! #######################################################################
  ip_mean_les = vindex(VA_MEAN_WRK)+(MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*inb_scal)*nstatavg*jmax

  IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) txc(:,ichi) = C_0_R

  IF ( iles_type_tran .EQ. LES_TRAN_NONE ) THEN 
     ssmflg = 0
  ELSE
     ssmflg = 1
  ENDIF

  IF ( iles_type_tran .EQ. LES_TRAN_NONE) THEN
  ELSE 
     CALL SGS_RECTN(i0, x,y,z,dx,dy,dz, q,z1, &
          txc(1,1),txc(1,2),txc(1,3),txc(1,4), h(1,1),h(1,2),h(1,3),h(1,4),zh1, vaux(ip_mean_les), &
          txc(1,5),txc(1,6),txc(1,7),txc(1,8),txc(1,9),txc(1,10),txc(1,11), &
          txc(1,12), txc(1,ichi), wrk1d,wrk2d,wrk3d, &
          vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), vaux(vindex(VA_LES_FLT0Z)),  &
          vaux(vindex(VA_LES_ARM_UF)),vaux(vindex(VA_LES_ARM_PF)),vaux(vindex(VA_LES_ARM_ZF)) )
  ENDIF

  IF ( iles_type_regu .EQ. LES_REGU_NONE) THEN
  ELSE IF ( iles_type_regu .EQ. LES_REGU_SMGSTA ) THEN
     CALL SGS_STA(x,dx,dy,dz, q,z1, h(1,1),h(1,2),h(1,3),h(1,4),zh1, vaux(ip_mean_les), &
          txc, txc(1,ichi), wrk1d,wrk2d)
     
  ELSE IF ( iles_type_regu .EQ. LES_REGU_SMGDYN .OR.&
       iles_type_regu .EQ. LES_REGU_SMGDYNRMS ) THEN
     CALL SGS_DYN(i1, i0, ssmflg, x,y,z,dx,dy,dz, q,z1, &
          txc(1,1),txc(1,2),txc(1,3),txc(1,4), h(1,1),h(1,2),h(1,3),h(1,4),zh1, txc(1,5), &
          vaux(ip_mean_les), txc(1,ichi), wrk1d,wrk2d,wrk3d, &
          vaux(vindex(VA_LES_FLT0X)), vaux(vindex(VA_LES_FLT0Y)), vaux(vindex(VA_LES_FLT0Z)), &
          vaux(vindex(VA_LES_FLT1X)), vaux(vindex(VA_LES_FLT1Y)), vaux(vindex(VA_LES_FLT1Z)), &
          vaux(vindex(VA_LES_FLT2X)), vaux(vindex(VA_LES_FLT2Y)), vaux(vindex(VA_LES_FLT2Z)), &
          vaux(vindex(VA_LES_ARM_UF)), vaux(vindex(VA_LES_ARM_UH)), &
          vaux(vindex(VA_LES_ARM_PF)), vaux(vindex(VA_LES_ARM_PH)), &
          vaux(vindex(VA_LES_ARM_ZF)), vaux(vindex(VA_LES_ARM_ZH)), &
          vaux(vindex(VA_LES_SGSCOEFU)), vaux(vindex(VA_LES_SGSCOEFE)), vaux(vindex(VA_LES_SGSCOEFZ)))
  ENDIF

  CALL LES_LOGS(i2)

! #######################################################################
! Initialize reactive part
! #######################################################################
  IF ( iles_type_chem .NE. CHEM_NONE ) THEN

     CPN2 = C_0_R
     DO icp=NCP_CHEMKIN, 1, -1
        CPN2 = CPN2 + THERMO_AI(icp,2,NSP)
     ENDDO

     LES_CHW_Qp = C_05_R*CHW_CE*CPN2/XIST/(1-XIST)
     TGFM = C_0_R

     WRITE(str,'(A8,1X,E12.5E3)') 'Q_P   = ', LES_CHW_Qp
     CALL IO_WRITE_ASCII(lfile, str)

! Impose exactly same value in all PEs
#ifdef PARALLEL
     CALL MPI_BCAST(LES_CHW_Qp, 1, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
#endif

! Read FDF table
     CALL CLIP(les_fdf_bs_file, l1, l2)
     IF ( les_fdf_bs_file(l1:l2) .NE. 'simplified' ) THEN
        OPEN(i23,file=les_fdf_bs_file(l1:l2),status='old',form='unformatted')
        READ(i23) p1,p2
        IF ( p1 .NE. les_fdf_bs_maxmean .OR. p2 .NE. les_fdf_bs_maxvar ) THEN
           CALL IO_WRITE_ASCII(efile,'LESINI: Error in FDF dimensions')
           CALL DNS_STOP(LES_ERROR_FDFDIM)    
        ENDIF
        p1 = vindex(VA_LES_FDF_BS)
        p2 = p1 + les_fdf_bs_maxmean
        p3 = p2 + les_fdf_bs_maxmean*les_fdf_bs_maxvar
        READ(i23) (vaux(p1-1+i),i=1,les_fdf_bs_maxmean)
! reading the information transposed
        READ(i23) ((vaux(p2-1+j+(i-1)*les_fdf_bs_maxvar),i=1,les_fdf_bs_maxmean),j=1,les_fdf_bs_maxvar)
        READ(i23) ((vaux(p3-1+j+(i-1)*les_fdf_bs_maxvar),i=1,les_fdf_bs_maxmean),j=1,les_fdf_bs_maxvar)
        CLOSE(i23)         
     ENDIF

  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LES_INI' )
#endif

  RETURN
END SUBROUTINE LES_INI
