#include "types.h"
#include "les_error.h"
#include "dns_const.h"
#include "les_const.h"

SUBROUTINE LES_READ_INI(inifile)

  USE DNS_GLOBAL
  USE LES_GLOBAL
  USE NFILE

  IMPLICIT NONE

  CHARACTER*(*) inifile

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  CHARACTER*64 sRes

! ###################################################################
  CALL IO_WRITE_ASCII(lfile, '#')
  CALL IO_WRITE_ASCII(lfile, '#[LES]')
  CALL IO_WRITE_ASCII(lfile, '#Active=<yes/no>')
  CALL SCANINICHAR(lfile, inifile, 'LES', 'Active', 'no', sRes)
  IF ( TRIM(ADJUSTL(sRes)) .EQ. 'yes' ) THEN
     iles = 1

! ###################################################################
! Global information
! ###################################################################
     CALL IO_WRITE_ASCII(lfile, '#Transport=<none/ssm/arm-rans/arm-sptl/arm-invs/gradient')
     CALL IO_WRITE_ASCII(lfile, '#Regularization=none/smg-static/smg-dynamic')
     CALL IO_WRITE_ASCII(lfile, '#Chemistry=<none/burke-schumann>')
     CALL IO_WRITE_ASCII(lfile, '#Dissipation=<none/diss-sgs>')
     CALL IO_WRITE_ASCII(lfile, '#Filter0Size=<value>')
     CALL IO_WRITE_ASCII(lfile, '#Filter1Size=<value>') 
     CALL IO_WRITE_ASCII(lfile, '#Inviscid=<yes/no>')

! Transport
     CALL SCANINICHAR(lfile, inifile, 'LES', 'Transport', 'ssm', sRes)
     IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'none'     ) THEN; iles_type_tran = LES_TRAN_NONE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'ssm'      ) THEN; iles_type_tran = LES_TRAN_SSM
                                                           arm_ucoef = C_1_R
                                                           arm_zcoef = C_1_R
                                                           arm_pcoef = C_1_R
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm-rans' ) THEN; iles_type_tran = LES_TRAN_ARMRANS
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm-sptl' ) THEN; iles_type_tran = LES_TRAN_ARMSPTL
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm-invs' ) THEN; iles_type_tran = LES_TRAN_ARMINVS
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'gradient' ) THEN
        CALL IO_WRITE_ASCII(efile,'Gradient model not developed')
        CALL DNS_STOP(LES_ERROR_TYPE)
     ELSE
        CALL IO_WRITE_ASCII(efile,'You must specify a transport model for LES')
        CALL DNS_STOP(LES_ERROR_TYPE)
     ENDIF

! Regularization
     CALL SCANINICHAR(lfile, inifile, 'LES', 'Regularization', 'smg-static', sRes)
     IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'none'            ) THEN; iles_type_regu = LES_REGU_NONE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'smg-static'      ) THEN; iles_type_regu = LES_REGU_SMGSTA
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'smg-dynamic'     ) THEN; iles_type_regu = LES_REGU_SMGDYN
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'smg-static-rms'  ) THEN; iles_type_regu = LES_REGU_SMGSTARMS
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'smg-dynamic-rms' ) THEN; iles_type_regu = LES_REGU_SMGDYNRMS
     ELSE
        CALL IO_WRITE_ASCII(efile,'You must specify a regularization model for LES')
        CALL DNS_STOP(LES_ERROR_TYPE)
     ENDIF

! Dissipation
     CALL SCANINICHAR(lfile, inifile, 'LES', 'Dissipation', 'none', sRes)
     IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'none'     ) THEN; iles_type_diss = LES_DISS_NONE
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'diss-sgs' ) THEN; iles_type_diss = LES_DISS_DISSSG ; ENDIF

! Chemistry:
! If active in DNS, then use corresponding LES model:
! CHEM_TYPE_XXX -> LES_CHEM_XXX
     IF ( ireactive .NE. CHEM_NONE ) THEN
        IF ( imech_type .EQ. CHEM_TYPE_QUASIBS ) THEN
           iles_type_chem = LES_CHEM_QUASIBS
        ENDIF
     ELSE
! if not, read a specific one
        CALL SCANINICHAR(lfile, inifile, 'LES', 'Chemistry', 'none', sRes)
        IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'none'   ) THEN; iles_type_chem = LES_CHEM_NONE
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm_bs' ) THEN; iles_type_chem = LES_CHEM_STAT_BS
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm_ps' ) THEN; iles_type_chem = LES_CHEM_STAT_PS
        ELSE
           CALL IO_WRITE_ASCII(efile,'You must specify a model for reactive LES')
           CALL DNS_STOP(LES_ERROR_TYPE)
        ENDIF
     ENDIF

! Filters
     CALL SCANINIINT(lfile, inifile, 'LES', 'Filter0Size', '2', isgs_f0size)
     CALL SCANINIINT(lfile, inifile, 'LES', 'Filter1Size', '4', isgs_f1size)
     IF ( MOD(isgs_f0size,i2) .NE. 0 ) THEN
        CALL IO_WRITE_ASCII(efile, 'LES: filter-0 size is not even')
        CALL DNS_STOP(LES_ERROR_EVEN)
     ENDIF
     IF ( MOD(isgs_f1size,i2) .NE. 0 ) THEN
        CALL IO_WRITE_ASCII(efile, 'LES: filter-1 size is not even')
        CALL DNS_STOP(LES_ERROR_EVEN)
     ENDIF

! Inviscid
     CALL SCANINICHAR(lfile, inifile, 'LES', 'Inviscid', 'no', sRes)
     IF ( TRIM(ADJUSTL(sRes)) .EQ. 'yes' ) THEN
        iles_inviscid = 1
        iviscous   = C_0_R
        idiffusion = C_0_R
     ELSE
        iles_inviscid = 0
     ENDIF

! control now in routines FILTH_Z and FILTH2_Z
!#ifdef PARALLEL
!     IF ( ims_npro .GT. 1 ) THEN
!        IF ( isgs_f0size .GT. 2*kmax ) THEN
!           CALL IO_WRITE_ASCII(efile, 'LES: f0 size greater than kmax')
!           CALL DNS_STOP(LES_ERROR_SIZEMPI)
!        ENDIF
!        IF ( isgs_f1size .GT. 2*kmax ) THEN
!           CALL IO_WRITE_ASCII(efile, 'LES: f1 size greater than kmax')
!           CALL DNS_STOP(LES_ERROR_SIZEMPI)
!        ENDIF
!        IF ( isgs_f0size+isgs_f1size .GT. 2*kmax ) THEN
!           CALL IO_WRITE_ASCII(efile, 'LES: f0+f1 size greater than kmax')
!           CALL DNS_STOP(LES_ERROR_SIZEMPI)
!        ENDIF
!     ENDIF
!#endif

! ###################################################################
! Nonreactive information
! ###################################################################
     CALL IO_WRITE_ASCII(lfile, '#SmagVariant=<deviatoric/other>')
     CALL IO_WRITE_ASCII(lfile, '#SmagTrans=<value>')
     CALL IO_WRITE_ASCII(lfile, '#SmagDelta=<value>')
     CALL IO_WRITE_ASCII(lfile, '#Alpha=<value>')
     CALL IO_WRITE_ASCII(lfile, '#Csm=<value>')
     CALL IO_WRITE_ASCII(lfile, '#Prandtl=<value>')
     CALL IO_WRITE_ASCII(lfile, '#Schmidt=<value>')

     CALL SCANINICHAR(lfile, inifile, 'LES', 'SmagVariant', 'deviatoric', sRes)
     IF ( TRIM(ADJUSTL(sRes)) .EQ. 'deviatoric' ) THEN; sgs_devsmag = 1
     ELSE;                                              sgs_devsmag = 0; ENDIF
     IF ( imode_sim .EQ. DNS_MODE_SPATIAL ) THEN
        CALL SCANINIREAL(lfile, inifile, 'LES', 'SmagTrans', '-1.0', sgs_smagtrans)
        CALL SCANINIREAL(lfile, inifile, 'LES', 'SmagDelta', '0.0', sgs_smagdelta)
     ENDIF

     CALL SCANINIREAL(lfile, inifile, 'LES', 'Alpha', '1.0', sgs_alpha)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'Csm', '0.13', sgs_csm)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'Prandtl', '0.6', smg_prandtl)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'Schmidt', '0.6', smg_schmidt)
! Value of sgs_pdil from Vreman et al., App. Scien. Research 54:191, 1995 
     CALL SCANINIREAL(lfile, inifile, 'LES', 'AlphaDil', '2.2', sgs_pdil)

! ###################################################################
! Reactive information
! ###################################################################
     IF ( iles_type_chem .NE. LES_CHEM_NONE ) THEN

! Conditonal dissipation model
        CALL SCANINICHAR(lfile, inifile, 'LES', 'ChemistryCondDissipation', 'mean', sRes)
        IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'mean' )            THEN; iles_type_disZchem = LES_DISZCHEM_MEAN
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'one-dimensional' ) THEN; iles_type_disZchem = LES_DISZCHEM_ONEDIM
        ELSE
           CALL IO_WRITE_ASCII(efile,'You must specify a model for ChemistryCondDissipation')
           CALL DNS_STOP(LES_ERROR_TYPE)
        ENDIF

! Dissipation model
        CALL SCANINICHAR(lfile, inifile, 'LES', 'ChemistryDissipation', 'gradient', sRes)
        IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'gradient' ) THEN; iles_type_dischem = LES_DISCHEM_GRAD
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'diss-sgs' ) THEN; iles_type_dischem = LES_DISCHEM_DISSSG
        ELSE
           CALL IO_WRITE_ASCII(efile,'You must specify a model for ChemistryDissipation')
           CALL DNS_STOP(LES_ERROR_TYPE)
        ENDIF

! FDF table
        CALL SCANINICHAR(lfile, inifile, 'LES', 'FDFfile', ' ', les_fdf_bs_file)
        IF ( TRIM(ADJUSTL(les_fdf_bs_file)) .EQ. 'simplified' ) THEN
           les_fdf_bs_maxmean = 1
           les_fdf_bs_maxvar = 1
        ELSE
           OPEN(i23,file=TRIM(ADJUSTL(les_fdf_bs_file)),status='old', form='unformatted')
           READ(i23) les_fdf_bs_maxmean, les_fdf_bs_maxvar
           CLOSE(i23)
        ENDIF

! Reconstruction level
        CALL SCANINICHAR(lfile, inifile, 'LES', 'ChemistryVariance', 'arm-rans', sRes)
        IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'ssm'      ) THEN; iles_type_recchem = LES_RECCHEM_SSM
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm-invs' ) THEN; iles_type_recchem = LES_RECCHEM_ARMINVS
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm-rans' ) THEN; iles_type_recchem = LES_RECCHEM_ARMRANS
        ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'arm-sptl' ) THEN; iles_type_recchem = LES_RECCHEM_ARMSPTL
        ELSE
           CALL IO_WRITE_ASCII(efile,'You must specify a model for ChemistryVariance')
           CALL DNS_STOP(LES_ERROR_TYPE)
        ENDIF

     ENDIF

     CALL IO_WRITE_ASCII(lfile, '#ARM_Spectrum=<Pope/Adhoc>')
     CALL IO_WRITE_ASCII(lfile, '#ARM_NL=<value>')
     CALL IO_WRITE_ASCII(lfile, '#ARM_NRe=<value>')
     CALL IO_WRITE_ASCII(lfile, '#ARM_ReMin=<value>')
     CALL IO_WRITE_ASCII(lfile, '#ARM_DeltaRe=<value>')
     CALL IO_WRITE_ASCII(lfile, '#ARM_c0=<value>')

!     IF ( iles_type_tran .EQ. LES_TRAN_ARMSPTL ) THEN
! ARM parameters
     CALL SCANINICHAR(lfile, inifile, 'LES', 'ARM_Spectrum', 'Pope',sRes)
     IF      ( TRIM(ADJUSTL(sRes)) .EQ. 'pope'  ) THEN; iarm_spc = 1
     ELSE IF ( TRIM(ADJUSTL(sRes)) .EQ. 'adhoc' ) THEN; iarm_spc = 2
     ELSE
        CALL IO_WRITE_ASCII(efile,'ARM spectrum not valid')
        CALL DNS_STOP(LES_ERROR_ARM_SPECTRUM)
     ENDIF
     CALL SCANINIINT(lfile, inifile, 'LES', 'ARM_NL',       '10',   iarm_nl)
     CALL SCANINIINT(lfile, inifile, 'LES', 'ARM_NRe',      '10',   iarm_nre)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_ReMin',   '10.0', arm_remin)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_DeltaRe', '10.0', arm_dre)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_ActivaT', '100.0',arm_tact)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_FlameT',  '10.0', arm_tflame)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_StechioZ','0.2',  arm_zst)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_SmoothZ', '0.1',  arm_smooth)
     CALL SCANINIREAL(lfile, inifile, 'LES', 'ARM_c0',      '4.72', arm_c0_inviscid)
!     ENDIF

  ELSE
     iles = 0
  ENDIF

  RETURN
END SUBROUTINE LES_READ_INI
