SUBROUTINE LES_SAVE_CHEMIJ( iterms, dx, dy, dz, txc, mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  
! #####################################################
! #
! # Runnning statistics for LES terms.
! # Careful with passed array txc
! #
! # ---------------------------------------------------
! # Modification history
! # 11/25/01 - Created 
! # 12/14/01 - Including the ARM statistics
! # 01/22/02 - Including SSM statistics
! # 06/25/02 - Extracted from les_save_avgij.f
! # 11/06/03 - Modified for LESCHEM_V1_1 
! #####################################################
  
#include "types.h"
#include "les_avgij_map.h"
  
  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

  TINTEGER iterms

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL txc(imax,jmax,kmax,*)

  TREAL mean1d_les_ch(nstatavg,jmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)

  TINTEGER i, j, n, ij, ij2, NNstat
  TINTEGER i0
  TREAL r13, r23

  i0 = 0
  r13 = C_1_R/C_3_R
  r23 = C_2_R/C_3_R
  NNstat = nstatavg*jmax

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LES_SAVE_CHEMIJ' )
#endif

! ###############################################################
! #                   ARM  model coefficient                    #
! ###############################################################
  IF ( iterms .EQ. 1 ) THEN

     DO j = 1,jmax
        DO n = 1,nstatavg
           i = statavg(n)
           ij = (j-1)*imax + i
           ij2 = (j-1)*nstatavg + n
           LC_ARM_c0(ij2) = LC_ARM_c0(ij2) +&
                kmax_total*wrk2d(ij,1) 
!           mean1d_les_ch(n,j,27) = mean1d_les_ch(n,j,27) + 
! $              kmax_total*wrk2d(ij,1) 
        ENDDO
     ENDDO

  ENDIF

! ##################################################
! #         Polynomial non-linearities             #
! ##################################################

! Resolved parts
  IF ( iterms .EQ. 2 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,3), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,2) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,3) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,5), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), &
          wrk2d(1,4) )

     DO j = 1,NNstat
        LC_ARM_Z2_SG(j)=LC_ARM_Z2_SG(j)+wrk2d(j,1)
        LC_ARM_Z4_SG(j)=LC_ARM_Z4_SG(j)+wrk2d(j,2)
        LC_ARM_Z8_SG(j)=LC_ARM_Z8_SG(j)+wrk2d(j,3)
     ENDDO
  ENDIF

! Subgrid parts
  IF ( iterms .EQ. 3 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,3), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,2) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,3) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,5), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), &
          wrk2d(1,4) )

     DO j = 1,NNstat
        LC_ARM_Z2_R(j)=LC_ARM_Z2_R(j)+wrk2d(j,1)
        LC_ARM_Z4_R(j)=LC_ARM_Z4_R(j)+wrk2d(j,2)
        LC_ARM_Z8_R(j)=LC_ARM_Z8_R(j)+wrk2d(j,3)
     ENDDO
  ENDIF

! ##########################################
! #       Arrhenius non-linearity          #
! ##########################################
  IF ( iterms .EQ. 4 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,3), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,2) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,3) )

     DO j = 1,NNstat
        LC_ARM_WZ_R(j) =LC_ARM_WZ_R(j) +&
             wrk2d(j,1)
        LC_ARM_WZ_SG(j)=LC_ARM_WZ_SG(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! ##########################################
! #       Temperature non-linearity        #
! ##########################################
  IF ( iterms .EQ. 5 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,3), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,2) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,3) )

     DO j = 1,NNstat
        LC_ARM_TZ_R(j) =LC_ARM_TZ_R(j) +&
             wrk2d(j,1)
        LC_ARM_TZ_SG(j)=LC_ARM_TZ_SG(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! ##########################################
! #         Density non-linearity          #
! ##########################################
  IF ( iterms .EQ. 6 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,3), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,2) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,4), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,3) )

     DO j = 1,NNstat
        LC_ARM_RZ_R(j) =LC_ARM_RZ_R(j) +&
             wrk2d(j,1)
        LC_ARM_RZ_SG(j)=LC_ARM_RZ_SG(j)+&
             wrk2d(j,2)
     ENDDO
  ENDIF

! ##########################################
! #      Convective terms in (Z^2)_sg      #
! ##########################################
  IF ( iterms .EQ. 7 ) THEN
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,1), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,1), &
          wrk2d(1,2) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,2), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,2), &
          wrk2d(1,3) )
     CALL REDUCE( imax, jmax, kmax, txc(1,1,1,3), nstatavg, &
          statavg, wrk3d )
     CALL SUM_K_V( NNstat, kmax, wrk3d, wrk2d(1,3), &
          wrk2d(1,4) )

     DO j = 1,NNstat
        LC_ARM_RZ2_SG(j)=LC_ARM_RZ2_SG(j) +&
             wrk2d(j,1)
        LC_ARM_CX_Z2_SG(j)=LC_ARM_CX_Z2_SG(j)+&
             wrk2d(j,2)
        LC_ARM_CY_Z2_SG(j)=LC_ARM_CY_Z2_SG(j)+&
             wrk2d(j,3)
     ENDDO
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LES_SAVE_CHEMIJ' )
#endif

  RETURN
END SUBROUTINE LES_SAVE_CHEMIJ
