      SUBROUTINE LESAVG_V1_0(itxc, x, y, mean1d,&
           mean1d_les, stat, wrk1d, wrk2d)

! ###############################################################
! # Postprocessing of averages for the LES quantities
! #
! # mean1d_les - array with the accumalated statistics for the les
! # mrho    - accumulated rho
! # mu      - accumulated rho*u
! # stat    - tmp array of size itxc
! #
! # Juan Pedro Mellado, May 2002
! ################################################################

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

      IMPLICIT NONE

      TINTEGER itxc
      TREAL x(imax)
      TREAL y(jmax)
      TREAL mean1d(nstatavg,jmax,*)
      TREAL mean1d_les(nstatavg,jmax,*)
      TREAL stat(nstatavg,jmax,*)
      TREAL wrk1d(nstatavg,*)
      TREAL wrk2d(nstatavg,jmax,5)

      TINTEGER i, j, n, NNstat, l1, l2, ll1, ll2
      TREAL U2, DU, fU_05
      TREAL pts, y_center, SimVar1
      CHARACTER*32 name
      CHARACTER*250 line1
      CHARACTER*500 line2
      TINTEGER i0, i1, i23

#define smg_cs(A,B)      stat(A,B,1)
#define smg_mu(A,B)      stat(A,B,2)
#define smg_tauxx(A,B)   stat(A,B,3)
#define smg_tauyy(A,B)   stat(A,B,4)
#define smg_tauzz(A,B)   stat(A,B,5)
#define smg_tauxy(A,B)   stat(A,B,6)
#define smg_tauxz(A,B)   stat(A,B,7) 
#define smg_tauyz(A,B)   stat(A,B,8)
#define smg_eps(A,B)     stat(A,B,9)

#define ssm_tauxx(A,B)   stat(A,B,10)
#define ssm_tauyy(A,B)   stat(A,B,11)
#define ssm_tauzz(A,B)   stat(A,B,12)
#define ssm_tauxy(A,B)   stat(A,B,13)
#define ssm_tauxz(A,B)   stat(A,B,14) 
#define ssm_tauyz(A,B)   stat(A,B,15)
#define ssm_eps(A,B)     stat(A,B,16)

#define smg_csp(A,B)     stat(A,B,17)
#define smg_rhoa(A,B)    stat(A,B,18)
#define smg_qpx(A,B)     stat(A,B,19)
#define smg_qpy(A,B)     stat(A,B,20)
#define smg_qpz(A,B)     stat(A,B,21)
#define ssm_qpx(A,B)     stat(A,B,22)
#define ssm_qpy(A,B)     stat(A,B,23)
#define ssm_qpz(A,B)     stat(A,B,24)

#define L_AVGMAX 24

#ifdef TRACE_ON
      CALL IO_WRITE_ASCII(tfile, 'ENTERING LESAVG_V1_0' )
#endif

      i0 = 0
      i1 = 1
      i23 = 23

      IF ( itxc .LT. nstatavg*jmax*L_AVGMAX) THEN
         CALL IO_WRITE_ASCII(efile,'LESAVG_V1_0: Not enough space in stat')
         CALL DNS_STOP(LES_ERROR_AVGTMP)
      ENDIF
      if ( nstatavg_points .EQ. 0 ) then
         CALL IO_WRITE_ASCII(efile,'LESAVG_V1_0: Zero number of points')
         CALL DNS_STOP(LES_ERROR_AVGPTS)
      ELSE
         pts = C_1_R/M_REAL(nstatavg_points)
      endif

      NNstat = jmax*nstatavg
      y_center = y(1) + C_05_R*scaley
      U2 = mean_u - C_05_R*delta_u

      DO j = 1,NNstat

         wrk2d(j,1,1) = MA_RU(j)/MA_R(j)

         smg_cs(j,1) = LA_SMG_CS_U(j)*pts
         smg_mu(j,1) = LA_SMG_RNU(j)*pts

         smg_tauxx(j,1) = LA_SMG_TAUxx(j)*pts
         smg_tauyy(j,1) = LA_SMG_TAUyy(j)*pts
         smg_tauzz(j,1) = LA_SMG_TAUzz(j)*pts
         smg_tauxy(j,1) = LA_SMG_TAUxy(j)*pts
         smg_tauxz(j,1) = LA_SMG_TAUxz(j)*pts
         smg_tauyz(j,1) = LA_SMG_TAUyz(j)*pts
         smg_eps(j,1)   = LA_SMG_EPS_U(j)*pts
         
         ssm_tauxx(j,1) = LA_SSM_TAUxx(j)*pts
         ssm_tauyy(j,1) = LA_SSM_TAUyy(j)*pts
         ssm_tauzz(j,1) = LA_SSM_TAUzz(j)*pts
         ssm_tauxy(j,1) = LA_SSM_TAUxy(j)*pts
         ssm_tauxz(j,1) = LA_SSM_TAUxz(j)*pts
         ssm_tauyz(j,1) = LA_SSM_TAUyz(j)*pts
         ssm_eps(j,1)   = LA_SSM_EPS_U(j)*pts
         
         smg_csp(j,1)   = LA_SMG_CS_P(j)*pts
         smg_rhoa(j,1)  = LA_SMG_RALPHA(j)*pts

         smg_qpx(j,1)   = LA_SMG_QPx(j)*pts
         smg_qpy(j,1)   = LA_SMG_QPy(j)*pts
         smg_qpz(j,1)   = LA_SMG_QPz(j)*pts
         ssm_qpx(j,1)   = LA_SSM_QPx(j)*pts
         ssm_qpy(j,1)   = LA_SSM_QPy(j)*pts
         ssm_qpz(j,1)   = LA_SSM_QPz(j)*pts

      ENDDO

! ######################################
! # Favre-averaged profiles of U       #
! ######################################
    
      IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

      ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Velocity Half width
         DO n = 1,nstatavg
            DU = wrk2d(n,jmax/2,1) - U2
            fU_05 = U2 + C_05_R*DU
            DO j = 1,jmax/2
               IF ( wrk2d(n,j  ,1) .LE. fU_05 .AND. &
                    wrk2d(n,j+1,1) .GT. fU_05 ) THEN
                  wrk1d(n,1) = y(j) + &
                       (fU_05-wrk2d(n,j,1))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,1)-wrk2d(n,j,1))
               ENDIF
            ENDDO
            wrk1d(n,1) = y_center - wrk1d(n,1)
            DU = wrk2d(n,jmax/2+1,1) - U2
            fU_05 = U2 + C_05_R*DU
            DO j = jmax/2+1,jmax
               IF ( wrk2d(n,j  ,1) .GT. fU_05 .AND. &
                    wrk2d(n,j+1,1) .LE. fU_05 ) THEN
                  wrk1d(n,2) = y(j) + &
                       (fU_05-wrk2d(n,j,1))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,1)-wrk2d(n,j,1))
               ENDIF
            ENDDO
            wrk1d(n,2) = wrk1d(n,2) - y_center
            wrk1d(n,1) = C_05_R*(wrk1d(n,1)+wrk1d(n,2))
         ENDDO
         
      ENDIF

! ############################
! # Scaling of the quatities #
! ############################

      DO n = 1,nstatavg

         IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
         ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
            DU = C_05_R*(wrk2d(n,jmax/2,1)+wrk2d(n,jmax/2+1,1)) - U2
         ENDIF

         DO j = 1,jmax
            smg_eps(n,j) = smg_eps(n,j)/(DU*DU*DU)*wrk1d(n,1)
            ssm_eps(n,j) = ssm_eps(n,j)/(DU*DU*DU)*wrk1d(n,1)
         ENDDO

      ENDDO

! ####################################
! # Saving the data in TkStat format #
! ####################################

      name = 'jlsavg                            '
      CALL CONCATI(name,itime)

#ifdef USE_RECLEN
      OPEN(UNIT=i23,RECL=500,FILE=name,STATUS='unknown')
#else
      OPEN(UNIT=i23,FILE=name,STATUS='unknown')
#endif     

      WRITE(i23,*) '#################'
      WRITE(i23,*) '#  Version 1.0  #'
      WRITE(i23,*) '#################'
      WRITE(i23,*) 'RTIME = ', rtime

      line2='I J X Y SU'
      CALL CLIP(line2,ll1,ll2)

      line1 ='Cs rho*nu_sg smg_tauxx smg_tauyy smg_tauzz '&
           //'smg_tauxy smg_tauxz smg_tauyz smg_eps '
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 1 SMG_U ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='ssm_tauxx ssm_tauyy ssm_tauzz '&
           //'ssm_tauxy ssm_tauxz ssm_tauyz ssm_eps '
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 2 SSM_U ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='CsP rho*alpha_sg smg_qpx smg_qpy smg_qpz '
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 3 SMG_P ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='ssm_qpx ssm_qpy ssm_qpz'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP 4 SSM_P ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      WRITE(i23,*) line2(ll1:ll2)
   
      DO n = 1,nstatavg
         i = statavg(n)
         DO j = 1,jmax
            IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
            ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
               SimVar1 = (y(j) - y_center)/wrk1d(n,1)
               WRITE(i23,1300) i, j, x(i), y(j), SimVar1,&
                    (stat(n,j,l1),l1=1,L_AVGMAX)
            ENDIF
         ENDDO
      ENDDO

      CLOSE(i23)

#ifdef TRACE_ON
      CALL IO_WRITE_ASCII(tfile, 'LEAVING LESAVG_V1_0' )
#endif

 1300 FORMAT(I3,1X,I3,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

      RETURN
      END
