      SUBROUTINE LESAVG_V1_1(itxc, x, y, mean1d,&
           mean1d_les, stat, wrk1d, wrk2d)

! ###############################################################
! # Postprocessing of averages for the LES quantities
! #
! # mean1d_les - array with the accumalated statistics for the les
! # mean1d     - array with the accumalated statistics for base flow
! # stat       - tmp array of size itxc
! #
! # 05/01/02 - Version 1.0
! # 07/19/03 - Version 1.1. Adding terms for the momentum and 
! #            energy balance equations.
! #
! ################################################################

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

      IMPLICIT NONE

      TINTEGER itxc
      TREAL x(imax)
      TREAL y(jmax)
      TREAL mean1d(nstatavg,jmax,*)
      TREAL mean1d_les(nstatavg,jmax,*)
      TREAL stat(nstatavg,jmax,*)
      TREAL wrk1d(nstatavg,*)
      TREAL wrk2d(nstatavg,jmax,5)

      TINTEGER i, j, n, NNstat, l1, l2, ll1, ll2
      TREAL U2, DU, fU_05
      TREAL pts, y_center, SimVar1
      TREAL favUx, favUy, favVx, favVy
      TREAL favU, favV, reyU, reyV
      TREAL SIMPSON_NU
      CHARACTER*32 name
      CHARACTER*250 line1
      CHARACTER*500 line2
      TINTEGER i0, i1, i23

#define smg_cs(A,B)      stat(A,B,1)
#define smg_mu(A,B)      stat(A,B,2)
#define smg_tauxx(A,B)   stat(A,B,3)
#define smg_tauyy(A,B)   stat(A,B,4)
#define smg_tauzz(A,B)   stat(A,B,5)
#define smg_tauxy(A,B)   stat(A,B,6)
#define smg_tauxz(A,B)   stat(A,B,7) 
#define smg_tauyz(A,B)   stat(A,B,8)
#define smg_eps(A,B)     stat(A,B,9)
#define smg_trans(A,B)   stat(A,B,10)
#define smg_force_u(A,B) stat(A,B,11)
#define smg_force_v(A,B) stat(A,B,12)

#define ssm_tauxx(A,B)   stat(A,B,13)
#define ssm_tauyy(A,B)   stat(A,B,14)
#define ssm_tauzz(A,B)   stat(A,B,15)
#define ssm_tauxy(A,B)   stat(A,B,16)
#define ssm_tauxz(A,B)   stat(A,B,17) 
#define ssm_tauyz(A,B)   stat(A,B,18)
#define ssm_eps(A,B)     stat(A,B,19)
#define ssm_trans(A,B)   stat(A,B,20)
#define ssm_force_u(A,B) stat(A,B,21)
#define ssm_force_v(A,B) stat(A,B,22)

#define smg_csp(A,B)     stat(A,B,23)
#define smg_rhoa(A,B)    stat(A,B,24)
#define smg_qpx(A,B)     stat(A,B,25)
#define smg_qpy(A,B)     stat(A,B,26)
#define smg_qpz(A,B)     stat(A,B,27)
#define ssm_qpx(A,B)     stat(A,B,28)
#define ssm_qpy(A,B)     stat(A,B,29)
#define ssm_qpz(A,B)     stat(A,B,30)

#define tot_eps(A,B)     stat(A,B,31)
#define tot_trans(A,B)   stat(A,B,32)
#define tot_mean(A,B)    stat(A,B,33)
#define tot_prod(A,B)    stat(A,B,34)

#define tot_force_u(A,B) stat(A,B,35)
#define tot_force_v(A,B) stat(A,B,36)

#define tot_tauxx(A,B)   stat(A,B,37)
#define tot_tauyy(A,B)   stat(A,B,38)
#define tot_tauzz(A,B)   stat(A,B,39)
#define tot_tauxy(A,B)   stat(A,B,40)
#define tot_tauxz(A,B)   stat(A,B,41)
#define tot_tauyz(A,B)   stat(A,B,42)

#define L_AVGMAX 42
#define LAST_INDEX 43

#define TotMom(A)       stat(A,1,LAST_INDEX)

#ifdef TRACE_ON
      CALL IO_WRITE_ASCII(tfile, 'ENTERING LESAVG_V1_1' )
#endif

      i0 = 0
      i1 = 1
      i23 = 23

      IF ( itxc .LT. nstatavg*jmax*LAST_INDEX) THEN
         CALL IO_WRITE_ASCII(efile,'LESAVG_V1_1: Not enough space in stat')
         CALL DNS_STOP(LES_ERROR_AVGTMP)
      ENDIF
      IF ( nstatavg_points .EQ. 0 ) THEN
         CALL IO_WRITE_ASCII(efile,'LESAVG_V1_1: Zero number of points')
         CALL DNS_STOP(LES_ERROR_AVGPTS)
      ELSE
         pts = C_1_R/M_REAL(nstatavg_points)
      ENDIF
      
      NNstat = jmax*nstatavg
      y_center = y(1) + C_05_R*scaley
      U2 = mean_u - C_05_R*delta_u

      DO j = 1,NNstat

         wrk2d(j,1,1) = MA_RU(j)/MA_R(j)
         wrk2d(j,1,2) = MA_RV(j)/MA_R(j)

         smg_cs(j,1) = LA_SMG_CS_U(j)*pts
         smg_mu(j,1) = LA_SMG_RNU(j)*pts

         smg_tauxx(j,1) = LA_SMG_TAUxx(j)*pts
         smg_tauyy(j,1) = LA_SMG_TAUyy(j)*pts
         smg_tauzz(j,1) = LA_SMG_TAUzz(j)*pts
         smg_tauxy(j,1) = LA_SMG_TAUxy(j)*pts
         smg_tauxz(j,1) = LA_SMG_TAUxz(j)*pts
         smg_tauyz(j,1) = LA_SMG_TAUyz(j)*pts
         smg_eps(j,1)   = LA_SMG_EPS_U(j)*pts
         smg_trans(j,1) = LA_SMG_TRA_U(j)*pts
         smg_force_u(j,1)= LA_SMG_FORCE_U(j)*pts
         smg_force_v(j,1)= LA_SMG_FORCE_V(j)*pts

         ssm_tauxx(j,1) = LA_SSM_TAUxx(j)*pts
         ssm_tauyy(j,1) = LA_SSM_TAUyy(j)*pts
         ssm_tauzz(j,1) = LA_SSM_TAUzz(j)*pts
         ssm_tauxy(j,1) = LA_SSM_TAUxy(j)*pts
         ssm_tauxz(j,1) = LA_SSM_TAUxz(j)*pts
         ssm_tauyz(j,1) = LA_SSM_TAUyz(j)*pts
         ssm_eps(j,1)   = LA_SSM_EPS_U(j)*pts
         ssm_trans(j,1) = LA_SSM_TRA_U(j)*pts
         ssm_force_u(j,1)= LA_SSM_FORCE_U(j)*pts
         ssm_force_v(j,1)= LA_SSM_FORCE_V(j)*pts
         
         smg_csp(j,1)   = LA_SMG_CS_P(j)*pts
         smg_rhoa(j,1)  = LA_SMG_RALPHA(j)*pts

         smg_qpx(j,1)   = LA_SMG_QPx(j)*pts
         smg_qpy(j,1)   = LA_SMG_QPy(j)*pts
         smg_qpz(j,1)   = LA_SMG_QPz(j)*pts
         ssm_qpx(j,1)   = LA_SSM_QPx(j)*pts
         ssm_qpy(j,1)   = LA_SSM_QPy(j)*pts
         ssm_qpz(j,1)   = LA_SSM_QPz(j)*pts

         tot_force_u(j,1) = smg_force_u(j,1)+&
              ssm_force_u(j,1)
         tot_force_v(j,1) = smg_force_v(j,1)+&
              ssm_force_v(j,1)

         tot_tauxx(j,1) = (smg_tauxx(j,1)+&
              ssm_tauxx(j,1))/&
              (MA_R(j)*pts)
         tot_tauyy(j,1) = (smg_tauyy(j,1)+&
              ssm_tauyy(j,1))/&
              (MA_R(j)*pts)
         tot_tauzz(j,1) = (smg_tauzz(j,1)+&
              ssm_tauzz(j,1))/&
              (MA_R(j)*pts)
         tot_tauxy(j,1) = (smg_tauxy(j,1)+&
              ssm_tauxy(j,1))/&
              (MA_R(j)*pts)
         tot_tauxz(j,1) = (smg_tauxz(j,1)+&
              ssm_tauxz(j,1))/&
              (MA_R(j)*pts)
         tot_tauyz(j,1) = (smg_tauyz(j,1)+&
              ssm_tauyz(j,1))/&
              (MA_R(j)*pts)
       
         favUx = (MA_RUx(j)+MA_URx(j)-&
              MA_Rx(j)*wrk2d(j,1,1))/MA_R(j)
         favUy = (MA_RUy(j)+MA_URy(j)-&
              MA_Ry(j)*wrk2d(j,1,1))/MA_R(j)
         favVx = (MA_RVx(j)+MA_VRx(j)-&
              MA_Rx(j)*wrk2d(j,1,2))/MA_R(j)
         favVy = (MA_RVy(j)+MA_VRy(j)-&
              MA_Ry(j)*wrk2d(j,1,2))/MA_R(j)
         favU = wrk2d(j,1,1)
         favV = wrk2d(j,1,2)
         reyU = MA_U(j)*pts
         reyV = MA_V(j)*pts

         tot_eps(j,1) = (LA_SMG_EPS_U(j)+&
              LA_SSM_EPS_U(j)-&
         (LA_SMG_TAUxx(j)+LA_SSM_TAUxx(j))*&
         favUx-&
         (LA_SMG_TAUxy(j)+LA_SSM_TAUxy(j))*&
         (favUy+favVx)-&
         (LA_SMG_TAUyy(j)+LA_SSM_TAUyy(j))*&
         favVy)/MA_R(j)

! watch out the sign of LS_FORCE !!
         tot_trans(j,1) =-tot_eps(j,1)-&
             (LA_SMG_TRA_U(j)+&
              LA_SSM_TRA_U(j)+&
         (LA_SMG_FORCE_U(j)+LA_SSM_FORCE_U(j))*&
         favU+&
         (LA_SMG_FORCE_V(j)+LA_SSM_FORCE_V(j))*&
         favV)/MA_R(j)

         tot_mean(j,1) = ((reyU-favU)*&
              tot_force_u(j,1)+&
              (reyV-favV)*tot_force_v(j,1))/&
              (MA_R(j)*pts)

         tot_prod(j,1) =-(&
              tot_tauxx(j,1)*favUx+&
              tot_tauxy(j,1)*&
              (favUy+favVx)+&
              tot_tauyy(j,1)*favVy)
      ENDDO

! ######################################
! # Favre-averaged profiles of U       #
! ######################################
    
      IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

      ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Velocity Half width
         DO n = 1,nstatavg
            DU = wrk2d(n,jmax/2,1) - U2
            fU_05 = U2 + C_05_R*DU
            DO j = 1,jmax/2
               IF ( wrk2d(n,j  ,1) .LE. fU_05 .AND. &
                    wrk2d(n,j+1,1) .GT. fU_05 ) THEN
                  wrk1d(n,1) = y(j) + &
                       (fU_05-wrk2d(n,j,1))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,1)-wrk2d(n,j,1))
               ENDIF
            ENDDO
            wrk1d(n,1) = y_center - wrk1d(n,1)
            DU = wrk2d(n,jmax/2+1,1) - U2
            fU_05 = U2 + C_05_R*DU
            DO j = jmax/2+1,jmax
               IF ( wrk2d(n,j  ,1) .GT. fU_05 .AND. &
                    wrk2d(n,j+1,1) .LE. fU_05 ) THEN
                  wrk1d(n,2) = y(j) + &
                       (fU_05-wrk2d(n,j,1))*(y(j+1)-y(j))/&
                       (wrk2d(n,j+1,1)-wrk2d(n,j,1))
               ENDIF
            ENDDO
            wrk1d(n,2) = wrk1d(n,2) - y_center
            wrk1d(n,1) = C_05_R*(wrk1d(n,1)+wrk1d(n,2))
         ENDDO
         
      ENDIF

! ###########################
! # Integral quantities jet #
! ###########################

      IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Excess of momentum
      l1 = ibuffer_ht-ibuffer_hb+1
      DO n = 1,nstatavg
         DO j = ibuffer_hb,ibuffer_ht
            wrk2d(j,1,3) = smg_tauxx(n,j)+&
                 ssm_tauxx(n,j)
         ENDDO
         TotMom(n) = SIMPSON_NU(l1, wrk2d(ibuffer_hb,1,3), &
              y(ibuffer_hb))
      ENDDO        
      ENDIF

! ############################
! # Scaling of the quatities #
! ############################

      DO n = 1,nstatavg

         IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
         ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
            DU = C_05_R*(wrk2d(n,jmax/2,1)+wrk2d(n,jmax/2+1,1)) - U2
         ENDIF

         DO j = 1,jmax
            tot_force_u(n,j) = tot_force_u(n,j)/(DU*DU)*wrk1d(n,1)
            tot_force_v(n,j) = tot_force_v(n,j)/(DU*DU)*wrk1d(n,1)

            tot_eps(n,j)  = tot_eps(n,j)/(DU*DU*DU)*wrk1d(n,1)
            tot_trans(n,j)= tot_trans(n,j)/(DU*DU*DU)*wrk1d(n,1)
            tot_mean(n,j) = tot_mean(n,j)/(DU*DU*DU)*wrk1d(n,1)
            tot_prod(n,j) = tot_prod(n,j)/(DU*DU*DU)*wrk1d(n,1)

            tot_tauxx(n,j) = SQRT(tot_tauxx(n,j))/DU
            tot_tauyy(n,j) = SQRT(tot_tauyy(n,j))/DU
            tot_tauzz(n,j) = SQRT(tot_tauzz(n,j))/DU
            tot_tauxy(n,j) = tot_tauxy(n,j)/(DU*DU)
            tot_tauxz(n,j) = tot_tauxz(n,j)/(DU*DU)
            tot_tauyz(n,j) = tot_tauyz(n,j)/(DU*DU)
         ENDDO

      ENDDO

! ####################################
! # Saving the data in TkStat format #
! ####################################

      name = 'jlsavg                            '
      CALL CONCATI(name,itime)

#ifdef USE_RECLEN
      OPEN(UNIT=i23,RECL=650,FILE=name,STATUS='unknown')
#else
      OPEN(UNIT=i23,FILE=name,STATUS='unknown')
#endif     

      WRITE(i23,*) '#################'
      WRITE(i23,*) '#  Version 1.1  #'
      WRITE(i23,*) '#################'
      WRITE(i23,*) 'RTIME = ', rtime

      line2='I J X Y SU'
      CALL CLIP(line2,ll1,ll2)

      line1 ='Cs rho*nu_sg smg_tauxx smg_tauyy smg_tauzz '&
           //'smg_tauxy smg_tauxz smg_tauyz smg_eps smg_trans '&
           //'smg_force_u smg_force_v'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = SMG_U ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='ssm_tauxx ssm_tauyy ssm_tauzz '&
           //'ssm_tauxy ssm_tauxz ssm_tauyz ssm_eps ssm_trans '&
           //'ssm_force_u ssm_force_v'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = SSM_U ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='CsP rho*alpha_sg smg_qpx smg_qpy smg_qpz '
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = SMG_P ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='ssm_qpx ssm_qpy ssm_qpz'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = SSM_P ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='Diss Tran MnFl Prod'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = TKE_Eqn ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='Reyn_u'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = U_Eqn ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='Reyn_v'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = V_Eqn ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      line1 ='Rxx Ryy Rzz Rxy Rxz Ryz'
      CALL CLIP(line1,l1,l2)
      WRITE(i23,*) 'GROUP = Reynolds_stresses ',line1(l1:l2)
      line2=line2(ll1:ll2)//' '//line1(l1:l2)
      CALL CLIP(line2,ll1,ll2)

      IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
         line1 = 'TotMom'
         CALL CLIP(line1,l1,l2)
         WRITE(i23,*) 'GROUP = 1D_Quantities ',line1(l1:l2)
         line2=line2(ll1:ll2)//' '//line1(l1:l2)
         CALL CLIP(line2,ll1,ll2)
      ENDIF

      WRITE(i23,*) line2(ll1:ll2)
   
      DO n = 1,nstatavg
         i = statavg(n)
         DO j = 1,jmax
            SimVar1 = (y(j) - y_center)/wrk1d(n,1)
            IF ( j .EQ. jmax/2 ) THEN
               IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
               ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
                  WRITE(i23,1400) i, j, x(i), y(j), SimVar1,&
                       (stat(n,j,l1),l1=1,L_AVGMAX),&
                       TotMom(n)
               ENDIF
            ELSE
               IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
               ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
                  WRITE(i23,1300) i, j, x(i), y(j), SimVar1,&
                       (stat(n,j,l1),l1=1,L_AVGMAX)
               ENDIF
            ENDIF
         ENDDO
      ENDDO

      CLOSE(i23)

#ifdef TRACE_ON
      CALL IO_WRITE_ASCII(tfile, 'LEAVING LESAVG_V1_1' )
#endif

 1300 FORMAT(I3,1X,I3,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))
 1400 FORMAT(I3,1X,I3,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3),&
           1X,E12.5E3)

      RETURN
      END
