SUBROUTINE LESCHEM_V1_1(is, itxc, x, y, mean1d, mean1d_sc, &
     mean1d_les_sc, mean1d_les_ch, stat, wrk1d, wrk2d)
  
! ###############################################################
! # Postprocessing of averages for the LES quantities
! #
! # mean1d_les_sc - array with the accumulated statistics for the scalar les
! # mean1d        - array with the accumulated statistics for base flow
! # mean1d_sc     - array with the accumulated statistics for base scalar flow
! # stat          - tmp array of size itxc
! #
! # 11/06/03 - Version 1.1. Extracted from LESSCBDG_V1_1
! #
! ################################################################
  
#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"
  
  USE DNS_GLOBAL
  USE LES_GLOBAL
  USE NFILE,     ONLY : efile

  IMPLICIT NONE

  TINTEGER is,itxc
  TREAL x(imax)
  TREAL y(jmax)
  TREAL mean1d(nstatavg,jmax,*)
  TREAL mean1d_sc(nstatavg,jmax,*)
  TREAL mean1d_les_sc(nstatavg,jmax,*)
  TREAL mean1d_les_ch(nstatavg,jmax,*)
  TREAL stat(nstatavg,jmax,*)
  TREAL wrk1d(nstatavg,*)
  TREAL wrk2d(nstatavg,jmax,5)

  TINTEGER i, j, n, NNstat, l1, l2, ll1, ll2
  TREAL S2, DS, U2, DU
  TREAL armpts, pts, y_center, SimVar1
  TREAL rR, dRdx, dRdy, favU, favV, r05
  CHARACTER*32 name
  CHARACTER*250 line1
  CHARACTER*500 line2
  TINTEGER i0, i1, i23

#define arm_c0(A,B)      stat(A,B,1)
#define arm_Z2_r(A,B)    stat(A,B,2)
#define arm_Z2_sg(A,B)   stat(A,B,3)
#define arm_Z4_r(A,B)    stat(A,B,4)
#define arm_Z4_sg(A,B)   stat(A,B,5)
#define arm_Z8_r(A,B)    stat(A,B,6)
#define arm_Z8_sg(A,B)   stat(A,B,7)
#define arm_TZ_r(A,B)    stat(A,B,8)
#define arm_TZ_sg(A,B)   stat(A,B,9)
#define arm_RZ_r(A,B)    stat(A,B,10)
#define arm_RZ_sg(A,B)   stat(A,B,11)
#define arm_WZ_r(A,B)    stat(A,B,12)
#define arm_WZ_sg(A,B)   stat(A,B,13)

#define arm_fZ2_sg(A,B)  stat(A,B,14)
#define arm_c_Z2_sg(A,B) stat(A,B,15)

#define L_AVGMAX 15

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING LESCHEM_V1_1' )
#endif

  i0 = 0
  i1 = 1
  i23 = 23

  r05 = C_05_R

  IF ( itxc .LT. nstatavg*jmax*L_AVGMAX) THEN
     CALL IO_WRITE_ASCII(efile,'LESCHEM_V1_1: Not enough space in stat')
     CALL DNS_STOP(LES_ERROR_AVGTMP)
  ENDIF
  if ( iarmavg_pts .EQ. 0 ) then
     CALL IO_WRITE_ASCII(efile,'LESCHEM_V1_1: Zero number of points')
     CALL DNS_STOP(LES_ERROR_AVGPTS)
  ELSE
     armpts = C_1_R/M_REAL(iarmavg_pts)
  ENDIF
  IF ( nstatavg_points .EQ. 0 ) then
     CALL IO_WRITE_ASCII(efile,'LESAVG_V1_1: Zero number of points')
     CALL DNS_STOP(LES_ERROR_AVGPTS)
  ELSE
     pts = C_1_R/M_REAL(nstatavg_points)
  ENDIF

  NNstat = jmax*nstatavg
  y_center = y(1) + C_05_R*scaley
  S2 = mean_z - C_05_R*delta_z
  U2 = mean_u - C_05_R*delta_u

  DO j = 1,NNstat

     wrk2d(j,1,1) = MA_RU(j)/MA_R(j)
     wrk2d(j,1,2) = MS_RS(j)/MA_R(j)


     arm_c0(j,1)    = LC_ARM_c0(j)*armpts
     arm_Z2_r(j,1)  = LC_ARM_Z2_R(j)*armpts
     arm_Z2_sg(j,1) = LC_ARM_Z2_SG(j)*armpts
     arm_Z4_r(j,1)  = LC_ARM_Z4_R(j)*armpts
     arm_Z4_sg(j,1) = LC_ARM_Z4_SG(j)*armpts
     arm_Z8_r(j,1)  = LC_ARM_Z8_R(j)*armpts
     arm_Z8_sg(j,1) = LC_ARM_Z8_SG(j)*armpts
     arm_TZ_r(j,1)  = LC_ARM_TZ_R(j)*armpts
     arm_TZ_sg(j,1) = LC_ARM_TZ_SG(j)*armpts
     arm_RZ_r(j,1)  = LC_ARM_RZ_R(j)*armpts
     arm_RZ_sg(j,1) = LC_ARM_RZ_SG(j)*armpts
     arm_WZ_r(j,1)  = LC_ARM_WZ_R(j)*armpts
     arm_WZ_sg(j,1) = LC_ARM_WZ_SG(j)*armpts

! Transport equation for (Z^2)_sg
     favU = MA_RU(j)/MA_R(j)
     favV = MA_RV(j)/MA_R(j)
     rR   = MA_R(j)*pts
     dRdx = MA_Rx(j)*pts
     dRdy = MA_Ry(j)*pts

     arm_fZ2_sg(j,1) = LC_ARM_RZ2_SG(j)*armpts&
          /rR

     arm_c_Z2_sg(j,1) = -favU*&
          (LC_ARM_CX_Z2_SG(j)*armpts&
          -arm_fZ2_sg(j,1)*dRdx)/rR&
          -favV*&
          (LC_ARM_CY_Z2_SG(j)*armpts-&
          arm_fZ2_sg(j,1)*dRdy)/rR

  ENDDO

! ######################################
! # Favre-averaged profiles of U and Z #
! ######################################

  IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN

  ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
! Velocity Half width
     CALL DELTA_X(nstatavg, jmax, y, wrk2d(1,1,1), wrk1d(1,1),&
          wrk1d(1,2), wrk1d(1,3), U2, r05)
! Scalar Half width
     CALL DELTA_X(nstatavg, jmax, y, wrk2d(1,1,2), wrk1d(1,2),&
          wrk1d(1,3), wrk1d(1,4), S2, r05)
  ENDIF

! ############################
! # Scaling of the quatities #
! ############################
  DO n = 1,nstatavg
     IF ( imode_flow .EQ. DNS_FLOW_SHEAR ) THEN
     ELSE IF ( imode_flow .EQ. DNS_FLOW_JET ) THEN
        DU = C_05_R*(wrk2d(n,jmax/2,1)+wrk2d(n,jmax/2+1,1)) - U2
        DS = C_05_R*(wrk2d(n,jmax/2,2)+wrk2d(n,jmax/2+1,2)) - S2
     ENDIF
     DO j = 1,jmax
        arm_fZ2_sg(n,j) = arm_fZ2_sg(n,j)/(DS*DS)
        arm_c_Z2_sg(n,j) = arm_c_Z2_sg(n,j)&
             /(DS*DS*DU)*wrk1d(n,1)
     ENDDO
  ENDDO

! ####################################
! # Saving the data in TkStat format #
! ####################################

  name = 'jls                            '
  CALL CONCATI(name, is)
  CALL CONCAT(name, 'chem')
  CALL CONCATI(name,itime)

#ifdef USE_RECLEN
  OPEN(UNIT=i23,RECL=326,FILE=name,STATUS='unknown')
#else
  OPEN(UNIT=i23,FILE=name,STATUS='unknown')
#endif     

  WRITE(i23,1010) '#################'
  WRITE(i23,1010) '#  Version 1.1  #'
  WRITE(i23,1010) '#################'
  WRITE(i23,'(A8,E14.7E3)') 'RTIME = ', rtime

  line2='I J X Y SS'
  CALL CLIP(line2,ll1,ll2)

  line1 ='arm_c0 arm_Z2_r arm_Z2_sg arm_Z4_r arm_Z4_sg '&
       //'arm_Z8_r arm_Z8_sg '&
       //'arm_T(Z)_r arm_T(Z)_sg arm_R(Z)_r arm_R(Z)_sg '&
       //'arm_W(Z)_r arm_W(Z)_sg'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = ARM '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  line1 ='fZ2_sg Conv'
  CALL CLIP(line1,l1,l2)
  WRITE(i23,1010) 'GROUP = (Z2)sg_Eqn '//line1(l1:l2)
  line2=line2(ll1:ll2)//' '//line1(l1:l2)
  CALL CLIP(line2,ll1,ll2)

  WRITE(i23,1010) line2(ll1:ll2)

1010 FORMAT(A)

  DO n = 1,nstatavg
     i = statavg(n)
     DO j = 1,jmax
        IF ( imode_flow .EQ. DNS_FLOW_SHEAR) THEN
        ELSE IF ( imode_flow .EQ. DNS_FLOW_JET) THEN
           SimVar1 = (y(j) - y_center)/wrk1d(n,2)
           WRITE(i23,1300) i, j, x(i), y(j), SimVar1,&
                (stat(n,j,l1),l1=1,L_AVGMAX)
        ENDIF
     ENDDO
  ENDDO

  CLOSE(i23)

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING LESCHEM_V1_1' )
#endif

1300 FORMAT(I3,1X,I3,3(1X,E12.5E3),L_AVGMAX(1X,E12.5E3))

  RETURN
END SUBROUTINE LESCHEM_V1_1
