      SUBROUTINE FDF_SINE_STO(n0, z_s, dx, dy, dz, rho, z1, z2sg, 
     $     rz_m, barrz_m, barvs, fdf_zs, txc, wrk3d,
     $     cf0x, cf0y, cf0z)

c     txc size is 2

      IMPLICIT NONE

#include "types.h"
#include "common.h"

      TINTEGER n0
      TREAL z_s
      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax_total)
      TREAL rho(imax,jmax,kmax)
      TREAL z1(imax,jmax,kmax)
      TREAL z2sg(imax,jmax,kmax)
      TREAL rz_m(imax,jmax,kmax)
      TREAL barrz_m(imax,jmax,kmax)
      TREAL barvs(imax,jmax,kmax)
      TREAL txc(imax,jmax,kmax,*)
      TREAL wrk3d(imax,jmax,kmax)
      TREAL fdf_zs(imax,jmax,kmax)
      TREAL cf0x(*), cf0y(*), cf0z(*)
      
      TINTEGER nmax
      PARAMETER(nmax=5)
      TINTEGER i, j, ij, k
      TREAL dtmp, eps, ln_Zs, ln_rZs
      TREAL MAT_W(nmax,nmax), MAT_B(nmax), MAT_W_LOC(nmax,nmax)
      TREAL r0
      TINTEGER i1, i3

      i1 = 1
      i3 = 3
      r0 = C_0_R

      eps = C_1EM6_R
      ln_Zs = LOG(z_s)
      ln_rZs = LOG(1-z_s)

#define Z3SG(i,j,k)   txc(i,j,k,1)
#define Z4SG(i,j,k)   txc(i,j,k,2)

c ##############################
c     Building up the matrix
c ##############################
      CALL DNS_UTIL_VSC(nmax, nmax, i1, MAT_W, r0)
c     First column (independent term)
      DO j = 1,nmax
         MAT_W(j,1) = C_1_R/M_REAL(j)
      ENDDO
c     First raw
      DO i = 1,nmax-1,2
         MAT_W(1,i+1) = C_2_R/C_PI_R/M_REAL(i)
      ENDDO
c     Second raw
      DO i = 1,nmax-1
         MAT_W(2,i+1) = (-1)**(i+1)/C_PI_R/M_REAL(i)
      ENDDO
c     Recurrence formula for the rest of the raws
      DO j = 3,nmax
         DO i = 1,nmax-1
            MAT_W(j,i+1) = MAT_W(2,i+1) -
     $           M_REAL((j-1)*(j-2))/
     $           ((C_PI_R*M_REAL(i))**2)*MAT_W(j-2,i+1)
         ENDDO
      ENDDO
c      DO j = 1,nmax
c         PRINT*,(MAT_W(j,i),i=1,nmax)
c      ENDDO

c ##############################
c     Higher moments using ARM
c ##############################
c     here I can use fdf as auxiliar array
      DO k = 1,kmax
         DO ij = 1,NN
            fdf_zs(ij,1,k) = (rz_m(ij,1,k)**3)/(rho(ij,1,k)**2)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n0, cf0x, cf0y, cf0z, fdf_zs, Z3SG(1,1,1), wrk3d)
      DO k = 1,kmax
         DO ij = 1,NN
            fdf_zs(ij,1,k) = (rz_m(ij,1,k)**4)/(rho(ij,1,k)**3)
         ENDDO
      ENDDO
      CALL FILTH(i3, imax, jmax, kmax, kmax_total, 
     $     iunifx, iunify, iunifz, i1bc, j1bc, k1bc,
     $     n0, cf0x, cf0y, cf0z, fdf_zs, Z4SG(1,1,1), wrk3d)
      DO k = 1,kmax
         DO ij = 1,NN
            Z3SG(ij,1,k) = (Z3SG(ij,1,k) -
     $           (barrz_m(ij,1,k)**3)*(barvs(ij,1,k)**2))/rho(ij,1,k)
            Z4SG(ij,1,k) = (Z4SG(ij,1,k) -
     $           (barrz_m(ij,1,k)**4)*(barvs(ij,1,k)**3))/rho(ij,1,k)
c     assuming a positive-definite filter
            dtmp = z1(ij,1,k)*(C_1_R-z1(ij,1,k)**2)
            Z3SG(ij,1,k) = MAX(MIN(Z3SG(ij,1,k),dtmp),C_0_R)
            dtmp = z1(ij,1,k)*(C_1_R-z1(ij,1,k)**3)
            Z4SG(ij,1,k) = MAX(MIN(Z4SG(ij,1,k),dtmp),C_0_R)
         ENDDO
      ENDDO

#ifdef _DEBUG
         CALL IO_WRITE_ASCII_FIELD('Z3SG.dat', imax, jmax, kmax, 
     $        Z3SG(1,1,1))
         CALL IO_WRITE_ASCII_FIELD('Z4SG.dat', imax, jmax, kmax, 
     $        Z4SG(1,1,1))
#endif
c ##############################
c     FDF
c ##############################
      DO k = 1,kmax
         DO ij = 1,NN

c     forcing term
            MAT_B(1) = C_1_R
            MAT_B(2) = z1(ij,1,k)
            MAT_B(3) = z2sg(ij,1,k) + z1(ij,1,k)**2
            MAT_B(4) = Z3SG(ij,1,k) + z1(ij,1,k)**3
            MAT_B(5) = Z4SG(ij,1,k) + z1(ij,1,k)**4
c     solving the system (MAT_A and MAT_B are overwritten)
            DO i = 1,nmax*nmax
               MAT_W_LOC(i,1) = MAT_W(i,1)
            ENDDO
            CALL GAUSS(nmax,MAT_W_LOC,MAT_B)

c     calculating FDF
            dtmp = C_0_R
            DO i = 1,nmax-1
               dtmp = dtmp + MAT_B(i+1)*
     $              SIN(C_PI_R*M_REAL(i)*z_s)
            ENDDO
            fdf_zs(ij,1,k) = MAT_B(1) + dtmp

         ENDDO
      ENDDO

#undef Z3SG
#undef Z4SG

      RETURN
      END
