      SUBROUTINE SGS_DYN_U2(ii, statflg, ssmflg, n0, n1, coefdelta,
     $     x, y, z, dx, dy, dz, rho, u, v, w, 
     $     sij, barsij, barvs, txc, mean1d_les, wrk1d, wrk2d, wrk3d, 
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh, numerator, denominator)

c ################################################
c # txc1 is bar1(bar0(u) |
c # txc2 is bar1(bar0(v)  > use in dilatation
c # txc3 is bar1(bar0(w) |
c #
c # barvs(1) = 1/bar1(bar0(rho))          <--- Always.
c # barvs(2) = 1/bar0(bar0(rho))             | Only
c # barvs(3) = 1/bar1(bar0(bar1(bar0(rho)))) | for Hij.
c #
c # txc size is 7, if ssmflg is 0
c # txc size is 8, if ssmflg is 1
c #
c # 07/29/02 - Introducing barvs to save time
c # 09/25/04 - Profiling and preparing for ARM: 1 more array.
c #            Introduncing 2 density arrays.
c ################################################

      IMPLICIT NONE

#include "types.h"
#include "dns_const.h"
#include "les_const.h"
#include "param.h"
#include "common.h"

      TINTEGER ii, statflg, ssmflg, n1, n0
      TREAL coefdelta

      TREAL rho(imax,jmax,kmax)
      TREAL u(imax,jmax,kmax)
      TREAL v(imax,jmax,kmax)
      TREAL w(imax,jmax,kmax)

      TREAL sij(imax,jmax,kmax)
      TREAL barsij(imax,jmax,kmax)
      TREAL barvs(imax,jmax,kmax,*)
      TREAL txc(imax,jmax,kmax,*)

      TREAL x(imax)
      TREAL y(jmax)
      TREAL z(kmax_total)
      TREAL dx(imax)
      TREAL dy(jmax)
      TREAL dz(kmax_total)

      TREAL mean1d_les(nstatavg,jmax,*)
      TREAL wrk1d(ijkmax,MX1D)
      TREAL wrk2d(ijmax, MX2D)
      TREAL wrk3d(imax,jmax,kmax)
      TREAL cf0x(*), cf0y(*), cf0z(*)
      TREAL cf1x(*), cf1y(*), cf1z(*)
      TREAL cf2x(*), cf2y(*), cf2z(*)
      TREAL carm_uf(*), carm_uh(*)
      TREAL numerator(*), denominator(*)

      TINTEGER itmp, imodel_loc, ij, k, j
      TREAL r23, r13, r0, AVG_IK, dummy
      TINTEGER i0, i1, i2, i3

      i0 = 0
      i1 = 1
      i2 = 2
      i3 = 3

      r0 = C_0_R
      r23 = C_2_R/C_3_R
      r13 = C_1_R/C_3_R
      
      imodel_loc = 1

#ifdef TRACE_ON
      CALL OUTPUT(trfile, 'ENTERING SGS_DYN_U')
#endif


c ######################################################################
c     PART 1: terms involving S_H
c ######################################################################

c --------------------------------------------------------------
c This section requires txc_size=7 if ssmflg is 0
c                       txc_size=8 is ssmflg is 1
c --------------------------------------------------------------
c#define DIL_LOC1(i,j,k)   txc(i,j,k,4)
c#define DIL_LOC2(i,j,k)   txc(i,j,k,5)
c#define TMP_LOC(i,j,k)    txc(i,j,k,6)
c      IF ( sgs_devsmag .EQ. 1 ) THEN
c         CALL DILATATION(dx, dy, dz, u, v, w, DIL_LOC1(1,1,1), 
c     $        TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d)
c         CALL DILATATION(dx, dy, dz, txc(1,1,1,1), txc(1,1,1,2), 
c     $        txc(1,1,1,3), DIL_LOC2(1,1,1), TMP_LOC(1,1,1), 
c     $        wrk1d, wrk2d, wrk3d)
c      ENDIF

#define SIJ_LOC(i,j,k)  txc(i,j,k,1)
#define FRACTION(i,j,k) txc(i,j,k,2)
#define TMP_LOC(i,j,k)  txc(i,j,k,5)

      itmp = imax*jmax*kmax*3
      CALL DNS_UTIL_VSC(itmp, i1, i1, FRACTION(1,1,1), r0)

c ############################
c #  Lxx, Hxx and Mxx terms  #
c ############################
      CALL SGS_DYN_NORMAL2(i1, ssmflg, n0, n1, 
     $     dx, dy, dz,
     $     rho, u, sij, barsij, barvs, FRACTION(1,1,1), 
     $     SIJ_LOC(1,1,1), dummy, TMP_LOC(1,1,1), 
     $     wrk1d, wrk2d, wrk3d,
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh)

c ############################
c #  Lyy, Hyy and Myy terms  #
c ############################
      CALL SGS_DYN_NORMAL2(i2, ssmflg, n0, n1, 
     $     dx, dy, dz,
     $     rho, v, sij, barsij, barvs, FRACTION(1,1,1), 
     $     SIJ_LOC(1,1,1), dummy,
     $     TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d,
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh)

      IF ( kmax_total .GT. 1 ) THEN
c ############################
c #  Lzz, Hzz and Mzz terms  #
c ############################
         CALL SGS_DYN_NORMAL2(i3, ssmflg, n0, n1, 
     $        dx, dy, dz, 
     $        rho, w, sij, barsij, barvs, FRACTION(1,1,1), 
     $        SIJ_LOC(1,1,1), dummy,
     $        TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d,
     $        cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $        carm_uf, carm_uh)
      ENDIF

c#undef DIL_LOC1
c#undef DIL_LOC2
c#undef TMP_LOC

c --------------------------------------------------------------
c This section requires txc_size=6 if ssmflg is 0
c                       txc_size=7 if ssmflg is 1
c --------------------------------------------------------------
c#define TMP_LOC(i,j,k)     txc(i,j,k,4)

c ############################
c #  Lxy, Hxy and Mxy terms  #
c ############################
      CALL SGS_DYN_CROSS2(i1, i2, ssmflg, n0, n1, 
     $     dx, dy, dz,
     $     rho, u, v, sij, barsij, barvs, FRACTION(1,1,1), 
     $     SIJ_LOC(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, 
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh, carm_uf, carm_uh)

      IF ( kmax_total .GT. 1 ) THEN
c ############################
c #  Lxz, Hxy and Mxz terms  #
c ############################
         CALL SGS_DYN_CROSS2(i1, i3, ssmflg, n0, n1, 
     $        dx, dy, dz,
     $        rho, u, w, sij, barsij, barvs, FRACTION(1,1,1), 
     $        SIJ_LOC(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, 
     $        cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $        carm_uf, carm_uh, carm_uf, carm_uh)

c ############################
c #  Lyz, Hyz and Myz terms  #
c ############################
         CALL SGS_DYN_CROSS2(i2, i3, ssmflg, n0, n1, 
     $        dx, dy, dz,
     $        rho, v, w, sij, barsij, barvs, FRACTION(1,1,1), 
     $        SIJ_LOC(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, 
     $        cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $        carm_uf, carm_uh, carm_uf, carm_uh)
      ENDIF

c     frc1 is LHSH
c     frc2 is NFSH
c     frc3 is NHSH
      IF ( isim_mode .EQ. DNS_MODE_TEMPORAL ) THEN
c         DO j = 1,jmax
c            numerator(j) = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,2), 
c     $           dx, dz, area)
c            denominator(j) = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,3),
c     $           dx, dz, area)
c            wrk1d(j,1) =  AVG_IK(imax, jmax, kmax, j, txc(1,1,1,4),
c     $           dx, dz, area)
c         ENDDO
         DO k = 1,kmax
            DO ij = 1,NN
               txc(ij,1,k,3) = txc(ij,1,k,3) - coefdelta*
     $              txc(ij,1,k,4)
            ENDDO
         ENDDO
      ENDIF

c#undef SIJ_LOC
c#undef FRACTION
c#undef TMP_LOC

      IF ( imodel_loc .EQ. 2 ) THEN

c ######################################################################
c     PART 2: terms involving S_F
c ######################################################################

c#define SIJ_LOC(i,j,k)  txc(i,j,k,1)
c#define FRACTION(i,j,k) txc(i,j,k,2)
c#define TMP_LOC(i,j,k)  txc(i,j,k,5)

      itmp = imax*jmax*kmax*3
      CALL DNS_UTIL_VSC(itmp, i1, i1, FRACTION(1,1,1), r0)

c ############################
c #  Lxx, Hxx and Mxx terms  #
c ############################
      CALL SGS_DYN_NORMAL3(i1, ssmflg, n0, n1, 
     $     dx, dy, dz,
     $     rho, u, sij, barsij, barvs, FRACTION(1,1,1), 
     $     SIJ_LOC(1,1,1), dummy, TMP_LOC(1,1,1), 
     $     wrk1d(1,2), wrk2d, wrk3d,
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh)

c ############################
c #  Lyy, Hyy and Myy terms  #
c ############################
      CALL SGS_DYN_NORMAL3(i2, ssmflg, n0, n1, 
     $     dx, dy, dz,
     $     rho, v, sij, barsij, barvs, FRACTION(1,1,1), 
     $     SIJ_LOC(1,1,1), dummy,
     $     TMP_LOC(1,1,1), wrk1d(1,2), wrk2d, wrk3d,
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh)

      IF ( kmax_total .GT. 1 ) THEN
c ############################
c #  Lzz, Hzz and Mzz terms  #
c ############################
         CALL SGS_DYN_NORMAL3(i3, ssmflg, n0, n1, 
     $        dx, dy, dz, 
     $        rho, w, sij, barsij, barvs, FRACTION(1,1,1), 
     $        SIJ_LOC(1,1,1), dummy,
     $        TMP_LOC(1,1,1), wrk1d(1,2), wrk2d, wrk3d,
     $        cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $        carm_uf, carm_uh)
      ENDIF

c ############################
c #  Lxy, Hxy and Mxy terms  #
c ############################
      CALL SGS_DYN_CROSS3(i1, i2, ssmflg, n0, n1, 
     $     dx, dy, dz,
     $     rho, u, v, sij, barsij, barvs, FRACTION(1,1,1), 
     $     SIJ_LOC(1,1,1), TMP_LOC(1,1,1), wrk1d(1,2), wrk2d, wrk3d, 
     $     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $     carm_uf, carm_uh, carm_uf, carm_uh)

      IF ( kmax_total .GT. 1 ) THEN
c ############################
c #  Lxz, Hxy and Mxz terms  #
c ############################
         CALL SGS_DYN_CROSS3(i1, i3, ssmflg, n0, n1, 
     $        dx, dy, dz,
     $        rho, u, w, sij, barsij, barvs, FRACTION(1,1,1), 
     $        SIJ_LOC(1,1,1), TMP_LOC(1,1,1), wrk1d(1,2), wrk2d, wrk3d, 
     $        cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $        carm_uf, carm_uh, carm_uf, carm_uh)

c ############################
c #  Lyz, Hyz and Myz terms  #
c ############################
         CALL SGS_DYN_CROSS3(i2, i3, ssmflg, n0, n1, 
     $        dx, dy, dz,
     $        rho, v, w, sij, barsij, barvs, FRACTION(1,1,1), 
     $        SIJ_LOC(1,1,1), TMP_LOC(1,1,1), wrk1d(1,2), wrk2d, wrk3d, 
     $        cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,
     $        carm_uf, carm_uh, carm_uf, carm_uh)
      ENDIF

c     frc1 is LHSF
c     frc2 is NFSF
c     frc3 is NHSF
      IF ( isim_mode .EQ. DNS_MODE_TEMPORAL ) THEN
         DO j = 1,jmax
            wrk1d(j,2) = AVG_IK(imax, jmax, kmax, j, txc(1,1,1,4), 
     $           dx, dz, area)
            numerator(j) =  numerator(j)*wrk1d(j,2)-
     $           wrk1d(j,1)*AVG_IK(imax, jmax, kmax, j, txc(1,1,1,2), 
     $           dx, dz, area)

            denominator(j) = denominator(j)*wrk1d(j,2)-
     $           wrk1d(j,1)*AVG_IK(imax, jmax, kmax, j, txc(1,1,1,3), 
     $           dx, dz, area)
         ENDDO
      ENDIF

#undef SIJ_LOC
#undef TMP_LOC

      ELSE
c         DO j = 1,jmax
c            denominator(j) = denominator(j)-
c     $           wrk1d(j,1)*coefdelta
c         ENDDO
      ENDIF

c ######################################################################

c     CAREFUL !!! the SGS diffusivity has to get out in txc(1)
#define DIFFSG(i,j,k) txc(i,j,k,1)

c #################################################
c # Calculate spatial average of 2*Cs             #
c # Clip it to positive values for stability      #        
c # Stored in wrk1d(1,1) or wrk2d(1,1)            #
c # txc(1) is 2*rho*nu_sg = rho*2*Cs*sgs_delta^2*S#
c #################################################
      CALL SGS_DYNCF2(ii, statflg, sgs_csdyn, sgs_nu, x,
     $     dx, dy, dz, rho, sij, FRACTION(1,1,1), DIFFSG(1,1,1), 
     $     wrk1d, wrk2d, wrk3d, numerator, denominator)

      sgs_csdyn = C_05_R*sgs_csdyn
      sgs_nu = C_05_R*sgs_nu

#ifdef _DEBUG
      IF ( ii .EQ. 1 ) THEN
#ifdef PARALLEL
         IF ( ims_pro .EQ. 0 ) THEN
#endif
            IF ( isim_mode .EQ. DNS_MODE_TEMPORAL ) THEN
               CALL IO_WRITE_ASCII_FIELD('dyn-U.dat', jmax, i1, i1, 
     $              wrk1d(1,1))
            ELSE IF ( isim_mode .EQ. DNS_MODE_SPATIAL ) THEN
               CALL IO_WRITE_ASCII_FIELD('dyn-U.dat', imax, jmax, i1, 
     $              wrk2d(1,1))
            ENDIF
#ifdef PARALLEL
         ENDIF
#endif
      ENDIF
#endif

c Running statistics
      IF ( statflg .EQ. 1 ) THEN
         CALL LES_SAVE_AVGIJ(i1, dx, dy, dz, u, v, w,
     $        DIFFSG(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
      ENDIF

#ifdef TRACE_ON
      CALL OUTPUT(trfile, 'LEAVING SGS_DYN_U' )
#endif

#undef FRACTION
#undef DIFFSG

      RETURN
      END
