SUBROUTINE SGS_ARM_CHEM_BS(ii, statflg, x, y, z, dx, dy, dz, rho, z1, &
     tmp1, tmp2, tmp3, tmp4, tmp5, tmp6, mean1d, &
     mean1d_sc, mean1d_les_sc, mean1d_les_ch, wrk1d, wrk2d, &
     wrk3d, arm_wrk, c0, cf0x, cf0y, cf0z, messageles)

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER ii, statflg

  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL rho(imax,jmax,kmax)
  TREAL z1(imax,jmax,kmax)
  TREAL tmp1(imax,jmax,kmax)
  TREAL tmp2(imax,jmax,kmax)
  TREAL tmp3(imax,jmax,kmax)
  TREAL tmp4(imax,jmax,kmax)
  TREAL tmp5(imax,jmax,kmax)
  TREAL tmp6(imax,jmax,kmax)
  TREAL mean1d(nstatavg,jmax,*)
  TREAL mean1d_sc(nstatavg,jmax,*)
  TREAL mean1d_les_sc(nstatavg,jmax,*)
  TREAL mean1d_les_ch(nstatavg,jmax,*)
  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL arm_wrk(*)
  TREAL c0(imax,jmax)
  TREAL cf0x(*), cf0y(*), cf0z(*)
  CHARACTER*(*) messageles

  TINTEGER ij, k, n0, NNstat, arrhflg
  TINTEGER pch
  TREAL aa, ab, ac0, ac1, ac2, ARRHENIUS

#ifdef PARALLEL
  INTEGER counts
#endif

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_ARM_CHEM_BS' )
#endif

  n0 = isgs_f0size
  NNstat = jmax*nstatavg
  pch = 1 + jmax*nstatavg*&
       (LA_MOMENTUM_SIZE+LS_SCALAR_SIZE*inb_scal)

  IF ( statflg .EQ. 1 ) THEN
     iarmavg_pts = iarmavg_pts + kmax_total
  ENDIF

! #########################################
! #           Model coefficient           #
! #########################################
  CALL SGS_ARM_CHEM_C0(ii, x, y, z, dx, dy, dz, &
       z1, mean1d, mean1d_sc, mean1d_les_sc, mean1d_les_ch, &
       wrk1d, wrk2d, wrk3d, arm_wrk, c0, messageles)
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_CHEMIJ(i1, dx, dy, dz, tmp1, &
          mean1d_les_ch, wrk1d, c0, wrk3d)
  ENDIF

! #########################################
! #           Intemediate field           #
! #########################################

! tmp1 is Z_m
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, z1, tmp1, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp1(ij,1,k) = z1(ij,1,k) + c0(ij,1)*&
             (z1(ij,1,k)-tmp1(ij,1,k))
        tmp1(ij,1,k) = MAX(C_0_R, MIN(tmp1(ij,1,k),C_1_R))
     ENDDO
  ENDDO

! tmp2 is bar(Z_m)      
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp1, tmp2, wrk3d)

! #####################################
! #     Polinomial nonlinearities     #
! #####################################

! P2 (Variance)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp3(ij,1,k) = tmp1(ij,1,k)*tmp1(ij,1,k)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp3, tmp4, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp3(ij,1,k) = tmp4(ij,1,k) - tmp2(ij,1,k)*tmp2(ij,1,k)
     ENDDO
  ENDDO

! P4
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp4(ij,1,k) = tmp1(ij,1,k)**C_4_R
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp4, tmp5, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp4(ij,1,k) = tmp5(ij,1,k) - tmp2(ij,1,k)**C_4_R
     ENDDO
  ENDDO

! P8
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp5(ij,1,k) = tmp1(ij,1,k)**C_8_R
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp5, tmp6, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp5(ij,1,k) = tmp6(ij,1,k) - tmp2(ij,1,k)**C_8_R
     ENDDO
  ENDDO

! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_CHEMIJ( i2, dx, dy, dz, tmp1, &
          mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  ENDIF

! Convective part of (Z2)_sg
  IF ( statflg .EQ. 1 ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           tmp3(ij,1,k) = tmp3(ij,1,k)*rho(ij,1,k)
        ENDDO
     ENDDO
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, tmp3, tmp4, i0, i0,&
          wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, tmp3, tmp5, i0, i0,&
          wrk1d, wrk2d, wrk3d)
     CALL LES_SAVE_CHEMIJ( i7, dx, dy, dz, tmp3, &
          mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  ENDIF

! Resolved contributions 
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp3(ij,1,k) = z1(ij,1,k)**C_2_R
        tmp4(ij,1,k) = z1(ij,1,k)**C_4_R
        tmp5(ij,1,k) = z1(ij,1,k)**C_8_R
     ENDDO
  ENDDO

! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_CHEMIJ( i3, dx, dy, dz, tmp1, &
          mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  ENDIF

! #######################
! #   Burke-Schumann    #
! #######################
  aa = (arm_tflame-C_1_R)/arm_zst
  ab = (arm_tflame-C_1_R)/(C_1_R-arm_zst)
  IF ( arm_smooth .NE. C_0_R ) THEN
     ac0 = C_1_R + EXP(-2*arm_zst/arm_smooth)
     ac1 = (aa-ab)*C_05_R
     ac2 = (aa+ab)*arm_smooth*C_05_R
  ENDIF

! Arrhenius nonlinearity
  IF ( arm_smooth .NE. C_0_R ) THEN
     arrhflg = 1
  ELSE
     arrhflg = 2
  ENDIF
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp3(ij,1,k) = ARRHENIUS(arrhflg, z1(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
        tmp4(ij,1,k) = ARRHENIUS(arrhflg, tmp1(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp4, tmp5, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp4(ij,1,k) = tmp5(ij,1,k) - &
             ARRHENIUS(arrhflg, tmp2(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_CHEMIJ( i4, dx, dy, dz, tmp1, &
          mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  ENDIF

! Temperature nonlinearity
  IF ( arm_smooth .NE. C_0_R ) THEN
     arrhflg = 7
  ELSE
     arrhflg = 8
  ENDIF
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp3(ij,1,k) = ARRHENIUS(arrhflg, z1(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
        tmp4(ij,1,k) = ARRHENIUS(arrhflg, tmp1(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp4, tmp5, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp4(ij,1,k) = tmp5(ij,1,k) - &
             ARRHENIUS(arrhflg, tmp2(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_CHEMIJ( i5, dx, dy, dz, tmp1, &
          mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  ENDIF

! Density nonlinearity
  IF ( arm_smooth .NE. C_0_R ) THEN
     arrhflg = 5
  ELSE
     arrhflg = 6
  ENDIF
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp3(ij,1,k) = ARRHENIUS(arrhflg, z1(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
        tmp4(ij,1,k) = ARRHENIUS(arrhflg, tmp1(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n0, cf0x, cf0y, cf0z, tmp4, tmp5, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        tmp4(ij,1,k) = tmp5(ij,1,k) - &
             ARRHENIUS(arrhflg, tmp2(ij,1,k), arm_tact, &
             arm_zst, arm_smooth, aa, ab, ac0, ac1, ac2)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_CHEMIJ( i6, dx, dy, dz, tmp1, &
          mean1d_les_ch, wrk1d, wrk2d, wrk3d )
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_ARM_CHEM_BS' )
#endif

  RETURN
END SUBROUTINE SGS_ARM_CHEM_BS
