SUBROUTINE SGS_ARM_CHEM_C0(ii, x, y, z, dx, dy, dz, &
     z1, mean1d, mean1d_sc, mean1d_les_sc, mean1d_les_ch, &
     wrk1d, wrk2d, wrk3d, arm_wrk, c0, messageles)

! ###############################################
! # Calculation of the ARM model coefficient
! #
! # 11/06/03 - extracted from SGS_ARM_CHEM
! ###############################################

#include "types.h"
#include "dns_const.h"
#include "dns_error.h"
#include "avgij_map.h"
#include "les_avgij_map.h"
#include "les_const.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif
  USE LES_GLOBAL

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TINTEGER ii

  TREAL x(imax)
  TREAL y(jmax)
  TREAL z(kmax_total)
  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)

  TREAL z1(imax,jmax,kmax)
  TREAL mean1d(nstatavg,jmax,*)
  TREAL mean1d_sc(nstatavg,jmax,*)
  TREAL mean1d_les_sc(nstatavg,jmax,*)
  TREAL mean1d_les_ch(nstatavg,jmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(isize_wrk2d,*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL arm_wrk(*)
  TREAL c0(imax,jmax)
  CHARACTER*(*) messageles

  TINTEGER ij, nj, i, j, k, n, n0, itmpmax, NNstat, pch, nxy
  TINTEGER i1
  TREAL pts, dummy1, r23, diff, inter1, inter2

#ifdef PARALLEL
  INTEGER counts
#endif

#ifdef SINGLE_PREC
  inter1 = 0.02e0
#else
  inter1 = 0.02d0
#endif
  inter2 = C_1_R-inter1

  i1 = 1
  r23 = C_2_R/C_3_R
  nxy = imax*jmax

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_ARM_CHEM_C0' )
#endif

  n0 = isgs_f0size
  NNstat = jmax*nstatavg
  pch = 1 + jmax*nstatavg*&
       (LA_MOMENTUM_SIZE+LS_SCALAR_SIZE*inb_scal)
  diff = visc/schmidt(inb_scal)

! #################################
! # Infinite Reynolds number case #
! #################################
  IF ( iles_inviscid .EQ. 1 ) THEN

! No statistical data
     IF ( nstatavg_points .EQ. 0 ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              IF ( inter1 .LE. z1(ij,1,k) .AND. &
                   z1(ij,1,k) .LE. inter2 ) THEN
                 wrk3d(ij,1,k) = C_1_R
              ELSE
                 wrk3d(ij,1,k) = C_0_R
              ENDIF
           ENDDO
        ENDDO
        CALL SUM_K_V(nxy, kmax, wrk3d, wrk2d(1,1), wrk2d(1,2))
        dummy1 = (arm_c0_inviscid-C_1_R)/M_REAL(kmax_total)
        DO ij = 1,imax*jmax
           c0(ij,1) = C_1_R+wrk2d(ij,1)*dummy1
        ENDDO

! Using statistical data
     ELSE
#ifdef PARALLEL
        IF ( ims_pro .EQ. 0 ) THEN
#endif
           dummy1 = (arm_c0_inviscid-C_1_R)/M_REAL(nstatavg_points)
           DO ij = 1,NNstat
! Intermittency x coefficient
              wrk2d(ij,7) = C_1_R+MS_GAMMA(ij)*dummy1
           ENDDO
! Interpolating
           itmpmax = imax*jmax*kmax
           DO n = 1,nstatavg
              wrk1d(n,1) = x(statavg(n))
           ENDDO
           dummy1 = wrk1d(nstatavg,1) - wrk1d(1,1)
           CALL INTERPOLATE_1D( nstatavg, jmax, imax, i1bc, dummy1, wrk1d(1,1), &
                x, wrk2d(1,7), c0, itmpmax, wrk3d )
! Possible negative values due to the interpolation 
           DO ij = 1,imax*jmax
              c0(ij,1) = MAX(C_0_R,c0(ij,1))
           ENDDO
#ifdef PARALLEL
        ENDIF
        counts = imax*jmax
        CALL MPI_BCAST(c0, counts, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
#endif
     ENDIF

! ###############################
! # Finite Reynolds number case #
! ###############################
  ELSE

! No statistical data, initializing to 1
     IF ( nstatavg_points .EQ. 0 ) THEN
        CALL IO_WRITE_ASCII(wfile,'SGS_ARM_Z: Zero number of points')
        DO ij = 1,imax*jmax
           c0(ij,1) = C_1_R
        ENDDO

! Using statistical data
     ELSE
#ifdef PARALLEL
        IF ( ims_pro .EQ. 0 ) THEN
#endif
           pts = C_1_R/M_REAL(nstatavg_points)         
           DO ij = 1,NNstat
! resolved TKE
              dummy1 = C_1_R/MA_R(ij)
              wrk2d(ij,1) = C_05_R*( MA_RUU(ij) -&
                   dummy1*MA_RU(ij)**2 + &
                   MA_RVV(ij) -&
                   dummy1*MA_RV(ij)**2 +&
                   MA_RWW(ij) -&
                   dummy1*MA_RW(ij)**2 )*dummy1
! resolved Z variance
              wrk2d(ij,2) = ( MS_RSS(ij) -&
                   dummy1*MS_RS(ij)**2)*dummy1
! SMG subgrid scalar dissipation
              wrk2d(ij,3) = C_2_R*ABS(LS_SMG_EPS_Z(ij))*pts
! Viscosity
              wrk2d(ij,4) = visc*MA_VIS(ij)*dummy1
           ENDDO
! SSM subgrid scalar dissipation
           IF ( iles_type_tran .NE. LES_TRAN_NONE ) THEN
              DO ij = 1,NNStat
                 wrk2d(ij,3) = wrk2d(ij,3) + &
                      C_2_R*ABS(LS_SSM_EPS_Z(ij))*pts
              ENDDO
           ENDIF

           CALL LES_DELTAF3D(imode_sim, i1bc, j1bc, k1bc, imax, jmax,&
                kmax_total, isize_wrk1d, n0, n0, n0, scalex, scaley, scalez, &
                x, y, z, wrk2d(1,5), wrk1d)
           DO j = 1,jmax
              DO n = 1,nstatavg
                 i = statavg(n)
                 ij = (j-1)*imax + i
                 nj = (j-1)*nstatavg + n
                 wrk2d(nj,5) = wrk2d(ij,5)
              ENDDO
           ENDDO

! Accounting for the subgrid parts
!#ifdef SINGLE_PREC
!     aa = 1.363e0
!#else
!     aa = 1.363d0
!#endif
!     DO j = 1,jmax
!        DO n = 1,nstatavg
!           ij = n + (j-1)*nstatavg
!           ijmd = n + (jmax/2-1)*nstatavg
!           dummy1 = SQRT(wrk2d(ijmd,1))*wrk2d(ijmd,2)/wrk2d(ijmd,3)
!           dummy1 = C_1_R - aa*MIN(wrk2d(ij,5)/dummy1,C_1_R)**r23
!           dummy1 = MIN(C_1_R,MAX(C_1EM3_R,dummy1))
!           wrk2d(ij,1) = wrk2d(ij,1)/dummy1
!        ENDDO
!     ENDDO

! Model coefficient
           IF ( MOD(itime,nitera_log) .EQ. 0 .AND. ii .EQ. 1 ) THEN
              n = 1
           ELSE
              n = 0
           ENDIF
           CALL SGS_ARM_SPTL_C0(n, itime, iarm_nl, iarm_nre, &
                wrk2d(1,4), &
                nstatavg, jmax, wrk2d(1,5), wrk2d(1,1), wrk2d(1,2), &
                wrk2d(1,3), arm_wrk, wrk2d(1,7), messageles)

! Interpolating
           itmpmax = imax*jmax*kmax
           DO n = 1,nstatavg
              wrk1d(n,1) = x(statavg(n))
           ENDDO
           dummy1 = wrk1d(nstatavg,1) - wrk1d(1,1)
           CALL INTERPOLATE_1D( nstatavg, jmax, imax, i1bc, dummy1, wrk1d(1,1), &
                x, wrk2d(1,7), c0, itmpmax, wrk3d )
! Possible negative values due to the interpolation
           DO ij = 1,imax*jmax
              c0(ij,1) = MAX(C_0_R,c0(ij,1))
           ENDDO
#ifdef PARALLEL
        ENDIF
        counts = imax*jmax
        CALL MPI_BCAST(c0, counts, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
#endif

     ENDIF
  ENDIF

#ifdef _DEBUG
  CALL IO_WRITE_ASCII_FIELD('c0.dat', imax, jmax, i1, c0)
#endif

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_ARM_CHEM_C0' )
#endif

  RETURN
END SUBROUTINE SGS_ARM_CHEM_C0
