!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/30 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Computes twice the dynamic coefficient into array sgscu
!#
!########################################################################
!# ARGUMENTS 
!#
!# sij     = rho * delta_F^2 * SQRT(2SijSij)
!# barsij  = rho * delta_f^2 * SQRT(2*bar(Sij)*bar(Sij))
!#
!# txc1 is bar1(bar0(u_sg) |
!# txc2 is bar1(bar0(v_sg)  > use in dilatation
!# txc3 is bar1(bar0(w_sg) |
!#
!# barvs(1) = 1/bar1(bar0(rho))          <--- Always.
!# barvs(2) = 1/bar0(bar0(rho))             | Only
!# barvs(3) = 1/bar1(bar0(bar1(bar0(rho)))) | for Hij.
!#
!# txc    In  size is 7, if ssmflg is 0
!#            size is 8, if ssmflg is 1
!#
!########################################################################
SUBROUTINE SGS_DYN_U(ssmflg, coefdelta, n0, n1, x,y,z,dx,dy,dz, rho,u,v,w, u_sg,v_sg,w_sg,&
     sij, barsij, barvs, txc, sgscu, wrk1d, wrk2d, wrk3d, &
     cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh)

#include "types.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER ssmflg, n1, n0
  TREAL coefdelta

  TREAL, DIMENSION(*)                :: x, y, z, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: u, v, w, rho, u_sg, v_sg, w_sg
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL sij(imax,jmax,kmax)
  TREAL barsij(imax,jmax,kmax)
  TREAL barvs(imax,jmax,kmax,*)

  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z
  TREAL, DIMENSION(*) :: carm_uf, carm_uh, sgscu

  TREAL, DIMENSION(*) :: wrk1d, wrk2d, wrk3d

! -----------------------------------------------------------------------
  TINTEGER ij, k
  TREAL r23, r13, dummy

  TREAL, DIMENSION(:,:,:,:), POINTER : FRACTION

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_DYN_U')
#endif

  r23 = C_2_R/C_3_R
  r13 = C_1_R/C_3_R

! --------------------------------------------------------------
! This section requires txc_size=7 if ssmflg is 0
!                   txc_size=8 is ssmflg is 1
! --------------------------------------------------------------
#define DIL_LOC1(i,j,k)   txc(i,j,k,4)
#define DIL_LOC2(i,j,k)   txc(i,j,k,5)
#define TMP_LOC(i,j,k)    txc(i,j,k,6)
  IF ( sgs_devsmag .EQ. 1 ) THEN

     CALL FI_INVARIANT_P(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
          dx, dy, dz, u_sg, v_sg, w_sg, &
          DIL_LOC1(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d)
     CALL FI_INVARIANT_P(imode_fdm, imax, jmax, kmax, i1bc, j1bc, k1bc, &
          dx, dy, dz, txc(1,1,1,1), txc(1,1,1,2), txc(1,1,1,3), &
          DIL_LOC2(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d)

! calculating coef*dilatation from the first invariant
     IF ( kmax_total .GT. 1 ) THEN
        dummy = C_1_R/C_3_R
     ELSE
        dummy = C_1_R/C_2_R
     ENDIF 
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           DIL_LOC1(ij,1,k) =-dummy*DIL_LOC1(ij,1,k)
           DIL_LOC2(ij,1,k) =-dummy*DIL_LOC2(ij,1,k)
        ENDDO
     ENDDO
 
  ENDIF

#define MIJ(i,j,k)   txc(i,j,k,1)

  FRACTION => txc(:,:,:,2:)

  FRACTION(:,:,:,1:2) = C_0_R

! ############################
! #  Lxx, Hxx and Mxx terms  #
! ############################
  CALL SGS_DYN_NORMAL(i1, ssmflg, n0, n1, coefdelta, dx, dy, dz, &
       rho, u, u_sg, sij, barsij, barvs, FRACTION(1,1,1), &
       MIJ(1,1,1), DIL_LOC1(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh)
!  CALL SGS_DYN_NORMAL(i1, ssmflg, n0, n1, coefdelta, &
!       dx, dy, dz,&
!       rho, u_sg, u_sg, sij, barsij, barvs, FRACTION(1,1,1), &
!       MIJ(1,1,1), DIL_LOC1(1,1,1), TMP_LOC(1,1,1), &
!       wrk1d, wrk2d, wrk3d,&
!       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!       carm_uf, carm_uh)

! ############################
! #  Lyy, Hyy and Myy terms  #
! ############################
  CALL SGS_DYN_NORMAL(i2, ssmflg, n0, n1, coefdelta, dx, dy, dz, &
       rho, v, v_sg, sij, barsij, barvs, FRACTION(1,1,1), &
       MIJ(1,1,1), DIL_LOC1(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh)
!  CALL SGS_DYN_NORMAL(i2, ssmflg, n0, n1, coefdelta, &
!       dx, dy, dz,&
!       rho, v_sg, v_sg, sij, barsij, barvs, FRACTION(1,1,1), &
!       MIJ(1,1,1), DIL_LOC1(1,1,1),&
!       TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d,&
!       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!       carm_uf, carm_uh)

  IF ( kmax_total .GT. 1 ) THEN
! ############################
! #  Lzz, Hzz and Mzz terms  #
! ############################
     CALL SGS_DYN_NORMAL(i3, ssmflg, n0, n1, coefdelta, dx, dy, dz, &
          rho, w, w_sg, sij, barsij, barvs, FRACTION(1,1,1), &
          MIJ(1,1,1), DIL_LOC1(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh)
!     CALL SGS_DYN_NORMAL(i3, ssmflg, n0, n1, coefdelta, &
!          dx, dy, dz, &
!          rho, w_sg, w_sg, sij, barsij, barvs, FRACTION(1,1,1), &
!          MIJ(1,1,1), DIL_LOC1(1,1,1),&
!          TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d,&
!          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!          carm_uf, carm_uh)
  ENDIF

#undef DIL_LOC1
#undef DIL_LOC2
#undef TMP_LOC

! --------------------------------------------------------------
! This section requires txc_size=6 if ssmflg is 0
!                   txc_size=7 if ssmflg is 1
! --------------------------------------------------------------
#define TMP_LOC(i,j,k)     txc(i,j,k,4)

! ############################
! #  Lxy, Hxy and Mxy terms  #
! ############################
  CALL SGS_DYN_CROSS(i1, i2, ssmflg, n0, n1, coefdelta, dx, dy, dz, rho, u, v, u_sg, v_sg, &
       sij, barsij, barvs, FRACTION(1,1,1), MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_uf, carm_uh)
!  CALL SGS_DYN_CROSS(i1, i2, ssmflg, n0, n1, coefdelta, &
!       dx, dy, dz, rho, u_sg, v_sg, u_sg, v_sg, &
!       sij, barsij, barvs, FRACTION(1,1,1), &
!       MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
!       cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!       carm_uf, carm_uh, carm_uf, carm_uh)

  IF ( kmax_total .GT. 1 ) THEN
! ############################
! #  Lxz, Hxy and Mxz terms  #
! ############################
     CALL SGS_DYN_CROSS(i1, i3, ssmflg, n0, n1, coefdelta, dx, dy, dz, rho, u, w, u_sg, w_sg, &
          sij, barsij, barvs, FRACTION(1,1,1), MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf,carm_uh,carm_uf,carm_uh)
!     CALL SGS_DYN_CROSS(i1, i3, ssmflg, n0, n1, coefdelta, &
!          dx, dy, dz, rho, u_sg, w_sg, u_sg, w_sg, &
!          sij, barsij, barvs, FRACTION(1,1,1), &
!          MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
!          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!          carm_uf, carm_uh, carm_uf, carm_uh)

! ############################
! #  Lyz, Hyz and Myz terms  #
! ############################
     CALL SGS_DYN_CROSS(i2, i3, ssmflg, n0, n1, coefdelta, dx, dy, dz, rho, v, w, v_sg, w_sg, &
          sij, barsij, barvs, FRACTION(1,1,1), MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z, carm_uf,carm_uh,carm_uf,carm_uh)
!     CALL SGS_DYN_CROSS(i2, i3, ssmflg, n0, n1, coefdelta, &
!          dx, dy, dz, rho, v_sg, w_sg, v_sg, w_sg,&
!          sij, barsij, barvs, FRACTION(1,1,1), &
!          MIJ(1,1,1), TMP_LOC(1,1,1), wrk1d, wrk2d, wrk3d, &
!          cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, cf2x, cf2y, cf2z,&
!          carm_uf, carm_uh, carm_uf, carm_uh)
  ENDIF

#undef TMP_LOC
#undef MIJ

! #######################################################################
! Calculate spatial average of 2*Cs; Clip it to positive values for stability
! #######################################################################
  CALL SGS_DYNCF(x, dx, dy, dz, FRACTION(1,1,1), sgscu, wrk1d, wrk2d, wrk3d)

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_DYN_U' )
#endif

  NULLIFY(FRACTION)

  RETURN
END SUBROUTINE SGS_DYN_U
