SUBROUTINE SGS_DYN_CROSS(idir1, idir2, ssmflg, n0, n1, coefdelta, &
     dx, dy, dz, rho, u, v, u_sg, v_sg, &
     sij, barsij, barvs, frc, mij, txc, wrk1d, &
     wrk2d, wrk3d, cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, &
     cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_vf, carm_vh)

! ###############################################################
! # frc1 is the numerator
! # frc2 is the denominator
! #
! # txc - required size of 3 if ssmflg is 0
! #       required size of 4 if ssmflg is 1
! #
! # mij stores in reality 2mij for the case of the velocity because
! #   the off-diagonal terms appear twice
! #
! # 07/29/02 - Introducing barvs and eliminating FILT_FAV to
! #            save time
! # 09/25/04 - Profiling and preparing for ARM: 2 more arrays.
! ###############################################################

#include "types.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER idir1, idir2, ssmflg, n0, n1
  TREAL coefdelta

  TREAL dx(imax)
  TREAL dy(jmax)
  TREAL dz(kmax_total)
  TREAL rho(imax,jmax,kmax)
  TREAL u(imax,jmax,kmax)
  TREAL v(imax,jmax,kmax)
  TREAL u_sg(imax,jmax,kmax)
  TREAL v_sg(imax,jmax,kmax)

  TREAL sij(imax,jmax,kmax)
  TREAL barsij(imax,jmax,kmax)
  TREAL barvs(imax,jmax,kmax,*)
  TREAL frc(imax,jmax,kmax,2)
  TREAL mij(imax,jmax,kmax)
  TREAL txc(imax,jmax,kmax,*)

  TREAL wrk1d(isize_wrk1d,*)
  TREAL wrk2d(*)
  TREAL wrk3d(imax,jmax,kmax)
  TREAL cf0x(*), cf0y(*), cf0z(*)
  TREAL cf1x(*), cf1y(*), cf1z(*)
  TREAL cf2x(*), cf2y(*), cf2z(*)
  TREAL carm_uf(*), carm_uh(*)
  TREAL carm_vf(*), carm_vh(*)

  TINTEGER ij, k

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_DYN_CROSS')
#endif

#define NUMERATOR(i,j,k)   frc(i,j,k,1)
#define DENOMINATOR(i,j,k) frc(i,j,k,2)

! #################
! # Calculate Mij #
! #################
! -------------------------------------------------------------
!                                                   level 1
! -------------------------------------------------------------
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        txc(ij,1,k,1) = rho(ij,1,k)*v_sg(ij,1,k)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        txc(ij,1,k,2) = txc(ij,1,k,2)*barvs(ij,1,k,1)
     ENDDO
  ENDDO
  IF ( idir1 .EQ. 1 ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, txc(1,1,1,2), txc(1,1,1,1), i0, i0,&
          wrk1d, wrk2d, wrk3d)
  ELSE IF ( idir1 .EQ. 2 ) THEN
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, txc(1,1,1,2), txc(1,1,1,1), i0, i0,&
          wrk1d, wrk2d, wrk3d)
  ELSE
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, txc(1,1,1,2), txc(1,1,1,1), i0, i0,&
          wrk1d, wrk2d, wrk3d)
  ENDIF

  IF ( idir2 .NE. 0 ) THEN
! Velocity case
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           txc(ij,1,k,2) = rho(ij,1,k)*u_sg(ij,1,k)
        ENDDO
     ENDDO
     CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
          iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
          n1, cf1x, cf1y, cf1z, txc(1,1,1,2), txc(1,1,1,3), wrk3d)
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           txc(ij,1,k,3) = txc(ij,1,k,3)*barvs(ij,1,k,1)
        ENDDO
     ENDDO
     IF ( idir2 .EQ. 1 ) THEN
        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
             dx, txc(1,1,1,3), txc(1,1,1,2), i0, i0,&
             wrk1d, wrk2d, wrk3d)
     ELSE IF (idir2 .EQ. 2 ) THEN
        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
             dy, txc(1,1,1,3), txc(1,1,1,2), i0, i0,&
             wrk1d, wrk2d, wrk3d)
     ELSE
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
             dz, txc(1,1,1,3), txc(1,1,1,2), i0, i0,&
             wrk1d, wrk2d, wrk3d)
     ENDIF

     DO k=1,kmax
        DO ij=1,imax*jmax
           mij(ij,1,k) = C_05_R*(txc(ij,1,k,1)+txc(ij,1,k,2))*&
                barsij(ij,1,k)*coefdelta
        ENDDO
     ENDDO
  ELSE
! Scalar case
     DO k=1,kmax
        DO ij=1,imax*jmax
           mij(ij,1,k) = txc(ij,1,k,1)*&
                barsij(ij,1,k)*coefdelta
        ENDDO
     ENDDO

  ENDIF

! -------------------------------------------------------------
!                                                   level 0
! -------------------------------------------------------------
  IF ( idir1 .EQ. 1 ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, v_sg, txc(1,1,1,1), i0, i0,&
          wrk1d, wrk2d, wrk3d)
  ELSE IF ( idir1 .EQ. 2 ) THEN
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, v_sg, txc(1,1,1,1), i0, i0,&
          wrk1d, wrk2d, wrk3d)
  ELSE
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, v_sg, txc(1,1,1,1), i0, i0,&
          wrk1d, wrk2d, wrk3d)
  ENDIF
  IF ( idir2 .NE. 0 ) THEN
! velocity case
     IF ( idir2 .EQ. 1 ) THEN
        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
             dx, u_sg, txc(1,1,1,2), i0, i0,&
             wrk1d, wrk2d, wrk3d)
     ELSE IF ( idir2 .EQ. 2 ) THEN
        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
             dy, u_sg, txc(1,1,1,2), i0, i0,&
             wrk1d, wrk2d, wrk3d)
     ELSE
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
             dz, u_sg, txc(1,1,1,2), i0, i0,&
             wrk1d, wrk2d, wrk3d)
     ENDIF
     DO k=1,kmax
        DO ij=1,imax*jmax
           txc(ij,1,k,1) = C_05_R*(txc(ij,1,k,1)+txc(ij,1,k,2))*&
                sij(ij,1,k)
        ENDDO
     ENDDO
  ELSE
! Scalar case
     DO k=1,kmax
        DO ij=1,imax*jmax
           txc(ij,1,k,1) = txc(ij,1,k,1)*sij(ij,1,k)
        ENDDO
     ENDDO
  ENDIF

  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)
  IF ( idir2 .NE. 0 ) THEN
! Velocity case
     DO k=1,kmax
        DO ij=1,imax*jmax
! mij stores in reality 2mij for the case of the velocity becasue
! the off-diagonal terms appear twice
           mij(ij,1,k) = C_2_R*(txc(ij,1,k,2) - mij(ij,1,k))
           DENOMINATOR(ij,1,k) = DENOMINATOR(ij,1,k) + &
                C_05_R*mij(ij,1,k)*mij(ij,1,k)
        ENDDO
     ENDDO
  ELSE
! Scalar case
     DO k=1,kmax
        DO ij=1,imax*jmax
           mij(ij,1,k) = txc(ij,1,k,2) - mij(ij,1,k)
           DENOMINATOR(ij,1,k) = DENOMINATOR(ij,1,k) + &
                mij(ij,1,k)*mij(ij,1,k)
        ENDDO
     ENDDO
  ENDIF

! ####################      
! # Calculate LijMij #
! ####################
! -------------------------------------------------------------
!                                                   level 0
! -------------------------------------------------------------
  DO k=1,kmax
     DO ij=1,imax*jmax
        txc(ij,1,k,1) = u(ij,1,k)*v(ij,1,k)*rho(ij,1,k)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n1, cf1x, cf1y, cf1z, txc(1,1,1,1), txc(1,1,1,2), wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        NUMERATOR(ij,1,k) = NUMERATOR(ij,1,k) + &
             txc(ij,1,k,2)*mij(ij,1,k)
     ENDDO
  ENDDO

! -------------------------------------------------------------
!                                                   level 1
! -------------------------------------------------------------
  DO k=1,kmax
     DO ij=1,imax*jmax
        txc(ij,1,k,2) = rho(ij,1,k)*u(ij,1,k)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n1, cf1x, cf1y, cf1z, txc(1,1,1,2), txc(1,1,1,1), wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        txc(ij,1,k,3) = rho(ij,1,k)*v(ij,1,k)
     ENDDO
  ENDDO
  CALL FILTH(i3, imax, jmax, kmax, kmax_total, &
       iunifx, iunify, iunifz, i1bc, j1bc, k1bc,&
       n1, cf1x, cf1y, cf1z, txc(1,1,1,3), txc(1,1,1,2), wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        NUMERATOR(ij,1,k) = NUMERATOR(ij,1,k) - &
             txc(ij,1,k,1)*txc(ij,1,k,2)*mij(ij,1,k)*&
             barvs(ij,1,k,1)
     ENDDO
  ENDDO

#undef NUMERATOR
#undef DENOMINATOR

! ####################
! # Calculate HijMij #
! ####################
  IF ( ssmflg .EQ. 1 ) THEN
     CALL SGS_DYN_HIJ(i0, n0, n1, dx, dy, dz,&
          rho, u, v, barvs, frc, mij, txc, &
          wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z, cf1x, cf1y, cf1z, &
          cf2x, cf2y, cf2z, carm_uf, carm_uh, carm_vf, carm_vh)
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_DYN_CROSS')
#endif

  RETURN
END SUBROUTINE SGS_DYN_CROSS
