!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/01/01 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate spatial average of dynamic coefficient
!# Clip it to positive values for stability  
!#    
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE SGS_DYNCF(x, dx, dy, dz, frc, coefficient, wrk1d, wrk2d, wrk3d)
  
#include "types.h"
#include "dns_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL, DIMENSION(*)                :: x, dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax,*) :: frc

  TREAL :: wrk1d(isize_wrk1d,*), wrk2d(isize_wrk2d,*), wrk3d(*), coefficient(*)

! -----------------------------------------------------------------------
  TINTEGER i, j, ij
  TREAL AVG_IK, dummy

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_DYNCF')
#endif

! -----------------------------------------------------------------------
! Temporal
! -----------------------------------------------------------------------
  IF ( imode_sim .EQ. DNS_MODE_TEMPORAL ) THEN
     DO j = 1,jmax
        wrk1d(j,1) = AVG_IK(imax, jmax, kmax, j, frc(1,1,1,1), dx, dz, area) ! numerator
        wrk1d(j,2) = AVG_IK(imax, jmax, kmax, j, frc(1,1,1,2), dx, dz, area) ! denominator
     ENDDO

     dummy = MAXVAL(wrk1d(1:jmax,2))
     IF ( ABS(dummy) .LT. C_SMALL_R ) THEN
        DO j = 1,jmax
           coefficient(j) = C_0_R
        ENDDO
     ELSE
        dummy = dummy*C_1EM6_R
        DO j = 1,jmax
           coefficient(j) = MAX(C_0_R,wrk1d(j,1))/MAX(wrk1d(j,2),dummy)
        ENDDO
     ENDIF

! diffusion is rho*nu_s = rho*coefficient*sgs_delta^2*S
!     DO k = 1,kmax
!        DO j = 1,jmax
!           DO i = 1,imax
!              diffusion(i,j,k) = coefficient(j)*sij(i,j,k)
!           ENDDO
!        ENDDO
!     ENDDO

! -----------------------------------------------------------------------
! Spatial
! -----------------------------------------------------------------------
  ELSE IF ( imode_sim .EQ. DNS_MODE_SPATIAL ) THEN
     CALL SUM_K_V(imax*jmax, kmax, frc(1,1,1,1), wrk2d(1,1), wrk2d(1,3)) ! numerator
     CALL SUM_K_V(imax*jmax, kmax, frc(1,1,1,2), wrk2d(1,2), wrk2d(1,3)) ! denominator

     dummy = MAXVAL(wrk2d(1:imax*jmax,2))
     IF ( ABS(dummy) .LE. C_SMALL_R ) THEN
        DO ij = 1,imax*jmax
           coefficient(ij) = C_0_R
        ENDDO
     ELSE
        dummy = dummy*C_1EM6_R
        DO ij = 1,imax*jmax
           coefficient(ij) = MAX(C_0_R,wrk2d(ij,1))/MAX(wrk2d(ij,2),dummy)
        ENDDO
     ENDIF

! Deactivation in the inflow zone for spatial mode
     IF ( sgs_smagtrans .GT. x(1) ) THEN
        IF ( sgs_smagdelta .EQ. C_0_R ) THEN
           DO i = 1,imax
              IF ( x(i) .LT. sgs_smagtrans ) THEN
                 wrk1d(i,1) = C_0_R
              ELSE
                 wrk1d(i,1) = C_1_R
              ENDIF
           ENDDO
        ELSE
           DO i = 1,imax
              wrk1d(i,1) = C_05_R*(C_1_R + TANH(C_05_R*(x(i)-sgs_smagtrans)/sgs_smagdelta) )
           ENDDO
        ENDIF
        DO j = 1,jmax
           DO i = 1,imax
              ij = (j-1)*imax + i
              coefficient(ij) = coefficient(ij)*wrk1d(i,1)
           ENDDO
        ENDDO
     ENDIF

! diffusion is rho*nu_s = rho*coefficient*sgs_delta^2*S 
!     DO k = 1,kmax
!        DO ij = 1,imax*jmax
!           diffusion(ij,1,k) = coefficient(ij)*sij(ij,1,k)
!        ENDDO
!     ENDDO

  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_DYNCF' )
#endif

  RETURN
END SUBROUTINE SGS_DYNCF
