!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/30 - J.P. Mellado
!#              Created
!# 2008/11/16 - J.P. Mellado
!#              New code version based on energy
!#
!########################################################################
!# DESCRIPTION
!#
!########################################################################
!# ARGUMENTS 
!#
!# barvs is the specific volume 1/bar0(bar0(rho))
!#
!# Written in terms of momentum per unit volume, m_i=rho*u_i:
!# barmu_m is bar0((rho*u)_model)
!# barmv_m is bar0((rho*v)_model)
!# barmw_m is bar0((rho*w)_model)
!# 
!# txc size is 5 if ARM
!#
!########################################################################
SUBROUTINE SGS_RECTN_U(n0, statflg, dx, dy, dz, rho, u, v, w, &
     mu_m, mv_m, mw_m, barvs, barmu_m, barmv_m, barmw_m, h1, h2, h3, h4, mean1d_les, &
     txc, wrk1d, wrk2d, wrk3d, cf0x, cf0y, cf0z)

#include "types.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER n0, statflg

  TREAL, DIMENSION(*)                :: dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: u, v, w, rho, h1, h2, h3, h4
  TREAL, DIMENSION(imax,jmax,kmax)   :: mu_m, mv_m, mw_m, barmu_m, barmv_m, barmw_m, barvs
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL mean1d_les(nstatavg,jmax,*)

  TREAL, DIMENSION(*) :: wrk1d, wrk2d, wrk3d
  TREAL, DIMENSION(*) :: cf0x, cf0y, cf0z

! -----------------------------------------------------------------------
  TINTEGER ij, k, sgs_dev_ssm, idissflg
  TREAL r13

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_RECTN_U' )
#endif

  r13 = C_1_R/C_3_R
! In compressible cases maybe some isotropic component is desired
  sgs_dev_ssm = 0

  IF ( iles_type_diss .EQ. LES_DISS_DISSSG .OR.&
       statflg .EQ. 1 ) THEN
     idissflg = 1
  ELSE
     idissflg = 0
  ENDIF

#define TRACE(i,j,k)    txc(i,j,k,1)
#define TMP1_LOC(i,j,k) txc(i,j,k,2)
#define TMP2_LOC(i,j,k) txc(i,j,k,3)

! #######################################################################
! Isotropic part of tauij
! #######################################################################
  IF ( sgs_dev_ssm .EQ. 1 ) THEN
     CALL SGS_RECTN_TAUIJ(n0, rho, mu_m, mu_m, &
          barvs, barmu_m, barmu_m, TRACE(1,1,1), &
          TMP2_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     CALL SGS_RECTN_TAUIJ(n0, rho, mv_m, mv_m, &
          barvs, barmv_m, barmv_m, TMP1_LOC(1,1,1), &
          TMP2_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     IF ( kmax_total .GT. 1 ) THEN
        DO k=1, kmax
           DO ij=1,imax*jmax
              TRACE(ij,1,k) = TRACE(ij,1,k) + TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
        CALL SGS_RECTN_TAUIJ(n0, rho, mw_m, mw_m, &
             barvs,barmw_m,barmw_m, TMP1_LOC(1,1,1), &
             TMP2_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
        DO k=1, kmax
           DO ij=1,imax*jmax
              TRACE(ij,1,k) = r13*(TRACE(ij,1,k) + TMP1_LOC(ij,1,k))
           ENDDO
        ENDDO
     ELSE
        DO k=1, kmax
           DO ij=1,imax*jmax
              TRACE(ij,1,k) = C_05_R*(TRACE(ij,1,k) + TMP1_LOC(ij,1,k))
           ENDDO
        ENDDO
     ENDIF
  ENDIF

#undef TMP1_LOC
#undef TMP2_LOC

#define TAU_M(i,j,k)    txc(i,j,k,2)
#define TMP1_LOC(i,j,k) txc(i,j,k,3)
#define TMP2_LOC(i,j,k) txc(i,j,k,4)
#define PHI(i,j,k)      txc(i,j,k,5)

! #######################################################################
! Momentum equation : diagonal terms
! #######################################################################
! -----------------------------------------------------------------    
!                                        tauxx = rho*(Um*Um)_sg
! -----------------------------------------------------------------    
  CALL SGS_RECTN_TAUIJ(n0, rho, mu_m, mu_m, barvs, barmu_m, barmu_m, &
       TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
  IF ( sgs_dev_ssm .EQ. 1 ) THEN
     DO k=1, kmax
        DO ij=1,imax*jmax
           TAU_M(ij,1,k) = TAU_M(ij,1,k) - TRACE(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
! Dissipation, -tau_xx*du/dx
  IF ( idissflg .EQ. 1 ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, u, &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              PHI(ij,1,k) = -TAU_M(ij,1,k)*TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_AVGIJ(i8, dx, dy, dz, u, v, w, &
             TAU_M(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
     ENDIF
  ENDIF
! Calculate -d/dx(tauxx)               (X-momentum)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
       TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k=1, kmax
     DO ij=1,imax*jmax
        h1(ij,1,k) = h1(ij,1,k) - sgs_alpha*TMP1_LOC(ij,1,k)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_AVGIJ(i21, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
          mean1d_les, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL LES_SAVE_AVGIJ(i30, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1),&
          mean1d_les, wrk1d, wrk2d, wrk3d)
  ENDIF

! -----------------------------------------------------------------    
!                                         tauyy = rho(Vm*Vm)_sg
! -----------------------------------------------------------------    
  CALL SGS_RECTN_TAUIJ(n0, rho, mv_m, mv_m, barvs, barmv_m, barmv_m, &
       TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
  IF ( sgs_dev_ssm .EQ. 1 ) THEN
     DO k=1, kmax
        DO ij=1,imax*jmax
           TAU_M(ij,1,k) = TAU_M(ij,1,k) - TRACE(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
! Dissipation, -tau_yy*dv/dy
  IF ( idissflg .EQ. 1 ) THEN
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, v, &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              PHI(ij,1,k) = PHI(ij,1,k) - TAU_M(ij,1,k)*TMP1_LOC(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_AVGIJ(i9, dx, dy, dz, u, v, w, &
             TAU_M(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
     ENDIF
  ENDIF
! Calculate -d/dy(tauyy)               (Y-momentum)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
       TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        h2(ij,1,k) = h2(ij,1,k) -  sgs_alpha*TMP1_LOC(ij,1,k)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_AVGIJ(i22, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
          mean1d_les, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL LES_SAVE_AVGIJ(i31, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
          mean1d_les, wrk1d, wrk2d, wrk3d)
  ENDIF

  IF ( kmax_total .GT. 1 ) THEN
! -----------------------------------------------------------------    
!                                        tauzz = rho*(Wm*Wm)_sg
! -----------------------------------------------------------------    
     CALL SGS_RECTN_TAUIJ(n0, rho, mw_m, mw_m, barvs, barmw_m, barmw_m, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
     IF ( sgs_dev_ssm .EQ. 1 ) THEN
        DO k=1, kmax
           DO ij=1,imax*jmax
              TAU_M(ij,1,k) = TAU_M(ij,1,k) - TRACE(ij,1,k)
           ENDDO
        ENDDO
     ENDIF
! Dissipation, tau_zz*dw/dz
     IF ( idissflg .EQ. 1 ) THEN
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, w, &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
           DO k = 1,kmax
              DO ij = 1,imax*jmax
                 PHI(ij,1,k) = PHI(ij,1,k) - TAU_M(ij,1,k)*TMP1_LOC(ij,1,k)
              ENDDO
           ENDDO
        ENDIF
        IF ( statflg .EQ. 1 ) THEN
           CALL LES_SAVE_AVGIJ(i10, dx, dy, dz, u, v, w, &
                TAU_M(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
        ENDIF
     ENDIF
! Calculate -d/dz(tauzz)               (Z-momentum)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1, kmax
        DO ij=1,imax*jmax
           h3(ij,1,k) = h3(ij,1,k) -  sgs_alpha*TMP1_LOC(ij,1,k)
        ENDDO
     ENDDO
! Statistics
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_AVGIJ(i23, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
             mean1d_les, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL LES_SAVE_AVGIJ(i32, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
             mean1d_les, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
             TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL LES_SAVE_AVGIJ(i33, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
             mean1d_les, wrk1d, wrk2d, wrk3d)
     ENDIF
  ENDIF

! #######################################################################
! Momentum equation : off-diagonal terms
! #######################################################################
! -----------------------------------------------------------------    
!                                         tauxy = rho(Um*Vm)_sg
! -----------------------------------------------------------------    
  CALL SGS_RECTN_TAUIJ(n0, rho, mu_m, mv_m, barvs, barmu_m, barmv_m, &
       TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
! Dissipation, -tau_xy*du/dy-tau_yx*dv/dx
  IF ( idissflg .EQ. 1 ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, v, &
          TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, u, &
          TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
        DO k = 1,kmax
           DO ij = 1,imax*jmax
              PHI(ij,1,k) = PHI(ij,1,k) - TAU_M(ij,1,k)*&
                   (TMP1_LOC(ij,1,k)+TMP2_LOC(ij,1,k))
           ENDDO
        ENDDO
     ENDIF
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_AVGIJ(i11, dx, dy, dz, u, v, w, &
             TAU_M(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
     ENDIF
  ENDIF
! Calculate  - d/dy(tauxy)             (X-momentum)
!       and  - d/dx(tauyx)             (Y-momentum)
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
       TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
       TAU_M(1,1,1), TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k = 1,kmax
     DO ij = 1,imax*jmax
        h2(ij,1,k) = h2(ij,1,k) -  sgs_alpha*TMP1_LOC(ij,1,k)
        h1(ij,1,k) = h1(ij,1,k) -  sgs_alpha*TMP2_LOC(ij,1,k)
     ENDDO
  ENDDO
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_AVGIJ(i24, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
          mean1d_les, wrk1d, wrk2d, wrk3d)
  ENDIF

  IF ( kmax_total .GT. 1 ) THEN
! -----------------------------------------------------------------    
!                                         tauxz = rho(Um*Wm)_sg
! -----------------------------------------------------------------    
     CALL SGS_RECTN_TAUIJ(n0, rho, mu_m, mw_m, barvs, barmu_m, barmw_m, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
! Dissipation, -tau_xz*du/dz-tau_zx*dw/dx
     IF ( idissflg .EQ. 1 ) THEN
        CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, w, &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, u, &
             TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
           DO k = 1,kmax
              DO ij = 1,imax*jmax
                 PHI(ij,1,k) = PHI(ij,1,k) - TAU_M(ij,1,k)*&
                      (TMP1_LOC(ij,1,k)+TMP2_LOC(ij,1,k))
              ENDDO
           ENDDO
        ENDIF
        IF ( statflg .EQ. 1 ) THEN
           CALL LES_SAVE_AVGIJ(i12, dx, dy, dz, u, v, w, &
                TAU_M(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
        ENDIF
     ENDIF
! Calculate  - d/dz(tauxz)             (X-momentum)
!       and  - d/dx(tauzx)             (Z-momentum)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc, dx, &
          TAU_M(1,1,1), TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1, kmax
        DO ij=1,imax*jmax
           h1(ij,1,k) = h1(ij,1,k) -  sgs_alpha*TMP1_LOC(ij,1,k)
           h3(ij,1,k) = h3(ij,1,k) -  sgs_alpha*TMP2_LOC(ij,1,k)
        ENDDO
     ENDDO
! Statistics
     IF ( statflg .EQ. 1 ) THEN
        CALL LES_SAVE_AVGIJ(i25, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1), &
             mean1d_les, wrk1d, wrk2d, wrk3d)
     ENDIF

! -----------------------------------------------------------------    
!                                         tauyz = rho(Vm*Wm)_sg
! -----------------------------------------------------------------    
     CALL SGS_RECTN_TAUIJ(n0, rho, mv_m, mw_m, barvs, barmv_m, barmw_m, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), wrk3d, cf0x, cf0y, cf0z)
! Dissipation, -tau_yz*dv/dz-tau_zy*dw/dy
     IF ( idissflg .EQ. 1 ) THEN
        CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, v, &
             TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, w, &
             TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
        IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
           DO k = 1,kmax
              DO ij = 1,imax*jmax
                 PHI(ij,1,k) = PHI(ij,1,k) - TAU_M(ij,1,k)*&
                      (TMP1_LOC(ij,1,k)+TMP2_LOC(ij,1,k))
              ENDDO
           ENDDO
        ENDIF
        IF ( statflg .EQ. 1 ) THEN
           CALL LES_SAVE_AVGIJ(i13, dx, dy, dz, u, v, w, &
                TAU_M(1,1,1), mean1d_les, wrk1d, wrk2d, wrk3d)
        ENDIF
     ENDIF
! Calculate  - d/dz(tauyz)             (Y-momentum)
!       and  - d/dy(tauzy)             (Z-momentum)
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc, dz, &
          TAU_M(1,1,1), TMP1_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc, dy, &
          TAU_M(1,1,1), TMP2_LOC(1,1,1), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k=1, kmax
        DO ij=1,imax*jmax
           h2(ij,1,k) = h2(ij,1,k) -  sgs_alpha*TMP1_LOC(ij,1,k)
           h3(ij,1,k) = h3(ij,1,k) -  sgs_alpha*TMP2_LOC(ij,1,k)
        ENDDO
     ENDDO
  ENDIF
! Statistics
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_AVGIJ(i26, dx, dy, dz, u, v, w, TMP1_LOC(1,1,1),&
          mean1d_les, wrk1d, wrk2d, wrk3d)
  ENDIF

! #######################################################################
! Dissipation function in energy equation
! #######################################################################
  IF ( iles_type_diss .EQ. LES_DISS_DISSSG ) THEN
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           h4(ij,1,k) = h4(ij,1,k) + MAX(C_0_R,PHI(ij,1,k))
        ENDDO
     ENDDO
  ENDIF

#undef TRACE
#undef TAU_M
#undef TMP1_LOC
#undef TMP2_LOC
#undef PHI

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_RECTN_U' )
#endif

  RETURN
END SUBROUTINE SGS_RECTN_U
