!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2001/11/30 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# This subroutine computes the additive residual stress term of the right
!# hand side of the filtered compressible scalar equations using a 
!# Smagorinsky model.
!#
!########################################################################
!# ARGUMENTS 
!#
!# txc     In    size 2
!#
!########################################################################
SUBROUTINE SGS_SMG_Z(statflg, dx, dy, dz, rho, &
     z1, z1_r, sgsdif, zh1, mean1d_les_sc, txc, chi, wrk1d, wrk2d, wrk3d)

#include "types.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL
  USE LES_GLOBAL

  IMPLICIT NONE

#include "integers.h"

  TINTEGER statflg

  TREAL, DIMENSION(*)                :: dx, dy, dz
  TREAL, DIMENSION(imax,jmax,kmax)   :: rho, z1, z1_r, sgsdif, zh1, chi
  TREAL, DIMENSION(imax,jmax,kmax,*) :: txc

  TREAL mean1d_les_sc(nstatavg,jmax,*)

  TREAL, DIMENSION(*) :: wrk1d, wrk2d, wrk3d

! -----------------------------------------------------------------------
  TINTEGER i1vsout, imxvsout
  TINTEGER j1vsout, jmxvsout
  TINTEGER k1vsout, kmxvsout
  TINTEGER ij, k

! #######################################################################
#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'ENTERING SGS_SMG_Z' )
#endif

#include "dns_bcs_out.h"

! --------------------------------------------------------
!                                   (d/dx)(rho*D * dz1/dx)
! --------------------------------------------------------
  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, z1, txc(1,1,1,2), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        txc(ij,1,k,1) = sgsdif(ij,1,k)*txc(ij,1,k,2)
     ENDDO
  ENDDO
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i2, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i2, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ENDIF

  IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
     CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
          dx, z1_r, txc(1,1,1,2), i1vsout, imxvsout, wrk1d, wrk2d, wrk3d)
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           chi(ij,1,k) = chi(ij,1,k) + txc(ij,1,k,1)*txc(ij,1,k,2)
        ENDDO
     ENDDO
  ENDIF

  CALL PARTIAL_X(imode_fdm, imax, jmax, kmax, i1bc,&
       dx, txc(1,1,1,1), txc(1,1,1,2), i1vsout, imxvsout, wrk1d, wrk2d, wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        zh1(ij,1,k) = zh1(ij,1,k) + txc(ij,1,k,2)
     ENDDO
  ENDDO
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i8, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i8, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ENDIF

! --------------------------------------------------------
!                                   (d/dy)(rho*D * dz1/dy)
! --------------------------------------------------------
  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, z1, txc(1,1,1,2), i0, i0, wrk1d, wrk2d, wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        txc(ij,1,k,1) = sgsdif(ij,1,k)*txc(ij,1,k,2)
     ENDDO
  ENDDO
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i3, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i3, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ENDIF

  IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
     CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
          dy, z1_r, txc(1,1,1,2), i0, i0, wrk1d, wrk2d, wrk3d)
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           chi(ij,1,k) = chi(ij,1,k)+txc(ij,1,k,1)*txc(ij,1,k,2)
        ENDDO
     ENDDO
  ENDIF

  CALL PARTIAL_Y(imode_fdm, imax, jmax, kmax, j1bc,&
       dy, txc(1,1,1,1), txc(1,1,1,2), j1vsout, jmxvsout, wrk1d, wrk2d, wrk3d)       
  DO k=1,kmax
     DO ij=1,imax*jmax
        zh1(ij,1,k) = zh1(ij,1,k) + txc(ij,1,k,2)
     ENDDO
  ENDDO
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i8, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i8, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ENDIF

! --------------------------------------------------------
!                                   (d/dz)(rho*D * dz1/dz)
! --------------------------------------------------------
  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, z1, txc(1,1,1,2), i0, i0, wrk1d, wrk2d, wrk3d)      
  DO k=1,kmax
     DO ij=1,imax*jmax
        txc(ij,1,k,1) = sgsdif(ij,1,k)*txc(ij,1,k,2)
     ENDDO
  ENDDO
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i4, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i4, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ENDIF

  IF ( iles_type_chem .EQ. LES_CHEM_QUASIBS ) THEN
     CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
          dz, z1_r, txc(1,1,1,2), i0, i0, wrk1d, wrk2d, wrk3d)      
     DO k = 1,kmax
        DO ij = 1,imax*jmax
           chi(ij,1,k) = chi(ij,1,k) + txc(ij,1,k,1)*txc(ij,1,k,2)
        ENDDO
     ENDDO
  ENDIF

  CALL PARTIAL_Z(imode_fdm, imax, jmax, kmax, k1bc,&
       dz, txc(1,1,1,1), txc(1,1,1,2), k1vsout, kmxvsout, wrk1d, wrk2d, wrk3d)
  DO k=1,kmax
     DO ij=1,imax*jmax
        zh1(ij,1,k) = zh1(ij,1,k) + txc(ij,1,k,2)
     ENDDO
  ENDDO
  IF ( statflg .EQ. 1 ) THEN
     CALL LES_SAVE_SCBDGIJ(i9, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ELSE IF ( statflg .EQ. -1 ) THEN
     CALL LES_SAVE_AVGPIJ(i9, dx, dy, dz, z1,&
          txc, mean1d_les_sc, wrk1d, wrk2d, wrk3d )
  ENDIF

#ifdef TRACE_ON
  CALL IO_WRITE_ASCII(tfile, 'LEAVING SGS_SMG_Z' )
#endif

  RETURN
END SUBROUTINE SGS_SMG_Z

