














		   



			   
               	           






!########################################################################
!# Tool/Library
!#
!########################################################################
!# HISTORY
!#
!# 2007/11/08 - J.P. Mellado
!#              Created
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculating the equilibrium T and q_l for given enthalpy and pressure,
!# iterative based on (rho,e,q_i)
!#
!# New created by Alberto to speed up. Only valid for air water . Assumed that HERMO_AI(6,1,1) = HERMO_AI(6,1,2) =0

!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE THERMO_AIRWATER_PHAL(nx, ny, nz, z1, p, h) 

  USE THERMO_GLOBAL, ONLY : THERMO_AI, THERMO_PSAT, NPSAT, WGHT_INV, dsmooth

  IMPLICIT NONE

INTEGER(4) i0, i1, i2, i3, i4, i5, i6, i7 ,i8, i9
  PARAMETER(i0=0)
  PARAMETER(i1=1)
  PARAMETER(i2=2)
  PARAMETER(i3=3)
  PARAMETER(i4=4)
  PARAMETER(i5=5)
  PARAMETER(i6=6)
  PARAMETER(i7=7)
  PARAMETER(i8=8)
  PARAMETER(i9=9)
  INTEGER(4) i10, i11, i12, i13, i14, i15, i16, i17 ,i18, i19
  PARAMETER(i10=10)
  PARAMETER(i11=11)
  PARAMETER(i12=12)
  PARAMETER(i13=13)
  PARAMETER(i14=14)
  PARAMETER(i15=15)
  PARAMETER(i16=16)
  PARAMETER(i17=17)
  PARAMETER(i18=18)
  PARAMETER(i19=19)
  INTEGER(4) i20, i21, i22, i23, i24, i25, i26, i27 ,i28, i29
  PARAMETER(i20=20)
  PARAMETER(i21=21)
  PARAMETER(i22=22)
  PARAMETER(i23=23)
  PARAMETER(i24=24)
  PARAMETER(i25=25)
  PARAMETER(i26=26)
  PARAMETER(i27=27)
  PARAMETER(i28=28)
  PARAMETER(i29=29)
  INTEGER(4) i30, i31, i32, i33, i34, i35, i36, i37 ,i38, i39
  PARAMETER(i30=30)
  PARAMETER(i31=31)
  PARAMETER(i32=32)
  PARAMETER(i33=33)
  PARAMETER(i34=34)
  PARAMETER(i35=35)
  PARAMETER(i36=36)
  PARAMETER(i37=37)
  PARAMETER(i38=38)
  PARAMETER(i39=39)
  INTEGER(4) i50, i51, i52, i53, i54, i55, i56, i57 ,i58, i59
  PARAMETER(i50=50)
  PARAMETER(i51=51)
  PARAMETER(i52=52)
  PARAMETER(i53=53)
  PARAMETER(i54=54)
  PARAMETER(i55=55)
  PARAMETER(i56=56)
  PARAMETER(i57=57)
  PARAMETER(i58=58)
  PARAMETER(i59=59)
  INTEGER(4) i150
  PARAMETER(i150=150)

  INTEGER(4) nx, ny, nz
  REAL(8) z1(nx*ny*nz,*), h(*) !Alberto
  REAL(8) p !Alberto p is now an scalar
  REAL(8) q_latent, cp_d, cp_v, cl, psat, qsat, dpdtsat, psat_coeff(20), dpsat_dT_coeff(20), rd_ov_rv
! -------------------------------------------------------------------
  INTEGER(4) ij, iter, ipsat, dum_npsat, niter
  REAL(8) t_loc, z1_loc(2), h_loc, dummy, der1, der2, func,dsmooth_loc,partial_ql_qt,deltaql



!$omp parallel default( none ) private( ij, z1_loc, h_loc, t_loc, psat, qsat, dummy, deltaql, iter, func, dpdtsat, ipsat, der1, der2, dsmooth_loc, cp_v, cp_d, cl, rd_ov_rv, q_latent, psat_coeff, dum_npsat, niter, dpsat_dT_coeff, partial_ql_qt) shared( z1, h, nx, ny, nz, dsmooth, THERMO_AI, WGHT_INV, NPSAT, THERMO_PSAT, p)
  
 

! ###################################################################
  niter = 4
  
  q_latent = -THERMO_AI(6,1,3)
  cp_d = THERMO_AI(1,1,2)
  cp_v = THERMO_AI(1,1,1)
  cl = THERMO_AI(1,1,3)
  rd_ov_rv = 1.0d0/WGHT_INV(1)
  dum_npsat = NPSAT
  DO ipsat = dum_npsat,1,-1
     psat_coeff(ipsat) = THERMO_PSAT(ipsat)/p  !Assuming pressure defined the code and in the definiction of PSAT in HPa 
  ENDDO
  DO ipsat = dum_npsat-1,1,-1
     dpsat_dT_coeff(ipsat) = psat_coeff(ipsat+1)*ipsat
  ENDDO


!$omp do
  DO ij = 1,nx*ny*nz
! -------------------------------------------------------------------
! initialize, q_l=0
! -------------------------------------------------------------------
     z1_loc(1) = z1(ij,1)
     z1_loc(2) = 0.0d0
     h_loc = h(ij)
     t_loc = h_loc/(cp_d+z1(ij,1)*(cp_v-cp_d)) !Temperature     
     psat = 0.0d0                              !Saturation pressure
     DO ipsat = dum_npsat,1,-1
	psat = psat*t_loc + psat_coeff(ipsat)
     ENDDO
     
     dummy = rd_ov_rv/(1.0d0/psat-1.0d0); 
     qsat = (1.0d0-z1_loc(1))*dummy
     z1_loc(2) = z1_loc(1) - qsat; !New liquid concentration
     
! -------------------------------------------------------------------
! iteration
! -------------------------------------------------------------------
     IF (z1_loc(2) .LT. 0.0d0) THEN !Unsaturated mixture
	z1_loc(2) = 0.0d0     !Reset liquid to zero
        deltaql = 0.0d0       !Needed by the smoothing function
     ELSE
	DO iter = 1,niter	
		der1 = cp_d*(1-z1_loc(1)) + cp_v*(z1_loc(1)-z1_loc(2)) + z1_loc(2)*cl !Partial derivative respective to the temperature
		func = der1*t_loc-z1_loc(2)*q_latent-h_loc;  ! Function to minimize
		dpdtsat = 0.0d0
		DO ipsat = dum_npsat-1,1,-1
			dpdtsat = dpdtsat*t_loc + dpsat_dT_coeff(ipsat)
		ENDDO        
		der2 = -((cl-cp_v)*t_loc-q_latent)*(1.0d0-z1_loc(1))*rd_ov_rv/((1.0d0-psat)*(1.0d0-psat))*dpdtsat; !Partial derivative respective to ql
		t_loc = t_loc - func/(der1+der2);  ! New guess for  Tnew using Newton (derivative = der1+der2)
		psat = 0.0d0          !Recalculate saturation pressure at the new temperature
		DO ipsat = dum_npsat,1,-1
	 		psat = psat*t_loc + psat_coeff(ipsat)
		ENDDO
		dummy = rd_ov_rv/(1.0d0/psat-1.0d0); 
                qsat = (1.0d0-z1_loc(1))*dummy
     		z1_loc(2) = z1_loc(1) - qsat; !New Liquid concentration		
	ENDDO
	 deltaql = -dummy*z1_loc(2) !Needed by the smoothing function
     ENDIF     

     IF ( dsmooth .GT. 0.0d0 ) THEN
           dsmooth_loc = dsmooth*qsat
	   partial_ql_qt = dummy+1.0d0
           z1(ij,2) = partial_ql_qt*dsmooth_loc*log(exp((z1_loc(1)-qsat)/dsmooth_loc)+1.0d0) + deltaql;
     ELSE
	   z1(ij,2) = z1_loc(2)
     ENDIF

     

   ENDDO
!$omp end do
!$omp end parallel

  RETURN
END SUBROUTINE THERMO_AIRWATER_PHAL
