!########################################################################
!# Tool/Library DNS
!#
!########################################################################
!# HISTORY
!#
!# 2007/06/09 - J.P. Mellado
!#              Created
!# 2007/06/20 - J.P. Mellado
!#              AIRWATER case included.
!#
!########################################################################
!# DESCRIPTION
!#
!# Computing energy from temperature and species mass fractions
!# according to the caloric equation of state.
!#
!# e=\sum Y_ih_i - TR_0\sum Y_i/W_i
!#
!# which in nondimensional form is
!#
!# e=\sum Y_ih_i - \sum Y_i T*(R_0/C_{p,0]W_0)*W_0/W_i
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################















		   



			   
               	           







! File formats

! Flow Types

! Flow Mode

! Equations mode

! Equation terms

 





! Finite-differences method


! Runge-Kutta method

! Boundary conditions


! Filters

! Mean profiles

! Chemistry Constants


! Chemistry Type

! Mixture Type


SUBROUTINE THERMO_CALORIC_ENERGY(nx,ny,nz, s,T, e)

  USE THERMO_GLOBAL, ONLY : imixture, gama0, GRATIO
  USE THERMO_GLOBAL, ONLY : NSP, NCP_CHEMKIN, WGHT_INV, THERMO_AI, THERMO_TLIM
  USE THERMO_GLOBAL, ONLY : YMASS

  IMPLICIT NONE

  INTEGER(4) nx,ny,nz
  REAL(8), DIMENSION(nx*ny*nz),   INTENT(IN)  :: T
  REAL(8), DIMENSION(nx*ny*nz,*), INTENT(IN)  :: s
  REAL(8), DIMENSION(nx*ny*nz),   INTENT(OUT) :: e

! -------------------------------------------------------------------
  INTEGER(4) i, is, im, icp
  REAL(8) ENTHALPY_I, WMEAN_INV
  REAL(8) ENERGY_V, ENERGY_D, ENERGY_L

! ###################################################################
! Single species
! ###################################################################
  IF ( imixture .EQ. 0 ) THEN
     DO i = 1, nx*ny*nz
        e(i) = T(i)/gama0
     ENDDO

! ###################################################################
! mixture 10
! ###################################################################
  ELSE IF ( imixture .EQ. 10 ) THEN
     DO i = 1, nx*ny*nz
        ENERGY_V = (THERMO_AI(1,1,1)- GRATIO*WGHT_INV(1))*T(i) + THERMO_AI(6,1,1)
        ENERGY_D = (THERMO_AI(1,1,2)- GRATIO*WGHT_INV(2))*T(i) + THERMO_AI(6,1,2)
        ENERGY_L = THERMO_AI(1,1,3)*T(i) + THERMO_AI(6,1,3)
        e(i) = (s(i,1)-s(i,2))*ENERGY_V + (1.0d0-s(i,1))*ENERGY_D + s(i,2)*ENERGY_L
     ENDDO

! ###################################################################
! General mixture of NSP species. s contains NSP-1 mass fractions
! ###################################################################
  ELSE
     e(1:nx*ny*nz) = 0.0d0
     DO i = 1, nx*ny*nz
! pass species to YMASS vector
        YMASS(NSP) = 1.0d0
        DO is = 1, NSP-1
           YMASS(is) = s(i,is)
           YMASS(NSP)= YMASS(NSP)-YMASS(is)
        ENDDO
! calculate enthalpy of the mixture
        WMEAN_INV = 0.0d0
        DO is = 1,NSP
           IF ( T(i) .LT. THERMO_TLIM(3,is) ) THEN
              im = 2
           ELSE
              im = 1
           ENDIF
           ENTHALPY_I = 0.0d0
           DO icp = NCP_CHEMKIN,1,-1
              ENTHALPY_I = ENTHALPY_I*T(i) + THERMO_AI(icp,im,is)/DBLE(icp)
           ENDDO
           ENTHALPY_I = ENTHALPY_I*T(i) + THERMO_AI(6,im,is)
           e(i)       = e(i)      + YMASS(is)*ENTHALPY_I
           WMEAN_INV  = WMEAN_INV + YMASS(is)*WGHT_INV(is)
        ENDDO
! go from enthalpy to energy
        e(i) = e(i) - GRATIO*WMEAN_INV*T(i)
     ENDDO

  ENDIF

  RETURN
END SUBROUTINE THERMO_CALORIC_ENERGY
