














		   



			   
               	           







! File formats

! Flow Types

! Flow Mode

! Equations mode

! Equation terms

 





! Finite-differences method


! Runge-Kutta method

! Boundary conditions


! Filters

! Mean profiles

! Chemistry Constants


! Chemistry Type

! Mixture Type


!########################################################################
!# Tool DNS
!#
!########################################################################
!# HISTORY
!#
!# 2007/05/22 - J.P. Mellado
!#              Created
!# 2007/07/04 - J.P. Mellado
!#              Including case AIRWATER.
!#
!########################################################################
!# DESCRIPTION
!#
!# Calculate entropy from T, p and composition. The reference state 
!# is T_0, which is set to 298 K in the case of multispecies, and 
!# p_init. Nondimensional with C_{p,0}.
!#
!########################################################################
!# ARGUMENTS 
!#
!########################################################################
SUBROUTINE THERMO_ENTROPY(nx,ny,nz, z1,T,p, s)

  USE DNS_GLOBAL, ONLY : p_init

  USE THERMO_GLOBAL, ONLY : imixture, gama0, GRATIO, MRATIO
  USE THERMO_GLOBAL, ONLY : NSP, NCP_CHEMKIN, WGHT_INV, THERMO_AI, THERMO_TLIM
  USE THERMO_GLOBAL, ONLY : YMASS

  IMPLICIT NONE

  INTEGER(4) nx,ny,nz
  REAL(8), DIMENSION(nx*ny*nz),   INTENT(IN)  :: T,p
  REAL(8), DIMENSION(nx*ny*nz,*), INTENT(IN)  :: z1
  REAL(8), DIMENSION(nx*ny*nz),   INTENT(OUT) :: s

! -------------------------------------------------------------------
  INTEGER(4) ij, is, im, icp
  REAL(8) WMEAN_INV, ENTROPY_I, XMOL_I, alpha

! ###################################################################
! Single species
! ###################################################################
  alpha = (gama0-1.0d0)/gama0
  IF ( imixture .EQ. 0 ) THEN
     DO ij = 1,nx*ny*nz
        s(ij) = log(T(ij)/(p(ij)/p_init)**alpha)
     ENDDO

! ###################################################################
! mixture 10
! ###################################################################
  ELSE IF ( imixture .EQ. 10 ) THEN
     DO ij = 1,nx*ny*nz
        YMASS(1) = z1(ij,1)-z1(ij,2) !q_v=q_t-q_l
        YMASS(2) = 1.0d0-z1(ij,1)    !q_d=1-q_t
! calculate temperature part
        WMEAN_INV = 0.0d0
        DO is = 1,2
           IF ( T(ij) .LT. THERMO_TLIM(3,is) ) THEN; im = 2
           ELSE;                                     im = 1; ENDIF
           ENTROPY_I = 0.0d0
           DO icp = NCP_CHEMKIN,2,-1
              ENTROPY_I = ENTROPY_I*T(ij) + THERMO_AI(icp,im,is)/DBLE(icp-1)
           ENDDO
           ENTROPY_I = ENTROPY_I*T(ij) + THERMO_AI(7,im,is) + &
                THERMO_AI(1,im,is)*log(T(ij))
           s(ij)      = s(ij)     + YMASS(is)*ENTROPY_I
           WMEAN_INV  = WMEAN_INV + YMASS(is)*WGHT_INV(is)
        ENDDO
! calculate pressure part
        DO is = 1,2
           XMOL_I = YMASS(is)*WGHT_INV(is)/WMEAN_INV
           IF ( XMOL_I .GT. 0.0d0 ) THEN
              s(ij) = s(ij) - GRATIO*YMASS(is)*WGHT_INV(is)*log(XMOL_I)
           ENDIF
        ENDDO
        s(ij) = s(ij) - GRATIO*WMEAN_INV*log(p(ij)/p_init)

        s(ij) = s(ij) + z1(ij,2)*(THERMO_AI(7,im,3)+THERMO_AI(1,1,3)*log(T(ij)))

     ENDDO

! ###################################################################
! General mixture of NSP species. z1 contains NSP-1 mass fractions
! ###################################################################
  ELSE 
     s(1:nx*ny*nz) = 0.0d0
     DO ij = 1,nx*ny*nz
! pass species to YMASS vector
        YMASS(NSP) = 1.0d0
        DO is = 1, NSP-1
           YMASS(is) = z1(ij,is)
           YMASS(NSP)= YMASS(NSP)-YMASS(is)
        ENDDO
! calculate temperature part
        WMEAN_INV = 0.0d0
        DO is = 1,NSP
           IF ( T(ij) .LT. THERMO_TLIM(3,is) ) THEN; im = 2
           ELSE;                                     im = 1; ENDIF
           ENTROPY_I = 0.0d0
           DO icp = NCP_CHEMKIN,2,-1
              ENTROPY_I = ENTROPY_I*T(ij) + THERMO_AI(icp,im,is)/DBLE(icp-1)
           ENDDO
           ENTROPY_I = ENTROPY_I*T(ij) + THERMO_AI(7,im,is) + &
                THERMO_AI(1,im,is)*log(T(ij))
           s(ij)     = s(ij)     + YMASS(is)*ENTROPY_I
           WMEAN_INV = WMEAN_INV + YMASS(is)*WGHT_INV(is)
        ENDDO
! calculate pressure part
        DO is = 1,NSP
           XMOL_I = YMASS(is)*WGHT_INV(is)/WMEAN_INV
           IF ( XMOL_I .GT. 0.0d0 ) THEN
              s(ij) = s(ij) - GRATIO*YMASS(is)*WGHT_INV(is)*log(XMOL_I)
           ENDIF
        ENDDO
        s(ij) = s(ij) - GRATIO*WMEAN_INV*log(p(ij)/p_init)
     ENDDO
  ENDIF

  RETURN
END SUBROUTINE THERMO_ENTROPY
