#include "types.h"
#include "dns_const.h"

SUBROUTINE FLAME_TIME(nmax, Z, dZ, Y, Yh, T, Th, rho, chi, eps, wrk1d, wrk2d, tmp)

  USE DNS_GLOBAL

  IMPLICIT NONE

  TINTEGER nmax
  TREAL Z(nmax)
  TREAL dZ(nmax)
  TREAL Y(nmax,NSP-1), Yh(nmax,NSP-1)
  TREAL T(nmax), Th(nmax)
  TREAL rho(nmax)
  TREAL chi, eps
  TREAL wrk1d(nmax,*)
  TREAL wrk2d(nmax,*)
  TREAL tmp(nmax,3)

  TINTEGER is, n, it, istop
  TREAL r0, delta_t, YN2, dum
  TINTEGER i0, i1, i23
  TREAL Yhmax(MAX_NSP), Thmax
  CHARACTER*128 line

  i0 = 0
  i1 = 1
  i23 = 23
  r0 = C_0_R

  ! constant density
  DO n = 1,nmax
     rho(n) = mean_rho
  ENDDO

  ! Loop
  DO it = 1, itend

     Yh = C_0_R
     Th = C_0_R

     ! diffusion
     DO is = 1,NSP-1
        CALL DZ3D(imode_fdm, i1, i1, nmax, Y(1,is), dZ, tmp(1,1),&
             i0, i0, wrk1d(1,1), wrk1d(1,2), wrk1d(1,3))
        CALL DZ3D(imode_fdm, i1, i1, nmax, tmp(1,1), dZ, tmp(1,2),&
             i0, i0, wrk1d(1,1), wrk1d(1,2), wrk1d(1,3))
        DO n = 1,nmax
           Yh(n,is) = Yh(n,is) + rho(n)*C_05_R*chi*tmp(n,2)
        ENDDO
     ENDDO
     CALL DZ3D(imode_fdm, i1, i1, nmax, T, dZ, tmp(1,1),&
          i0, i0, wrk1d(1,1), wrk1d(1,2), wrk1d(1,3))
     CALL DZ3D(imode_fdm, i1, i1, nmax, tmp(1,1), dZ, tmp(1,2),&
          i0, i0, wrk1d(1,1), wrk1d(1,2), wrk1d(1,3))
     DO n = 1,nmax
        Th(n) = Th(n) + rho(n)*C_05_R*chi*tmp(n,2)
     ENDDO

     ! reaction
     CALL CHEM_BILGER1997(i1, rho, T, dum, Y, Yh, Th, tmp)

     ! time step
     delta_t = C_025_R*dZ(1)*dZ(1)/(C_05_R*chi)
     dum     = C_025_R/TGFM
     delta_t = MIN(delta_t,dum)

     DO is = 1,NSP-1
        DO n = 1,nmax
           Yh(n,is) = delta_t*Yh(n,is)/rho(n)
        ENDDO
     ENDDO
     DO n = 1,nmax
        Th(n) = delta_t*Th(n)/rho(n)
     ENDDO

     ! advance in time
     DO is = 1,NSP-1
        DO n = 2,nmax-1
           Y(n,is) = Y(n,is) + Yh(n,is)
        ENDDO
     ENDDO
     DO n = 2,nmax-1
        T(n) = T(n) + Th(n)
     ENDDO

     !     DO n = 1,nmax
     !        WMEAN = C_0_R
     !        YN2 = C_1_R
     !        DO is=1,NSP-1
     !           YN2 = YN2 - Y(n,is)
     !           WMEAN = WMEAN + Y(n,is)/WGHT(is)
     !        ENDDO
     ! radical
     !        YN2 = YN2 - tmp(n,1)
     !        WMEAN = WMEAN + YN2/WGHT(NSP)
     !        WMEAN = C_1_R/(WMEAN*WREF)
     !        rho(n) = MRATIO*p_init*WMEAN/T(n)
     !     ENDDO

     ! convergence (infinity-norm)
     DO is = 1,NSP-1
        !            Yhmax(is) = Q9SMAX(Yh(2,is),nmax-2) / Q9SMAX(Y(2,is),nmax-2)
        Yhmax(is) = MAXVAL(Yh(2:nmax-1,is)) / MAXVAL(Y(2:nmax-1,is))
     ENDDO
     !         Thmax = Q9SMAX(Th(2),nmax-2) / Q9SMAX(T(2),nmax-2)
     Thmax = MAXVAL(Th(2:nmax-1)) / MAXVAL(T(2:nmax-1))

     istop = 0
     DO is = 1,NSP-1
        IF ( ABS(Yhmax(is)) .GT. eps ) THEN
           istop = 1
        ENDIF
     ENDDO
     IF ( ABS(Thmax) .GT. eps ) THEN
        istop = 1
     ENDIF

     ! output
     IF ( MOD(it,nitera_log) .EQ. 0 ) THEN
        WRITE(line,1000) it, delta_t, (ABS(Yhmax(is)),is=1,NSP-1), ABS(Thmax)
        CALL IO_WRITE_ASCII(ofile,line)
     ENDIF

     IF ( istop .EQ. 0 ) EXIT

  ENDDO

  WRITE(*,*) line

  OPEN(i23,file='flame_final.dat')
  DO n = 1,nmax
     WRITE(i23,'(13f12.6)') Z(n), &
          (Y(n,is),is=1,NSP-1),T(n),rho(n),&
          (tmp(n,is),is=1,3)
  ENDDO
  CLOSE(i23)

  RETURN
1000 FORMAT(I5,10(1X,E8.2E2))
END SUBROUTINE FLAME_TIME
