PROGRAM DECONVOLUTION

#include "types.h"
#include "dns_const.h"
#include "les_const.h"

  USE DNS_GLOBAL

  IMPLICIT NONE

#include "integers.h"
#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL x(:)
  ALLOCATABLE x
  TREAL y(:)
  ALLOCATABLE y
  TREAL z(:)
  ALLOCATABLE z
  TREAL dx(:)
  ALLOCATABLE dx
  TREAL dy(:)
  ALLOCATABLE dy
  TREAL dz(:)
  ALLOCATABLE dz

  TREAL wrk1d(:,:)
  ALLOCATABLE wrk1d
  TREAL wrk2d(:)
  ALLOCATABLE wrk2d
  TREAL wrk3d(:,:,:)
  ALLOCATABLE wrk3d

  TREAL txc1(:)
  ALLOCATABLE txc1
  TREAL txc2(:)
  ALLOCATABLE txc2
  TREAL txc3(:)
  ALLOCATABLE txc3
  TREAL txc4(:)
  ALLOCATABLE txc4
  TREAL txc5(:)
  ALLOCATABLE txc5
  TREAL txc6(:)
  ALLOCATABLE txc6
  TREAL txc7(:)
  ALLOCATABLE txc7
  TREAL txc8(:)
  ALLOCATABLE txc8

  TREAL mean(:)
  ALLOCATABLE mean

  TREAL cf0x(:)
  ALLOCATABLE cf0x
  TREAL cf0y(:)
  ALLOCATABLE cf0y
  TREAL cf0z(:)
  ALLOCATABLE cf0z

#ifdef PARALLEL
  INTEGER icount
#endif

  TINTEGER nx0, fsize_max
  TINTEGER n_cf0x_size, n_cf0y_size, n_cf0z_size

  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')

#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) 'Integral Iteration ?'
     READ(*,*) itime
     WRITE(*,*) 'Filter size ?'
     READ(*,*) nx0
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST(itime, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
       ims_err)
  CALL MPI_BCAST(nx0, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
       ims_err)
#endif

  isgs_f0size = nx0
! temporally !!!
  isgs_f1size = isgs_f0size

! top-hat filter at level 0
  IF ( iunifx .EQ. 0 ) THEN
     IF ( i1bc .EQ. 0 ) THEN
        n_cf0x_size = 1
     ELSE
        n_cf0x_size = isgs_f0size
     ENDIF
  ELSE
     n_cf0x_size = imax*(isgs_f0size + 1)
  ENDIF
  IF ( iunify .EQ. 0 ) THEN
     IF ( j1bc .EQ. 0 ) THEN
        n_cf0y_size = 1
     ELSE
        n_cf0y_size = isgs_f0size
     ENDIF
  ELSE
     n_cf0y_size = jmax*(isgs_f0size + 1)
  ENDIF
  IF ( iunifz .EQ. 0 ) THEN
     IF ( k1bc .EQ. 0 ) THEN
        n_cf0z_size = 1
     ELSE
        n_cf0z_size = isgs_f0size
     ENDIF
  ELSE
     n_cf0z_size = kmax_total*(isgs_f0size + 1)
  ENDIF

  ALLOCATE(x(imax_total))
  ALLOCATE(y(jmax_total))
  ALLOCATE(z(kmax_total))
  ALLOCATE(dx(imax_total*inb_grid))
  ALLOCATE(dy(jmax_total*inb_grid))
  ALLOCATE(dz(kmax_total*inb_grid))

  ALLOCATE(txc1(imax*jmax*kmax))
  ALLOCATE(txc2(imax*jmax*kmax))
  ALLOCATE(txc3(imax*jmax*kmax))
  ALLOCATE(txc4(imax*jmax*kmax))
  ALLOCATE(txc5(imax*jmax*kmax))
  ALLOCATE(txc6(imax*jmax*kmax))
  ALLOCATE(txc7(imax*jmax*kmax))
  ALLOCATE(txc8(imax*jmax*kmax))

  ALLOCATE(wrk1d(isize_wrk1d,inb_wrk1d))
  ALLOCATE(wrk2d(isize_wrk2d*inb_wrk2d))

  fsize_max = isgs_f0size + isgs_f1size + kmax
  ALLOCATE(wrk3d(imax,jmax,fsize_max))

  ALLOCATE(mean(42*jmax))

  ALLOCATE(cf0x(n_cf0x_size))
  ALLOCATE(cf0y(n_cf0y_size))
  ALLOCATE(cf0z(n_cf0z_size))

! Read the grid 
#include "dns_read_grid.h"

! Initializing filters
#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     CALL FILTHINI(iunifx, i1bc, imax, nx0, scalex, x, cf0x, wrk1d)
     CALL FILTHINI(iunify, j1bc, jmax, nx0, scaley, y, cf0y, wrk1d)
     IF ( kmax_total .GT. 1 ) THEN
        CALL FILTHINI(iunifz, k1bc, kmax_total, nx0, scalez, z, cf0z, wrk1d)
     ENDIF
#ifdef PARALLEL
  ENDIF
  icount = n_cf0x_size
  CALL MPI_BCAST(cf0x, icount, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
  icount = n_cf0y_size
  CALL MPI_BCAST(cf0y, icount, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
  icount = n_cf0z_size
  CALL MPI_BCAST(cf0z, icount, MPI_REAL8, 0, MPI_COMM_WORLD, ims_err)
#endif

  CALL DEC_FLUX_SG(nx0, x, y, z, dx, dy, dz, txc1, txc2,&
       txc3, txc4, txc5, txc6, txc7, txc8, wrk1d, wrk2d, wrk3d, mean, cf0x, cf0y, cf0z)

  CALL DNS_END(0)

  STOP
END PROGRAM DECONVOLUTION

