PROGRAM CORRYY
  
#include "types.h"
#include "dns_const.h"
#include "avgij_map.h"

  USE DNS_GLOBAL
#ifdef PARALLEL
  USE DNS_MPI
#endif

  IMPLICIT NONE

#ifdef PARALLEL
#include "mpif.h"
#endif

  TREAL x(:)
  ALLOCATABLE x
  TREAL y(:)
  ALLOCATABLE y
  TREAL z(:)
  ALLOCATABLE z
  TREAL dx(:)
  ALLOCATABLE dx
  TREAL dy(:)
  ALLOCATABLE dy
  TREAL dz(:)
  ALLOCATABLE dz
  TREAL wrk1d(:,:)
  ALLOCATABLE wrk1d
  TREAL t_dat(:,:)
  ALLOCATABLE t_dat
  TREAL p_dat(:,:,:,:,:)
  ALLOCATABLE p_dat
  TREAL crl(:,:,:)
  ALLOCATABLE crl
  TREAL txc(:,:,:,:)
  ALLOCATABLE txc
  TREAL mean(:)
  ALLOCATABLE mean

  TINTEGER nrestloc, itxc, MXTMP_LOC
  CHARACTER*32 cfile
  CHARACTER*128 line
  TINTEGER i, l, n, l1, l2, flgstat

  LOGICAL lfilexist

  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')

#ifdef PARALLEL
  CALL DNS_MPI_INITIALIZE
#endif

  MXTMP_LOC = 3

! ###############
! # Read inputs #
! ###############

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) 'Integral Iteration ?'
     READ(*,*) itime
#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST( itime, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, ims_err)
#endif

! Flag for use of st**** file

#ifdef PARALLEL
  IF ( ims_pro .EQ. 0 ) THEN
#endif
     WRITE(*,*) 'Use global statistics (0-no/1-yes)?'
     READ(*,*) flgstat
     IF ( flgstat .NE. 1 ) THEN
        flgstat = 0
     ENDIF

     WRITE(cfile,*) itime; cfile='st'//TRIM(ADJUSTL(cfile))

     INQUIRE(file=cfile,EXIST=lfilexist)
     IF ( .NOT. lfilexist ) THEN
        line = TRIM(ADJUSTL(cfile))//' does not exit. Going to local.'
        CALL IO_WRITE_ASCII(wfile,line)
        flgstat = 0
     ENDIF

     DO n = 1, nstatpln
        i = statpln(n)
        DO l = 1,nstatavg
           IF ( statavg(l) .EQ. i ) EXIT
        ENDDO
        IF ( l .EQ. nstatavg+1 ) THEN
           WRITE(line,*) i; line='Avg-plane'//TRIM(ADJUSTL(line))//' not present. Going to local.'
           CALL IO_WRITE_ASCII(wfile,line)
           flgstat = 0
        ENDIF
     ENDDO

#ifdef PARALLEL
  ENDIF
  CALL MPI_BCAST( flgstat, 1, MPI_INTEGER4, 0, MPI_COMM_WORLD, &
       ims_err)
#endif

  ALLOCATE(x(imax_total))
  ALLOCATE(y(jmax_total))
  ALLOCATE(z(kmax_total))
  ALLOCATE(dx(imax_total*inb_grid))
  ALLOCATE(dy(jmax_total*inb_grid))
  ALLOCATE(dz(kmax_total*inb_grid))

  ALLOCATE(wrk1d(isize_wrk1d,inb_wrk1d))
  ALLOCATE(t_dat(2,nspa_rest))
  ALLOCATE(p_dat(nspa_rest,nstatpln,jmax,nstatplnvars,kmax))
  ALLOCATE(crl(jmax/2+1,nstatplnvars,nstatpln))
  ALLOCATE(txc(imax,jmax,kmax,MXTMP_LOC))
  IF ( flgstat .EQ. 1 ) THEN
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        ALLOCATE(mean(nstatavg*jmax*inb_mean_spatial))
#ifdef PARALLEL
     ENDIF
#endif
  ENDIF

! ################
! #   Grid data  #
! ################
#include "dns_read_grid.h"

! ###########################
! # Temporary evolving case #
! ###########################

! ###########################
! # Spatially evolving case #
! ###########################

  IF (imode_sim .EQ. DNS_MODE_SPATIAL ) THEN

     cfile='pl                              '
     CALL CONCATI(cfile, itime)

     nrestloc = nspa_rest
     itxc = imax*jmax*kmax*MXTMP_LOC
     CALL DNS_READ_PLANE( cfile, itime, rtime, nrestloc, &
          nstatpln, statpln, jmax, nstatplnvars, kmax, kmax_total, &
          itxc, t_dat, p_dat, txc )

#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif

        WRITE(*,*) '#############################'
        WRITE(*,*) '# Plane file ....: ', cfile
#ifdef PARALLEL
     ENDIF
#endif
     IF ( flgstat .EQ. 1 ) THEN
        cfile='st                             '
        CALL CONCATI(cfile, itime)
#ifdef PARALLEL
        IF ( ims_pro .EQ. 0 ) THEN
#endif
           WRITE(*,*) '# Stat. file    : ', cfile
#ifdef PARALLEL
        ENDIF
#endif
        CALL DNS_READ_AVGIJ ( cfile, mean )
     ENDIF
#ifdef PARALLEL
     IF ( ims_pro .EQ. 0 ) THEN
#endif
        cfile='ycr                             '
        CALL CONCATI(cfile, itime)
        WRITE(*,*) '# Y Correlation .: ', cfile
        WRITE(*,*) '#############################'
#ifdef PARALLEL
     ENDIF
#endif

     CALL FAVFLUCT_PLANE( imax, jmax, kmax, kmax_total, &
          flgstat, nstatavg, nstatavg_points, nstatpln, &
          nspa_rest, nstatplnvars, inb_scal, statavg, statpln, mean, &
          mean((MA_MOMENTUM_SIZE+MS_SCALAR_SIZE*(inb_scal-1))*&
          nstatavg*jmax),&
          p_dat, p_dat )

     itxc = imax*jmax*kmax*MXTMP_LOC
     CALL CORRY_SPA( itime, itxc, imax, jmax, kmax, kmax_total, &
          x, y, z, dy, inb_scal+5, &
          nstatpln, statpln, nrestloc, &
          t_dat, p_dat, crl, wrk1d, txc )

  ENDIF

  CALL DNS_END(0)

  STOP
END PROGRAM CORRYY

