PROGRAM FLAMELET

#include "types.h"
#include "dns_error.h"

  USE DNS_GLOBAL

  IMPLICIT NONE

  TREAL wrk(:)
  ALLOCATABLE wrk
  TREAL weight(:)
  ALLOCATABLE weight
  TREAL abc(:)
  ALLOCATABLE abc
  TREAL rho(:)
  ALLOCATABLE rho
  TREAL yomega(:)
  ALLOCATABLE yomega
  TREAL cp(:)
  ALLOCATABLE cp
  TREAL xi(:)
  ALLOCATABLE xi
  TREAL tmp(:)
  ALLOCATABLE tmp
  TREAL Z(:)
  ALLOCATABLE Z
  TREAL x(:)
  ALLOCATABLE x
  TINTEGER IPIV(:)
  ALLOCATABLE IPIV
  TREAL F(:)
  ALLOCATABLE F
  TREAL J(:)
  ALLOCATABLE J

  CHARACTER*64 sRes
  TINTEGER IT, ITMAX
  TINTEGER i, is, n, in
  TREAL ETOL, ERROR, RHO_CST, r0, r1
  TREAL WOX, WFUEL, TOX, TFUEL
  TREAL xi0
  TINTEGER ixsize, NZ, i1, INFO
  TREAL YBCOX(MAX_NSP), YBCFUEL(MAX_NSP)
  TREAL lewis(MAX_NSP), sclv(MAX_NSP)
  CHARACTER*(24) name
  CHARACTER*(24) inifile
  TREAL wgtY, wgtT, rg, XPER, XTOL, TMAX
  TINTEGER iwstep, ijac, ijstep, ijstep_start, iload_ini
  TINTEGER ifltmax, iomega, itemp, ijtstep

  i1 = 1
  r0 = C_0_R
  r1 = C_1_R

  CALL DNS_INITIALIZE

  CALL DNS_READ_GLOBAL('dns.ini')

  inifile = 'flamelet.ini'

  CALL SCANINICHAR(lfile, inifile, 'Parameters', 'LoadIniFile', 'no', sRes)
  IF ( TRIM(ADJUSTL(sRes)) .EQ. 'yes' ) THEN
     iload_ini = 1
  ELSE
     iload_ini = 0
  ENDIF
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'NZ', '100', NZ)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', 'Xst', '1.0', xi0)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', 'GridStretch', &
       '0.96', rg)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', 'WeightY', '0.1',&
       wgtY)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', 'WeightT', '0.02',&
       wgtT)
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'IteraMax', '1000',&
       ITMAX)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', 'TolError', &
       '1.0e-10', ETOL)
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'WriteStep', '10',&
       iwstep)
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'JacobianStep', '5',&
       ijstep)
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'JacobianStepStart', &
       '50', ijstep_start)
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'MaxFilterItera', &
       '100', ifltmax)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', &
       'JacobianPercentage',&
       '1.0e-5', XPER)
  CALL SCANINIREAL(lfile, inifile, 'Parameters', &
       'JacobianTolerance', &
       '1.0e-12', XTOL)
  CALL SCANINIINT(lfile, inifile, 'Parameters', 'JacobianTempStep', &
       '10', ijtstep)

  ixsize = NSP*NZ

  ALLOCATE(abc(NZ*3))
  ALLOCATE(xi(NZ))
  ALLOCATE(tmp(ixsize))
  ALLOCATE(Z(NZ+2))
  ALLOCATE(rho(NZ))
  ALLOCATE(yomega(ixsize))
  ALLOCATE(cp(NZ))
  ALLOCATE(x(ixsize))
  ALLOCATE(IPIV(ixsize))
  ALLOCATE(F(ixsize))
  ALLOCATE(wrk(ixsize))
  ALLOCATE(weight(ixsize))
  ALLOCATE(J(ixsize*ixsize))

! Generate mixture fraction grid
! Create alpha, beta, gama derivative coefficients
  CALL FLAMELET_GRID(NZ, XIST, rg, Z)

  IF ( iload_ini .EQ. 1 ) THEN
!    Read Flamelet
     CALL FLAMELET_READ('flamelet-ini.dat', NZ, NSP, Z(2), x)
  ELSE
!    Inititalize Data and solution
     CALL FLAMELET_INI(NZ, Z(2), x)
  ENDIF

  Z(1) = C_0_R
  Z(NZ+2) = C_1_R
  CALL FLAMELET_ABC(NZ, Z, abc)

! Compute Xi
  CALL FLAMELET_XI(NZ, xi0, XIST, Z(2), xi)

! Write Solution
  OPEN(UNIT=23, FILE='flamelet-xi.dat', STATUS='unknown')
  DO i=1, NZ
     WRITE(23,121) Z(i+1), xi(i)
  ENDDO
  CLOSE(23)
121 FORMAT(2(1X,E12.5E3))

! Boundary Conditions
  DO is=1, NSP
     YBCOX(is) = C_0_R
     YBCFUEL(is) = C_0_R
     lewis(is) = schmidt(is)/prandtl
  ENDDO
! Set fixed boundary conditions
  lewis(NSP) = C_1_R
  YBCOX(2) = YOX
  YBCFUEL(1) = YFUEL

  RHO_CST = MRATIO*p_init
  WOX = C_1_R/(WREF*(YOX/WGHT(2)+(C_1_R-YOX)/WGHT(NSP)))
  WFUEL = C_1_R/(WREF*(YFUEL/WGHT(1)+(C_1_R-YFUEL)/WGHT(NSP)))
  TOX = RHO_CST*WOX/(mean_rho - C_05_R*delta_rho)
  TFUEL = RHO_CST*WFUEL/(mean_rho + C_05_R*delta_rho)
  YBCOX(NSP) = TOX
  YBCFUEL(NSP) = TFUEL

! Do not update the temperature as fast
  DO n=1, (NSP-1)*NZ
     weight(n) = wgtY
  ENDDO
  DO n=(NSP-1)*NZ+1, NSP*NZ
     weight(n) = wgtT
  ENDDO

! Newton-Rapson loop
  imax = NZ
  jmax = 1
  kmax = 1
  kmax_total = 1
  ERROR = ETOL*C_2_R
  IT = 1

  PRINT *, 'Damkohler number = ', damkohler(inb_scal)
  PRINT *, 'Temp(OX)         = ', TOX
  PRINT *, 'Temp(FUEL)       = ', TFUEL

  CALL FLAMELET_F(i1, i1, NZ, YBCOX, YBCFUEL, lewis, abc, &
       sclv, Z(2), rho, cp, xi, yomega, x, F, tmp)

! Write Solution
  CALL FLAMELET_WRITE('flamelet-0.dat', NZ, NSP, Z(2), x)
  CALL FLAMELET_WRITE('yomega-0.dat', NZ, NSP, Z(2), &
       yomega)

  iomega = 1
  itemp = 0

  DO WHILE ( ERROR .GT. ETOL )

     IF ( IT .GT. ijstep_start ) THEN
        IF ( MOD(IT,ijstep) .EQ. 0 ) THEN
           ijac = 1
        ELSE
           ijac = 0
        ENDIF
     ELSE
        ijac = 1
     ENDIF

     IF ( MOD(IT,ijtstep) .EQ. 0 .AND. wgtT .GT. C_0_R ) THEN
        itemp = 1
     ELSE
        itemp = 0
     ENDIF

!    Assemble function and jacobian
     CALL FLAMELET_ASSEMBLE(ijac, iomega, itemp, NZ, YBCOX, &
          YBCFUEL, XPER, XTOL,&
          lewis, abc, Z(2), rho, cp, xi, yomega, x, F, J, wrk, tmp)

!    Solve system of equations
!     IF ( ijac .EQ. 1 ) THEN
#ifdef HAS_LAPACK
     CALL DGESV(ixsize, i1, J, ixsize, IPIV, F, ixsize, INFO)
#else
     CALL IO_WRITE_ASCII(efile, 'You need LAPACK to run this program')
     STOP
#endif
!     ELSE
!        CALL DGETRS('No transpose', ixsize, i1, J, ixsize, IPIV, F, 
! $           ixsize, INFO)
!     ENDIF

     IF ( INFO .NE. 0 ) THEN
        WRITE(name,*) IT; name = 'flamelet-'//TRIM(ADJUST(IT))//'.dat'
        CALL FLAMELET_WRITE(name, NZ, NSP, Z(2), x)
        WRITE(*,*) 'Error in linear solver: ', INFO
        WRITE(*,*) 'Saving restart file: ', IT
        STOP
     ENDIF

!    Check convergence error and update unknown vector
     ERROR = C_0_R
     in = 0
     DO n=1, ixsize
        IF ( weight(n) .GT. C_0_R ) THEN
           in = in + 1
           ERROR = ERROR + ABS(F(n)) !/(ETOL+ABS(x(n)))
           x(n) = x(n) + weight(n)*F(n)
!          Ensure positive values
           x(n) = MAX(x(n), C_0_R)
        ENDIF
     ENDDO

     ERROR = ERROR/M_REAL(in)

     IF ( MOD(IT,iwstep) .EQ. 0 ) THEN
        WRITE(name,*) IT; name = 'flamelet-'//TRIM(ADJUST(IT))//'.dat'
        CALL FLAMELET_WRITE(name, NZ, NSP, Z(2), x)
        WRITE(name,*) IT; name = 'yomega-'//TRIM(ADJUST(IT))//'.dat'
        CALL FLAMELET_WRITE(name, NZ, NSP, Z(2), yomega)
        WRITE(name,*) IT; name = 'flamelet-post'//TRIM(ADJUST(IT))//'.dat'
        CALL FLAMELET_POST(name, NZ, Z(2), x, lewis, xi)
     ENDIF

!     IF ( IT .LT. ifltmax ) THEN
!        DO i=1,10
!           CALL FLAMELET_FILTER(NZ, NSP, x, tmp)
!        ENDDO
!     ENDIF

     TMAX=C_0_R
     in = 0
     DO n=1, NZ
        IF ( x(n+(NSP-1)*NZ) .GT. TMAX ) THEN
           TMAX = x(n+(NSP-1)*NZ)
           in = n
        ENDIF
     ENDDO



     WRITE(*,12) IT, ERROR, TMAX

     IT = IT + 1
!    Check max iterations
     IF ( IT .GT. ITMAX ) THEN
        WRITE(*,*) 'Maximum iterations reached'
        STOP
     ENDIF
  ENDDO

12 FORMAT(1(I4,1X),2(2X,E12.5E3))

! Write final solution
  CALL FLAMELET_WRITE('flamelet.dat', NZ, NSP, Z(2), x)

  CALL DNS_END(0)

  STOP
END PROGRAM FLAMELET
