SUBROUTINE FLAMELET_F(iomega, isclv, NZ, YBCOX, YBCFUEL, lewis, &
     abc, sclv, z, rho, cp, xi, yomega, x, F, tmp)

  IMPLICIT NONE

#include "types.h"
#include "common.h"

  TINTEGER iomega, isclv
  TINTEGER NZ

  TREAL YBCOX(*), YBCFUEL(*), lewis(*)

  TREAL x(NZ,*)
  TREAL F(NZ,*)
  TREAL z(*)
  TREAL abc(NZ,3)
  TREAL sclv(*)
  TREAL xi(*), rho(*), cp(*), tmp(*)

  TINTEGER i, is, icp, im
  TREAL YV(3)
  TREAL yomega(NZ,*)
  TREAL RHO_CST, r0, r1
  TINTEGER i1

  RHO_CST = MRATIO*p_init
  r0 = C_0_R
  r1 = C_1_R
  i1 = 1

  DO i=1, NZ
     !    Pass species to YMASS vector
     YMASS(NSP) = C_1_R
     DO is=1, NSP-1
        YMASS(is) = x(i,is)
        YMASS(NSP) = YMASS(NSP)-YMASS(is)
     ENDDO

     !    Compute mean molecular weight of mayor species
     WMEAN = C_0_R
     DO is=1, NSP
        WMEAN = WMEAN + YMASS(is)/WGHT(is)
     ENDDO
     WMEAN = C_1_R/(WMEAN*WREF)

     cp(i) = C_0_R
     DO is=1, NSP
        IF ( x(i,NSP) .LT. THERMO_TLIM(3,is) ) THEN
           im = 2
        ELSE
           im = 1
        ENDIF
        CHW_CP = C_0_R
        DO icp=NCP_CHEMKIN, 1, -1
           CHW_CP = CHW_CP*x(i,NSP) &
                + THERMO_AI(icp,im,is)
        ENDDO
        cp(i) = cp(i) + CHW_CP*YMASS(is)
     ENDDO

     rho(i) = RHO_CST*WMEAN/x(i,NSP)
  ENDDO

  ! Erase arrays
  F(:,1:NSP) = C_0_R
  tmp(1:NZ) = C_1_R

  ! Compute omega
  IF ( iomega .EQ. 1 ) THEN
     yomega(:,1:NSP) = C_0_R
     CALL CHEM_PETERS1988(rho, x(1,NSP), tmp, x, yomega, tmp)
     DO is=1, NSP-1
        DO i=1, NZ
           yomega(i,is) = yomega(i,is)*C_2_R/(rho(i)*xi(i))
        ENDDO
     ENDDO

     !     DO is=1, NSP-1
     !        DO i=1, NZ
     !        IF ( is .LT. 3 ) THEN
     !           cst1 = -1.0e5
     !        ELSE
     !           cst1 = 1.0e5
     !        ENDIF
     !        yomega(i,is) = cst1*rho(i)*x(i,1)*x(i,2)
     ! $           *EXP(-10.0e0/x(i,NSP))
     !        ENDDO
     !     ENDDO
  ENDIF

  ! Loop all variables
  DO is=1, NSP

     !    Loop points in grid
     DO i=1, NZ

        !       Assemble F
        YV(1) = x(i+1,is)
        YV(2) = x(i,is)
        YV(3) = x(i-1,is)
        !       BC 
        IF ( i .EQ. 1 ) THEN
           YV(3) = YBCOX(is)
        ELSE IF ( i .EQ. NZ ) THEN
           YV(1) = YBCFUEL(is)
        ENDIF

        F(i,is) = F(i,is) + &
             (abc(i,1)*YV(1)+abc(i,2)*YV(2)&
             +abc(i,3)*YV(3))/lewis(is)

        IF ( is .LT. NSP ) THEN

           F(i,is) = F(i,is)+yomega(i,is)

           !          Enthalpy
           IF ( x(i,NSP) .LT. THERMO_TLIM(3,is) ) THEN
              im = 2
           ELSE
              im = 1
           ENDIF
           CHW_HI = C_0_R
           DO icp=NCP_CHEMKIN, 1, -1
              CHW_HI = CHW_HI*x(i,NSP)&
                   + THERMO_AI(icp,im,is)/M_REAL(icp)
           ENDDO
           CHW_HI = CHW_HI*x(i,NSP) + THERMO_AI(6,im,is)
           F(i,NSP) = F(i,NSP) - yomega(i,is)*CHW_HI/cp(i)

        ENDIF

     ENDDO

  ENDDO

  IF ( isclv .EQ. 1 ) THEN
     DO is=1, NSP
        sclv(is) = C_0_R
        DO i=1, NZ
           sclv(is) = MAX(sclv(is),ABS(F(i,is)))
        ENDDO
        IF ( sclv(is) .EQ. C_0_R ) THEN
           sclv(is) = C_1_R
        ENDIF
     ENDDO
  ENDIF

  DO is=1,NSP
     DO i=1, NZ
        F(i,is) = F(i,is)/sclv(is)
     ENDDO
  ENDDO

#ifdef _DEBUG
  DO is=1, NSP
     WRITE(*,31) 'F(', is, ')'
     DO i=1, NZ, 5
        WRITE(*,32) (F(i+im,is),im=0,4)
     ENDDO
  ENDDO
31 FORMAT(A2,I3,A1)
32 FORMAT(5(E12.5E3,1X))
  STOP
#endif

  RETURN
END SUBROUTINE FLAMELET_F
