module mod_DEle
  
  implicit none
  save

  !     arrays
  double precision, dimension(:,:,:,:), allocatable :: dEleTraj
  integer,          dimension(:),       allocatable :: dEleNoTraj
  integer,          dimension(:,:),     allocatable :: dEleTrajSize
  integer,          dimension(:,:),     allocatable :: subdomainX, subdomainY, subdomainZ
  logical,          dimension(:,:,:),   allocatable :: dEleTrajMark

  !     output files
  character(len=30) :: fo_dEleInfo, fo_dEleTraj
  character(len=30) :: fi_dEleInfo

  parameter( fo_dEleInfo   = 'output/dEle.info',   &
             fo_dEleTraj   = 'output/dEle.trajs')
  parameter( fi_dEleInfo   = 'input/dEle.info' )

contains

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/30 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Load element data from file dEle.info and allocate basic data
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine loadDEleInfo(ielem_vec)
    use mod_data

    integer :: ielem_vec(*)

    !------------------------------------------------------------------------
    integer :: alloc_err=0, io_eof=0
    integer :: ielem_loc, dE, k0

    !########################################################################
    ! allocate basic data
    allocate(dEleMinMax(dEleSets_nodEles, 2, 3),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *dEleMinMax*. ERROR', alloc_err
    dEleMinMax(:,:,:)=0
    
    allocate(dEleNoTraj(dEleSets_nodEles ),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *dEleNoTraj*. ERROR', alloc_err
    dEleNoTraj(:)=0

    allocate(subdomainX(dEleSets_nodEles,2),STAT=alloc_err) ! subdomainX(elementId,minX/maxX)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for  *subdomainX*. ERROR', alloc_err
    subdomainX(:,:)=0

    allocate(subdomainY(dEleSets_nodEles,2),STAT=alloc_err) ! subdomainY(elementId,minY/maxY)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *subdomainY*. ERROR', alloc_err
    subdomainY(:,:)=0

    allocate(subdomainZ(dEleSets_nodEles,2),STAT=alloc_err) ! subdomainZ(elementId,minZ/maxZ)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *subdomainZ*. ERROR', alloc_err
    subdomainZ(:,:)=0

    !########################################################################
    ! read information about dissipation elements in dEle.info and fill
    ! previous arrays.
    ! format like in routine saveDEleInfo()
    !########################################################################
    open(unit=61, file=fi_dEleInfo, err=51, status='old', form='formatted', action='READ')
    rewind(61)
    do k0=1,7 ! advance record
       read(61,*) 
    enddo

    dE=0
21  read(61,*,iostat=io_eof)
    read(61,'(i6)',iostat=io_eof) ielem_loc
    if ( io_eof .eq. 0 ) then
       do k0 = 1,dEleSets_nodEles
          if ( ielem_loc .eq. ielem_vec(k0) ) then
             dE=dE+1
             read(61,'(i4,i4,i4)',iostat=io_eof) &
                  subdomainX(dE,1),subdomainY(dE,1),subdomainZ(dE,1)
             read(61,'(i4,i4,i4)',iostat=io_eof) &
                  subdomainX(dE,2),subdomainY(dE,2),subdomainZ(dE,2)
             read(61,*,iostat=io_eof) &
                  dEleMinMax(dE,1,1),dEleMinMax(dE,1,2),dEleMinMax(dE,1,3)
             read(61,*,iostat=io_eof) &
                  dEleMinMax(dE,2,1),dEleMinMax(dE,2,2),dEleMinMax(dE,2,3)
             read(61,'(i10)',iostat=io_eof) dEleNoTraj(dE)
             if(dEleSets_maxNoDETraj < dEleNoTraj(dE)) dEleSets_maxNoDETraj = dEleNoTraj(dE)
             exit
          endif
       enddo
       if ( k0 .gt. dEleSets_nodEles ) then ! advance record if nothing found
          read(61,*,iostat=io_eof) 
          read(61,*,iostat=io_eof) 
          read(61,*,iostat=io_eof) 
          read(61,*,iostat=io_eof) 
          read(61,*,iostat=io_eof) 
       endif
       goto 21
    endif

    close(61)

    ! reset
    dEleNoTraj(:)=0

    ! fixed number of trajectories
    if ( dEleSets_fixNoDETraj .gt. 0 ) &
         dEleSets_maxNoDETraj=MIN(dEleSets_maxNoDETraj,dEleSets_fixNoDETraj)

    return

51  write(*,*) 'Could not OPEN ',fi_dEleInfo, ' ... exiting'
    stop

  end subroutine loadDEleInfo

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/20 - J. H. Goebbert
  !#              Created
  !# 2007/11/30 - J.P. Mellado
  !#              Part of it moved to loadDEleInfo
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Allocation of (possibly) big array dEleTraj, dEleTrajSize and dEleTrajMark
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine initDEle()
    use mod_Data

    integer :: alloc_err=0

    ! dEleTraj-array
    allocate(dEleTraj(dEleSets_nodEles,      &
                      dEleSets_maxNoDETraj,  &
                      dEleSets_noPtsPerTraj, &
                      dEleSets_ipCondField+dEleSets_noMapField),STAT=alloc_err)
    if( (alloc_err.NE.0) ) then
       write(*,*) 'Not enough memory for *dEleTraj*. ERROR', alloc_err
       write(*,*) 'Number of elements .....:', dEleSets_nodEles
       write(*,*) 'Number of trajectories .:', dEleSets_maxNoDETraj
       write(*,*) 'Number of points .......:', dEleSets_noPtsPerTraj
       stop
    endif
    dEleTraj(:,:,:,:)=0
    
    ! dEleTrajSize-array
    allocate(dEleTrajSize(dEleSets_nodEles,     &
                          dEleSets_maxNoDETraj),STAT=alloc_err)
    if( (alloc_err.NE.0) ) then
       write(*,*) 'Not enough memory for *dEleTrajSize*. ERROR', alloc_err
       stop
    endif
    dEleTrajSize(:,:)=0

    ! array marking if the ijk trajectories already stored
    allocate(dEleTrajMark(ndimx,ndimy,ndimz))
    dEleTrajMark(:,:,:) = .false.

    return

  end subroutine initDEle

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/20 - J. H. Goebbert
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine finishDEle()

    if(allocated(dEleTraj))     deallocate(dEleTraj)
    if(allocated(dEleNoTraj))   deallocate(dEleNoTraj)
    if(allocated(dEleTrajSize)) deallocate(dEleTrajSize)

  end subroutine finishDEle

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/20 - J. H. Goebbert
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Calculate elements and store data into disk
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !# control flag    In    Type of elements
  !#                       +1 Internal elements
  !#                       -1 External (boundary) elements
  !#
  !########################################################################
  subroutine saveDEleInfo( numends_loc, signs, points, numbering, control_flag )
    use mod_data

    integer,                                          intent(inout) :: numends_loc
    logical,          dimension(:),      allocatable, intent(inout) :: signs
    double precision, dimension(:,:),    allocatable, intent(inout) :: points
    integer(type_numbering), dimension(:,:,:,:),allocatable, intent(inout) :: numbering
    integer,                                          intent(in)    :: control_flag

    !------------------------------------------------------------------------
    integer :: localMaxId, localMinId
    integer :: numpairs, numpairsmax, numpairs_lower, numpairs_upper
    integer :: i,j,k,k0, k0_loc
    integer :: alloc_err
    parameter(numpairsmax=100000)

    double precision :: pSign=0, pimage
    character(len=30):: fname_loc

    integer, dimension(:,:),   allocatable :: pairing   ! elementId-maxId/minId connnections
    integer, dimension(:,:,:), allocatable :: note	! elementId a ijk-trajectory belongs to
    integer, dimension(:),     allocatable :: sort_pairs! ordering array pairing

    !########################################################################
    allocate(pairing(3,numpairsmax),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *pairing*. ERROR', alloc_err
    pairing(:,:)=0

    allocate(note(ndimx,ndimy,ndimz),STAT=alloc_err)   ! 128^3=> 8MB; 256^3=> 64MB; 512^3=> 512MB
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *note*. ERROR', alloc_err
    note(:,:,:)=0

    allocate(subdomainX(numpairsmax,2),STAT=alloc_err) ! subdomainX(elementId,minX/maxX) 781 kB
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for  *subdomainX*. ERROR', alloc_err
    subdomainX(:,:)=0

    allocate(subdomainY(numpairsmax,2),STAT=alloc_err) ! subdomainY(elementId,minY/maxY)  781 kB
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *subdomainY*. ERROR', alloc_err
    subdomainY(:,:)=0

    allocate(subdomainZ(numpairsmax,2),STAT=alloc_err) ! subdomainZ(elementId,minZ/maxZ)  781 kB
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *subdomainZ*. ERROR', alloc_err
    subdomainZ(:,:)=0

    !------------------------------------------------------------------------
    numpairs=0

    !########################################################################
    ! define elements
    ! assign each grid point to an element 
    ! save the subdomain of each element
    !########################################################################
    write(*,*) 'searching for dissipation elements...'
    do i=subdomain(1,1),subdomain(1,2)
       write(*,*) 'i=',i
       do k=subdomain(3,1),subdomain(3,2)
          do j=subdomain(2,1),subdomain(2,2)

             ! Shear layer cropping
             if( (j.lt.jmin_loc(i,k)).or.(j.gt.jmax_loc(i,k)) ) goto 35

             ! control flag = 1 for internal elements
             ! control flag =-1 for external elements (superlayer,boundary)
             if(        ( numbering(i,j,k,1).gt.0 .or. numbering(i,j,k,2).gt.0 ) &
                  .and. ( numbering(i,j,k,1)*numbering(i,j,k,2)*control_flag .gt. 0 ) ) then

                localMaxId=numbering(i,j,k,1)
                localMinId=numbering(i,j,k,2)

                ! first pair
                if(numpairs.eq.0) then
                   numpairs=1
                   pairing(1,1)=localMinId
                   pairing(2,1)=localMaxId
                   pairing(3,1)=1
                   note(i,j,k)=1

                   subdomainX(1,1)=i
                   subdomainX(1,2)=i
                   subdomainY(1,1)=j
                   subdomainY(1,2)=j
                   subdomainZ(1,1)=k
                   subdomainZ(1,2)=k

                   goto 35	! exit k-loop

                ! second pair to last pair
                else
                   do k0=1,numpairs ! search for an existing pair
                      if(localMinId.eq.pairing(1,k0) .and. localMaxId.eq.pairing(2,k0)) then
                         note(i,j,k)=k0
                         pairing(3,k0)=pairing(3,k0)+1 ! found new grid-point for this pair

                         if(i.lt.subdomainX(k0,1)) subdomainX(k0,1)=i
                         if(i.gt.subdomainX(k0,2)) subdomainX(k0,2)=i
                         if(j.lt.subdomainY(k0,1)) subdomainY(k0,1)=j
                         if(j.gt.subdomainY(k0,2)) subdomainY(k0,2)=j
                         if(k.lt.subdomainZ(k0,1)) subdomainZ(k0,1)=k
                         if(k.gt.subdomainZ(k0,2)) subdomainZ(k0,2)=k

                         go to 35 ! exit k-loop
                      endif
                   enddo
                endif

                ! no existing pair found - create a new one
                numpairs=numpairs+1
                if (numpairs.GT.numpairsmax) THEN
                   WRITE(*,*) 'Increase numpairsmax',numpairsmax
                ENDIF
                pairing(1,numpairs)=localMinId
                pairing(2,numpairs)=localMaxId
                pairing(3,numpairs)=1
                note(i,j,k)=numpairs

                subdomainX(numpairs,1)=i
                subdomainX(numpairs,2)=i
                subdomainY(numpairs,1)=j
                subdomainY(numpairs,2)=j
                subdomainZ(numpairs,1)=k
                subdomainZ(numpairs,2)=k

             endif
35        enddo	! k-loop
       enddo  	! j-loop
    enddo	! i-loop

    !########################################################################
    ! sort elements
    !########################################################################
    allocate(sort_pairs(numpairs),STAT=alloc_err)
    if( (alloc_err.NE.0) ) write(*,*) 'Not enough memory for *sort_pairs*. ERROR', alloc_err

    ! external (boundary) elements into lower superlayer and upper superlayer
    if ( control_flag .EQ. -1 ) then
       numpairs_lower=0
       do k0 = 1,numpairs
          if ( pairing(1,k0) .gt. 0 ) then ! maximum lays outside => lower layer
             numpairs_lower=numpairs_lower+1
             sort_pairs(numpairs_lower)=k0
          endif
       enddo

       numpairs_upper=0
       do k0 = 1,numpairs
          if ( pairing(2,k0) .gt. 0 ) then ! minimum lays outside => upper layer
             numpairs_upper=numpairs_upper+1
             sort_pairs(numpairs_lower+numpairs_upper)=k0
          endif
       enddo
       
       if ( numpairs_upper+numpairs_lower .ne. numpairs ) then
          write(*,*) 'Element ordering failed. ',numpairs_lower,numpairs_upper
          stop
       endif

    ! default for internal elements
    else 
       do k0 = 1,numpairs
          sort_pairs(k0)=k0
       enddo

    endif

    !########################################################################
    ! save information about dissipation elements
    !########################################################################
    !------------------------------------------------------------------------
    ! Data file
    !------------------------------------------------------------------------
    if ( control_flag .EQ. -1 ) then
       fname_loc = 'output/dEle.ext.info'
    else
       fname_loc = 'output/dEle.int.info'
    endif
    write(*,*) '         ...writing info to ', fname_loc

    open(unit=61,file=fname_loc,err=101,status='unknown',form='formatted',action='WRITE')
    rewind(61)

    if ( control_flag .EQ. -1 ) then     ! into lower superlayer and upper superlayer
       write(61,*) 'number of dissipation elements ',numpairs, &
                               ', lower/upper ',numpairs_lower, numpairs_upper
    else
       write(61,*) 'number of dissipation elements',numpairs
    endif
    write(61,*) '# eId'
    write(61,*) '#  subdomain minXYZPt'
    write(61,*) '#  subdomain maxXYZPt'
    write(61,*) '#  xyzCoords of element minimum'
    write(61,*) '#  xyzCoords of element maximum'
    write(61,*) '#  number of gridpoints belonging to element'
    do k0=1, numpairs
       k0_loc=sort_pairs(k0) ! sorting data
       write(61,*)
       write(61,'(i6)') k0
       write(61,'(i4,i4,i4)') subdomainX(k0_loc,1),subdomainY(k0_loc,1),subdomainZ(k0_loc,1)
       write(61,'(i4,i4,i4)') subdomainX(k0_loc,2),subdomainY(k0_loc,2),subdomainZ(k0_loc,2)
       if ( pairing(1,k0_loc) .gt. 0 ) then ! minimum lays inside
          write(61,*) &
               points(1,pairing(1,k0_loc)),points(2,pairing(1,k0_loc)),points(3,pairing(1,k0_loc))
       else                                 ! minimum lays outside
          pimage=REAL(ndimy)
          write(61,*) &
               points(1,pairing(2,k0_loc)),pimage,points(3,pairing(2,k0_loc))
       endif
       if ( pairing(2,k0_loc) .gt. 0 ) then ! maximum lays inside
          write(61,*) &
               points(1,pairing(2,k0_loc)),points(2,pairing(2,k0_loc)),points(3,pairing(2,k0_loc))
       else                                 ! maximum lays outside
          pimage=1.
          write(61,*) points(1,pairing(1,k0_loc)),pimage,points(3,pairing(1,k0_loc))
       endif
       write(61,'(i10)') pairing(3,k0_loc)
    enddo
    close(61)

    !------------------------------------------------------------------------
    ! OpenDx file with the points and connectivities
    !------------------------------------------------------------------------
    if ( control_flag .EQ. -1 ) then ! external elements
       fname_loc = 'output/extrPts.ext.dx'
    else
       fname_loc = 'output/extrPts.int.dx'
    endif
    write(*,*) '         ...writing ',numends_loc,' extrem points to dx-file'

    open(unit=63, file=fname_loc, err=101, status='unknown', form='formatted', action='write')
    rewind(63)

    ! extrem point coordinates
    if ( control_flag .eq. -1 ) then ! external elements. Projection on the upper/lower planes
       write(63,*) &
            'object 1 class array type float rank 1 shape 3 items ',3*numends_loc,' data follows'
    else
       write(63,*) &
            'object 1 class array type float rank 1 shape 3 items ',numends_loc,' data follows'
    endif
    do k0=1,numends_loc
       write(63,'(es14.4,es14.4,es14.4)') &
            points(1,k0)*deltax, points(2,k0)*deltay, points(3,k0)*deltaz
    enddo
    if ( control_flag .eq. -1 ) then ! external elements. Image point offset of 5 units 
       do k0=1,numends_loc
          write(63,'(es14.4,es14.4,es14.4)') &
               points(1,k0)*deltax, (points(2,k0)-ndimy/10.)*deltay, points(3,k0)*deltaz
       enddo
       do k0=1,numends_loc
          write(63,'(es14.4,es14.4,es14.4)') &
               points(1,k0)*deltax, (points(2,k0)+ndimy/10.)*deltay, points(3,k0)*deltaz
       enddo
    endif

    write(63,*) 'attribute "dep" string "positions"'
    
    ! extrem point signs
    if ( control_flag .eq. -1 ) then ! external elements. Projection on the upper/lower planes
       write(63,*) &
            'object 2 class array type float rank 0 items ',3*numends_loc,' data follows'
    else
       write(63,*) &
            'object 2 class array type float rank 0 items ',numends_loc,' data follows'
    endif
    do k0=1,numends_loc
       if(signs(k0)) then; pSign=1; else; pSign=-1; endif
       write(63,'(es14.4)') pSign
    enddo
    if ( control_flag .eq. -1 ) then ! external elements. Projection on the upper/lower planes
       pSign=-2
       do k0=1,numends_loc
          write(63,'(es14.4)') pSign
       enddo
       pSign=+2
       do k0=1,numends_loc
          write(63,'(es14.4)') pSign
       enddo
    endif
    write(63,*) 'attribute "dep" string "positions"'

    ! min-max connection for each grid point
    write(63,*) &
         'object 3 class array type int rank 1 shape 2 items ',numpairs,' data follows'
    if ( control_flag .eq. -1 ) then ! external elements. Projection on the upper/lower planes
       do k0=1,numpairs
          ! lower external region, max. is outside, offset=numends_loc
          if ( pairing(2,k0) .eq. -1 ) then
             write(63,*) pairing(1,k0)-1, (pairing(1,k0)-1) + numends_loc
          ! upper external region, min. is outside, offset=2*numends_loc
          else if ( pairing(1,k0) .eq. -1 ) then
             write(63,*) (pairing(2,k0)-1) + 2*numends_loc, pairing(2,k0)-1 
          endif
       enddo
    else
       do k0=1,numpairs
          write(63,*) pairing(1,k0)-1, pairing(2,k0)-1
       enddo
    endif
    write(63,*) 'attribute "element type" string "lines"'
    write(63,*) 'attribute "ref" string "positions"'

    ! finish
    write(63,*) 'object "Connectivity" class field'
    write(63,*) 'component "positions" value 1'
    write(63,*) 'component "sign" value 2'
    write(63,*) 'component "connections" value 3'
    close(63)

    deallocate(pairing)
    deallocate(note)
    deallocate(subdomainX)
    deallocate(subdomainY)
    deallocate(subdomainZ)

    return

101 write(*,*) 'Could not OPEN ',fname_loc, ' ... exiting'
    return

  end subroutine saveDEleInfo

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/20 - J. H. Goebbert
  !#              Created
  !# 2007/11/30 - J.P. Mellado
  !#              Adding mapped fields
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Move data from local trajectory array to general elements array
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine storeDEleTraj(i,j,k,numbering, traject, trajMaxMinIds, traj_err)
    use mod_data

    integer,          intent(in)                                  :: i,j,k
    integer(type_numbering), dimension(:,:,:,:), allocatable, intent(in) :: numbering
    double precision, intent(in)                                  :: traject(-10*ndimx:10*ndimx,*)
    integer,          intent(in)                                  :: trajMaxMinIds(2)
    double precision, intent(in)                                  :: traj_err

    !------------------------------------------------------------------------
    integer          :: eleId,pT,curPt,fieldId
    double precision :: distance, px, py, pz

    !########################################################################
    ! array marking if the ijk trajectories already stored
    if (.not.dEleTrajMark(i,j,k)) then
       dEleTrajMark(i,j,k) = .true.

       do eleId=1, ubound(dEleMinMax,1)
          
          ! shall we store this trajectory?
          if( dEleNoTraj(eleId) .lt. dEleSets_maxNoDETraj ) then

             !------------------------------------------------------------------------
             ! check minimum
             !------------------------------------------------------------------------
             ! external point, minimum lays outside. Use maximum vertical projection
             if ( numbering(i,j,k,2) .lt. 0 ) then 
                pT = trajMaxMinIds(1)
                px = traject(pT,1); py = REAL(ndimy); pz = traject(pT,3)
             ! regular minimum
             else
                pT = trajMaxMinIds(2)
                px = traject(pT,1); py = traject(pT,2); pz = traject(pT,3)
             endif
             distance = dsqrt( (px-dEleMinMax(eleId,1,1))**2     &
                             + (py-dEleMinMax(eleId,1,2))**2     &
                             + (pz-dEleMinMax(eleId,1,3))**2 )
             if( distance.lt.traj_err ) then

                !------------------------------------------------------------------------
                ! check maximum
                !------------------------------------------------------------------------
                ! external point, maximum lays outside. Use minimum vertical projection
                if ( numbering(i,j,k,1) .lt. 0 ) then
                   pT = trajMaxMinIds(2)
                   px = traject(pT,1); py = 1.; pz = traject(pT,3)
                ! regular maximum
                else
                   pT = trajMaxMinIds(1)
                   px = traject(pT,1); py = traject(pT,2); pz = traject(pT,3)
                endif
                distance = dsqrt( (px-dEleMinMax(eleId,2,1))**2     &
                                + (py-dEleMinMax(eleId,2,2))**2     &
                                + (pz-dEleMinMax(eleId,2,3))**2 )
                if( distance.lt.traj_err ) then
                   
                   !------------------------------------------------------------------------
                   ! found a trajectory belonging to a dissipation element which shall be saved
                   !------------------------------------------------------------------------
                   curPt = 0
                   dEleNoTraj(eleId) = dEleNoTraj(eleId)+1
                   do pT=trajMaxMinIds(2), trajMaxMinIds(1)
                      curPt = curPt+1
                      if ( curPt .gt. dEleSets_noPtsPerTraj ) goto 201
                      if ( dEleSets_ipCondField .eq. 4 ) then ! positions
                         dEleTraj(eleId,dEleNoTraj(eleId),curPt,1) = traject(pT,1)*deltax ! x
                         dEleTraj(eleId,dEleNoTraj(eleId),curPt,2) = traject(pT,2)*deltay ! y
                         dEleTraj(eleId,dEleNoTraj(eleId),curPt,3) = traject(pT,3)*deltaz ! z
                      endif
                      ! conditioning field
                      dEleTraj(eleId,dEleNoTraj(eleId),curPt,dEleSets_ipCondField) = &
                           traject(pT,4)
                      ! mapped fields
                      do fieldId = 1,dEleSets_noMapField
                         dEleTraj(eleId,dEleNoTraj(eleId),curPt,dEleSets_ipCondField+fieldId) = &
                              traject(pT,4+fieldId)
                      enddo
                   enddo
                   dEleTrajSize(eleId,dEleNoTraj(eleId)) = curPt
                   
                   if ( dEleSets_fixNoDETraj .gt. 0 .and. &
                        dEleNoTraj(eleId) .eq. dEleSets_maxNoDETraj ) itraj_stop=.true.
                   
                endif
             endif
          
          endif
       enddo

    endif

    return

    ! if not enough space for this trajectory, drop it and continue
201 open(unit=61,file='delements.warning', &
         status='unknown',form='formatted',action='write',position='append')
    write(61,*) 'Not enough dEleSets_noPtsPerTraj at element ',eleId, &
         ' trajectory', dEleNoTraj(eleId)
    close(61)
    dEleNoTraj(eleId) = dEleNoTraj(eleId)-1
    return

  end subroutine storeDEleTraj

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/20 - J. H. Goebbert
  !#              Created
  !# 2007/11/30 - J.P. Mellado
  !#              Adding mapped fields
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Save element/trajectories data to disk
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine saveDEleTraj(fileType)
    use mod_Data

    integer, intent(in) :: fileType

    !------------------------------------------------------------------------
    integer           :: eleId, trajId, ptId, fieldId
    integer           :: noPts, noTrajs
    integer           :: trajSkip, ptSkip
    integer           :: lastPtId
    character(len=30) :: fo_dEleTraj

    !########################################################################
!!$       trajSkip = 10
!!$       ptSkip   = 10
    trajSkip = 1
    ptSkip   = 1
    
    !########################################################################
    ! private txt-format
    !########################################################################
    if (fileType .eq. 1) then
       fo_dEleTraj = 'output/dEleTraj.txt'
       write(*,*) '         ...writing trajectories to private-format file', fo_dEleTraj

       open(unit=61,file=fo_dEleTraj,err=108,status='unknown',form='formatted',action='WRITE')
       rewind(61)

       write(61,*,err=109) dEleSets_nodEles, ' ! number of diss. elements'
       write(61,*,err=109)

       do eleId=1, dEleSets_nodEles ! loop over elements
          write(61,*,err=109) eleId, ' ! diss. element id'
          write(61,*,err=109) dEleTrajSize(eleId,dEleNoTraj(eleId)), ' ! number of trajectories'

          do trajId=1, dEleNoTraj(eleId) ! loop over trajectories
             write(61,*,err=109) trajId, ' ! trajectory id'
             write(61,*,err=109)  dEleTrajSize(eleId,trajId), ' ! number of points'
             do ptId=1, dEleTrajSize(eleId,trajId) ! loop over points on trajectory
                write(61,*,err=109) &
                     (dEleTraj( eleId, trajId, ptId, fieldId), fieldId=1,dEleSets_noMapField+4)
             enddo
          enddo
       enddo
       close(61)

    !########################################################################
    ! ensight txt-format
    !########################################################################
    else if(fileType .eq. 2) then
       fo_dEleTraj = 'output/dEleTraj.case'
       write(*,*) '         ...writing data to ENSIGHT-fortmat file', fo_dEleTraj

       open(unit=61,file=fo_dEleTraj,err=108,status='unknown',form='formatted',action='WRITE')
       rewind(61)

       write(61,fmt=*,err=109) '# Description: dissipation elements'
       write(61,fmt=*,err=109) '# Date       :'
       write(61,fmt=*,err=109) '# Case File  : dissipEle.case'
       write(61,fmt=*,err=109)
       write(61,fmt=*,err=109) 'FORMAT'
       write(61,fmt=*,err=109) 'type: ensight gold'
       write(61,fmt=*,err=109)
       write(61,fmt=*,err=109) 'GEOMETRY'
       write(61,fmt=*,err=109) 'model:  1  dEleTraj.geo_001'
       !write(61,fmt=*,err=109)
       !write(61,fmt=*,err=109) 'VARIABLE'
       !write(61,fmt=*,err=109) 'vector per node:  1 velocity disipEle.velocity_*'
       !write(61,fmt=*,err=109) 'scalar per node:  1 eleId    disipEle.eleId_*'
       !write(61,fmt=*,err=109)
       !write(61,fmt=*,err=109) 'TIME'
       !write(61,fmt=*,err=109) 'time set:          1'
       !write(61,fmt=*,err=109) 'number of steps:  1'
       !write(61,fmt=*,err=109) 'filename numbers:'
       !write(61,fmt=*,err=109) '001'
       !write(61,fmt=*,err=109) 'time values:'
       !write(61,fmt=*,err=109) '0.0'
       !write(61,fmt=*,err=109)

       close(61)
       
       ! write geometry file
       fo_dEleTraj = 'output/dEleTraj.geo_001'
       write(*,*) '         ...writing data to ENSIGHT-fortmat file', fo_dEleTraj

       open(unit=62,file=fo_dEleTraj,err=108,status='unknown',form='formatted',action='WRITE')
       rewind(62)

       write(62,fmt=*,err=109) &
            'This is the 1st description line of the EnSight Gold geometry example'
       write(62,fmt=*,err=109) &
            'This is the 2nd description line of the EnSight Gold geometry example'
       write(62,fmt=*,err=109) 'node id off'
       write(62,fmt=*,err=109) 'element id off'
       !write(62,fmt=*,err=109) 'extents'
       !write(62,fmt='(2E12.5)',err=109) 0.0, 0.0, 0.0
       !write(62,fmt='(2E12.5)',err=109) pi2, pi2, pi2

       do eleId=1, dEleSets_nodEles ! loop over dissipation elements

          write(*,*) eleId, dEleNoTraj(eleId)

          ! part header
          write(62,fmt=*,err=109) 'part'
          write(62,fmt='(I10)',err=109) eleId
          write(62,fmt=*,err=109) 'description line for part 1'
          write(62,fmt=*,err=109) 'coordinates'

          noPts   = 0
          noTrajs = 0
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories for x-coordinate
             noTrajs = noTrajs + 1
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                noPts = noPts + 1
             enddo
          enddo
          write(62,fmt='(I10)',err=109) noPts

          ! part coordinates
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories for x-coordinate
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                write(62,fmt='(E12.5)',err=109) dEleTraj( eleId, trajId, ptId, 1) ! x
             enddo
          enddo
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories for y-coordinate
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                write(62,fmt='(E12.5)',err=109) dEleTraj( eleId, trajId, ptId, 2) ! y
             enddo
          enddo
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories for z-coordinate
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                write(62,fmt='(E12.5)',err=109) dEleTraj( eleId, trajId, ptId, 3) ! z
             enddo
          enddo

          ! part cells
          write(62,fmt=*,err=109) 'bar2'	! cell type
          write(62,fmt='(I10)',err=109) noPts-noTrajs ! number of cells
          lastPtId = 0
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories
             lastPtId = lastPtId+1
             do ptId=1, dEleTrajSize(eleId,trajId)-ptSkip, ptSkip ! loop over points on trajectory
                write(62,fmt='(I10,I10)',err=109) lastPtId, lastPtId+1
                lastPtId = lastPtId+1
             enddo
          enddo

       enddo
       close(62)

    !########################################################################
    ! opendx
    !########################################################################
    else if(fileType .eq. 3) then

       !------------------------------------------------------------------------
       ! Trajectories file
       !------------------------------------------------------------------------
       fo_dEleTraj = 'output/dEleTraj.dx'
       write(*,*) '         ...writing data to OpenDx-fortmat file', fo_dEleTraj

       open(unit=62,file=fo_dEleTraj,err=108,status='unknown',form='formatted',action='write')
       rewind(62)
       
       noPts   = 0
       noTrajs = 0
       do eleId=1, dEleSets_nodEles
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories for x-coordinate
             noTrajs = noTrajs + 1
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                noPts = noPts + 1
             enddo
          enddo
       enddo

       ! point coordinates
       write(62,*) &
            'object 1 class array type float rank 1 shape 3 items ',noPts,' data follows'
       do eleId=1, dEleSets_nodEles
          do trajId=1, dEleNoTraj(eleId), trajSkip
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                write(62,'(es14.4,es14.4,es14.4)') dEleTraj(eleId, trajId, ptId, 1), &
                                                   dEleTraj(eleId, trajId, ptId, 2), &
                                                   dEleTraj(eleId, trajId, ptId, 3)
             enddo
          enddo
       enddo
       write(62,*) 'attribute "dep" string "positions"'
       
       ! scalar values of conditioning field
       write(62,*) &
            'object 2 class array type float rank 0 items ',noPts,' data follows'
       do eleId=1, dEleSets_nodEles
          do trajId=1, dEleNoTraj(eleId), trajSkip
             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                write(62,'(es14.4,es14.4,es14.4)') dEleTraj(eleId, trajId, ptId, 4)
             enddo
          enddo
       enddo
       write(62,*) 'attribute "dep" string "positions"'
       
       ! scalar values of mapped fields
       do fieldId=1, dEleSets_noMapField
          ptId=fieldId+2
          write(62,*) &
               'object ',ptId,' class array type float rank 0 items ',noPts,' data follows'
          do eleId=1, dEleSets_nodEles
             do trajId=1, dEleNoTraj(eleId), trajSkip
                do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                   write(62,'(es14.4,es14.4,es14.4)') dEleTraj(eleId, trajId, ptId, 4+fieldId)
                enddo
             enddo
          enddo
          write(62,*) 'attribute "dep" string "positions"'
       enddo

       ! min-max connection for each grid point
       ptId=dEleSets_noMapField+3
       write(62,*) &
            'object ',ptId,' class array type int rank 1 shape 2 items ',noPts-noTrajs,' data follows'
       lastPtId = 0
       do eleId=1, dEleSets_nodEles
          do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories
             lastPtId = lastPtId+1
             ! loop over points on trajectory: OpenDX counter from 0
             do ptId=1, dEleTrajSize(eleId,trajId)-ptSkip, ptSkip 
                write(62,fmt='(I10,I10)',err=109) (lastPtId)-1, (lastPtId+1)-1
                lastPtId = lastPtId+1
             enddo
          enddo
       enddo
       write(62,*) 'attribute "element type" string "lines"'
       write(62,*) 'attribute "ref" string "positions"'
       
       ! finish
       write(62,*) 'object "trajectories" class field'
       write(62,*) 'component "positions" value 1'
       write(62,*) 'component "conditioning field" value 2'
       do fieldId=1, dEleSets_noMapField
          write(62,*) 'component "',TRIM(ADJUSTL(mappedfield_name(fieldId))),'" value ',fieldId+2
       enddo
       write(62,*) 'component "connections" value ',dEleSets_noMapField+3
       close(62)

       !------------------------------------------------------------------------
       ! extrem point coordinates file
       !------------------------------------------------------------------------
       fo_dEleTraj = 'output/dEleTrajPts.dx'
       write(*,*) '         ...writing data to OpenDx-fortmat file', fo_dEleTraj

       open(unit=63,file=fo_dEleTraj,err=108,status='unknown',form='formatted',action='write')
       rewind(63)

       noPts = 0
       do eleId=1,dEleSets_nodEles
          if ( dEleMinMax(eleId,1,2) .lt. jmax_loc_max ) noPts=noPts+1
          if ( dEleMinMax(eleId,2,2) .gt. jmin_loc_min ) noPts=noPts+1
       enddo
       write(63,*) &
            'object 1 class array type float rank 1 shape 3 items ',noPts,' data follows'

       do eleId=1,dEleSets_nodEles
          if ( dEleMinMax(eleId,1,2) .lt. jmax_loc_max ) then
             write(63,'(es14.4,es14.4,es14.4)') dEleMinMax(eleId,1,1)*deltax, &
                                                dEleMinMax(eleId,1,2)*deltay, &
                                                dEleMinMax(eleId,1,3)*deltaz
          endif
          if ( dEleMinMax(eleId,2,2) .gt. jmin_loc_min ) then
             write(63,'(es14.4,es14.4,es14.4)') dEleMinMax(eleId,2,1)*deltax, &
                                                dEleMinMax(eleId,2,2)*deltay, &
                                                dEleMinMax(eleId,2,3)*deltaz
          endif
       enddo

       write(63,*) 'attribute "dep" string "positions"'
    
       ! finish
       write(63,*) 'object "extrema" class field'
       write(63,*) 'component "positions" value 1'
       close(63)

    endif

    return

108 write(*,*) 'Could not OPEN ',fo_dEleTraj, ' ... exiting'
    return

109 write(*,*) 'Could not WRITE ',fo_dEleTraj, ' ... exiting'
    return

  end subroutine saveDEleTraj

  !########################################################################
  !# HISTORY
  !#
  !# 2007/11/25 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Save only extremal points data to disk in OpenDx format
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine saveDElePts()
    use mod_Data

    !------------------------------------------------------------------------
    integer           :: eleId, noPts
    double precision  :: pSign
    character(len=30) :: fo_dElePts

    !########################################################################
    fo_dElePts = 'output/dEleTrajPts.dx'

    open(unit=63,file=fo_dElePts,err=110,status='unknown',form='formatted',action='write')
    rewind(63)

    noPts = 0
    do eleId=1,dEleSets_nodEles
       if ( dEleMinMax(eleId,1,2) .lt. jmax_loc_max ) noPts=noPts+1
       if ( dEleMinMax(eleId,2,2) .gt. jmin_loc_min ) noPts=noPts+1
    enddo

    ! extrem point positions
    write(63,*) &
         'object 1 class array type float rank 1 shape 3 items ',noPts,' data follows'
    do eleId=1,dEleSets_nodEles
       if ( dEleMinMax(eleId,1,2) .lt. jmax_loc_max ) then
          write(63,'(es14.4,es14.4,es14.4)') dEleMinMax(eleId,1,1)*deltax, &
                                             dEleMinMax(eleId,1,2)*deltay, &
                                             dEleMinMax(eleId,1,3)*deltaz
       endif
       if ( dEleMinMax(eleId,2,2) .gt. jmin_loc_min ) then
          write(63,'(es14.4,es14.4,es14.4)') dEleMinMax(eleId,2,1)*deltax, &
                                             dEleMinMax(eleId,2,2)*deltay, &
                                             dEleMinMax(eleId,2,3)*deltaz
       endif
    enddo
    write(63,*) 'attribute "dep" string "positions"'
    
    ! extrema point signs
    write(63,*) &
         'object 2 class array type float rank 0 items ',noPts,' data follows'
    do eleId=1,dEleSets_nodEles
       if ( dEleMinMax(eleId,1,2) .lt. jmax_loc_max ) then
          pSign = -1
          write(63,'(es14.4)') pSign
       endif
       if ( dEleMinMax(eleId,2,2) .gt. jmin_loc_min ) then
          pSign = +1
          write(63,'(es14.4)') pSign
       endif
    enddo
    write(63,*) 'attribute "dep" string "positions"'

    ! finish
    write(63,*) 'object "extrema" class field'
    write(63,*) 'component "positions" value 1'
    write(63,*) 'component "signs" value 2'
    close(63)

    return

110 write(*,*) 'Could not OPEN ',fo_dElePts, ' ... exiting'
    return
    
  end subroutine saveDElePts

  !########################################################################
  !# HISTORY
  !#
  !# 2007/12/04 - J.P. Mellado
  !#              Created
  !#
  !########################################################################
  !# DESCRIPTION
  !#
  !# Calculate PDFs of mapped fields conditioned on the normalized scalar
  !#
  !########################################################################
  !# ARGUMENTS 
  !#
  !########################################################################
  subroutine pdfsDEleTraj(nz, nbins)
    use mod_Data

    integer           :: nz, nbins

    !------------------------------------------------------------------------
    integer           :: eleId, trajId, ptId, fieldId, sampId
    integer           :: noPts, noTrajs
    integer           :: trajSkip, ptSkip
    character(len=32) :: fname
    character(len=32) :: varname(1)
    integer           :: alloc_err=0
    double precision  :: norm, z1min, z1max, dummy

    double precision                          :: pdf(nbins,dEleSets_noMapField),wrk1d(nbins,2)
    double precision                          :: moments(dEleSets_noMapField,nz,2)
    double precision, dimension(:),   pointer :: sample1
    double precision, dimension(:,:), pointer :: sample2

    integer(1)        :: igate, gate
    
    !########################################################################
    trajSkip = 1
    ptSkip   = 1
    
    !########################################################################
    noPts   = 0
    noTrajs = 0
    do eleId=1, dEleSets_nodEles
       do trajId=1, dEleNoTraj(eleId), trajSkip ! loop over trajectories for x-coordinate
          noTrajs = noTrajs + 1
          do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
             noPts = noPts + 1
          enddo
       enddo
    enddo

    !########################################################################
    ! extract PDF scalar difference
    !########################################################################
    allocate(sample1(noTrajs),STAT=alloc_err)
    if( (alloc_err.NE.0) ) goto 301
    sample1(:)=0.

    sampId = 0
    do eleId=1, dEleSets_nodEles
       do trajId=1, dEleNoTraj(eleId), trajSkip
          sampId=sampId+1
          sample1(sampId) =dEleTraj(eleId,trajId,dEleTrajSize(eleId,trajId),dEleSets_ipCondField)&
                          -dEleTraj(eleId, trajId, 1, dEleSets_ipCondField)
       enddo
    enddo

    varname(1) = 'scalar'
    fname='pdfTr '
    CALL PDF3D_N(fname, varname, 1, 0, itime_loc, rtime_loc, noTrajs, &
         1, 1, 1, nbins, sample1, pdf, wrk1d)

    deallocate(sample1)

    !########################################################################
    ! conditional PDFs
    !########################################################################
    if ( dEleSets_noMapField .gt. 0 ) then

       allocate(sample2(noPts,1+dEleSets_noMapField),STAT=alloc_err)
       if( (alloc_err.NE.0) ) goto 302
       sample2(:,:)=0.

       !------------------------------------------------------------------------
       ! construct the sample
       !------------------------------------------------------------------------
       sampId = 0
       do eleId=1, dEleSets_nodEles
          do trajId=1, dEleNoTraj(eleId), trajSkip
             z1max = dEleTraj(eleId,trajId,dEleTrajSize(eleId,trajId),dEleSets_ipCondField)
             z1min = dEleTraj(eleId,trajId,1,dEleSets_ipCondField)
             norm = 1.d0/( z1max-z1min )

             do ptId=1, dEleTrajSize(eleId,trajId), ptSkip ! loop over points on trajectory
                sampId=sampId+1

                ! Normalize conditioning field
                sample2(sampId,1) = norm* &
                   (min(max(dEleTraj(eleId,trajId,ptId,dEleSets_ipCondField),z1min),z1max)-z1min)

                ! correction if maximum is outside (lower boundary) so that 
                ! normalized scalar goes from 0 in the outer region to 1 in the
                ! turbulent region
                if ( INT(dEleMinMax(eleId,2,2)) .eq. 1 ) then 
                   sample2(sampId,1) = 1.d0 - sample2(sampId,1)
                endif

                ! mapped fields
                do fieldId = 1,dEleSets_noMapField
                   sample2(sampId,fieldId+1) = &
                        dEleTraj(eleId,trajId,ptId,dEleSets_ipCondField+fieldId)
                   if ( INT(dEleMinMax(eleId,2,2)) .eq. 1 .and. & ! symmetry of lower boundary
                        (.not.mappedfield_symmetry(fieldId)) ) then 
                      sample2(sampId,fieldId+1) =-sample2(sampId,fieldId+1)
                   endif
                enddo

             enddo

          enddo
       enddo
       
       !------------------------------------------------------------------------
       ! PDFs
       !------------------------------------------------------------------------
       fname = 'cpdfTr '
       igate = 0 ! no intermittency
       call CPDF3D_N(fname, mappedfield_name, igate, 1, 0, itime_loc, rtime_loc, noPts, &
            1, 1, dEleSets_noMapField, nz, nbins, gate, sample2(1,1), sample2(1,2), pdf, wrk1d)
       
       !------------------------------------------------------------------------
       ! Conditional averages: first two moments
       !------------------------------------------------------------------------
       WRITE(fname,*) itime_loc; fname = 'cavgTr'//TRIM(ADJUSTL(fname))
       igate = 0 ! no intermittency
       call CAVG3D_N(fname, 1, mappedfield_name, igate, itime_loc, rtime_loc, noPts, &
            1, 1, dEleSets_noMapField, nz, 2, dummy, dummy, gate, sample2(1,1), sample2(1,2), &
            moments, wrk1d)
              
       deallocate(sample2)

    endif

    return

301 write(*,*) 'Not enough memory for *sample1*', alloc_err
    return

302 write(*,*) 'Not enough memory for *sample2*', alloc_err
    write(*,*) 'Space required (points)', noPts*(1+dEleSets_noMapField)
    return

400 format(I3,1X,I3,1X,E10.3E3,20(1X,E10.3E3),20(1X,E10.3E3))

  end subroutine pdfsDEleTraj

end module mod_DEle
